@@ -21,9 +21,8 @@
 #ifndef __GST_A52DEC_H__
 #define __GST_A52DEC_H__
 
-#include <config.h>
 #include <gst/gst.h>
-#include <gst/gstbytestream.h>
+#include <gst/bytestream/bytestream.h>
 
 G_BEGIN_DECLS
 
@@ -63,8 +62,6 @@ struct _GstA52Dec {
 
   GstClockTime	last_ts;
   GstClockTime	current_ts;
-
-  GstCaps	*streaminfo;
 };
 
 struct _GstA52DecClass {
