@@ -29,22 +29,22 @@
 
 #define NTSC_HEIGHT 480
 #define NTSC_BUFFER 120000
+#define NTSC_FRAMERATE 29.997
+
 #define PAL_HEIGHT 576
 #define PAL_BUFFER 144000
+#define PAL_FRAMERATE 25.0
 
 /* The ElementDetails structure gives a human-readable description
  * of the plugin, as well as author and version data.
  */
-static GstElementDetails dvdec_details = {
+static GstElementDetails dvdec_details = GST_ELEMENT_DETAILS (
   "DV (smpte314) decoder plugin",
-  "Codec/Video/Decoder",
-  "LGPL",
+  "Codec/Decoder/Video",
   "Uses libdv to decode DV video (libdv.sourceforge.net)",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@tvd.be>",
-  "(C) 2001-2002",
-};
+  "Wim Taymans <wim.taymans@tvd.be>"
+);
 
 
 /* These are the signals that this element can fire.  They are zero-
@@ -71,128 +71,88 @@ enum {
  * can have.  They can be quite complex, but for this dvdec plugin
  * they are rather simple.
  */
-GST_PAD_TEMPLATE_FACTORY (sink_temp,
-  "sink",
+static GstStaticPadTemplate sink_temp =
+GST_STATIC_PAD_TEMPLATE 
+(
+  "sink" , 
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "dv_dec_sink",
-    "video/x-dv",
-    "systemstream", GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS (
+    "video/x-dv, systemstream = (boolean) true"
   )
-)
+);
 
-
-GST_PAD_TEMPLATE_FACTORY (video_src_temp,
+static GstStaticPadTemplate video_src_temp =
+GST_STATIC_PAD_TEMPLATE
+(
   "video",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "dv_dec_src",
-    "video/x-raw-yuv",
-    "format",   	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','U','Y','2')),
-    "width",    	GST_PROPS_INT (720),
-    "height",  		GST_PROPS_LIST (
-			  GST_PROPS_INT (NTSC_HEIGHT),
-			  GST_PROPS_INT (PAL_HEIGHT)
-			),
-    "framerate",	GST_PROPS_LIST (
-			  GST_PROPS_FLOAT (25.),
-			  GST_PROPS_FLOAT (30/1.001)
-			)
-  ),
-  GST_CAPS_NEW (
-    "dv_dec_src",
-    "video/x-raw-rgb",
-    "bpp", 		GST_PROPS_INT(32),
-    "depth", 		GST_PROPS_INT(32),
-    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
-    "red_mask", 	GST_PROPS_INT(0x000000ff),
-    "green_mask", 	GST_PROPS_INT(0x0000ff00),
-    "blue_mask", 	GST_PROPS_INT(0x00ff0000),
-    "width",     	GST_PROPS_INT (720),
-    "height",  		GST_PROPS_LIST (
-			  GST_PROPS_INT (NTSC_HEIGHT),
-			  GST_PROPS_INT (PAL_HEIGHT)
-			),
-    "framerate",	GST_PROPS_LIST (
-			  GST_PROPS_FLOAT (25.),
-			  GST_PROPS_FLOAT (30/1.001)
-			)
-  ),
-  GST_CAPS_NEW (
-    "dv_dec_src",
-    "video/x-raw-rgb",
-    "bpp", 		GST_PROPS_INT(24),
-    "depth", 		GST_PROPS_INT(24),
-    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
-    "red_mask", 	GST_PROPS_INT(0x0000ff),
-    "green_mask", 	GST_PROPS_INT(0x00ff00),
-    "blue_mask", 	GST_PROPS_INT(0xff0000),
-    "width",     	GST_PROPS_INT (720),
-    "height",  		GST_PROPS_LIST (
-			  GST_PROPS_INT (NTSC_HEIGHT),
-			  GST_PROPS_INT (PAL_HEIGHT)
-			),
-    "framerate",	GST_PROPS_LIST (
-			  GST_PROPS_FLOAT (25.),
-			  GST_PROPS_FLOAT (30/1.001)
-			)
-  )
-)
+  GST_STATIC_CAPS (
+    "video/x-raw-yuv, "
+    "format = (fourcc) YUY2, " 
+    "width = (int) 720, "
+    "height = (int) { " 
+    	G_STRINGIFY(NTSC_HEIGHT) ", "
+	G_STRINGIFY(PAL_HEIGHT)
+    " }, "
+    "framerate = (double) { "
+        G_STRINGIFY(PAL_FRAMERATE) ", "
+	G_STRINGIFY(NTSC_FRAMERATE) 
+    " }; "
 
-GST_PAD_TEMPLATE_FACTORY ( audio_src_temp,
+    "video/x-raw-rgb, "
+    "bpp = (int) 32, "
+    "depth = (int) 32, "
+    "endianness = (int) " G_STRINGIFY(G_BIG_ENDIAN) ", "
+    "red_mask =   (int) 0x000000ff, "
+    "green_mask = (int) 0x0000ff00, "
+    "blue_mask =  (int) 0x00ff0000, "
+    "width = (int) 720, "
+    "height = (int) { "
+    	G_STRINGIFY(NTSC_HEIGHT) ", "
+	G_STRINGIFY(PAL_HEIGHT)
+    " }, "
+    "framerate = (double) { "
+        G_STRINGIFY(PAL_FRAMERATE) ", "
+	G_STRINGIFY(NTSC_FRAMERATE) 
+    " }; "
+
+    "video/x-raw-rgb, "
+    "bpp = (int) 24, "
+    "depth = (int) 24, "
+    "endianness = (int) " G_STRINGIFY(G_BIG_ENDIAN) ", "
+    "red_mask =   (int) 0x000000ff, "
+    "green_mask = (int) 0x0000ff00, "
+    "blue_mask =  (int) 0x00ff0000, "
+    "width = (int) 720, "
+    "height = (int) { "
+    	G_STRINGIFY(NTSC_HEIGHT) ", "
+	G_STRINGIFY(PAL_HEIGHT)
+    " }, "
+    "framerate = (double) { "
+        G_STRINGIFY(PAL_FRAMERATE) ", "
+	G_STRINGIFY(NTSC_FRAMERATE) 
+    " }"
+  )
+);
+
+static GstStaticPadTemplate audio_src_temp =
+GST_STATIC_PAD_TEMPLATE
+(
   "audio",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "arts_sample",
-    "audio/x-raw-int",
-    "depth",   		GST_PROPS_INT (16),
-    "width",    	GST_PROPS_INT (16),
-    "signed",   	GST_PROPS_BOOLEAN (TRUE),
-    "channels", 	GST_PROPS_INT (2),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
-    "rate",   		GST_PROPS_INT_RANGE (4000, 48000)
+  GST_STATIC_CAPS (
+    "audio/x-raw-int, "
+    "depth = (int) 16, "
+    "width = (int) 16, "
+    "signed = (boolean) TRUE, "
+    "channels = (int) 2, "
+    "endianness = (int) " G_STRINGIFY(G_LITTLE_ENDIAN) ", "
+    "rate = (int) [ 4000, 48000 ]"
   )
-)
-
-/* typefind stuff */
-static GstCaps*
-dv_type_find (GstByteStream *bs, gpointer private)
-{
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 5) == 5) {
-    guint32 head = GUINT32_FROM_BE (*((guint32 *) GST_BUFFER_DATA (buf)));
-
-    /* check for DIF  and DV flag */
-    if ((head & 0xffffff00) == 0x1f070000 &&
-        !(GST_BUFFER_DATA (buf)[4] & 0x01)) {
-      gchar *format;
-
-      if ((head & 0x000000ff) & 0x80)
-        format = "PAL";
-      else
-        format = "NTSC";
-    
-      new = GST_CAPS_NEW ("dv_type_find",
-                          "video/x-dv",
-                          "systemstream", GST_PROPS_BOOLEAN (TRUE));
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
-static GstTypeDefinition dv_definition = {
-  "dv_video/dv", "video/x-dv", ".dv", dv_type_find 
-};
+);
 
 #define GST_TYPE_DVDEC_QUALITY (gst_dvdec_quality_get_type())
 GType
@@ -212,6 +172,7 @@ gst_dvdec_quality_get_type (void)
 }
 
 /* A number of functon prototypes are given so we can refer to them later. */
+static void		gst_dvdec_base_init		(gpointer g_class);
 static void		gst_dvdec_class_init		(GstDVDecClass *klass);
 static void		gst_dvdec_init			(GstDVDec *dvdec);
 
@@ -262,7 +223,7 @@ gst_dvdec_get_type (void)
   if (!dvdec_type) {
     static const GTypeInfo dvdec_info = {
       sizeof (GstDVDecClass),      
-      NULL,      
+      gst_dvdec_base_init,      
       NULL,
       (GClassInitFunc) gst_dvdec_class_init,
       NULL,
@@ -276,6 +237,23 @@ gst_dvdec_get_type (void)
   return dvdec_type;
 }
 
+static void
+gst_dvdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  /* The pad templates can be easily generated from the factories above,
+   * and then added to the list of padtemplates for the elementfactory.
+   * Note that the generated padtemplates are stored in static global
+   * variables, for the gst_dvdec_init function to use later on.
+   */
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get(&sink_temp));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get(&video_src_temp));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get(&audio_src_temp));
+
+  gst_element_class_set_details (element_class, &dvdec_details);
+}
+
 /* In order to create an instance of an object, the class must be
  * initialized by this function.  GObject will take care of running
  * it, based on the pointer to the function provided above.
@@ -325,13 +303,13 @@ gst_dvdec_init(GstDVDec *dvdec)
 {
   gint i;
 
-  dvdec->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_temp), "sink");
+  dvdec->sinkpad = gst_pad_new_from_template (gst_static_pad_template_get (&sink_temp), "sink");
   gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->sinkpad);
   gst_pad_set_query_function (dvdec->sinkpad, NULL);
   gst_pad_set_convert_function (dvdec->sinkpad, GST_DEBUG_FUNCPTR (gst_dvdec_sink_convert));
   gst_pad_set_formats_function (dvdec->sinkpad, GST_DEBUG_FUNCPTR (gst_dvdec_get_formats));
 
-  dvdec->videosrcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (video_src_temp), "video");
+  dvdec->videosrcpad = gst_pad_new_from_template (gst_static_pad_template_get (&video_src_temp), "video");
   gst_element_add_pad (GST_ELEMENT (dvdec), dvdec->videosrcpad);
   gst_pad_set_query_function (dvdec->videosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_src_query));
   gst_pad_set_query_type_function (dvdec->videosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_get_src_query_types));
@@ -340,7 +318,7 @@ gst_dvdec_init(GstDVDec *dvdec)
   gst_pad_set_convert_function (dvdec->videosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_src_convert));
   gst_pad_set_formats_function (dvdec->videosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_get_formats));
 
-  dvdec->audiosrcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET(audio_src_temp), "audio");
+  dvdec->audiosrcpad = gst_pad_new_from_template (gst_static_pad_template_get (&audio_src_temp), "audio");
   gst_element_add_pad(GST_ELEMENT(dvdec),dvdec->audiosrcpad);
   gst_pad_set_query_function (dvdec->audiosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_src_query));
   gst_pad_set_query_type_function (dvdec->audiosrcpad, GST_DEBUG_FUNCPTR (gst_dvdec_get_src_query_types));
@@ -351,7 +329,6 @@ gst_dvdec_init(GstDVDec *dvdec)
 
   gst_element_set_loop_function (GST_ELEMENT (dvdec), gst_dvdec_loop);
 
-  dvdec->pool = NULL;
   dvdec->length = 0;
   dvdec->next_ts = 0LL;
   dvdec->end_position = -1LL;
@@ -728,7 +705,7 @@ gst_dvdec_loop (GstElement *element)
   dvdec->PAL = dv_system_50_fields (dvdec->decoder);
 
   dvdec->framerate = (dvdec->PAL ? 2500 : 2997);
-  fps = (dvdec->PAL ? 25. : 30/1.001);
+  fps = (dvdec->PAL ? PAL_FRAMERATE : NTSC_FRAMERATE);
   dvdec->height = height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
 
@@ -746,43 +723,52 @@ gst_dvdec_loop (GstElement *element)
 
   /* if we did not negotiate yet, do it now */
   if (!GST_PAD_CAPS (dvdec->videosrcpad)) {
-    GstCaps *allowed;
-    GstCaps *trylist;
+    GstCaps *caps = NULL;
+    GstCaps *negotiated_caps = NULL;
+    GstPadTemplate *src_pad_template;
+    int i;
     
-    /* we what we are allowed to do */
-    allowed = gst_pad_get_allowed_caps (dvdec->videosrcpad);
-
     /* try to fix our height */
-    trylist = gst_caps_intersect (allowed, gst_caps_append (
-            GST_CAPS_NEW (
-	      "dvdec_negotiate",
-	      "video/x-raw-yuv",
-	        "height",  	GST_PROPS_INT (height),
-                "framerate",	GST_PROPS_FLOAT (fps)
-	    ), GST_CAPS_NEW (
-	      "dvdec_negotiate",
-	      "video/x-raw-rgb",
-	        "height",  	GST_PROPS_INT (height),
-                "framerate",	GST_PROPS_FLOAT (fps)
-            )));
-    
-    /* prepare for looping */
-    trylist = gst_caps_normalize (trylist);
+    src_pad_template = gst_static_pad_template_get (&video_src_temp);
+    caps = gst_caps_copy(gst_pad_template_get_caps (src_pad_template));
 
-    while (trylist) {
-      GstCaps *to_try = gst_caps_copy_1 (trylist);
+    for (i = 0; i < gst_caps_get_size (caps); i++)
+    {
+	GstStructure *structure = gst_caps_get_structure (caps, i);
+	gst_structure_set(structure, 
+	        "height", G_TYPE_INT, height,
+                "framerate", G_TYPE_INT, fps, NULL
+	      );
+    }
+
+    for (i=0; i < gst_caps_get_size(caps); i++) {
+      GstStructure *to_try_struct = gst_caps_get_structure (caps, i);
+      GstCaps *try_caps = 
+        gst_caps_new_full (gst_structure_copy(to_try_struct), NULL);
 
       /* try each format */
-      if (gst_pad_try_set_caps (dvdec->videosrcpad, to_try) > 0) {
-	guint32 fourcc;
+      if (gst_pad_try_set_caps (dvdec->videosrcpad, try_caps) > 0) {
+        negotiated_caps = try_caps;
+        break;
+      }
+
+      gst_caps_free(try_caps);
+    }
+
+    gst_caps_free (caps);
+
+    /* Check if we negotiated caps successfully */
+    if (negotiated_caps != NULL) {
+        GstStructure *structure = gst_caps_get_structure (negotiated_caps, 0);
+        guint32 fourcc;
 
 	/* it worked, try to find what it was again */
-	gst_caps_get_fourcc_int (to_try, "format", &fourcc);
+	gst_structure_get_fourcc (structure, "format", &fourcc);
 
 	if (fourcc == GST_STR_FOURCC ("RGB ")) {
           gint bpp;
 
-	  gst_caps_get_int (to_try, "bpp", &bpp);
+	  gst_structure_get_int (structure, "bpp", &bpp);
 	  if (bpp == 24) {
             dvdec->space = e_dv_color_rgb;
             dvdec->bpp = 3;
@@ -796,13 +782,8 @@ gst_dvdec_loop (GstElement *element)
           dvdec->space = e_dv_color_yuv;
           dvdec->bpp = 2;
 	}
-	break;
-      }
-      trylist = trylist->next;
-    }
-    /* oops list exhausted an nothing was found... */
-    if (!trylist) {
-      gst_element_error (element, "could not negotiate");
+    } else {
+      GST_ELEMENT_ERROR (element, CORE, NEGOTIATION, (NULL), (NULL));
       return;
     }
   }
@@ -821,15 +802,14 @@ gst_dvdec_loop (GstElement *element)
     /* if we did not negotiate yet, do it now */
     if (!GST_PAD_CAPS (dvdec->audiosrcpad)) {
       gst_pad_try_set_caps (dvdec->audiosrcpad,
-	GST_CAPS_NEW (
-	  "dvdec_audio_caps",
-	  "audio/x-raw-int",
-    	    "rate",   		GST_PROPS_INT (dvdec->decoder->audio->frequency),
-	    "depth",   		GST_PROPS_INT (16),
-	    "width",    	GST_PROPS_INT (16),
-	    "signed",   	GST_PROPS_BOOLEAN (TRUE),
-	    "channels", 	GST_PROPS_INT (dvdec->decoder->audio->num_channels),
-	    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN)
+	gst_caps_new_simple (
+	  "audio/x-raw-int", 
+    	    "rate", G_TYPE_INT, dvdec->decoder->audio->frequency,
+	    "depth", G_TYPE_INT,  16,
+	    "width", G_TYPE_INT,  16,
+	    "signed", G_TYPE_BOOLEAN, TRUE,
+	    "channels", G_TYPE_INT, dvdec->decoder->audio->num_channels,
+	    "endianness", G_TYPE_INT, G_LITTLE_ENDIAN
   	  ));
     }
 
@@ -853,23 +833,7 @@ gst_dvdec_loop (GstElement *element)
     guint8 *outframe_ptrs[3];
     gint outframe_pitches[3];
 
-    /* try to grab a pool */
-    if (!dvdec->pool) {
-      dvdec->pool = gst_pad_get_bufferpool (dvdec->videosrcpad);
-    }
-
-    outbuf = NULL;
-    /* try to get a buffer from the pool if we have one */
-    if (dvdec->pool) {
-      outbuf = gst_buffer_new_from_pool (dvdec->pool, 0, 0);
-    }
-    /* no buffer from pool, allocate one ourselves */
-    if (!outbuf) {
-      outbuf = gst_buffer_new ();
-    
-      GST_BUFFER_SIZE (outbuf) = (720 * height) * dvdec->bpp;
-      GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
-    }
+    outbuf = gst_buffer_new_and_alloc ((720 * height) * dvdec->bpp);
     
     outframe = GST_BUFFER_DATA (outbuf);
 
@@ -920,9 +884,6 @@ gst_dvdec_change_state (GstElement *element)
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-      if (dvdec->pool)
-	gst_buffer_pool_unref (dvdec->pool);
-      dvdec->pool = NULL;
       break;
     case GST_STATE_PAUSED_TO_READY:
       dv_decoder_free (dvdec->decoder);
@@ -1001,40 +962,25 @@ gst_dvdec_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
  * this function is called to register everything that the plugin provides.
  */
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
-  /* We need to create an ElementFactory for each element we provide.
-   * This consists of the name of the element, the GType identifier,
-   * and a pointer to the details structure at the top of the file.
-   */
-  factory = gst_element_factory_new("dvdec", GST_TYPE_DVDEC, &dvdec_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-  /* The pad templates can be easily generated from the factories above,
-   * and then added to the list of padtemplates for the elementfactory.
-   * Note that the generated padtemplates are stored in static global
-   * variables, for the gst_dvdec_init function to use later on.
-   */
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(sink_temp));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(video_src_temp));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(audio_src_temp));
-
-  /* The very last thing is to register the elementfactory with the plugin. */
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  type = gst_type_factory_new (&dv_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
+  if (!gst_element_register (plugin, "dvdec", GST_RANK_PRIMARY, gst_dvdec_get_type()))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "dvdec",
-  plugin_init
-};
+  "Uses libdv to decode DV video (libdv.sourceforge.net)",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+);
