@@ -31,7 +31,7 @@
 #include <gst/gst.h>
 
 #include <dvdnav/dvdnav.h>
-#include <dvdread/nav_print.h>
+#include <dvdnav/nav_print.h>
 
 #define GST_TYPE_DVDNAVSRC \
   (dvdnavsrc_get_type())
@@ -51,7 +51,7 @@ struct _DVDNavSrc {
   GstElement element;
 
   /* pads */
-  GstPad *srcpad;
+  GstPad   *srcpad;
   GstCaps *streaminfo;
 
   /* location */
@@ -59,7 +59,6 @@ struct _DVDNavSrc {
 
   gboolean did_seek;
   gboolean need_flush;
-  GstBufferPool *bufferpool;
 
   int title, chapter, angle;
   dvdnav_t *dvdnav;
@@ -261,7 +260,7 @@ dvdnavsrc_class_init (DVDNavSrcClass *klass)
                      0,99,1,G_PARAM_READWRITE));
   g_object_class_install_property(gobject_class, ARG_CHAPTER,
     g_param_spec_int("chapter", "chapter", "chapter",
-                     1,99,1,G_PARAM_READWRITE));
+                     0,99,1,G_PARAM_READWRITE));
   g_object_class_install_property(gobject_class, ARG_ANGLE,
     g_param_spec_int("angle", "angle", "angle",
                      1,9,1,G_PARAM_READWRITE));
@@ -293,8 +292,6 @@ dvdnavsrc_init (DVDNavSrc *src)
 
   gst_element_add_pad (GST_ELEMENT (src), src->srcpad);
 
-  src->bufferpool = gst_buffer_pool_get_default (DVD_VIDEO_LB_LEN, 2);
-
   src->location = g_strdup("/dev/dvd");
   src->did_seek = FALSE;
   src->need_flush = FALSE;
@@ -305,17 +302,6 @@ dvdnavsrc_init (DVDNavSrc *src)
   src->buttoninfo = NULL;
 }
 
-/* FIXME: this code is not being used */
-#ifdef PLEASEFIXTHISCODE
-static void
-dvdnavsrc_destroy (DVDNavSrc *dvdnavsrc)
-{
-  /* FIXME */
-  g_print("FIXME\n");
-  gst_buffer_pool_destroy (dvdnavsrc->bufferpool);
-}
-#endif
-
 static gboolean 
 dvdnavsrc_is_open (DVDNavSrc *src)
 {
@@ -518,49 +504,36 @@ static void
 dvdnavsrc_update_streaminfo (DVDNavSrc *src)
 {
   GstCaps *caps;
-  GstProps *props;
-  GstPropsEntry *entry;
+  GstStructure *structure;
   gint64 value;
 
-  props = gst_props_empty_new ();
-
-  /*
-  entry = gst_props_entry_new ("title_string", GST_PROPS_STRING (""));
-  gst_props_add_entry (props, entry);
-  */
+  caps = gst_caps_new_empty();
+  structure = gst_structure_empty_new ("application/x-gst-streaminfo");
+  gst_caps_append_structure (caps, structure);
 
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_TOTAL, &title_format, &value)) {
-    entry = gst_props_entry_new ("titles", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "titles", G_TYPE_INT, value, NULL);
   }
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_POSITION, &title_format, &value)) {
-    entry = gst_props_entry_new ("title", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "title", G_TYPE_INT, value, NULL);
   }
 
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_TOTAL, &chapter_format, &value)) {
-    entry = gst_props_entry_new ("chapters", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "chapters", G_TYPE_INT, value, NULL);
   }
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_POSITION, &chapter_format, &value)) {
-    entry = gst_props_entry_new ("chapter", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "chapter", G_TYPE_INT, value, NULL);
   }
 
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_TOTAL, &angle_format, &value)) {
-    entry = gst_props_entry_new ("angles", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "angles", G_TYPE_INT, value, NULL);
   }
   if (dvdnavsrc_query(src->srcpad, GST_QUERY_POSITION, &angle_format, &value)) {
-    entry = gst_props_entry_new ("angle", GST_PROPS_INT (value));
-    gst_props_add_entry (props, entry);
+    gst_caps_set_simple (caps, "angle", G_TYPE_INT, value, NULL);
   }
 
-  caps = gst_caps_new ("dvdnavsrc_streaminfo",
-      "application/x-gst-streaminfo",
-      props);
   if (src->streaminfo) {
-    gst_caps_unref (src->streaminfo);
+    gst_caps_free (src->streaminfo);
   }
   src->streaminfo = caps;
   g_object_notify (G_OBJECT (src), "streaminfo");
@@ -570,24 +543,30 @@ static void
 dvdnavsrc_update_buttoninfo (DVDNavSrc *src)
 {
   GstCaps *caps;
-  GstProps *props;
-  GstPropsEntry *entry;
   pci_t *pci;
 
   pci = dvdnav_get_current_nav_pci(src->dvdnav);
-  fprintf(stderr, "update button info total:%d\n", pci->hli.hl_gi.btn_ns);
 
-  props = gst_props_empty_new ();
-
-  entry = gst_props_entry_new ("total", GST_PROPS_INT (pci->hli.hl_gi.btn_ns));
-  gst_props_add_entry (props, entry);
-
-  caps = gst_caps_new ("dvdnavsrc_buttoninfo",
-      "application/x-gst-dvdnavsrc-buttoninfo",
-      props);
   if (src->buttoninfo) {
-    gst_caps_unref (src->buttoninfo);
+    gint btn_ns;
+
+    /* Don't notify if there is no actual change */
+    if (gst_structure_get_int (gst_caps_get_structure (src->buttoninfo, 0),
+			       "total", &btn_ns) 
+        && (btn_ns == pci->hli.hl_gi.btn_ns))
+    {
+      return;
+    }
   }
+
+  caps = gst_caps_new_simple (
+      "application/x-gst-dvdnavsrc-buttoninfo",
+      "total", G_TYPE_INT, (gint) pci->hli.hl_gi.btn_ns, 
+      NULL);
+
+  if (src->buttoninfo)
+    gst_caps_free (src->buttoninfo);
+
   src->buttoninfo = caps;
   g_object_notify (G_OBJECT (src), "buttoninfo");
 }
@@ -701,8 +680,8 @@ dvdnavsrc_user_op (DVDNavSrc *src, int op)
   }
   return;
 naverr:
-  gst_element_error(GST_ELEMENT(src), "user op %d failure: %d",
-      op, dvdnav_err_to_string(src->dvdnav));
+  GST_ELEMENT_ERROR (src, LIBRARY, TOO_LAZY, (NULL),
+                     ("user op %d failure: %s", op, dvdnav_err_to_string(src->dvdnav)));
 
 }
 
@@ -870,19 +849,17 @@ dvdnavsrc_get (GstPad *pad)
   /* loop processing blocks until data is pushed */
   have_buf = FALSE;
   while (!have_buf) {
-    /* allocate a pool for the buffer data */
-    /* FIXME: mem leak on non BLOCK_OK events */
-    buf = gst_buffer_new_from_pool (src->bufferpool, DVD_VIDEO_LB_LEN, 0);
+    buf = gst_buffer_new_and_alloc (DVD_VIDEO_LB_LEN);
     if (!buf) {
-      gst_element_error (GST_ELEMENT (src), "Failed to create a new GstBuffer");
+      GST_ELEMENT_ERROR (src, CORE, TOO_LAZY, (NULL), ("Failed to create a new GstBuffer"));
       return NULL;
     }
     data = GST_BUFFER_DATA(buf);
 
     if (dvdnav_get_next_block (src->dvdnav, data, &event, &len) !=
         DVDNAV_STATUS_OK) {
-      gst_element_error (GST_ELEMENT (src), "dvdnav_get_next_block error: %s\n",
-          dvdnav_err_to_string(src->dvdnav));
+      GST_ELEMENT_ERROR (src, STREAM, DECODE, (NULL),
+                         ("dvdnav_get_next_block error: %s\n", dvdnav_err_to_string(src->dvdnav)));
       return NULL;
     }
 
@@ -899,8 +876,8 @@ dvdnavsrc_get (GstPad *pad)
          * dvdnav_still_skip */
         dvdnavsrc_print_event (src, data, event, len);
         if (dvdnav_still_skip (src->dvdnav) != DVDNAV_STATUS_OK) {
-          gst_element_error (GST_ELEMENT (src), "dvdnav_still_skip error: %s\n",
-              dvdnav_err_to_string(src->dvdnav));
+          GST_ELEMENT_ERROR (src, STREAM, TOO_LAZY, (NULL), ("dvdnav_still_skip error: %s\n",
+              dvdnav_err_to_string(src->dvdnav)));
           /* FIXME: close the stream??? */
         }
         break;
@@ -915,19 +892,38 @@ dvdnavsrc_get (GstPad *pad)
         dvdnavsrc_update_streaminfo (src);
         break;
       case DVDNAV_NAV_PACKET:
-        if (0) dvdnavsrc_update_buttoninfo (src);
+        dvdnavsrc_update_buttoninfo (src);
         break;
       case DVDNAV_WAIT:
 	/* FIXME: supposed to make sure all the data has made 
 	 * it to the sinks before skipping the wait
 	 */
+        dvdnavsrc_print_event (src, data, event, len);
         dvdnav_wait_skip(src->dvdnav);
-      case DVDNAV_VTS_CHANGE:
-      case DVDNAV_SPU_STREAM_CHANGE:
-      case DVDNAV_AUDIO_STREAM_CHANGE:
-      case DVDNAV_HIGHLIGHT:
-      case DVDNAV_SPU_CLUT_CHANGE:
+	break;
       case DVDNAV_HOP_CHANNEL:
+	/* Indicates a time discontinuity, and the downstream should
+	 * flush
+	 */
+        dvdnavsrc_print_event (src, data, event, len);
+        buf = GST_BUFFER (gst_event_new (GST_EVENT_DISCONTINUOUS));
+        have_buf = TRUE;
+	break;
+      case DVDNAV_HIGHLIGHT:
+        dvdnavsrc_print_event (src, data, event, len);
+	break;
+
+	/* SPU_STREAM_CHANGE provides MPEG stream numbers for different
+	 * formats of the video, eg letterbox/pan&scan
+	 */
+      case DVDNAV_SPU_STREAM_CHANGE:
+	/* AUDIO_STREAM_CHANGE indicates that the user selected an alternate
+	 * audio stream (from a menu)
+	 */
+      case DVDNAV_AUDIO_STREAM_CHANGE:
+	/* VTS_CHANGE Indicates a change in VTS (Video Title Set) */ 
+      case DVDNAV_VTS_CHANGE:
+      case DVDNAV_SPU_CLUT_CHANGE:
       default:
         dvdnavsrc_print_event (src, data, event, len);
         break;
@@ -1072,12 +1068,50 @@ dvdnavsrc_get_event_mask (GstPad *pad)
 	                     GST_SEEK_METHOD_END | 
 		             GST_SEEK_FLAG_FLUSH },
                              */
+    {GST_EVENT_NAVIGATION, GST_EVENT_FLAG_NONE },
     {0,}
   };
 
   return masks;
 }
 
+static gboolean
+dvdnav_handle_navigation_event (DVDNavSrc *src, GstEvent *event)
+{
+  GstStructure *structure = event->event_data.structure.structure;
+  const char *event_type = gst_structure_get_string (structure, "event");
+
+  g_return_val_if_fail (event != NULL, FALSE);
+	
+  if (strcmp (event_type, "key-press") == 0) 
+  {
+    const char *key = gst_structure_get_string(structure, "key");
+
+    g_assert (key != NULL);
+    g_print ("dvdnavsrc got a keypress: %s\n", key);
+  }
+  else if (strcmp (event_type, "mouse-move") == 0) 
+  {
+    double x, y;
+
+    gst_structure_get_double(structure, "pointer_x", &x);
+    gst_structure_get_double(structure, "pointer_y", &y);
+
+    dvdnavsrc_pointer_select (src, (int) x, (int) y);	
+  }
+  else if (strcmp (event_type, "mouse-button-release") == 0) 
+  {
+    double x, y;
+
+    gst_structure_get_double(structure, "pointer_x", &x);
+    gst_structure_get_double(structure, "pointer_y", &y);
+
+    dvdnavsrc_pointer_activate (src, (int) x, (int) y);	
+  }
+
+  return TRUE;
+}
+
 static gboolean
 dvdnavsrc_event (GstPad *pad, GstEvent *event)
 {
@@ -1205,6 +1239,8 @@ dvdnavsrc_event (GstPad *pad, GstEvent *event)
       src->need_flush = GST_EVENT_SEEK_FLAGS(event) & GST_SEEK_FLAG_FLUSH;
       break;
     }
+    case GST_EVENT_NAVIGATION:
+      res = dvdnav_handle_navigation_event (src, event);
     case GST_EVENT_FLUSH:
       src->need_flush = TRUE;
       break;
