@@ -28,6 +28,9 @@
 #include <unistd.h>
 #include <errno.h>
 
+GST_DEBUG_CATEGORY_EXTERN (esd_debug);
+#define GST_CAT_DEFAULT esd_debug
+
 /* elementfactory information */
 static GstElementDetails esdsink_details = {
   "Esound audio sink",
@@ -50,43 +53,29 @@ enum {
   ARG_FALLBACK,
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_factory,
-  "sink",					/* the name of the pads */
-  GST_PAD_SINK,				/* type of the pad */
-  GST_PAD_ALWAYS,			/* ALWAYS/SOMETIMES */
-  GST_CAPS_NEW (
-    "esdsink_sink",				/* the name of the caps */
-    "audio/x-raw-int",				/* the mime type of the caps */
-      /* Properties follow: */
-        "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-	"signed",     GST_PROPS_LIST (
-			GST_PROPS_BOOLEAN (TRUE),
-			GST_PROPS_BOOLEAN (FALSE)
-		      ),
-        "width",      GST_PROPS_LIST (
-			GST_PROPS_INT (8),
-			GST_PROPS_INT (16)
-		      ),
-	"depth",      GST_PROPS_LIST (
-			GST_PROPS_INT (8),
-			GST_PROPS_INT (16)
-		      ),
-	"rate",       GST_PROPS_INT_RANGE (8000, 96000),
-     	"channels",   GST_PROPS_LIST (
-			GST_PROPS_INT (1),
-			GST_PROPS_INT (2)
-		      )
+static GstStaticPadTemplate sink_factory = 
+GST_STATIC_PAD_TEMPLATE (
+  "sink",
+  GST_PAD_SINK,
+  GST_PAD_ALWAYS,
+  GST_STATIC_CAPS (
+    "audio/x-raw-int, "
+      "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
+      "signed = (boolean) TRUE, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = 44100, "
+      "channels = 2"
   )
 );
 
 static void                     gst_esdsink_base_init           (gpointer g_class);
-static void			gst_esdsink_class_init		(GstEsdsinkClass *klass);
-static void			gst_esdsink_init		(GstEsdsink *esdsink);
+static void			gst_esdsink_class_init		(gpointer g_class, gpointer class_data);
+static void			gst_esdsink_init		(GTypeInstance *instance, gpointer g_class);
 
 static gboolean			gst_esdsink_open_audio		(GstEsdsink *sink);
 static void			gst_esdsink_close_audio		(GstEsdsink *sink);
 static GstElementStateReturn	gst_esdsink_change_state	(GstElement *element);
-static GstPadLinkReturn		gst_esdsink_sinkconnect		(GstPad *pad, GstCaps *caps);
 
 static GstClockTime		gst_esdsink_get_time		(GstClock *clock, gpointer data);
 static GstClock *		gst_esdsink_get_clock		(GstElement *element);
@@ -111,12 +100,12 @@ gst_esdsink_get_type (void)
       sizeof(GstEsdsinkClass),
       gst_esdsink_base_init,
       NULL,
-      (GClassInitFunc)gst_esdsink_class_init,
+      gst_esdsink_class_init,
       NULL,
       NULL,
       sizeof(GstEsdsink),
       0,
-      (GInstanceInitFunc)gst_esdsink_init,
+      gst_esdsink_init,
     };
     esdsink_type = g_type_register_static(GST_TYPE_ELEMENT, "GstEsdsink", &esdsink_info, 0);
   }
@@ -128,33 +117,34 @@ gst_esdsink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (sink_factory));
+  gst_element_class_add_pad_template (element_class, 
+      gst_static_pad_template_get (&sink_factory));
   gst_element_class_set_details (element_class, &esdsink_details);
 }
 
 static void
-gst_esdsink_class_init (GstEsdsinkClass *klass)
+gst_esdsink_class_init (gpointer g_class, gpointer class_data)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gobject_class = (GObjectClass*)klass;
-  gstelement_class = (GstElementClass*)klass;
+  parent_class = g_type_class_peek_parent (g_class);
 
-  parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
-
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_MUTE,
+  g_object_class_install_property(gobject_class, ARG_MUTE,
     g_param_spec_boolean("mute","mute","mute",
                          TRUE,G_PARAM_READWRITE)); /* CHECKME */
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_HOST,
+  g_object_class_install_property(gobject_class, ARG_HOST,
     g_param_spec_string("host","host","host",
                         NULL, G_PARAM_READWRITE)); /* CHECKME */
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_SYNC,
+  g_object_class_install_property(gobject_class, ARG_SYNC,
     g_param_spec_boolean("sync","sync","Synchronize output to clock",
-                         FALSE,G_PARAM_READWRITE));
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_FALLBACK,
+                         TRUE,G_PARAM_READWRITE));
+#if 0
+  /* This option is disabled because it is dumb in GStreamer's architecture. */
+  g_object_class_install_property(gobject_class, ARG_FALLBACK,
     g_param_spec_boolean("fallback","fallback","Fall back to using OSS if Esound daemon is not present",
                          FALSE,G_PARAM_READWRITE));
+#endif
 
   gobject_class->set_property = gst_esdsink_set_property;
   gobject_class->get_property = gst_esdsink_get_property;
@@ -165,74 +155,64 @@ gst_esdsink_class_init (GstEsdsinkClass *klass)
 }
 
 static void
-gst_esdsink_init(GstEsdsink *esdsink)
+gst_esdsink_init(GTypeInstance *instance, gpointer g_class)
 {
+  GstEsdsink *esdsink = GST_ESDSINK (instance);
+  
   esdsink->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_factory), "sink");
+      gst_element_class_get_pad_template (GST_ELEMENT_GET_CLASS (instance), "sink"), 
+      "sink");
   gst_element_add_pad(GST_ELEMENT(esdsink), esdsink->sinkpad);
   gst_pad_set_chain_function(esdsink->sinkpad, GST_DEBUG_FUNCPTR(gst_esdsink_chain));
-  gst_pad_set_link_function(esdsink->sinkpad, gst_esdsink_sinkconnect);
 
   GST_FLAG_SET (esdsink, GST_ELEMENT_EVENT_AWARE);
 
   esdsink->mute = FALSE;
   esdsink->fd = -1;
   /* FIXME: get default from somewhere better than just putting them inline. */
-  esdsink->negotiated = FALSE;
-  esdsink->format = -1;
-  esdsink->depth = -1;
-  esdsink->channels = -1;
-  esdsink->frequency = -1;
+  /*esdsink->negotiated = FALSE;*/
+  /* we have static caps on our template, so it always is negotiated */
+  esdsink->negotiated = TRUE;
+  esdsink->format = 16;
+  esdsink->depth = 16;
+  esdsink->channels = 2;
+  esdsink->frequency = 44100;
+  esdsink->bytes_per_sample = esdsink->channels * (esdsink->depth/8);
   esdsink->host = getenv ("ESPEAKER");
   esdsink->provided_clock = gst_audio_clock_new("esdclock", gst_esdsink_get_time, esdsink);
   gst_object_set_parent(GST_OBJECT(esdsink->provided_clock), GST_OBJECT(esdsink));
-  esdsink->sync = FALSE;
+  esdsink->sync = TRUE;
   esdsink->fallback = FALSE;
 }
 
+#ifdef unused
 static GstPadLinkReturn
-gst_esdsink_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_esdsink_link (GstPad *pad, const GstCaps *caps)
 {
   GstEsdsink *esdsink;
-  gboolean sign;
+  GstStructure *structure;
 
   esdsink = GST_ESDSINK (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-
-  gst_caps_get_int (caps, "depth", &esdsink->depth);
-  gst_caps_get_int (caps, "signed", &sign);
-  gst_caps_get_int (caps, "channels", &esdsink->channels);
-  gst_caps_get_int (caps, "rate", &esdsink->frequency);
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "depth", &esdsink->depth);
+  gst_structure_get_int (structure, "channels", &esdsink->channels);
+  gst_structure_get_int (structure, "rate", &esdsink->frequency);
 
   esdsink->bytes_per_sample = esdsink->channels * (esdsink->depth/8);
 
-  /* only u8/s16 */
-  if ((sign == FALSE && esdsink->depth != 8) ||
-      (sign == TRUE  && esdsink->depth != 16)) {
-    return GST_PAD_LINK_REFUSED;
-  }
-
   gst_esdsink_close_audio (esdsink);
   if (gst_esdsink_open_audio (esdsink)) {
     esdsink->negotiated = TRUE;
     return GST_PAD_LINK_OK;
   }
+  /* FIXME: is it supposed to be correct to have closed audio when caps nego 
+     failed? */
 
+  GST_DEBUG ("esd link function could not negotiate, returning delayed");
   return GST_PAD_LINK_REFUSED;
 }
-
-static int
-gst_esdsink_get_latency (GstEsdsink *esdsink)
-{
-  /* esd_get_latency() doesn't actually work.  So we return a
-   * fake value */
-  return 44100/2;
-#if 0
-  return esd_get_latency (esdsink->fd);
 #endif
-}
 
 static GstClockTime
 gst_esdsink_get_time (GstClock *clock, gpointer data)
@@ -240,8 +220,8 @@ gst_esdsink_get_time (GstClock *clock, gpointer data)
   GstEsdsink *esdsink = GST_ESDSINK(data);
   GstClockTime res;
 
-  res = (esdsink->handled - gst_esdsink_get_latency(esdsink))
-    * GST_SECOND / esdsink->frequency;
+  res = (esdsink->handled * GST_SECOND) / esdsink->frequency;
+    //- GST_SECOND * 2;
 
   return res;
 }
@@ -275,7 +255,8 @@ gst_esdsink_chain (GstPad *pad, GstData *_data)
   esdsink = GST_ESDSINK (gst_pad_get_parent (pad));
 
   if (!esdsink->negotiated) {
-    gst_element_error (GST_ELEMENT (esdsink), "not negotiated");
+    GST_ELEMENT_ERROR (esdsink, CORE, NEGOTIATION, (NULL),
+                       ("element wasn't negotiated before chain function"));
     goto done;
   }
 
@@ -288,20 +269,6 @@ gst_esdsink_chain (GstPad *pad, GstData *_data)
 	    FALSE);
 	gst_pad_event_default (pad, event);
 	return;
-      case GST_EVENT_DISCONTINUOUS:
-      {
-	gint64 value;
-
-	if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
-	  if (!gst_clock_handle_discont (esdsink->clock, value)){
-	    gst_audio_clock_set_active (GST_AUDIO_CLOCK (esdsink->provided_clock),
-		FALSE);
-	  }
-	  esdsink->handled = 0;
-	}
-	esdsink->resync = TRUE;
-	break;
-      }
       default:
 	gst_pad_event_default(pad, event);
 	return;
@@ -316,35 +283,10 @@ gst_esdsink_chain (GstPad *pad, GstData *_data)
       gint size = GST_BUFFER_SIZE (buf);
       gint to_write = 0;
 
-      if (esdsink->clock){
-	gint delay = 0;
-	gint64 queued;
-	GstClockTimeDiff jitter;
+      to_write = size;
 
-	delay = gst_esdsink_get_latency (esdsink);
-	queued = delay * GST_SECOND / esdsink->frequency;
-
-	if (esdsink->resync && esdsink->sync) {
-	  GstClockID id = gst_clock_new_single_shot_id (esdsink->clock,
-	      GST_BUFFER_TIMESTAMP (buf) - queued);
-
-	  gst_element_clock_wait (GST_ELEMENT (esdsink), id, &jitter);
-	  gst_clock_id_free (id);
-
-	  if (jitter >= 0){
-	    gst_clock_handle_discont (esdsink->clock,
-		GST_BUFFER_TIMESTAMP (buf) - queued + jitter);
-	    to_write = size;
-	    gst_audio_clock_set_active ((GstAudioClock *)esdsink->provided_clock, TRUE);
-	    esdsink->resync = FALSE;
-	  }
-	}else{
-	  to_write = size;
-	}
-      }
-
-      GST_DEBUG ("esdsink: fd=%d data=%p size=%d",
-		 esdsink->fd, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+      GST_LOG ("fd=%d data=%p size=%d",
+                esdsink->fd, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
       while (to_write > 0){
 	int done;
 
@@ -366,7 +308,7 @@ gst_esdsink_chain (GstPad *pad, GstData *_data)
   }
 
   gst_audio_clock_update_time ((GstAudioClock *)esdsink->provided_clock,
-      GST_BUFFER_TIMESTAMP (buf));
+      gst_esdsink_get_time (esdsink->provided_clock, esdsink));
 
 done:
   gst_buffer_unref (buf);
@@ -452,27 +394,31 @@ gst_esdsink_open_audio (GstEsdsink *sink)
   if (sink->depth == 16) esdformat |= ESD_BITS16;
   else if (sink->depth == 8) esdformat |= ESD_BITS8;
   else {
-    GST_DEBUG ("esdsink: invalid bit depth (%d)", sink->depth);
+    GST_ELEMENT_ERROR (sink, STREAM, FORMAT, (NULL),
+                       ("invalid bit depth (%d)", sink->depth));
     return FALSE;
   }
 
   if (sink->channels == 2) esdformat |= ESD_STEREO;
   else if (sink->channels == 1) esdformat |= ESD_MONO;
   else {
-    GST_DEBUG ("esdsink: invalid number of channels (%d)", sink->channels);
+    GST_ELEMENT_ERROR (sink, STREAM, FORMAT, (NULL),
+                       ("invalid number of channels (%d)", sink->channels));
     return FALSE;
   }
 
-  GST_DEBUG ("esdsink: attempting to open connection to esound server");
+  GST_INFO ("attempting to open connection to esound server");
   if(sink->fallback){
     sink->fd = esd_play_stream_fallback(esdformat, sink->frequency, sink->host, connname);
   }else{
     sink->fd = esd_play_stream(esdformat, sink->frequency, sink->host, connname);
   }
   if ( sink->fd < 0 ) {
-    GST_DEBUG ("esdsink: can't open connection to esound server");
+    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
+                       ("can't open connection to esound server"));
     return FALSE;
   }
+  GST_INFO ("successfully opened connection to esound server");
 
   return TRUE;
 }
@@ -486,7 +432,7 @@ gst_esdsink_close_audio (GstEsdsink *sink)
   close(sink->fd);
   sink->fd = -1;
 
-  GST_DEBUG ("esdsink: closed sound device");
+  GST_INFO ("esdsink: closed sound device");
 }
 
 static GstElementStateReturn
@@ -498,10 +444,15 @@ gst_esdsink_change_state (GstElement *element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
+      if (!gst_esdsink_open_audio (GST_ESDSINK (element))) {
+        return GST_STATE_FAILURE;
+      }
       break;
     case GST_STATE_READY_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
+      gst_audio_clock_set_active (GST_AUDIO_CLOCK (esdsink->provided_clock),
+	  TRUE);
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       gst_audio_clock_set_active (GST_AUDIO_CLOCK (esdsink->provided_clock),
@@ -509,10 +460,9 @@ gst_esdsink_change_state (GstElement *element)
       esdsink->resync = TRUE;
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_esdsink_close_audio (GST_ESDSINK (element));
-      esdsink->negotiated = FALSE;
       break;
     case GST_STATE_READY_TO_NULL:
+      gst_esdsink_close_audio (GST_ESDSINK (element));
       break;
     default:
       break;
