@@ -51,10 +51,7 @@ struct _GstFFMpegDemux {
 
 typedef struct _GstFFMpegDemuxClassParams {
   AVInputFormat 	*in_plugin;
-  GstPadTemplate	*sinktempl;
-  GstPadTemplate	*videosrctempl;
-  GstPadTemplate	*audiosrctempl;
-  GstPluginFeature	*typefind_feature;
+  GstCaps		*sinkcaps, *videosrccaps, *audiosrccaps;
 } GstFFMpegDemuxClassParams;
 
 typedef struct _GstFFMpegDemuxClass GstFFMpegDemuxClass;
@@ -66,7 +63,6 @@ struct _GstFFMpegDemuxClass {
   GstPadTemplate	*sinktempl;
   GstPadTemplate	*videosrctempl;
   GstPadTemplate	*audiosrctempl;
-  GstPluginFeature	*typefind_feature;
 };
 
 #define GST_TYPE_FFMPEGDEC \
@@ -90,10 +86,11 @@ enum {
   /* FILL ME */
 };
 
-static GHashTable *global_plugins, *typefind;
+static GHashTable *global_plugins;
 
 /* A number of functon prototypes are given so we can refer to them later. */
 static void	gst_ffmpegdemux_class_init	(GstFFMpegDemuxClass *klass);
+static void	gst_ffmpegdemux_base_init	(GstFFMpegDemuxClass *klass);
 static void	gst_ffmpegdemux_init		(GstFFMpegDemux *ffmpegdemux);
 static void	gst_ffmpegdemux_dispose		(GObject *object);
 
@@ -106,27 +103,68 @@ static GstElementClass *parent_class = NULL;
 
 /*static guint gst_ffmpegdemux_signals[LAST_SIGNAL] = { 0 }; */
 
+static void
+gst_ffmpegdemux_base_init (GstFFMpegDemuxClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstFFMpegDemuxClassParams *params;
+  GstElementDetails *details;
+  GstPadTemplate *sinktempl, *audiosrctempl, *videosrctempl;
+
+  params = g_hash_table_lookup (global_plugins,
+		GINT_TO_POINTER (G_OBJECT_CLASS_TYPE (gobject_class)));
+  if (!params)
+    params = g_hash_table_lookup (global_plugins,
+		GINT_TO_POINTER (0));
+  g_assert (params);
+
+  /* construct the element details struct */
+  details = g_new0 (GstElementDetails, 1);
+  details->longname = g_strdup_printf("FFMPEG %s demuxer",
+				      params->in_plugin->name);
+  details->klass = g_strdup("Codec/Demuxer");
+  details->description = g_strdup_printf("FFMPEG %s decoder",
+					 params->in_plugin->name);
+  details->author = g_strdup("Wim Taymans <wim.taymans@chello.be>\n"
+			     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
+  /* pad templates */
+  sinktempl = gst_pad_template_new ("sink",
+				    GST_PAD_SINK,
+				    GST_PAD_ALWAYS,
+				    params->sinkcaps);
+  videosrctempl = gst_pad_template_new ("video_%02d",
+					GST_PAD_SRC,
+					GST_PAD_SOMETIMES,
+					params->videosrccaps);
+  audiosrctempl = gst_pad_template_new ("audio_%02d",
+					GST_PAD_SRC,
+					GST_PAD_SOMETIMES,
+					params->audiosrccaps);
+
+  gst_element_class_add_pad_template (element_class, videosrctempl);
+  gst_element_class_add_pad_template (element_class, audiosrctempl);
+  gst_element_class_add_pad_template (element_class, sinktempl);
+  gst_element_class_set_details (element_class, details);
+
+  klass->in_plugin = params->in_plugin;
+  klass->videosrctempl = videosrctempl;
+  klass->audiosrctempl = audiosrctempl;
+  klass->sinktempl = sinktempl;
+}
+
 static void
 gst_ffmpegdemux_class_init (GstFFMpegDemuxClass *klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
-  GstFFMpegDemuxClassParams *params;
 
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  params = g_hash_table_lookup (global_plugins,
-		GINT_TO_POINTER (G_OBJECT_CLASS_TYPE (gobject_class)));
-
-  klass->in_plugin = params->in_plugin;
-  klass->typefind_feature = params->typefind_feature;
-  klass->videosrctempl = params->videosrctempl;
-  klass->audiosrctempl = params->audiosrctempl;
-  klass->sinktempl = params->sinktempl;
-
   gstelement_class->change_state = gst_ffmpegdemux_change_state;
   gobject_class->dispose = gst_ffmpegdemux_dispose;
 }
@@ -160,45 +198,28 @@ gst_ffmpegdemux_dispose (GObject *object)
   }
 }
 
-static GstCaps*
-gst_ffmpegdemux_typefind (GstByteStream *bs,
-			  gpointer       priv)
+#define GST_FFMPEG_TYPE_FIND_SIZE 4096
+static void
+gst_ffmpegdemux_type_find (GstTypeFind *tf, gpointer priv)
 {
-  GstFFMpegDemuxClassParams *params;
-  AVInputFormat *in_plugin;
+  guint8 *data;
+  GstFFMpegDemuxClassParams *params = (GstFFMpegDemuxClassParams *) priv;
+  AVInputFormat *in_plugin = params->in_plugin;
   gint res = 0;
-  gint required = AVPROBE_SCORE_MAX * 0.8; /* 80% certainty enough? */
-  gint size_required = 4096;
-  GstBuffer *buf = NULL;
   
-  params = g_hash_table_lookup (typefind, priv);
-
-  in_plugin = params->in_plugin;
-
   if (in_plugin->read_probe &&
-      gst_bytestream_peek (bs, &buf, size_required) == size_required) {
+      (data = gst_type_find_peek (tf, 0, GST_FFMPEG_TYPE_FIND_SIZE)) != NULL) {
     AVProbeData probe_data;
 
     probe_data.filename = "";
-    probe_data.buf = GST_BUFFER_DATA (buf);
-    probe_data.buf_size = GST_BUFFER_SIZE (buf);
+    probe_data.buf = data;
+    probe_data.buf_size = GST_FFMPEG_TYPE_FIND_SIZE;
 
     res = in_plugin->read_probe (&probe_data);
-    if (res >= required) {
-      GstCaps *caps;
-      caps = GST_PAD_TEMPLATE_CAPS (params->sinktempl);
-      /* make sure we still hold a refcount to this caps */
-      gst_caps_ref (caps);
-      gst_buffer_unref (buf);
-      return caps;
-    }
+    res = res * GST_TYPE_FIND_MAXIMUM / AVPROBE_SCORE_MAX;
+    if (res > 0) 
+      gst_type_find_suggest (tf, res, params->sinkcaps);
   }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return NULL;
 }
 
 static void
@@ -220,8 +241,8 @@ gst_ffmpegdemux_loop (GstElement *element)
 					       ffmpegdemux->sinkpad),
 			      oclass->in_plugin, 0, NULL);
     if (res < 0) {
-      gst_element_error (GST_ELEMENT (ffmpegdemux),
-			 "Failed to open demuxer/file context");
+      GST_ELEMENT_ERROR (ffmpegdemux, LIBRARY, TOO_LAZY, (NULL),
+			 ("Failed to open demuxer/file context"));
       return;
     }
 
@@ -295,21 +316,21 @@ gst_ffmpegdemux_loop (GstElement *element)
     pad = gst_pad_new_from_template (templ, padname);
     g_free (padname);
 
+    gst_pad_use_explicit_caps (pad);
     /* FIXME: convert() and query() functions for pad */
 
     /* store pad internally */
     ffmpegdemux->srcpads[pkt.stream_index] = pad;
-    gst_element_add_pad (GST_ELEMENT (ffmpegdemux), pad);
 
     /* get caps that belongs to this stream */
     caps = gst_ffmpeg_codecid_to_caps (st->codec.codec_id,
 				       &st->codec);
-    if (gst_pad_try_set_caps (pad, caps) <= 0) {
-      GST_DEBUG (
-		 "Failed to set caps from ffdemuxer on next element");
-      /* we continue here, in the next pad-is-usable check,
-       * we'll return nonetheless */
-    }
+    gst_pad_set_explicit_caps (pad, caps);
+
+    gst_element_add_pad (GST_ELEMENT (ffmpegdemux), pad);
+
+    /* we continue here, in the next pad-is-usable check,
+     * we'll return nonetheless */
   }
 
   /* shortcut to pad belonging to this stream */
@@ -361,12 +382,14 @@ gst_ffmpegdemux_change_state (GstElement *element)
 gboolean
 gst_ffmpegdemux_register (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type_factory;
-  GstTypeDefinition *type_definition;
+  GType type;
+  AVInputFormat *in_plugin;
+  GstFFMpegDemuxClassParams *params;
+  AVCodec *in_codec;
+  gchar **extensions;
   GTypeInfo typeinfo = {
     sizeof(GstFFMpegDemuxClass),      
-    NULL,
+    (GBaseInitFunc)gst_ffmpegdemux_base_init,
     NULL,
     (GClassInitFunc)gst_ffmpegdemux_class_init,
     NULL,
@@ -375,19 +398,14 @@ gst_ffmpegdemux_register (GstPlugin *plugin)
     0,
     (GInstanceInitFunc)gst_ffmpegdemux_init,
   };
-  GType type;
-  GstElementDetails *details;
-  AVInputFormat *in_plugin;
-  GstFFMpegDemuxClassParams *params;
-  AVCodec *in_codec;
+  GstCaps *any_caps = gst_caps_new_any ();
   
   in_plugin = first_iformat;
 
   global_plugins = g_hash_table_new (NULL, NULL);
-  typefind = g_hash_table_new (NULL, NULL);
 
   while (in_plugin) {
-    gchar *type_name;
+    gchar *type_name, *typefind_name;
     gchar *p;
     GstCaps *sinkcaps, *audiosrccaps, *videosrccaps;
 
@@ -409,19 +427,20 @@ gst_ffmpegdemux_register (GstPlugin *plugin)
       }
       switch (in_codec->type) {
         case CODEC_TYPE_VIDEO:
-          videosrccaps = gst_caps_append (videosrccaps, temp);
+          gst_caps_append (videosrccaps, temp);
           break;
         case CODEC_TYPE_AUDIO:
-          audiosrccaps = gst_caps_append (audiosrccaps, temp);
+          gst_caps_append (audiosrccaps, temp);
           break;
         default:
-          gst_caps_unref (temp);
+          gst_caps_free (temp);
           break;
       }
     }
 
     /* construct the type */
     type_name = g_strdup_printf("ffdemux_%s", in_plugin->name);
+    typefind_name = g_strdup_printf("fftype_%s", in_plugin->name);
 
     p = type_name;
 
@@ -435,76 +454,38 @@ gst_ffmpegdemux_register (GstPlugin *plugin)
       g_free(type_name);
       goto next;
     }
-
-    /* create the type now */
-    type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
-
-    /* construct the element details struct */
-    details = g_new0 (GstElementDetails, 1);
-    details->longname = g_strdup (in_plugin->long_name);
-    details->klass = g_strdup ("Codec/Demuxer");
-    details->license = g_strdup ("LGPL");
-    details->description = g_strdup_printf ("FFMPEG %s demuxer",
-					    in_plugin->name);
-    details->version = g_strdup (VERSION);
-    details->author = g_strdup ("The FFMPEG crew, "
-				"Wim Taymans <wim.taymans@chello.be>, "
-				"Ronald Bultje <rbultje@ronald.bitfreak.net>");
-    details->copyright = g_strdup ("(c) 2002-2003");
-
-    /* register the plugin with gstreamer */
-    factory = gst_element_factory_new(type_name,type,details);
-    g_return_val_if_fail(factory != NULL, FALSE);
-
-    /* typefind info */
-    type_definition = g_new0 (GstTypeDefinition, 1);
-    type_definition->name = g_strdup_printf ("fftype_%s",
-					     in_plugin->name);
-    type_definition->mime = g_strdup (gst_caps_get_mime (sinkcaps));
-    type_definition->exts = g_strdup (in_plugin->extensions);
-    type_definition->typefindfunc = gst_ffmpegdemux_typefind;
-
-    type_factory = gst_type_factory_new (type_definition);
-
+    
     /* create a cache for these properties */
     params = g_new0 (GstFFMpegDemuxClassParams, 1);
     params->in_plugin = in_plugin;
-    params->typefind_feature = GST_PLUGIN_FEATURE (type_factory);
-    params->sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK,
-					      GST_PAD_ALWAYS,
-					      sinkcaps, NULL);
-    gst_element_factory_add_pad_template (factory,
-					  params->sinktempl);
-    params->audiosrctempl = gst_pad_template_new ("audio_%02d",
-						  GST_PAD_SRC,
-						  GST_PAD_SOMETIMES,
-						  audiosrccaps, NULL);
-    gst_element_factory_add_pad_template (factory,
-					  params->audiosrctempl);
-    params->videosrctempl = gst_pad_template_new ("video_%02d",
-						  GST_PAD_SRC,
-						  GST_PAD_SOMETIMES,
-						  videosrccaps, NULL);
-    gst_element_factory_add_pad_template (factory,
-					  params->videosrctempl);
+    params->sinkcaps = sinkcaps;
+    params->videosrccaps = videosrccaps;
+    params->audiosrccaps = audiosrccaps;
+
+    g_hash_table_insert (global_plugins, 
+		         GINT_TO_POINTER (0), 
+			 (gpointer) params);
+
+    /* create the type now */
+    type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
 
     g_hash_table_insert (global_plugins, 
 		         GINT_TO_POINTER (type), 
 			 (gpointer) params);
 
-    g_hash_table_insert (typefind,
-			 (gpointer) type_factory,
-			 (gpointer) params);
-
-    gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_MARGINAL);
-
-    /* The very last thing is to register the elementfactory with the plugin. */
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type_factory));
+    extensions = g_strsplit (in_plugin->extensions, " ", 0);
+    if (!gst_element_register (plugin, type_name, GST_RANK_MARGINAL, type) ||
+        !gst_type_find_register (plugin, typefind_name, GST_RANK_MARGINAL,
+				 gst_ffmpegdemux_type_find,
+				 extensions, any_caps, params))
+      return FALSE;
+    g_strfreev (extensions);
 
 next:
     in_plugin = in_plugin->next;
   }
+  gst_caps_free (any_caps);
+  g_hash_table_remove (global_plugins, GINT_TO_POINTER (0));
 
   return TRUE;
 }
