@@ -17,7 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <assert.h>
 #include <string.h>
@@ -123,7 +125,7 @@ static void	gst_ffmpegenc_init		(GstFFMpegEnc *ffmpegenc);
 static void	gst_ffmpegenc_dispose		(GObject *object);
 
 static GstPadLinkReturn
-		gst_ffmpegenc_connect		(GstPad *pad, GstCaps *caps);
+		gst_ffmpegenc_connect		(GstPad *pad, const GstCaps *caps);
 static void	gst_ffmpegenc_chain_video	(GstPad *pad, GstData *_data);
 static void	gst_ffmpegenc_chain_audio	(GstPad *pad, GstData *_data);
 
@@ -175,9 +177,9 @@ gst_ffmpegenc_base_init (GstFFMpegEncClass *klass)
 
   /* pad templates */
   sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK,
-				    GST_PAD_ALWAYS, params->sinkcaps, NULL);
+				    GST_PAD_ALWAYS, params->sinkcaps);
   srctempl = gst_pad_template_new ("src", GST_PAD_SRC,
-				   GST_PAD_ALWAYS, params->srccaps, NULL);
+				   GST_PAD_ALWAYS, params->srccaps);
 
   gst_element_class_add_pad_template (element_class, srctempl);
   gst_element_class_add_pad_template (element_class, sinktempl);
@@ -241,6 +243,7 @@ gst_ffmpegenc_init(GstFFMpegEnc *ffmpegenc)
   ffmpegenc->sinkpad = gst_pad_new_from_template (oclass->sinktempl, "sink");
   gst_pad_set_link_function (ffmpegenc->sinkpad, gst_ffmpegenc_connect);
   ffmpegenc->srcpad = gst_pad_new_from_template (oclass->srctempl, "src");
+  gst_pad_use_explicit_caps (ffmpegenc->srcpad);
 
   gst_element_add_pad (GST_ELEMENT (ffmpegenc), ffmpegenc->sinkpad);
   gst_element_add_pad (GST_ELEMENT (ffmpegenc), ffmpegenc->srcpad);
@@ -282,15 +285,14 @@ gst_ffmpegenc_dispose (GObject *object)
 
 static GstPadLinkReturn
 gst_ffmpegenc_connect (GstPad  *pad,
-		       GstCaps *caps)
+		       const GstCaps *caps)
 {
+  GstCaps *other_caps;
+  GstCaps *allowed_caps;
+  GstCaps *icaps;
+  enum PixelFormat pix_fmt;
   GstFFMpegEnc *ffmpegenc = (GstFFMpegEnc *) gst_pad_get_parent (pad);
-  GstFFMpegEncClass *oclass = (GstFFMpegEncClass*)(G_OBJECT_GET_CLASS(ffmpegenc));
-  GstCaps *ret_caps;
-  GstPadLinkReturn ret;
-
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  GstFFMpegEncClass *oclass = (GstFFMpegEncClass *) G_OBJECT_GET_CLASS(ffmpegenc);
 
   /* close old session */
   if (ffmpegenc->opened) {
@@ -315,53 +317,64 @@ gst_ffmpegenc_connect (GstPad  *pad,
   /* no edges */
   ffmpegenc->context->flags |= CODEC_FLAG_EMU_EDGE;
 
-  for (ret_caps = caps; ret_caps != NULL; ret_caps = ret_caps->next) {
-    enum PixelFormat pix_fmt;
+  /* fetch pix_fmt and so on */
+  gst_ffmpeg_caps_to_codectype (oclass->in_plugin->type,
+				caps, ffmpegenc->context);
 
-    /* fetch pix_fmt and so on */
-    gst_ffmpeg_caps_to_codectype (oclass->in_plugin->type,
-				  caps, ffmpegenc->context);
+  pix_fmt = ffmpegenc->context->pix_fmt;
 
-    pix_fmt = ffmpegenc->context->pix_fmt;
-
-    /* open codec */
-    if (avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0) {
-      GST_DEBUG ("ffenc_%s: Failed to open FFMPEG codec",
-		 oclass->in_plugin->name);
-      continue;
-    }
-
-    /* is the colourspace correct? */
-    if (pix_fmt != ffmpegenc->context->pix_fmt) {
-      avcodec_close (ffmpegenc->context);
-      GST_DEBUG ("ffenc_%s: AV wants different colourspace (%d given, %d wanted)",
-		 oclass->in_plugin->name, pix_fmt, ffmpegenc->context->pix_fmt);
-      continue;
-    }
-
-    break;
+  /* open codec */
+  if (avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0) {
+    avcodec_close (ffmpegenc->context);
+    GST_DEBUG ("ffenc_%s: Failed to open FFMPEG codec",
+	       oclass->in_plugin->name);
+    return GST_PAD_LINK_REFUSED;
   }
 
-  if (ret_caps == NULL) {
+  /* is the colourspace correct? */
+  if (pix_fmt != ffmpegenc->context->pix_fmt) {
+    avcodec_close (ffmpegenc->context);
+    GST_DEBUG ("ffenc_%s: AV wants different colourspace (%d given, %d wanted)",
+	       oclass->in_plugin->name, pix_fmt, ffmpegenc->context->pix_fmt);
     return GST_PAD_LINK_REFUSED;
   }
 
   /* try to set this caps on the other side */
-  ret_caps = gst_ffmpeg_codecid_to_caps (oclass->in_plugin->id,
+  other_caps = gst_ffmpeg_codecid_to_caps (oclass->in_plugin->id,
 					 ffmpegenc->context);
-  if (!ret_caps) {
+  if (!other_caps) {
     avcodec_close (ffmpegenc->context);
     GST_DEBUG ("Unsupported codec - no caps found");
     return GST_PAD_LINK_REFUSED;
   }
 
-  if ((ret = gst_pad_try_set_caps (ffmpegenc->srcpad, ret_caps)) <= 0) {
-    avcodec_close (ffmpegenc->context);
-    GST_DEBUG ("Failed to set caps on next element for ffmpeg encoder (%s)",
-               oclass->in_plugin->name);
-    return ret;
+  allowed_caps = gst_pad_get_allowed_caps (ffmpegenc->srcpad);
+  icaps = gst_caps_intersect (allowed_caps, other_caps);
+  gst_caps_free (allowed_caps);
+  gst_caps_free (other_caps);
+
+  if (gst_caps_is_empty (icaps)) {
+    gst_caps_free (icaps);
+    return GST_PAD_LINK_REFUSED;
   }
 
+  if (gst_caps_get_size (icaps) > 1) {
+    GstCaps *newcaps;
+
+    newcaps = gst_caps_new_full (gst_structure_copy (gst_caps_get_structure (icaps, 0)), NULL);
+    gst_caps_free (icaps);
+    icaps = newcaps;
+  }
+
+  /* FIXME set_explicit_caps is not supposed to be used in a pad link
+   * function. */
+  if (!gst_pad_set_explicit_caps (ffmpegenc->srcpad, icaps)) {
+    avcodec_close (ffmpegenc->context);
+    gst_caps_free (icaps);
+    return GST_PAD_LINK_REFUSED;
+  }
+  gst_caps_free (icaps);
+
   /* success! */
   ffmpegenc->opened = TRUE;
 
@@ -482,6 +495,7 @@ gst_ffmpegenc_chain_audio (GstPad  *pad,
       g_warning("ffenc_%s: failed to encode buffer",
 		oclass->in_plugin->name);
       gst_buffer_unref (inbuf);
+      gst_buffer_unref (outbuf);
       gst_buffer_unref (subbuf);
       return;
     }
@@ -647,8 +661,12 @@ gst_ffmpegenc_register (GstPlugin *plugin)
 
     /* create the glib type now */
     type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
-    if (!gst_element_register (plugin, type_name, GST_RANK_NONE, type))
+    if (!gst_element_register (plugin, type_name, GST_RANK_NONE, type)) {
+      g_free (type_name);
       return FALSE;
+    }
+
+    g_free (type_name);
 
     g_hash_table_insert (enc_global_plugins, 
 		         GINT_TO_POINTER (type), 
