@@ -17,25 +17,27 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
-#include <sys/soundcard.h>
 
 /*#define DEBUG_ENABLED */
 #include "gstflacdec.h"
+#include <gst/gsttaginterface.h>
+
+#include <gst/tag/tag.h>
 
 #include "flac_compat.h"
 
-extern GstPadTemplate *gst_flacdec_src_template, *gst_flacdec_sink_template;
+static GstPadTemplate *src_template, *sink_template;
 
 /* elementfactory information */
 GstElementDetails flacdec_details = {
   "FLAC decoder",
-  "Codec/Audio/Decoder",
-  "LGPL",
+  "Codec/Decoder/Audio",
   "Decodes FLAC lossless audio streams",
-  VERSION,
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
 };
 
 /* FlacDec signals and args */
@@ -49,6 +51,7 @@ enum {
   ARG_METADATA
 };
 
+static void             gst_flacdec_base_init           (gpointer g_class);
 static void 		gst_flacdec_class_init		(FlacDecClass *klass);
 static void 		gst_flacdec_init		(FlacDec *flacdec);
 
@@ -65,14 +68,6 @@ static gboolean		gst_flacdec_src_query 		(GstPad *pad, GstQueryType type,
 static const GstEventMask* 
 			gst_flacdec_get_src_event_masks (GstPad *pad);
 static gboolean 	gst_flacdec_src_event 		(GstPad *pad, GstEvent *event);
-static void             gst_flacdec_get_property        (GObject *object, 
-                                                         guint prop_id, 
-                                                         GValue *value, 
-                                                         GParamSpec *pspec);
-static void             gst_flacdec_set_property        (GObject *object, 
-                                                         guint prop_id, 
-                                                         const GValue *value, 
-                                                         GParamSpec *pspec);
 
 static FLAC__SeekableStreamDecoderReadStatus 	
 			gst_flacdec_read 		(const FLAC__SeekableStreamDecoder *decoder, 
@@ -111,7 +106,7 @@ flacdec_get_type(void) {
   if (!flacdec_type) {
     static const GTypeInfo flacdec_info = {
       sizeof(FlacDecClass),
-      NULL,
+      gst_flacdec_base_init,
       NULL,
       (GClassInitFunc)gst_flacdec_class_init,
       NULL,
@@ -125,6 +120,45 @@ flacdec_get_type(void) {
   return flacdec_type;
 }
 
+static GstCaps*
+flac_caps_factory (void)
+{
+  return gst_caps_new_simple ("audio/x-flac", NULL);
+  /* "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+   * "channels", 	GST_PROPS_INT_RANGE (1, 2), */
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return gst_caps_new_simple ("audio/x-raw-int",
+      "endianness", 	G_TYPE_INT, G_BYTE_ORDER,
+      "signed", 	G_TYPE_BOOLEAN, TRUE,
+      "width", 		G_TYPE_INT, 16,
+      "depth",    	G_TYPE_INT, 16,
+      "rate",     	GST_TYPE_INT_RANGE, 11025, 48000,
+      "channels", 	GST_TYPE_INT_RANGE, 1, 2,
+      NULL);
+}
+
+static void
+gst_flacdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *flac_caps;
+
+  raw_caps = raw_caps_factory ();
+  flac_caps = flac_caps_factory ();
+
+  sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
+      GST_PAD_ALWAYS, flac_caps);
+  src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+      GST_PAD_ALWAYS, raw_caps);
+  gst_element_class_add_pad_template (element_class, sink_template);
+  gst_element_class_add_pad_template (element_class, src_template);
+  gst_element_class_set_details (element_class, &flacdec_details);
+}
+
 static void
 gst_flacdec_class_init (FlacDecClass *klass) 
 {
@@ -135,26 +169,19 @@ gst_flacdec_class_init (FlacDecClass *klass)
   gobject_class = (GObjectClass*) klass;
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
-	
-  g_object_class_install_property (gobject_class, ARG_METADATA,
-  g_param_spec_boxed ("metadata", "Metadata", "(logical) Stream metadata",
-                      GST_TYPE_CAPS, G_PARAM_READABLE));
 
-  gobject_class->get_property = gst_flacdec_get_property;
-  gobject_class->set_property = gst_flacdec_set_property;
-	
   gstelement_class->change_state = gst_flacdec_change_state;
 }
 
 static void 
 gst_flacdec_init (FlacDec *flacdec) 
 {
-  flacdec->sinkpad = gst_pad_new_from_template (gst_flacdec_sink_template, "sink");
+  flacdec->sinkpad = gst_pad_new_from_template (sink_template, "sink");
   gst_element_add_pad (GST_ELEMENT (flacdec), flacdec->sinkpad);
   gst_pad_set_convert_function (flacdec->sinkpad, NULL);
 
   gst_element_set_loop_function (GST_ELEMENT (flacdec), gst_flacdec_loop);
-  flacdec->srcpad = gst_pad_new_from_template (gst_flacdec_src_template, "src");
+  flacdec->srcpad = gst_pad_new_from_template (src_template, "src");
   gst_element_add_pad (GST_ELEMENT (flacdec), flacdec->srcpad);
   gst_pad_set_formats_function (flacdec->srcpad, gst_flacdec_get_src_formats);
   gst_pad_set_convert_function (flacdec->srcpad, gst_flacdec_convert_src);
@@ -162,18 +189,18 @@ gst_flacdec_init (FlacDec *flacdec)
   gst_pad_set_query_function (flacdec->srcpad, gst_flacdec_src_query);
   gst_pad_set_event_mask_function (flacdec->srcpad, gst_flacdec_get_src_event_masks);
   gst_pad_set_event_function (flacdec->srcpad, gst_flacdec_src_event);
+  gst_pad_use_explicit_caps (flacdec->srcpad);
 
   flacdec->decoder = FLAC__seekable_stream_decoder_new ();
   flacdec->total_samples = 0;
   flacdec->init = TRUE;
   flacdec->eos = FALSE;
   flacdec->seek_pending = FALSE;
-  flacdec->metadata = NULL;
 
   FLAC__seekable_stream_decoder_set_read_callback (flacdec->decoder, gst_flacdec_read);
   FLAC__seekable_stream_decoder_set_seek_callback (flacdec->decoder, gst_flacdec_seek);
   FLAC__seekable_stream_decoder_set_tell_callback (flacdec->decoder, gst_flacdec_tell);
-  FLAC__seekable_stream_decoder_set_length_callback (flacdec->decoder, gst_flacdec_length);
+  FLAC__seekable_stream_decoder_set_length_callback (flacdec->decoder, gst_flacdec_length);  
   FLAC__seekable_stream_decoder_set_eof_callback (flacdec->decoder, gst_flacdec_eof);
 #if FLAC_VERSION >= 0x010003
   FLAC__seekable_stream_decoder_set_write_callback (flacdec->decoder, gst_flacdec_write);
@@ -195,51 +222,44 @@ gst_flacdec_init (FlacDec *flacdec)
 static gboolean
 gst_flacdec_update_metadata (FlacDec *flacdec, const FLAC__StreamMetadata *metadata)
 {
+  GstTagList *list;
   guint32 number_of_comments, cursor, str_len;
   gchar *p_value, *value, *name, *str_ptr;
-  GstProps *props = NULL;
-  GstPropsEntry *entry;
-	
-  /* clear old one */
-  if (flacdec->metadata) {
-    gst_caps_unref (flacdec->metadata);
-    flacdec->metadata = NULL;
+
+  list = gst_tag_list_new ();
+  if (list == NULL) {
+    return FALSE;
   }
 
-  /* create props to hold the key/value pairs */
-  props = gst_props_empty_new ();
-  
   number_of_comments = metadata->data.vorbis_comment.num_comments;
   value = NULL;
-  GST_DEBUG (GST_CAT_CAPS, "%d tag(s) found",  number_of_comments);
+  GST_DEBUG ("%d tag(s) found",  number_of_comments);
   for (cursor = 0; cursor < number_of_comments; cursor++)
   {			
     str_ptr = metadata->data.vorbis_comment.comments[cursor].entry;
-    str_len = GUINT32_FROM_LE (metadata->data.vorbis_comment.comments[cursor].length);
+    str_len = metadata->data.vorbis_comment.comments[cursor].length;
     p_value = g_strstr_len ( str_ptr, str_len , "=" );
     if (p_value)
     {			
       name = g_strndup (str_ptr, p_value - str_ptr);
-      value = g_strndup (p_value + 1, str_ptr + str_len - p_value);
+      value = g_strndup (p_value + 1, str_ptr + str_len - p_value - 1);
 			
-      entry = gst_props_entry_new (name, GST_PROPS_STRING_TYPE, value);
-      gst_props_add_entry (props, (GstPropsEntry *) entry);
-
-      GST_DEBUG (GST_CAT_CAPS, "%s : %s", name, value);
-
+      GST_DEBUG ("%s : %s", name, value);
+      gst_vorbis_tag_add (list, name, value);
       g_free (name);
       g_free (value);
     }
   }
-	
-  flacdec->metadata = gst_caps_new ("vorbisfile_metadata",
-                                    "application/x-gst-metadata",
-                                    props);
-  g_object_notify (G_OBJECT (flacdec), "metadata");
-	
+
+
+  gst_element_found_tags (GST_ELEMENT (flacdec), list);
+  if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+    gst_pad_push (flacdec->srcpad, GST_DATA (gst_event_new_tag (list)));
+  }
   return TRUE;
 }
 
+
 static void 
 gst_flacdec_metadata_callback (const FLAC__SeekableStreamDecoder *decoder,
 			       const FLAC__StreamMetadata *metadata, void *client_data)
@@ -285,9 +305,7 @@ gst_flacdec_error_callback (const FLAC__SeekableStreamDecoder *decoder,
       break;
   }
 
-  GST_DEBUG (0, error);
-				  
-  gst_element_error (GST_ELEMENT (flacdec), error);
+  GST_ELEMENT_ERROR (flacdec, STREAM, DECODE, (NULL), (error));
 }
 
 static FLAC__SeekableStreamDecoderSeekStatus 	
@@ -298,7 +316,7 @@ gst_flacdec_seek (const FLAC__SeekableStreamDecoder *decoder,
 
   flacdec = GST_FLACDEC (client_data);
 
-  GST_DEBUG (0, "seek %" G_GINT64_FORMAT, position);
+  GST_DEBUG ("seek %" G_GINT64_FORMAT, position);
   if (!gst_bytestream_seek (flacdec->bs, position, GST_SEEK_METHOD_SET)) {
     return FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR;
   }
@@ -317,12 +335,11 @@ gst_flacdec_tell (const FLAC__SeekableStreamDecoder *decoder,
   if (*position == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "tell %" G_GINT64_FORMAT, *position);
+  GST_DEBUG ("tell %" G_GINT64_FORMAT, *position);
 
   return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
 }
 
-
 static FLAC__SeekableStreamDecoderLengthStatus 	
 gst_flacdec_length (const FLAC__SeekableStreamDecoder *decoder, 
 		    FLAC__uint64 *length, void *client_data)
@@ -335,7 +352,7 @@ gst_flacdec_length (const FLAC__SeekableStreamDecoder *decoder,
   if (*length == -1)
     return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;
 
-  GST_DEBUG (0, "length %" G_GINT64_FORMAT, *length);
+  GST_DEBUG ("length %" G_GINT64_FORMAT, *length);
 
   return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
 }
@@ -347,7 +364,7 @@ gst_flacdec_eof (const FLAC__SeekableStreamDecoder *decoder,
   FlacDec *flacdec;
 
   flacdec = GST_FLACDEC (client_data);
-  GST_DEBUG (0, "eof %d", flacdec->eos);
+  GST_DEBUG ("eof %d", flacdec->eos);
 
   return flacdec->eos;
 }
@@ -375,7 +392,7 @@ gst_flacdec_read (const FLAC__SeekableStreamDecoder *decoder,
 
       switch (GST_EVENT_TYPE (event)) {
         case GST_EVENT_EOS:
-          GST_DEBUG (0, "eos");
+          GST_DEBUG ("eos");
           flacdec->eos = TRUE; 
 	  gst_event_unref (event);
           if (avail == 0) {
@@ -383,7 +400,7 @@ gst_flacdec_read (const FLAC__SeekableStreamDecoder *decoder,
           }
           break;
         case GST_EVENT_DISCONTINUOUS:
-          GST_DEBUG (0, "discont");
+          GST_DEBUG ("discont");
 
 	  /* we are not yet sending the discont, we'll do that in the next write operation */
 	  flacdec->need_discont = TRUE;
@@ -428,43 +445,41 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder *decoder,
     GstEvent *discont;
 
     flacdec->need_discont = FALSE;
+    
+    if (!GST_PAD_CAPS (flacdec->srcpad)) {
+      if (flacdec->seek_pending) {
+        flacdec->total_samples = flacdec->seek_value;
+      }
 
-    if (flacdec->seek_pending) {
-      flacdec->total_samples = flacdec->seek_value;
-    }
+      if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+        GST_DEBUG ("send discont");
 
-    if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
-      GST_DEBUG (0, "send discont");
-
-      format = GST_FORMAT_TIME;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_UNITS,
-                       flacdec->total_samples, &format, &time);
-      format = GST_FORMAT_BYTES;
-      gst_pad_convert (flacdec->srcpad, GST_FORMAT_UNITS,
-                       flacdec->total_samples, &format, &bytes);
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
-		                         GST_FORMAT_BYTES, bytes,
-		                         GST_FORMAT_UNITS, flacdec->total_samples, 
+        format = GST_FORMAT_TIME;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+                         flacdec->total_samples, &format, &time);
+        format = GST_FORMAT_BYTES;
+        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+                         flacdec->total_samples, &format, &bytes);
+        discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
+	  	                             GST_FORMAT_BYTES, bytes,
+		                             GST_FORMAT_DEFAULT, flacdec->total_samples, 
 					 NULL);
 	  
-      gst_pad_push (flacdec->srcpad, GST_BUFFER (discont));
+        gst_pad_push (flacdec->srcpad, GST_DATA (discont));
+      }
     }
   }
   
   if (!GST_PAD_CAPS (flacdec->srcpad)) {
-    gst_pad_try_set_caps (flacdec->srcpad,
-		    GST_CAPS_NEW (
-		      "flac_caps",
-		      "audio/raw",
-		        "format", 	GST_PROPS_STRING ("int"),
-                         "law",         GST_PROPS_INT (0),
-                         "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-                         "signed",      GST_PROPS_BOOLEAN (TRUE),
-                         "width",       GST_PROPS_INT (depth),
-                         "depth",       GST_PROPS_INT (depth),
-                         "rate",     	GST_PROPS_INT (frame->header.sample_rate),
-                         "channels", 	GST_PROPS_INT (channels)
-		    ));
+    gst_pad_set_explicit_caps (flacdec->srcpad,
+        gst_caps_new_simple ("audio/x-raw-int",
+          "endianness",  G_TYPE_INT, G_BYTE_ORDER,
+          "signed",      G_TYPE_BOOLEAN, TRUE,
+          "width",       G_TYPE_INT, depth,
+          "depth",       G_TYPE_INT, depth,
+          "rate",     	 G_TYPE_INT, frame->header.sample_rate,
+          "channels", 	 G_TYPE_INT, channels,
+          NULL));
 
     flacdec->depth = depth;
     flacdec->channels = channels;
@@ -499,7 +514,7 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder *decoder,
       g_warning ("flacdec: invalid depth %d found\n", depth);
       return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
     }
-    gst_pad_push (flacdec->srcpad, outbuf);
+    gst_pad_push (flacdec->srcpad, GST_DATA (outbuf));
   }
   flacdec->total_samples += samples;
 
@@ -514,56 +529,68 @@ gst_flacdec_loop (GstElement *element)
 
   flacdec = GST_FLACDEC (element);
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: entering loop");
-  if (flacdec->init) {
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: initializing decoder");
-    FLAC__seekable_stream_decoder_init (flacdec->decoder);
-    /* FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder); */
+  GST_DEBUG ("flacdec: entering loop");
+  if (flacdec->init) {    
+    FLAC__StreamDecoderState res;
+    GST_DEBUG ("flacdec: initializing decoder");    
+    res = FLAC__seekable_stream_decoder_init (flacdec->decoder);
+    if (res != FLAC__SEEKABLE_STREAM_DECODER_OK) {
+      GST_ELEMENT_ERROR (flacdec, LIBRARY, INIT, (NULL),
+                         (FLAC__SeekableStreamDecoderStateString[res]));
+      return;
+    }
+    /*    FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder);*/
     flacdec->init = FALSE;
   }
 
   if (flacdec->seek_pending) {
-    GST_DEBUG (GST_CAT_EVENT, "perform seek to sample %" G_GINT64_FORMAT, 
+    GST_DEBUG ("perform seek to sample %" G_GINT64_FORMAT, 
 		              flacdec->seek_value);
 
     if (FLAC__seekable_stream_decoder_seek_absolute (flacdec->decoder, 
 			                             flacdec->seek_value)) 
     {
       flacdec->total_samples = flacdec->seek_value;
-      GST_DEBUG (GST_CAT_EVENT, "seek done");
+      GST_DEBUG ("seek done");
     }
     else {
-      GST_DEBUG (GST_CAT_EVENT, "seek failed");
+      GST_DEBUG ("seek failed");
     }
     flacdec->seek_pending = FALSE;
   }
 
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: processing single");
+  GST_DEBUG ("flacdec: processing single");
   res = FLAC__seekable_stream_decoder_process_single (flacdec->decoder);
-  GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: checking for EOS");
+  GST_DEBUG ("flacdec: checking for EOS");
   if (FLAC__seekable_stream_decoder_get_state (flacdec->decoder) == 
 		  FLAC__SEEKABLE_STREAM_DECODER_END_OF_STREAM) 
   {
     GstEvent *event;
 
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: sending EOS event");
-    FLAC__seekable_stream_decoder_finish(flacdec->decoder);
-    flacdec->init = TRUE;
+    GST_DEBUG ("flacdec: sending EOS event");
+    FLAC__seekable_stream_decoder_reset(flacdec->decoder);
 
     if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
       event = gst_event_new (GST_EVENT_EOS);
-      gst_pad_push (flacdec->srcpad, GST_BUFFER (event));
+      gst_pad_push (flacdec->srcpad, GST_DATA (event));
     }
     gst_element_set_eos (element);
   }
-   GST_DEBUG (GST_CAT_PLUGIN_INFO, "flacdec: _loop end");
+  GST_DEBUG ("flacdec: _loop end");
 }
 
-GST_PAD_FORMATS_FUNCTION (gst_flacdec_get_src_formats,
-  GST_FORMAT_UNITS,
-  GST_FORMAT_BYTES,
-  GST_FORMAT_TIME
-)
+
+static const GstFormat *
+gst_flacdec_get_src_formats (GstPad *pad)
+{ 
+  static const GstFormat formats[] = {
+    GST_FORMAT_DEFAULT,
+    GST_FORMAT_BYTES,
+    GST_FORMAT_TIME,
+    0,
+  };
+  return formats;
+} 
 
 static gboolean
 gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
@@ -579,13 +606,11 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           if (bytes_per_sample == 0)
             return FALSE;
           *dest_value = src_value / bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	{
           gint byterate = bytes_per_sample * flacdec->frequency;
@@ -599,13 +624,11 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (flacdec->frequency == 0)
 	    return FALSE;
@@ -617,11 +640,9 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
           scale = bytes_per_sample;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * flacdec->frequency / GST_SECOND;
           break;
         default:
@@ -634,10 +655,16 @@ gst_flacdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-GST_PAD_QUERY_TYPE_FUNCTION (gst_flacdec_get_src_query_types,
-  GST_QUERY_TOTAL,
-  GST_QUERY_POSITION
-)
+static const GstQueryType *
+gst_flacdec_get_src_query_types (GstPad *pad)
+{ 
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0,
+  };
+  return types;
+} 
 
 static gboolean
 gst_flacdec_src_query (GstPad *pad, GstQueryType type,
@@ -657,14 +684,14 @@ gst_flacdec_src_query (GstPad *pad, GstQueryType type,
         samples = flacdec->stream_samples;
 
       gst_pad_convert (flacdec->srcpad, 
-		       GST_FORMAT_UNITS, 
+		       GST_FORMAT_DEFAULT, 
 		       samples, 
 		       format, value);
       break;
     }
     case GST_QUERY_POSITION:
       gst_pad_convert (flacdec->srcpad, 
-		       GST_FORMAT_UNITS, 
+		       GST_FORMAT_DEFAULT, 
 		       flacdec->total_samples, 
 		       format, value);
       break;
@@ -676,9 +703,15 @@ gst_flacdec_src_query (GstPad *pad, GstQueryType type,
   return res;
 }
 	  
-GST_PAD_EVENT_MASK_FUNCTION (gst_flacdec_get_src_event_masks,
-    { GST_EVENT_SEEK, GST_SEEK_FLAG_ACCURATE }
-);
+static const GstEventMask *
+gst_flacdec_get_src_event_masks (GstPad *pad)
+{
+  static const GstEventMask masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_FLAG_ACCURATE },
+    { 0, 0 },
+  };
+  return masks;
+}
 
 static gboolean
 gst_flacdec_src_event (GstPad *pad, GstEvent *event)
@@ -689,7 +722,7 @@ gst_flacdec_src_event (GstPad *pad, GstEvent *event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-      format = GST_FORMAT_UNITS;
+      format = GST_FORMAT_DEFAULT;
 
       if (gst_pad_convert (flacdec->srcpad, 
 			   GST_EVENT_SEEK_FORMAT (event), 
@@ -718,9 +751,10 @@ gst_flacdec_change_state (GstElement *element)
       flacdec->bs = gst_bytestream_new (flacdec->sinkpad);
       flacdec->seek_pending = FALSE;
       flacdec->total_samples = 0;
-      flacdec->init = TRUE;
       flacdec->eos = FALSE;
-      flacdec->metadata = NULL;
+      if (flacdec->init == FALSE) {
+	FLAC__seekable_stream_decoder_reset (flacdec->decoder);
+      }
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       flacdec->eos = FALSE;
@@ -739,38 +773,3 @@ gst_flacdec_change_state (GstElement *element)
 
   return GST_STATE_SUCCESS;
 }
-
-static void
-gst_flacdec_set_property (GObject *object, guint prop_id, 
-                          const GValue *value, GParamSpec *pspec)
-{
-  FlacDec *flacdec;
-	      
-  g_return_if_fail (GST_IS_FLACDEC (object));
-
-  flacdec = GST_FLACDEC (object);
-
-  switch (prop_id) {
-    default:
-      g_warning ("Unknown property id\n");
-  }
-}
-
-static void 
-gst_flacdec_get_property (GObject *object, guint prop_id, 
-                          GValue *value, GParamSpec *pspec)
-{
-  FlacDec *flacdec;
-	      
-  g_return_if_fail (GST_IS_FLACDEC(object));
-
-  flacdec = GST_FLACDEC (object);
-
-  switch (prop_id) {
-    case ARG_METADATA:
-      g_value_set_boxed (value, flacdec->metadata);
-      break;
-    default:
-      g_warning ("Unknown property id\n");
-  }
-}
