@@ -25,7 +25,8 @@
 #include <string.h>
 
 #include <gstflacenc.h>
-
+#include <gst/tag/tag.h>
+#include <gst/gsttaginterface.h>
 #include "flac_compat.h"
 
 static GstPadTemplate *src_template, *sink_template;
@@ -67,7 +68,7 @@ static void		gst_flacenc_class_init		(FlacEncClass *klass);
 static void 		gst_flacenc_dispose 		(GObject *object);
 
 static GstPadLinkReturn
-			gst_flacenc_sinkconnect 	(GstPad *pad, GstCaps *caps);
+			gst_flacenc_sinkconnect 	(GstPad *pad, const GstCaps *caps);
 static void		gst_flacenc_chain		(GstPad *pad, GstData *_data);
 
 static gboolean 	gst_flacenc_update_quality 	(FlacEnc *flacenc, gint quality);
@@ -108,7 +109,15 @@ flacenc_get_type (void)
       0,
       (GInstanceInitFunc)gst_flacenc_init,
     };
+
+    static const GInterfaceInfo tag_setter_info = {
+      NULL,
+      NULL,
+      NULL
+    };
+
     flacenc_type = g_type_register_static (GST_TYPE_ELEMENT, "FlacEnc", &flacenc_info, 0);
+    g_type_add_interface_static (flacenc_type, GST_TYPE_TAG_SETTER, &tag_setter_info);
   }
   return flacenc_type;
 }
@@ -170,31 +179,22 @@ gst_flacenc_quality_get_type (void)
 static GstCaps*
 flac_caps_factory (void)
 {
-  return
-   gst_caps_new (
-  	"flac_flac",
-  	"application/x-flac",
-	/* gst_props_new (
- 	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
-    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
-	    NULL) */NULL);
+  return gst_caps_new_simple ("audio/x-flac", NULL);
+  /* "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+   * "channels", 	GST_PROPS_INT_RANGE (1, 2), */
 }
 
 static GstCaps*
 raw_caps_factory (void)
 {
-  return
-   gst_caps_new (
-  	"flac_raw",
-  	"audio/x-raw-int",
-	gst_props_new (
-    	    "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
-    	    "signed", 		GST_PROPS_BOOLEAN (TRUE),
-    	    "width", 		GST_PROPS_INT (16),
-    	    "depth",    	GST_PROPS_INT (16),
-    	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
-    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
-	    NULL));
+  return gst_caps_new_simple ("audio/x-raw-int",
+      "endianness", 	G_TYPE_INT, G_BYTE_ORDER,
+      "signed", 	G_TYPE_BOOLEAN, TRUE,
+      "width", 		G_TYPE_INT, 16,
+      "depth",    	G_TYPE_INT, 16,
+      "rate",     	GST_TYPE_INT_RANGE, 11025, 48000,
+      "channels", 	GST_TYPE_INT_RANGE, 1, 2,
+      NULL);
 }
 
 static void
@@ -205,13 +205,13 @@ gst_flacenc_base_init (gpointer g_class)
 
   raw_caps = raw_caps_factory ();
   flac_caps = flac_caps_factory ();
-  
+
   sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
 					GST_PAD_ALWAYS, 
-					raw_caps, NULL);
+					raw_caps);
   src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
 				       GST_PAD_ALWAYS, 
-				       flac_caps, NULL);
+				       flac_caps);
   gst_element_class_add_pad_template (element_class, sink_template);
   gst_element_class_add_pad_template (element_class, src_template);
   gst_element_class_set_details (element_class, &flacenc_details);
@@ -326,38 +326,12 @@ gst_flacenc_init (FlacEnc *flacenc)
 
   flacenc->encoder = FLAC__seekable_stream_encoder_new();
 
-  FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
-		                        gst_flacenc_write_callback);
-  FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
-		                        gst_flacenc_seek_callback);
-  					
-  FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
-		                        flacenc);
-
-  flacenc->metadata = GST_CAPS_NEW (
-                  "flacenc_metadata",
-                  "application/x-gst-metadata",
-                    "DESCRIPTION",      GST_PROPS_STRING ("Track encoded with GStreamer"),
-                    "DATE",             GST_PROPS_STRING (""),
-                    "TRACKNUMBER",      GST_PROPS_STRING (""),
-                    "TITLE",            GST_PROPS_STRING (""),
-                    "ARTIST",           GST_PROPS_STRING (""),
-                    "ALBUM",            GST_PROPS_STRING (""),
-                    "GENRE",            GST_PROPS_STRING (""),
-                    "VERSION",          GST_PROPS_STRING (""),
-                    "COPYRIGHT",        GST_PROPS_STRING (""),
-                    "LICENSE",          GST_PROPS_STRING (""),
-                    "ORGANISATION",     GST_PROPS_STRING (""),
-                    "LOCATION",         GST_PROPS_STRING (""),
-                    "CONTACT",          GST_PROPS_STRING (""),
-                    "ISRC",             GST_PROPS_STRING ("")
-                  );
-		  
   flacenc->negotiated = FALSE;
   flacenc->first = TRUE;
   flacenc->first_buf = NULL;
   flacenc->data = NULL;
   gst_flacenc_update_quality (flacenc, DEFAULT_QUALITY);
+  flacenc->tags = gst_tag_list_new ();
 }
 
 static void
@@ -371,24 +345,23 @@ gst_flacenc_dispose (GObject *object)
 }
 
 static GstPadLinkReturn
-gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_flacenc_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstPadLinkReturn ret;
   FlacEnc *flacenc;
+  GstStructure *structure;
 
   flacenc = GST_FLACENC (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
 
-  gst_caps_get_int (caps, "channels", &flacenc->channels);
-  gst_caps_get_int (caps, "depth", &flacenc->depth);
-  gst_caps_get_int (caps, "rate", &flacenc->sample_rate);
+  gst_structure_get_int (structure, "channels", &flacenc->channels);
+  gst_structure_get_int (structure, "depth", &flacenc->depth);
+  gst_structure_get_int (structure, "rate", &flacenc->sample_rate);
   
-  caps = GST_CAPS_NEW ("flacenc_srccaps",
-                       "application/x-flac",
-                         "channels", GST_PROPS_INT (flacenc->channels),
-                         "rate", GST_PROPS_INT (flacenc->sample_rate));
+  caps = gst_caps_new_simple ("audio/x-flac",
+      "channels", G_TYPE_INT, flacenc->channels,
+      "rate", G_TYPE_INT, flacenc->sample_rate, NULL);
   ret = gst_pad_try_set_caps (flacenc->srcpad, caps);
   if (ret <= 0) {
     return ret;
@@ -492,46 +465,52 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
   return FLAC__STREAM_ENCODER_OK;
 }
 
-static void
-gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
+static void 
+add_one_tag (const GstTagList *list, const gchar *tag, 
+	     gpointer user_data)
 {
-  guint indice;
-  guint comments;
-  const gchar *meta_types[] = { "TITLE", "VERSION", "ALBUM", "TRACKNUMBER",
-                                "ARTIST", "PERFORMER", "COPYRIGHT", "LICENSE",
-                                "ORGANISATION", "DESCRIPTION", "GENRE", "DATE",
-                                "LOCATION", "CONTACT", "ISRC", NULL };
+      GList *comments;
+      GList *it;
+      FlacEnc *flacenc = GST_FLACENC (user_data);
+
+      comments = gst_tag_to_vorbis_comments (list, tag);
+      for (it = comments; it != NULL; it = it->next) {
+	      FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
+	      commment_entry.length = strlen(it->data);
+	      commment_entry.entry  =  it->data;
+	      FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], 
+								  flacenc->meta[0]->data.vorbis_comment.num_comments, 
+								  commment_entry, 
+								  TRUE);
+	      g_free (it->data);
+      }
+      g_list_free (comments);
+}
+
+static void
+gst_flacenc_set_metadata (FlacEnc *flacenc)
+{
+  const GstTagList *user_tags;
+  GstTagList *copy;
 
   g_return_if_fail (flacenc != NULL);
-
+  user_tags = gst_tag_setter_get_list (GST_TAG_SETTER (flacenc));
+  if ((flacenc->tags == NULL) && (user_tags == NULL)) {
+    return;
+  }
+  copy = gst_tag_list_merge (user_tags, flacenc->tags, 
+			     gst_tag_setter_get_merge_mode (GST_TAG_SETTER (flacenc)));
   flacenc->meta = g_malloc (sizeof (FLAC__StreamMetadata **));
 
   flacenc->meta[0] = FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
-  
-  for ( indice = 0, comments=0; meta_types[indice] != NULL; indice++) {
-    if (gst_caps_has_property(caps, meta_types[indice])) {
-      const gchar *entry;
-      gchar *data;
-      FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
-      
-      gst_caps_get_string(caps, (gchar *)meta_types[indice], &entry);
+  gst_tag_list_foreach (copy, add_one_tag, flacenc);
 
-      if (!strcmp (entry, ""))
-	continue;
-   
-      data = g_strdup_printf("%s=%s", meta_types[indice], entry);
-      commment_entry.length = GUINT32_TO_LE (strlen(entry) + strlen(meta_types[indice]) + 1);
-      commment_entry.entry  =  g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
-
-      FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], comments++, commment_entry, TRUE);
-    }
-  }
-
-  FLAC__seekable_stream_encoder_set_metadata(flacenc->encoder, flacenc->meta, 1);
+  if (FLAC__seekable_stream_encoder_set_metadata(flacenc->encoder, flacenc->meta, 1) != true)
+    g_warning ("Dude, i'm already initialized!");
+  gst_tag_list_free (copy);
 }
 
 
-
 static void
 gst_flacenc_chain (GstPad *pad, GstData *_data)
 {
@@ -553,16 +532,25 @@ gst_flacenc_chain (GstPad *pad, GstData *_data)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
 	FLAC__seekable_stream_encoder_finish(flacenc->encoder);
+	break;
+      case GST_EVENT_TAG:
+	if (flacenc->tags) {
+	  gst_tag_list_insert (flacenc->tags, gst_event_tag_get_list (event), 
+		  GST_TAG_MERGE_REPLACE);
+	} else {
+	  g_assert_not_reached ();
+	}
+	break;
       default:
-	gst_pad_event_default (pad, event);
         break;
     }
+    gst_pad_event_default (pad, event);
     return;
   }
 
   if (!flacenc->negotiated) {
-    gst_element_error (GST_ELEMENT (flacenc),
-		    "format not negotiated");
+    GST_ELEMENT_ERROR (flacenc, CORE, NEGOTIATION, (NULL),
+		       ("format wasn't negotiated before chain function"));
     return;
   }
 
@@ -576,11 +564,19 @@ gst_flacenc_chain (GstPad *pad, GstData *_data)
   {
     FLAC__SeekableStreamEncoderState state;
 
-    gst_flacenc_set_metadata (flacenc, flacenc->metadata);
+    FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
+                                          gst_flacenc_write_callback);
+    FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
+                                          gst_flacenc_seek_callback);
+                                          
+    FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
+                                          flacenc);
+		  
+    gst_flacenc_set_metadata (flacenc);
     state = FLAC__seekable_stream_encoder_init (flacenc->encoder);
     if (state != FLAC__STREAM_ENCODER_OK) {
-      gst_element_error (GST_ELEMENT (flacenc),
-		         "could not initialize encoder (wrong parameters?)");
+      GST_ELEMENT_ERROR (flacenc, LIBRARY, INIT, NULL,
+		         ("could not initialize encoder (wrong parameters?)"));
       return;
     }
   }
@@ -614,8 +610,7 @@ gst_flacenc_chain (GstPad *pad, GstData *_data)
   flacenc->data = NULL;
 
   if (!res) {
-    gst_element_error (GST_ELEMENT (flacenc),
-		         "encoding error");
+    GST_ELEMENT_ERROR (flacenc, STREAM, ENCODE, NULL, NULL);
   }
 }
 
