@@ -25,6 +25,7 @@
 #endif
 #include <gst/gst.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gst/video/video.h>
 #include <string.h>
 
 #include "gstgdkpixbuf.h"
@@ -47,50 +48,39 @@ enum {
   ARG_SILENT
 };
 
-GST_PAD_TEMPLATE_FACTORY (gst_gdk_pixbuf_sink_factory,
+static GstStaticPadTemplate gst_gdk_pixbuf_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/png", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/jpeg", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/gif", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-icon", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "application/x-navi-animation", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-cmu-raster", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-sun-raster", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-pixmap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/tiff", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-anymap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-bitmap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-graymap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-pixmap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/bmp", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-bmp", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-MS-bmp", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/vnd.wap.wbmp", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-bitmap", NULL),
-  GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-tga", NULL)
+  GST_STATIC_CAPS (
+    "image/png; "
+    "image/jpeg; "
+    "image/gif; "
+    "image/x-icon; "
+    "application/x-navi-animation; "
+    "image/x-cmu-raster; "
+    "image/x-sun-raster; "
+    "image/x-pixmap; "
+    "image/tiff; "
+    "image/x-portable-anymap; "
+    "image/x-portable-bitmap; "
+    "image/x-portable-graymap; "
+    "image/x-portable-pixmap; "
+    "image/bmp; "
+    "image/x-bmp; "
+    "image/x-MS-bmp; "
+    "image/vnd.wap.wbmp; "
+    "image/x-bitmap; "
+    "image/x-tga")
 );
 
-GST_PAD_TEMPLATE_FACTORY (gst_gdk_pixbuf_src_factory,
+static GstStaticPadTemplate gst_gdk_pixbuf_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW(
-    "gdk_pixbuf_src",
-    "video/x-raw-rgb",
-      "width", GST_PROPS_INT_RANGE(1,INT_MAX),
-      "height", GST_PROPS_INT_RANGE(1,INT_MAX),
-      /* well, it's needed for connectivity but this
-       * doesn't really make sense... */
-      "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-      "bpp", GST_PROPS_INT(32),
-      "depth", GST_PROPS_INT(24),
-      "endianness", GST_PROPS_INT(G_BIG_ENDIAN),
-      "red_mask", GST_PROPS_INT(0x00ff0000),
-      "green_mask", GST_PROPS_INT(0x0000ff00),
-      "blue_mask", GST_PROPS_INT(0x000000ff)
-  )
+  GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB)
 );
 
 static void     gst_gdk_pixbuf_base_init (gpointer g_class);
@@ -110,7 +100,7 @@ static void     gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore);
 static GstElementClass *parent_class = NULL;
 
 static GstPadLinkReturn
-gst_gdk_pixbuf_sink_link (GstPad *pad, GstCaps *caps)
+gst_gdk_pixbuf_sink_link (GstPad *pad, const GstCaps *caps)
 {
   GstGdkPixbuf *filter;
 
@@ -119,12 +109,11 @@ gst_gdk_pixbuf_sink_link (GstPad *pad, GstCaps *caps)
   g_return_val_if_fail (GST_IS_GDK_PIXBUF (filter),
                         GST_PAD_LINK_REFUSED);
 
-  if (GST_CAPS_IS_FIXED (caps))
-  {
-    return GST_PAD_LINK_OK;
-  }
+  filter->framerate = 0.0;
+  gst_structure_get_double (gst_caps_get_structure (caps, 0), "framerate",
+      &filter->framerate);
 
-  return GST_PAD_LINK_DELAYED;
+  return GST_PAD_LINK_OK;
 }
 
 #if GDK_PIXBUF_MAJOR == 2 && GDK_PIXBUF_MINOR < 2
@@ -134,31 +123,8 @@ gst_gdk_pixbuf_sink_link (GstPad *pad, GstCaps *caps)
  * library. */
 static GstCaps *gst_gdk_pixbuf_get_capslist(void)
 {
-  GstCaps *capslist;
-
-  capslist = gst_caps_chain(
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/png", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/jpeg", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/gif", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-icon", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "application/x-navi-animation", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-cmu-raster", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-sun-raster", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-pixmap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/tiff", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-anymap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-bitmap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-graymap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-portable-pixmap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/bmp", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-bmp", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-MS-bmp", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/vnd.wap.wbmp", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-bitmap", NULL),
-    GST_CAPS_NEW("gdk_pixbuf_sink", "image/x-tga", NULL),
-    NULL);
-
-  return capslist;
+  return gst_caps_copy (gst_static_caps_get (
+        &gst_gdk_pixbuf_sink_template.static_caps));
 }
 #else
 static GstCaps *gst_gdk_pixbuf_get_capslist(void)
@@ -168,31 +134,27 @@ static GstCaps *gst_gdk_pixbuf_get_capslist(void)
   GdkPixbufFormat *pixbuf_format;
   char **mimetypes;
   char **mimetype;
-  static GstCaps *capslist = NULL;
+  GstCaps *capslist = NULL;
 
-  if(capslist==NULL){
-    slist0 = gdk_pixbuf_get_formats();
+  capslist = gst_caps_new_empty ();
+  slist0 = gdk_pixbuf_get_formats();
 
-    for(slist = slist0;slist;slist=g_slist_next(slist)){
-      pixbuf_format = slist->data;
-      mimetypes = gdk_pixbuf_format_get_mime_types(pixbuf_format);
-      for(mimetype = mimetypes; *mimetype; mimetype++){
-        capslist = gst_caps_append(capslist, gst_caps_new("ack",*mimetype,NULL));
-      }
-      g_free(mimetypes);
+  for(slist = slist0;slist;slist=g_slist_next(slist)){
+    pixbuf_format = slist->data;
+    mimetypes = gdk_pixbuf_format_get_mime_types(pixbuf_format);
+    for(mimetype = mimetypes; *mimetype; mimetype++){
+      gst_caps_append_structure (capslist,
+          gst_structure_new (*mimetype,NULL));
     }
-    gst_caps_ref(capslist);
-    gst_caps_sink(capslist);
-    g_slist_free(slist0);
-
-    g_print("%s\n",gst_caps_to_string(capslist));
+    g_free(mimetypes);
   }
+  g_slist_free(slist0);
 
   return capslist;
 }
 #endif
 
-static GstCaps *gst_gdk_pixbuf_sink_getcaps(GstPad *pad, GstCaps *caps)
+static GstCaps *gst_gdk_pixbuf_sink_getcaps(GstPad *pad)
 {
   GstGdkPixbuf *filter;
 
@@ -234,8 +196,10 @@ gst_gdk_pixbuf_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, gst_gdk_pixbuf_src_factory ());
-  gst_element_class_add_pad_template (element_class, gst_gdk_pixbuf_sink_factory ());
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get( &gst_gdk_pixbuf_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get( &gst_gdk_pixbuf_sink_template));
   gst_element_class_set_details (element_class, &plugin_details);
 }
 
@@ -262,17 +226,20 @@ gst_gdk_pixbuf_class_init (GstGdkPixbufClass *klass)
 static void
 gst_gdk_pixbuf_init (GstGdkPixbuf *filter)
 {
-  filter->sinkpad = gst_pad_new_from_template (gst_gdk_pixbuf_sink_factory (),
-                                               "sink");
+  filter->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get( &gst_gdk_pixbuf_sink_template), "sink");
   gst_pad_set_link_function (filter->sinkpad, gst_gdk_pixbuf_sink_link);
   gst_pad_set_getcaps_function (filter->sinkpad, gst_gdk_pixbuf_sink_getcaps);
-  filter->srcpad = gst_pad_new_from_template (gst_gdk_pixbuf_src_factory (),
-                                              "src");
+
+  filter->srcpad = gst_pad_new_from_template (
+      gst_static_pad_template_get( &gst_gdk_pixbuf_src_template), "src");
+  gst_pad_use_explicit_caps (filter->srcpad);
 
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
   gst_pad_set_chain_function (filter->sinkpad, gst_gdk_pixbuf_chain);
 
+  GST_FLAG_SET (GST_ELEMENT (filter), GST_ELEMENT_EVENT_AWARE);
 }
 
 static void
@@ -280,12 +247,12 @@ gst_gdk_pixbuf_chain (GstPad *pad, GstData *_data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstGdkPixbuf *filter;
-  GdkPixbufLoader *pixbuf_loader;
-  GdkPixbuf *pixbuf;
-  GstBuffer *outbuf;
   GError *error = NULL;
+  gboolean push_buffer = FALSE;
+  gboolean dump_buffer = FALSE;
+  gboolean got_eos = FALSE;
 
-  g_print("gst_gdk_pixbuf_chain\n");
+  GST_DEBUG ("gst_gdk_pixbuf_chain");
 
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (buf != NULL);
@@ -293,46 +260,96 @@ gst_gdk_pixbuf_chain (GstPad *pad, GstData *_data)
   filter = GST_GDK_PIXBUF (GST_OBJECT_PARENT (pad));
   g_return_if_fail (GST_IS_GDK_PIXBUF (filter));
 
-  pixbuf_loader = gdk_pixbuf_loader_new();
-  //pixbuf_loader = gdk_pixbuf_loader_new_with_type("gif");
-  
-  gdk_pixbuf_loader_write(pixbuf_loader, GST_BUFFER_DATA(buf),
-      GST_BUFFER_SIZE(buf), &error);
-  gdk_pixbuf_loader_close(pixbuf_loader, &error);
-  pixbuf = gdk_pixbuf_loader_get_pixbuf(pixbuf_loader);
+  if (GST_IS_EVENT (_data)) {
+    GstEvent *event = GST_EVENT (buf);
 
-  g_print("width=%d height=%d\n", gdk_pixbuf_get_width(pixbuf),
-      gdk_pixbuf_get_height(pixbuf));
-
-  if(filter->image_size == 0){
-    GstCaps *caps;
-
-    filter->width = gdk_pixbuf_get_width(pixbuf);
-    filter->height = gdk_pixbuf_get_height(pixbuf);
-    filter->rowstride = gdk_pixbuf_get_rowstride(pixbuf);
-    filter->image_size = filter->rowstride * filter->height;
-
-    caps = gst_pad_get_caps(filter->srcpad);
-    gst_caps_set(caps, "width", GST_PROPS_INT(filter->width));
-    gst_caps_set(caps, "height", GST_PROPS_INT(filter->height));
-    gst_caps_set(caps, "framerate", GST_PROPS_FLOAT(0.));
-
-    gst_pad_try_set_caps(filter->srcpad, caps);
+    switch (GST_EVENT_TYPE (event)) {
+      case GST_EVENT_EOS:
+        push_buffer = TRUE;
+        got_eos = TRUE;
+        break;
+      case GST_EVENT_DISCONTINUOUS:
+        dump_buffer = TRUE;
+        break;
+      default:
+        gst_pad_event_default (pad, event);
+        return;
+    }
   }
 
-  outbuf = gst_buffer_new();
-  GST_BUFFER_TIMESTAMP(outbuf) = GST_BUFFER_TIMESTAMP(buf);
-  GST_BUFFER_DURATION(outbuf) = GST_BUFFER_DURATION(buf);
-  
-  GST_BUFFER_SIZE(outbuf) = filter->image_size;
-  GST_BUFFER_DATA(outbuf) = g_malloc(filter->image_size);
+  if (filter->last_timestamp != GST_BUFFER_TIMESTAMP (buf)) {
+    push_buffer = TRUE;
+  }
 
-  memcpy(GST_BUFFER_DATA(outbuf), gdk_pixbuf_get_pixels(pixbuf),
-      filter->image_size);
+  if (push_buffer) {
+    if (filter->pixbuf_loader != NULL) {
+      GstBuffer *outbuf;
+      GdkPixbuf *pixbuf;
+      GError *error;
 
-  g_object_unref(G_OBJECT(pixbuf_loader));
+      if (gdk_pixbuf_loader_close (filter->pixbuf_loader, &error)) {
+        GST_ELEMENT_ERROR (filter, LIBRARY, SHUTDOWN, (NULL), (error->message));
+        g_error_free (error);
+        return;
+      }
 
-  gst_pad_push (filter->srcpad, GST_DATA (outbuf));
+      pixbuf = gdk_pixbuf_loader_get_pixbuf (filter->pixbuf_loader);
+
+      if(filter->image_size == 0){
+        GstCaps *caps;
+
+        filter->width = gdk_pixbuf_get_width(pixbuf);
+        filter->height = gdk_pixbuf_get_height(pixbuf);
+        filter->rowstride = gdk_pixbuf_get_rowstride(pixbuf);
+        filter->image_size = filter->rowstride * filter->height;
+
+        caps = gst_caps_copy (gst_pad_get_pad_template_caps (filter->srcpad));
+        gst_caps_set_simple (caps,
+            "width", G_TYPE_INT, filter->width,
+            "height", G_TYPE_INT, filter->height,
+            "framerate", G_TYPE_DOUBLE, filter->framerate, NULL);
+
+        gst_pad_set_explicit_caps (filter->srcpad, caps);
+      }
+
+      outbuf = gst_pad_alloc_buffer (filter->srcpad, GST_BUFFER_OFFSET_NONE,
+          filter->image_size);
+      GST_BUFFER_TIMESTAMP(outbuf) = GST_BUFFER_TIMESTAMP(buf);
+      GST_BUFFER_DURATION(outbuf) = GST_BUFFER_DURATION(buf);
+      
+      memcpy(GST_BUFFER_DATA(outbuf), gdk_pixbuf_get_pixels(pixbuf),
+          filter->image_size);
+
+      gst_pad_push (filter->srcpad, GST_DATA (outbuf));
+
+      g_object_unref(G_OBJECT(filter->pixbuf_loader));
+      filter->pixbuf_loader = NULL;
+      dump_buffer = FALSE;
+    }
+  }
+
+  if (dump_buffer) {
+    if (filter->pixbuf_loader != NULL) {
+      gdk_pixbuf_loader_close (filter->pixbuf_loader, NULL);
+      g_object_unref(G_OBJECT(filter->pixbuf_loader));
+      filter->pixbuf_loader = NULL;
+    }
+  }
+
+  if (GST_IS_BUFFER (_data)) {
+    if (filter->pixbuf_loader == NULL) {
+      filter->pixbuf_loader = gdk_pixbuf_loader_new();
+      filter->last_timestamp = GST_BUFFER_TIMESTAMP (buf);
+    }
+    
+    gdk_pixbuf_loader_write(filter->pixbuf_loader, GST_BUFFER_DATA(buf),
+        GST_BUFFER_SIZE(buf), &error);
+    gst_buffer_unref (buf);
+  }
+
+  if (got_eos) {
+    gst_pad_event_default (pad, GST_EVENT (_data));
+  }
 }
 
 static void
@@ -396,10 +413,13 @@ gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
   format = gdk_pixbuf_loader_get_format (pixbuf_loader);
 
   if (format != NULL) {
+    GstCaps *caps;
     gchar **mlist = gdk_pixbuf_format_get_mime_types(format);
 
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MINIMUM,
-        gst_caps_new ("gdk_pixbuf_type_find", mlist[0], NULL));
+    caps = gst_caps_new_simple (mlist[0], NULL);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MINIMUM, caps);
+    gst_caps_free (caps);
+    g_strfreev (mlist);
   }
 
   gdk_pixbuf_loader_close (pixbuf_loader, NULL);
@@ -414,11 +434,16 @@ gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
 static gboolean
 plugin_init (GstPlugin *plugin)
 {
+  GstCaps *caps;
+  
   if (!gst_element_register (plugin, "gdkpixbufdec", GST_RANK_NONE, GST_TYPE_GDK_PIXBUF))
     return FALSE;
 
+  caps = GST_CAPS_ANY;
   gst_type_find_register (plugin, "image/*", GST_RANK_MARGINAL,
-			  gst_gdk_pixbuf_type_find, NULL, GST_CAPS_ANY, NULL);
+			  gst_gdk_pixbuf_type_find, NULL,
+                          caps, NULL);
+  gst_caps_free (caps);
 
   /* plugin initialisation succeeded */
   return TRUE;
