@@ -23,6 +23,8 @@
 #  include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
+
 #include "gstgnomevfs.h"
 #include <gst/gst.h>
 
@@ -36,6 +38,11 @@ plugin_init(GstPlugin *plugin)
     return FALSE;
   }
 
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return TRUE;
 }
 
