@@ -30,6 +30,8 @@
 #include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
+
 #include "gstgnomevfs.h"
 
 #include <sys/types.h>
@@ -95,7 +97,6 @@ struct _GstGnomeVFSSrc {
 	GnomeVFSFileOffset curoffset;	/* current offset in file */
 	gulong bytes_per_read;		/* bytes per read */
 	gboolean new_seek;
-	gboolean in_first_get;
 
 	/* icecast/audiocast metadata extraction handling */
 	gboolean iradio_mode;
@@ -131,23 +132,42 @@ static GstElementDetails gst_gnomevfssrc_details = GST_ELEMENT_DETAILS (
 	"Bastien Nocera <hadess@hadess.net>"
 );
 
-GST_PAD_FORMATS_FUNCTION (gst_gnomevfssrc_get_formats,
-	GST_FORMAT_BYTES
-)
 
-GST_PAD_QUERY_TYPE_FUNCTION (gst_gnomevfssrc_get_query_types,
-	GST_QUERY_TOTAL,
-	GST_QUERY_POSITION
-)
+static const GstFormat *
+gst_gnomevfssrc_get_formats (GstPad *pad)
+{
+  static const GstFormat formats[] = {
+    GST_FORMAT_BYTES,
+    0,
+  };
+  return formats;
+}
 
-GST_PAD_EVENT_MASK_FUNCTION (gst_gnomevfssrc_get_event_mask,
-  { GST_EVENT_SEEK, GST_SEEK_METHOD_CUR |
-                    GST_SEEK_METHOD_SET |
-                    GST_SEEK_METHOD_END |
-                    GST_SEEK_FLAG_FLUSH },
-  { GST_EVENT_FLUSH, 0 },
-  { GST_EVENT_SIZE, 0 }
-)
+static const GstQueryType *
+gst_gnomevfssrc_get_query_types (GstPad *pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0,
+  };
+  return types;
+}
+
+static const GstEventMask *
+gst_gnomevfssrc_get_event_mask (GstPad *pad)
+{
+  static const GstEventMask masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_METHOD_CUR |
+                      GST_SEEK_METHOD_SET |
+                      GST_SEEK_METHOD_END |
+                      GST_SEEK_FLAG_FLUSH },
+    { GST_EVENT_FLUSH, 0 },
+    { GST_EVENT_SIZE, 0 },
+    { 0, 0 },
+  };
+  return masks;
+}
 
 /* GnomeVFSSrc signals and args */
 enum {
@@ -156,6 +176,7 @@ enum {
 
 enum {
 	ARG_0,
+	ARG_HANDLE,
 	ARG_LOCATION,
 	ARG_BYTESPERREAD,
 	ARG_IRADIO_MODE,
@@ -246,6 +267,13 @@ static void gst_gnomevfssrc_class_init(GstGnomeVFSSrcClass *klass)
 
 	gobject_class->dispose         = gst_gnomevfssrc_dispose;
 
+	g_object_class_install_property (gobject_class,
+					 ARG_HANDLE,
+					 g_param_spec_pointer ("handle",
+							       "GnomeVFSHandle",
+							       "Handle for GnomeVFS",
+							       G_PARAM_READWRITE));
+
 	/* icecast stuff */
 	g_object_class_install_property (gobject_class,
 					 ARG_IRADIO_MODE,
@@ -318,7 +346,6 @@ static void gst_gnomevfssrc_init(GstGnomeVFSSrc *gnomevfssrc)
 	gnomevfssrc->curoffset = 0;
 	gnomevfssrc->bytes_per_read = 4096;
 	gnomevfssrc->new_seek = FALSE;
-	gnomevfssrc->in_first_get = TRUE;
 
 	gnomevfssrc->icy_metaint = 0;
 
@@ -412,6 +439,9 @@ static void gst_gnomevfssrc_set_property(GObject *object, guint prop_id, const G
 			gst_gnomevfssrc_open_file(src);
 		}
 		break;
+	case ARG_HANDLE:
+		src->handle = g_value_get_pointer (value);
+		break;
 	case ARG_BYTESPERREAD:
 		src->bytes_per_read = g_value_get_int (value);
 		break;
@@ -440,6 +470,9 @@ static void gst_gnomevfssrc_get_property(GObject *object, guint prop_id, GValue
 	case ARG_BYTESPERREAD:
 		g_value_set_int (value, src->bytes_per_read);
 		break;
+	case ARG_HANDLE:
+		g_value_set_pointer (value, src->handle);
+		break;
 	case ARG_IRADIO_MODE:
 		g_value_set_boolean (value, src->iradio_mode);
 		break;
@@ -510,16 +543,9 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	if (!src->iradio_mode)
 		return TRUE;
 	GST_DEBUG ("audiocast: registering listener");
-	if (audiocast_register_listener(&src->audiocast_port, &src->audiocast_fd) < 0)
-	{
-		char *escaped;
-
-		escaped = gnome_vfs_unescape_string_for_display (src->filename);
-		gst_element_error(GST_ELEMENT(src),
-				  "opening vfs file \"%s\" (%s)",
-				  escaped,
-				  "unable to register UDP port");
-		g_free (escaped);
+	if (audiocast_register_listener(&src->audiocast_port, &src->audiocast_fd) < 0) {
+		GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+                                     ("Unable to listen on UDP port %d", src->audiocast_port));
 		close(src->audiocast_fd);
 		return FALSE;
 	}
@@ -535,13 +561,8 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	GST_DEBUG ("audiocast: creating audiocast thread");
 	src->audiocast_thread = g_thread_create((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
 	if (error != NULL) {
-		char *escaped;
-		escaped = gnome_vfs_unescape_string_for_display (src->filename);
-		gst_element_error(GST_ELEMENT(src),
-				  "opening vfs file \"%s\" (unable to create thread: %s)",
-				  escaped,
-				  error->message);
-		g_free (escaped);
+		GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL),
+				  ("Unable to create thread: %s", error->message));
 		close(src->audiocast_fd);
 		return FALSE;
 	}
@@ -957,56 +978,50 @@ static GstData *gst_gnomevfssrc_get(GstPad *pad)
 		g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 
 		GST_BUFFER_SIZE (buf) = 0;
-		/* FIXME GROSS HACK: We try to read in at least 8000
-		 * bytes of data so that mp3 typefinding will work. */
-		do
-		{
-			GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
-			result = gnome_vfs_read (src->handle, data,
-						 src->icy_metaint - src->icy_count,
-						 &readbytes);
-
-			/* EOS? */
-			if (readbytes == 0) {
-				gst_buffer_unref (buf);
-				gst_element_set_eos (GST_ELEMENT (src));
-				src->in_first_get = FALSE;
-				return GST_DATA (gst_event_new (GST_EVENT_EOS));
-			}
-			
-			src->icy_count += readbytes;
-			GST_BUFFER_OFFSET (buf) = src->curoffset;
-			GST_BUFFER_SIZE (buf) += readbytes;
-		      	data += readbytes;
-			src->curoffset += readbytes;
-			
-			if (src->icy_count == src->icy_metaint) {
-				gst_gnomevfssrc_get_icy_metadata (src);
-				src->icy_count = 0;
-			}
-		} while (src->in_first_get
-			 && GST_BUFFER_OFFSET (buf) < 8000 &&
-			 src->icy_metaint - src->icy_count >= 8000);
-		src->in_first_get = FALSE;
+		GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
+		result = gnome_vfs_read (src->handle, data,
+					 src->icy_metaint - src->icy_count,
+					 &readbytes);
+		
+		/* EOS? */
+		if (readbytes == 0) {
+		  gst_buffer_unref (buf);
+		  gst_element_set_eos (GST_ELEMENT (src));
+		  return GST_DATA (gst_event_new (GST_EVENT_EOS));
+		}
+		
+		src->icy_count += readbytes;
+		GST_BUFFER_OFFSET (buf) = src->curoffset;
+		GST_BUFFER_SIZE (buf) += readbytes;
+		data += readbytes;
+		src->curoffset += readbytes;
+		
+		if (src->icy_count == src->icy_metaint) {
+		  gst_gnomevfssrc_get_icy_metadata (src);
+		  src->icy_count = 0;
+		}
 	} else {
 		/* allocate the space for the buffer data */
 		GST_BUFFER_DATA(buf) = g_malloc(src->bytes_per_read);
 		g_return_val_if_fail(GST_BUFFER_DATA(buf) != NULL, NULL);
 
-		if (src->new_seek)
-		{
-			GstEvent *event;
-
-			gst_buffer_unref (buf);
-			GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
-			src->new_seek = FALSE;
-
-			GST_DEBUG ("gnomevfssrc sending discont");
-			event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
-			src->need_flush = FALSE;
-
-			return GST_DATA (event);
-		}
+                if (src->need_flush) {
+                  GstEvent *event = gst_event_new_flush ();
+                  src->need_flush = FALSE;
+                  gst_buffer_unref (buf);
+                  GST_DEBUG ("gnomevfssrc sending flush");
+                  return GST_DATA (event);
+                }
+                
+                if (src->new_seek) {
+                  GstEvent *event;
+                  gst_buffer_unref (buf);
+                  GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
+                  src->new_seek = FALSE;
+                  GST_DEBUG ("gnomevfssrc sending discont");
+                  event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
+                  return GST_DATA (event);
+                }
 
 		result = gnome_vfs_read(src->handle, GST_BUFFER_DATA(buf),
 				   src->bytes_per_read, &readbytes);
@@ -1043,12 +1058,14 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 	g_return_val_if_fail(!GST_FLAG_IS_SET(src, GST_GNOMEVFSSRC_OPEN),
 			     FALSE);
 
-	/* create the uri */
-	src->uri = gnome_vfs_uri_new(src->filename);
-	if (!src->uri) {
-		gst_element_error(GST_ELEMENT(src), "creating uri \"%s\" (%s)",
-				  src->filename, strerror (errno));
-		return FALSE;
+	if (src->filename) {
+		/* create the uri */
+		src->uri = gnome_vfs_uri_new(src->filename);
+		if (!src->uri) {
+			GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+                                           (_("Could not open vfs file \"%s\" for reading."), src->filename), GST_ERROR_SYSTEM);
+			return FALSE;
+		}
 	}
 
 	if (!audiocast_init(src))
@@ -1056,24 +1073,25 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 			
 	gst_gnomevfssrc_push_callbacks (src);
 	
-	result = gnome_vfs_open_uri(&(src->handle), src->uri,
-				    GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK)
-	{
+	if (src->filename)
+		result = gnome_vfs_open_uri(&(src->handle), src->uri,
+					    GNOME_VFS_OPEN_READ);
+	else
+		result = GNOME_VFS_OK;
+	if (result != GNOME_VFS_OK) {
 		char *escaped;
-		
+
 		gst_gnomevfssrc_pop_callbacks (src);
 		audiocast_thread_kill(src);
 
 		escaped = gnome_vfs_unescape_string_for_display (src->filename);
-		gst_element_error(GST_ELEMENT(src),
-				  "opening vfs file \"%s\" (%s)",
-				  escaped,
-				  gnome_vfs_result_to_string(result));
+		GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+				   (_("Could not open vfs file \"%s\" for reading."), escaped),
+				   (gnome_vfs_result_to_string (result)));
 		g_free (escaped);
 		return FALSE;
 	}
-	
+
 	info = gnome_vfs_file_info_new ();
 	if (gnome_vfs_get_file_info_from_handle (src->handle, info,
 						 GNOME_VFS_FILE_INFO_DEFAULT)
@@ -1093,10 +1111,8 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 	
 	GST_DEBUG ("open result: %s", gnome_vfs_result_to_string (result));
 
-	src->in_first_get = TRUE;
-
 	if(gnome_vfs_seek(src->handle, GNOME_VFS_SEEK_CURRENT, 0)
-	    == GNOME_VFS_OK){
+	    == GNOME_VFS_OK) {
 		src->seekable = TRUE;
 	}else{
 		src->seekable = FALSE;
@@ -1113,10 +1129,12 @@ static void gst_gnomevfssrc_close_file(GstGnomeVFSSrc *src)
 
 	gst_gnomevfssrc_pop_callbacks (src);
 	audiocast_thread_kill(src);
-
-	gnome_vfs_close(src->handle);
-
-	gnome_vfs_uri_unref(src->uri);
+	
+	if (src->filename) {
+		gnome_vfs_close(src->handle);
+	  
+		gnome_vfs_uri_unref(src->uri);
+	}
 	src->size = 0;
 	src->curoffset = 0;
 	src->new_seek = FALSE;
@@ -1169,10 +1187,18 @@ gst_gnomevfssrc_srcpad_query (GstPad *pad, GstQueryType type,
 		*value = src->size;
 		break;
 	case GST_QUERY_POSITION:
-		if (*format != GST_FORMAT_BYTES) {
-			return FALSE;
+		switch (*format) {
+			case GST_FORMAT_BYTES:
+				*value = src->curoffset;
+		                break;
+			case GST_FORMAT_PERCENT:
+				if (src->size == 0)
+					return FALSE;
+				*value = src->curoffset * GST_FORMAT_PERCENT_MAX / src->size;
+				break;
+			default:
+				return FALSE;
 		}
-		*value = src->curoffset;
 		break;
 	default:
 		return FALSE;
