@@ -50,10 +50,8 @@ static void             gst_jpegenc_base_init   (gpointer g_class);
 static void		gst_jpegenc_class_init	(GstJpegEnc *klass);
 static void		gst_jpegenc_init	(GstJpegEnc *jpegenc);
 
-static void		gst_jpegenc_chain	(GstPad *pad,GstData *_data);
-static GstPadLinkReturn	gst_jpegenc_link	(GstPad *pad, GstCaps *caps);
-
-static GstData	*gst_jpegenc_get	(GstPad *pad);
+static void		gst_jpegenc_chain	(GstPad *pad, GstData *_data);
+static GstPadLinkReturn	gst_jpegenc_link	(GstPad *pad, const GstCaps *caps);
 
 static void		gst_jpegenc_resync	(GstJpegEnc *jpegenc);
 
@@ -86,26 +84,17 @@ gst_jpegenc_get_type (void)
 static GstCaps*
 jpeg_caps_factory (void) 
 {
-  return
-    gst_caps_new (
-  	"jpeg_jpeg",
-  	"video/x-jpeg",
-  	    gst_props_new (
-		"width",     GST_PROPS_INT_RANGE (16, 4096),
-		"height",    GST_PROPS_INT_RANGE (16, 4096),
-		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-                NULL));
+  return gst_caps_new_simple ("video/x-jpeg",
+      "width",     GST_TYPE_INT_RANGE, 16, 4096,
+      "height",    GST_TYPE_INT_RANGE, 16, 4096,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
 }
 
 static GstCaps*
 raw_caps_factory (void)
 {
-  return
-    gst_caps_new (
-  	"jpeg_raw",
-  	"video/x-raw-yuv",
-	GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
-	  GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0'))));
+  return gst_caps_from_string (GST_VIDEO_CAPS_YUV ("I420"));
 }
 
 static void
@@ -119,10 +108,10 @@ gst_jpegenc_base_init (gpointer g_class)
 
   jpegenc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
 						GST_PAD_ALWAYS, 
-						raw_caps, NULL);
+						raw_caps);
   jpegenc_src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
 					       GST_PAD_ALWAYS, 
-					       jpeg_caps, NULL);
+					       jpeg_caps);
 
   gst_element_class_add_pad_template (element_class, jpegenc_sink_template);
   gst_element_class_add_pad_template (element_class, jpegenc_src_template);
@@ -173,7 +162,6 @@ gst_jpegenc_init (GstJpegEnc *jpegenc)
   gst_element_add_pad(GST_ELEMENT(jpegenc),jpegenc->sinkpad);
   gst_pad_set_chain_function(jpegenc->sinkpad,gst_jpegenc_chain);
   gst_pad_set_link_function(jpegenc->sinkpad, gst_jpegenc_link);
-  gst_pad_set_get_function(jpegenc->sinkpad,gst_jpegenc_get);
   jpegenc->srcpad = gst_pad_new("src",GST_PAD_SRC);
   gst_element_add_pad(GST_ELEMENT(jpegenc),jpegenc->srcpad);
 
@@ -202,24 +190,21 @@ gst_jpegenc_init (GstJpegEnc *jpegenc)
 }
 
 static GstPadLinkReturn
-gst_jpegenc_link (GstPad *pad, GstCaps *caps)
+gst_jpegenc_link (GstPad *pad, const GstCaps *caps)
 {
   GstJpegEnc *jpegenc = GST_JPEGENC (gst_pad_get_parent (pad));
+  GstStructure *structure;
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-
-  gst_caps_get (caps,
-		"framerate", &jpegenc->fps,
-		"width",     &jpegenc->width,
-		"height",    &jpegenc->height,
-		NULL);
-
-  caps = GST_CAPS_NEW ("jpegdec_srccaps",
-		       "video/x-jpeg",
-			 "width",     GST_PROPS_INT (jpegenc->width),
-			 "height",    GST_PROPS_INT (jpegenc->height),
-			 "framerate", GST_PROPS_FLOAT (jpegenc->fps));
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_double (structure, "framerate", &jpegenc->fps);
+  gst_structure_get_int (structure, "width",     &jpegenc->width);
+  gst_structure_get_int (structure, "height",    &jpegenc->height);
+  
+  caps = gst_caps_new_simple ("video/x-jpeg",
+      "width",     G_TYPE_INT, jpegenc->width,
+      "height",    G_TYPE_INT, jpegenc->height,
+      "framerate", G_TYPE_DOUBLE, jpegenc->fps,
+      NULL);
 
   return gst_pad_try_set_caps (jpegenc->srcpad, caps);
 }
@@ -288,31 +273,6 @@ gst_jpegenc_resync (GstJpegEnc *jpegenc)
   GST_DEBUG ("gst_jpegenc_resync: resync done");
 }
 
-static GstData*
-gst_jpegenc_get (GstPad *pad)
-{
-  GstJpegEnc *jpegenc;
-  GstBuffer *newbuf;
-
-  GST_DEBUG ("gst_jpegenc_chain: pull buffer");
-
-  g_return_val_if_fail (pad != NULL, NULL);
-  g_return_val_if_fail (GST_IS_PAD (pad), NULL);
-
-  jpegenc = GST_JPEGENC (GST_OBJECT_PARENT (pad));
-
-  if (jpegenc->buffer == NULL || GST_BUFFER_REFCOUNT_VALUE(jpegenc->buffer) != 1) {
-    if (jpegenc->buffer) gst_buffer_unref(jpegenc->buffer);
-    GST_DEBUG ("gst_jpegenc_chain: new buffer");
-    newbuf = jpegenc->buffer = gst_buffer_new();
-    GST_BUFFER_DATA(newbuf) = g_malloc(jpegenc->bufsize);
-    GST_BUFFER_SIZE(newbuf) = jpegenc->bufsize;
-  }
-  gst_buffer_ref(jpegenc->buffer);
-
-  return GST_DATA (jpegenc->buffer);
-}
-
 static void
 gst_jpegenc_chain (GstPad *pad, GstData *_data)
 {
