@@ -111,24 +111,22 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (id3_tag_src_template_factory,
+static GstStaticPadTemplate id3_tag_src_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   /* FIXME: for spider - GST_PAD_ALWAYS, */
   GST_PAD_SOMETIMES,
-  NULL
-)
+  GST_STATIC_CAPS ("ANY")
+);
 
-GST_PAD_TEMPLATE_FACTORY (id3_tag_sink_template_factory,
+static GstStaticPadTemplate id3_tag_sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "id3_tag_data_sink",
-    "application/x-id3",
-    NULL
-  )
-)
+  GST_STATIC_CAPS ("application/x-id3")
+);
 
 
 static void		gst_id3_tag_base_init		(gpointer		g_class);
@@ -155,6 +153,8 @@ static gboolean		gst_id3_tag_src_query		(GstPad *		pad,
 
 static void		gst_id3_tag_chain		(GstPad *		pad,
 							 GstData *		data);
+static GstPadLinkReturn	gst_id3_tag_src_link		(GstPad *		pad,
+							 const GstCaps *	caps);
 
 static GstElementStateReturn gst_id3_tag_change_state	(GstElement *		element);
 
@@ -200,9 +200,9 @@ gst_id3_tag_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_id3_tag_details);
 
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (id3_tag_sink_template_factory));
+		  gst_static_pad_template_get (&id3_tag_sink_template_factory));
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (id3_tag_src_template_factory));
+		  gst_static_pad_template_get (&id3_tag_src_template_factory));
 }
 static void
 gst_id3_tag_class_init (GstID3TagClass *klass)
@@ -230,30 +230,34 @@ gst_id3_tag_class_init (GstID3TagClass *klass)
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_id3_tag_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_id3_tag_get_property);
 }
+
 static GstCaps *
-gst_id3_tag_get_caps (GstPad *pad, GstCaps *unused)
+gst_id3_tag_get_caps (GstPad *pad)
 {
   GstID3Tag *tag = GST_ID3_TAG (gst_pad_get_parent (pad));
 
   if (tag->found_caps) {
-    GstCaps *caps = GST_CAPS_NEW ("gstid3tag", "application/x-gst-tags", NULL);
-    caps = gst_caps_append (caps, GST_CAPS_NEW ("gstid3tag", "application/x-id3", NULL));
-    caps = gst_caps_append (caps, gst_caps_copy (tag->found_caps));
+    GstCaps *caps;
+    caps = gst_caps_from_string ("application/x-gst-tags; application/x-id3");
+    gst_caps_append (caps, gst_caps_copy (tag->found_caps));
     return caps;
   } else {
-    return GST_CAPS_ANY;
+    return gst_caps_new_any ();
   }
 }
+
 static void
 gst_id3_tag_add_src_pad (GstID3Tag *tag)
 {
-  tag->srcpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (id3_tag_src_template_factory), "src");
+  g_assert (tag->srcpad == NULL);
+  tag->srcpad = gst_pad_new_from_template (gst_static_pad_template_get (
+        &id3_tag_src_template_factory), "src");
   gst_pad_set_event_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_src_event));
   gst_pad_set_event_mask_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_get_event_masks));
   gst_pad_set_query_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_src_query));
   gst_pad_set_query_type_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_get_query_types));
   gst_pad_set_getcaps_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_get_caps));
+  gst_pad_set_link_function (tag->srcpad, GST_DEBUG_FUNCPTR (gst_id3_tag_src_link));
   gst_element_add_pad (GST_ELEMENT (tag), tag->srcpad);
 }
 static void
@@ -261,7 +265,7 @@ gst_id3_tag_init (GstID3Tag *tag)
 {
   /* create the sink and src pads */
   tag->sinkpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (id3_tag_sink_template_factory), "sink");
+		  gst_static_pad_template_get (&id3_tag_sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (tag), tag->sinkpad);
   gst_pad_set_chain_function (tag->sinkpad, GST_DEBUG_FUNCPTR (gst_id3_tag_chain));
 
@@ -464,6 +468,12 @@ gst_mad_id3_to_tag_list(const struct id3_tag *tag)
       utf8 = id3_ucs4_utf8duplicate(ucs4);
       if (utf8 == 0)
 	continue;
+      
+      if (!g_utf8_validate (utf8, -1, NULL)) {
+	g_warning ("converted string is not valid utf-8");
+	free (utf8);
+	continue;
+      }
 
       /* be sure to add non-string tags here */
       switch (gst_tag_get_type (tag_name)) {
@@ -471,14 +481,30 @@ gst_mad_id3_to_tag_list(const struct id3_tag *tag)
 	{
 	  guint tmp;
 	  gchar *check;
+
 	  tmp = strtoul (utf8, &check, 10);
-	  if (*check != '\0') break;
+
 	  if (strcmp (tag_name, GST_TAG_DATE) == 0) {
+	    GDate *d;
+
+	    if (*check != '\0') break;
 	    if (tmp == 0) break;
-	    GDate *d = g_date_new_dmy (1, 1, tmp);
+	    d = g_date_new_dmy (1, 1, tmp);
 	    tmp = g_date_get_julian (d);
 	    g_date_free (d);
+	  } else if (strcmp(tag_name,GST_TAG_TRACK_NUMBER) == 0) {
+	    if (*check == '/') {
+	      guint total;
+
+	      check++;
+	      total = strtoul (check, &check, 10);
+	      if (*check != '\0') break;
+
+	      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, GST_TAG_TRACK_COUNT, total, NULL);
+	    }
 	  }
+
+	  if (*check != '\0') break;		
 	  gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, tmp, NULL);
 	  break;
 	}
@@ -602,7 +628,7 @@ gst_id3_tag_handle_event (GstPad *pad, GstEvent *event)
 	  gst_event_discont_get_value (event, GST_FORMAT_BYTES, &value);
 	  if (value != (tag->buffer ? GST_BUFFER_OFFSET (tag->buffer) + GST_BUFFER_SIZE (tag->buffer)
 				    : 0))
-	    gst_element_error (GST_ELEMENT (tag), "Seek during ID3v2 tag reading");
+	    GST_ELEMENT_ERROR (tag, CORE, EVENT, (NULL), ("Seek during ID3v2 tag reading"));
 	  gst_data_unref (GST_DATA (event));
 	  break;
 	}
@@ -611,7 +637,7 @@ gst_id3_tag_handle_event (GstPad *pad, GstEvent *event)
 	  gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_READING_V1_TAG);
 	  break;
 	case GST_ID3_TAG_STATE_READING_V1_TAG:
-	  gst_element_error (GST_ELEMENT (tag), "Seek during ID3v1 tag reading");
+	  GST_ELEMENT_ERROR (tag, CORE, EVENT, (NULL), ("Seek during ID3v1 tag reading"));
 	  gst_data_unref (GST_DATA (event));
 	  break;
 	case GST_ID3_TAG_STATE_SEEKING_TO_NORMAL:
@@ -695,12 +721,12 @@ simple_find_peek (gpointer data, gint64 offset, guint size)
   return NULL;
 }
 static void
-simple_find_suggest (gpointer data, guint probability, GstCaps *caps)
+simple_find_suggest (gpointer data, guint probability, const GstCaps *caps)
 {
   SimpleTypeFind *find = (SimpleTypeFind *) data;
 
   if (probability > find->best_probability) {
-    gst_caps_replace (&find->caps, caps);
+    gst_caps_replace (&find->caps, gst_caps_copy (caps));
     find->best_probability = probability;
   }
 }
@@ -735,58 +761,54 @@ gst_id3_tag_do_typefind (GstID3Tag *tag, GstBuffer *buffer)
   if (find.best_probability > 0) {
     return find.caps;
   } else {
-    gst_element_error (GST_ELEMENT (tag), "no caps found");
+    GST_ELEMENT_ERROR (tag, CORE, CAPS, (NULL), ("no caps found"));
     return NULL;
   }
 }
 static gboolean
 gst_id3_tag_do_caps_nego (GstID3Tag *tag, GstBuffer *buffer)
 {
-  GstCaps *caps;
-
   if (buffer != NULL) {
-    g_assert (tag->found_caps == NULL);
     tag->found_caps = gst_id3_tag_do_typefind (tag, buffer);
     if (!tag->found_caps) {
       return FALSE;
     } 
   }
-	
   if (!tag->srcpad)
     gst_id3_tag_add_src_pad (tag);
+  if (!gst_pad_is_linked (tag->srcpad)) {
+    tag->parse_mode = GST_ID3_TAG_PARSE_TAG;
+    return TRUE;
+  } else {
+    return gst_pad_renegotiate (tag->srcpad) != GST_PAD_LINK_REFUSED;
+  }
+}
 
-  do {
-    caps = GST_CAPS_NEW ("id3_tag_data_src", "application/x-id3", NULL);
-    if (gst_pad_try_set_caps (tag->srcpad, caps) != GST_PAD_LINK_REFUSED) {
-      tag->parse_mode = GST_ID3_TAG_PARSE_WRITE;
-      GST_LOG_OBJECT (tag, "normal operation, using application/x-id3 output");
-    } else {
-      caps = GST_CAPS_NEW ("id3_tag_tag_src", "application/x-gst-tags", NULL);
-      if (gst_pad_try_set_caps (tag->srcpad, caps) != GST_PAD_LINK_REFUSED) {
-	tag->parse_mode = GST_ID3_TAG_PARSE_TAG;
-	GST_LOG_OBJECT (tag, "fast operation, just outputting tags");
-      } else {
-	g_assert (tag->found_caps);
-	if (gst_pad_try_set_caps (tag->srcpad, tag->found_caps) != GST_PAD_LINK_REFUSED) {
-	  tag->parse_mode = GST_ID3_TAG_PARSE_PARSE;
-	  GST_LOG_OBJECT (tag, "parsing operation, extracting tags"); 
-	} else {
-	  caps = GST_CAPS_NEW ("id3_tag_data_src", "application/x-id3", NULL);
-	  caps = gst_caps_append (caps,
-		  GST_CAPS_NEW ("id3_tag_tag_src", "application/x-gst-tags", NULL));
-	  caps = gst_caps_append (caps, tag->found_caps);
-	  if (gst_pad_recover_caps_error (tag->srcpad, caps)) {
-	    tag->parse_mode = GST_ID3_TAG_PARSE_UNKNOWN;
-	    continue;
-	  } else {
-	    return FALSE;
-	  }
-	}
-      }
-    }
-  } while (FALSE);
+static GstPadLinkReturn
+gst_id3_tag_src_link (GstPad *pad, const GstCaps *caps)
+{
+  GstID3Tag *tag;
+  const gchar *mimetype;
+
+  tag = GST_ID3_TAG (gst_pad_get_parent (pad));
+
+  if (!tag->found_caps)
+    return GST_PAD_LINK_DELAYED;
   
-  return TRUE;
+  mimetype = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+
+  if (strcmp (mimetype, "application/x-id3") == 0) {
+    tag->parse_mode = GST_ID3_TAG_PARSE_WRITE;
+    GST_LOG_OBJECT (tag, "normal operation, using application/x-id3 output");
+  } else if (strcmp (mimetype, "application/x-gst-tags") == 0) {
+    tag->parse_mode = GST_ID3_TAG_PARSE_TAG;
+    GST_LOG_OBJECT (tag, "fast operation, just outputting tags");
+  } else {
+    tag->parse_mode = GST_ID3_TAG_PARSE_PARSE;
+    GST_LOG_OBJECT (tag, "parsing operation, extracting tags"); 
+  }
+
+  return GST_PAD_LINK_OK;
 }
 static void
 gst_id3_tag_send_tag_event (GstID3Tag *tag)
@@ -867,7 +889,7 @@ gst_id3_tag_chain (GstPad *pad, GstData *data)
 						    GST_SEEK_FLAG_FLUSH, tag->v2tag_size))) {
 	  gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_SEEKING_TO_NORMAL);
 	} else {
-	  gst_element_error (GST_ELEMENT (tag), "can't seek back to beginning from reading ID3v1 tag");
+	  GST_ELEMENT_ERROR (tag, CORE, SEEK, (NULL), ("can't seek back to beginning from reading ID3v1 tag"));
 	}
       } else {
 	gst_id3_tag_send_tag_event (tag);
@@ -915,12 +937,14 @@ gst_id3_tag_chain (GstPad *pad, GstData *data)
 	}
       }
       /* caps nego and typefinding */
+      GST_LOG_OBJECT (tag, "removing first %ld bytes, because they're the ID3v2 tag", tag->v2tag_size);
       buffer = gst_buffer_create_sub (tag->buffer, tag->v2tag_size, 
 				     GST_BUFFER_SIZE (tag->buffer) - tag->v2tag_size);
       gst_data_unref (GST_DATA (tag->buffer));
       tag->buffer = NULL;
-      if (!gst_id3_tag_do_caps_nego (tag, buffer))
-	return;
+      if (tag->found_caps == NULL)
+	if (!gst_id3_tag_do_caps_nego (tag, buffer))
+	  return;
       /* seek to ID3v1 tag */
       if (gst_pad_send_event (GST_PAD_PEER (tag->sinkpad),
 			  gst_event_new_seek (GST_FORMAT_BYTES | GST_SEEK_METHOD_END | 
@@ -1022,7 +1046,7 @@ gst_id3_tag_change_state (GstElement *element)
 	tag->buffer = NULL;
       }
       if (tag->found_caps) {
-	gst_caps_unref (tag->found_caps);
+	gst_caps_free (tag->found_caps);
 	tag->found_caps = NULL;
       }
       tag->parse_mode = GST_ID3_TAG_PARSE_UNKNOWN;
