@@ -23,6 +23,7 @@
 #endif
 #include "gstswfdec.h"
 #include <string.h>
+#include <gst/video/video.h>
 
 /* elementfactory information */
 static GstElementDetails gst_swfdec_details = GST_ELEMENT_DETAILS (
@@ -43,50 +44,35 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (video_template_factory,
+static GstStaticPadTemplate video_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "video_00",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "swfdec_videosrc",
-    "video/x-raw-rgb",
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096),
-      "bpp",		GST_PROPS_INT (24),
-      "depth",		GST_PROPS_INT (24),
-      "endianness",	GST_PROPS_INT (G_BIG_ENDIAN),
-      "red_mask",	GST_PROPS_INT (0xff0000),
-      "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0x0000ff),
-      "framerate",	GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
-  )
+  GST_STATIC_CAPS (GST_VIDEO_CAPS_BGRx)
 );
 
-GST_PAD_TEMPLATE_FACTORY (audio_template_factory,
+static GstStaticPadTemplate audio_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "audio_00",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "swfdec_audiosrc",
-    "audio/x-raw-int",
-      "endianness",	GST_PROPS_INT(G_BYTE_ORDER),
-      "signed",		GST_PROPS_BOOLEAN(TRUE),
-      "width",		GST_PROPS_INT(16),
-      "depth",		GST_PROPS_INT(16),
-      "rate",		GST_PROPS_INT(44100),
-      "channels",	GST_PROPS_INT(2)
-  )
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+    "rate = (int) 44100, "
+    "channels = (int) 2, "
+    "endianness = (int) BYTE_ORDER, "
+    "width = (int) 16, "
+    "depth = (int) 16, "
+    "signed = (boolean) true, "
+    "buffer-frames = (int) [ 1, MAX ]")
 );
 
-GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
+static GstStaticPadTemplate sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "swfdec_sink",
-    "application/x-shockwave-flash",
-    NULL
-  )
+  GST_STATIC_CAPS ( "application/x-shockwave-flash")
 );
 
 static void	gst_swfdec_base_init		(gpointer g_class);
@@ -148,11 +134,11 @@ gst_swfdec_base_init(gpointer g_class)
   gst_element_class_set_details (element_class, &gst_swfdec_details);
 
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (video_template_factory));
+		  gst_static_pad_template_get (&video_template_factory));
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (audio_template_factory));
+		  gst_static_pad_template_get (&audio_template_factory));
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (sink_template_factory));
+		  gst_static_pad_template_get (&sink_template_factory));
 }
 static void
 gst_swfdec_class_init(GstSwfdecClass *klass)
@@ -172,33 +158,76 @@ gst_swfdec_class_init(GstSwfdecClass *klass)
   gstelement_class->change_state = gst_swfdec_change_state;
 }
 
-static GstCaps *gst_swfdec_videosrc_getcaps(GstPad *pad, GstCaps *caps)
+static GstCaps *
+gst_swfdec_video_getcaps (GstPad *pad)
 {
   GstSwfdec *swfdec;
-  GstCaps *c;
+  GstCaps *caps;
 
   swfdec = GST_SWFDEC (gst_pad_get_parent (pad));
 
-  c = GST_CAPS_NEW (
-    "swfdec_videosrc",
-    "video/x-raw-rgb",
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096),
-      "bpp",		GST_PROPS_INT (24),
-      "depth",		GST_PROPS_INT (24),
-      "endianness",	GST_PROPS_INT (G_BIG_ENDIAN),
-      "red_mask",	GST_PROPS_INT (0xff0000),
-      "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0x0000ff),
-      "framerate",	GST_PROPS_FLOAT (swfdec->frame_rate)
-  );
-
-  if(swfdec->height){
-    gst_caps_set(c,"height",GST_PROPS_INT(swfdec->height));
-    gst_caps_set(c,"width",GST_PROPS_INT(swfdec->width));
+  caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  if (swfdec->have_format) {
+    gst_caps_set_simple (caps,
+        "framerate", G_TYPE_DOUBLE, swfdec->frame_rate,
+        NULL);
   }
 
-  return c;
+  return caps;
+}
+
+static GstPadLinkReturn
+gst_swfdec_video_link (GstPad *pad, const GstCaps *caps)
+{
+  GstSwfdec *swfdec;
+  GstStructure *structure;
+  int width, height;
+  int ret;
+
+  swfdec = GST_SWFDEC (gst_pad_get_parent (pad));
+
+  if (!swfdec->have_format) {
+    return GST_PAD_LINK_DELAYED;
+  }
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int (structure, "width", &width);
+  gst_structure_get_int (structure, "height", &height);
+
+  if (swfdec->height == height && swfdec->width == width) {
+    return GST_PAD_LINK_OK;
+  }
+
+  ret = swfdec_decoder_set_image_size (swfdec->state, width, height);
+  if (ret == SWF_OK) {
+    swfdec->width = width;
+    swfdec->height = height;
+
+    return GST_PAD_LINK_OK;
+  }
+
+  return GST_PAD_LINK_REFUSED;
+}
+
+static void
+copy_image (void *dest, void *src, int width, int height)
+{
+  guint8 *d = dest;
+  guint8 *s = src;
+  int x,y;
+
+  for(y=0;y<height;y++){
+    for(x=0;x<width;x++){
+      d[0] = s[2];
+      d[1] = s[1];
+      d[2] = s[0];
+      d[3] = 0;
+      d+=4;
+      s+=3;
+    }
+  }
+  
 }
 
 static void
@@ -239,14 +268,31 @@ gst_swfdec_loop(GstElement *element)
 	}
 
 	if(ret==SWF_CHANGE){
+	  	GstCaps *caps;
+                GstPadLinkReturn link_ret;
+
 		swfdec_decoder_get_image_size(swfdec->state,
 			&swfdec->width, &swfdec->height);
 		swfdec_decoder_get_rate(swfdec->state, &swfdec->rate);
 		swfdec->interval = GST_SECOND / swfdec->rate;
 
-		gst_caps_debug(gst_swfdec_videosrc_getcaps(swfdec->videopad,NULL),"ack");
-		gst_pad_try_set_caps(swfdec->videopad,
-			gst_swfdec_videosrc_getcaps(swfdec->videopad,NULL));
+		caps = gst_caps_copy (gst_pad_get_pad_template_caps (
+		      swfdec->videopad));
+                swfdec_decoder_get_rate (swfdec->state, &swfdec->frame_rate);
+		gst_caps_set_simple (caps,
+		      "framerate", G_TYPE_DOUBLE, swfdec->frame_rate,
+		      "height",G_TYPE_INT,swfdec->height,
+		      "width",G_TYPE_INT,swfdec->width,
+		      NULL);
+		link_ret = gst_pad_try_set_caps (swfdec->videopad, caps);
+                if (GST_PAD_LINK_SUCCESSFUL (link_ret)){
+                  /* good */
+                } else {
+                  GST_ELEMENT_ERROR (swfdec, CORE, NEGOTIATION, (NULL), (NULL));
+                  return;
+                }
+                swfdec->have_format = TRUE;
+
 		return;
 	}
 
@@ -256,11 +302,17 @@ gst_swfdec_loop(GstElement *element)
 		int len;
 
 		/* video stuff */
-		newbuf = gst_buffer_new();
-		GST_BUFFER_SIZE(newbuf) = swfdec->width * swfdec->height * 3;
+		//newbuf = gst_buffer_new();
+		//GST_BUFFER_SIZE(newbuf) = swfdec->width * swfdec->height * 3;
+
+                newbuf = gst_pad_alloc_buffer (swfdec->videopad, GST_BUFFER_OFFSET_NONE,
+                    swfdec->width * 4 * swfdec->height);
 
 		swfdec_decoder_get_image(swfdec->state, &data);
-		GST_BUFFER_DATA(newbuf) = data;
+                copy_image (GST_BUFFER_DATA (newbuf), data, swfdec->width,
+                    swfdec->height);
+                free (data);
+		//GST_BUFFER_DATA(newbuf) = data;
 
 		swfdec->timestamp += swfdec->interval;
 		GST_BUFFER_TIMESTAMP(newbuf) = swfdec->timestamp;
@@ -294,20 +346,20 @@ gst_swfdec_init (GstSwfdec *swfdec)
 {
   /* create the sink and src pads */
   swfdec->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_template_factory), "sink");
+		  gst_static_pad_template_get (&sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (swfdec), swfdec->sinkpad);
 
   swfdec->videopad = gst_pad_new_from_template(
-		GST_PAD_TEMPLATE_GET(video_template_factory),
+		gst_static_pad_template_get (&video_template_factory),
 		"video_00");
   gst_pad_set_query_function (swfdec->videopad,
 		GST_DEBUG_FUNCPTR (gst_swfdec_src_query));
-  gst_pad_set_getcaps_function (swfdec->videopad,
-		GST_DEBUG_FUNCPTR (gst_swfdec_videosrc_getcaps));
+  gst_pad_set_getcaps_function (swfdec->videopad, gst_swfdec_video_getcaps);
+  gst_pad_set_link_function (swfdec->videopad, gst_swfdec_video_link);
   gst_element_add_pad(GST_ELEMENT(swfdec), swfdec->videopad);
 
   swfdec->audiopad = gst_pad_new_from_template(
-		GST_PAD_TEMPLATE_GET(audio_template_factory),
+		gst_static_pad_template_get (&audio_template_factory),
 		"audio_00");
   gst_pad_set_query_function (swfdec->audiopad,
 		GST_DEBUG_FUNCPTR (gst_swfdec_src_query));
@@ -563,10 +615,10 @@ gst_swfdec_change_state (GstElement *element)
       //swfdec->decoder->is_sequence_needed = 1;
       //swfdec->decoder->frame_rate_code = 0;
       swfdec->timestamp = 0;
-      swfdec->pool = NULL;
       swfdec->closed = FALSE;
 
       /* reset the initial video state */
+      swfdec->have_format = FALSE;
       swfdec->format = -1;
       swfdec->width = -1;
       swfdec->height = -1;
@@ -574,19 +626,8 @@ gst_swfdec_change_state (GstElement *element)
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
-      /* try to get a bufferpool */
-#if 0
-      swfdec->pool = gst_pad_get_bufferpool (swfdec->videopad);
-      if (swfdec->pool)
-        GST_INFO ( "got pool %p", swfdec->pool);
-#endif
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-      /* need to clear things we get from other plugins, since we could be reconnected */
-      if (swfdec->pool) {
-	gst_buffer_pool_unref (swfdec->pool);
-	swfdec->pool = NULL;
-      }
       break;
     case GST_STATE_PAUSED_TO_READY:
       /* if we are not closed by an EOS event do so now, this cen send a few frames but
@@ -652,7 +693,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
