@@ -17,9 +17,40 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * @file dvdata.h
+ * Constants for DV codec.
+ */
+
+/* 
+ * DVprofile is used to express the differences between various 
+ * DV flavors. For now it's primarily used for differentiating
+ * 525/60 and 625/50, but the plans are to use it for various
+ * DV specs as well (e.g. SMPTE314M vs. IEC 61834).
+ */
+typedef struct DVprofile {
+    int              dsf;                 /* value of the dsf in the DV header */
+    int              frame_size;          /* total size of one frame in bytes */
+    int              difseg_size;         /* number of DIF segments */
+    int              frame_rate;      
+    int              frame_rate_base;
+    int              ltc_divisor;         /* FPS from the LTS standpoint */
+    int              height;              /* picture height in pixels */
+    int              width;               /* picture width in pixels */
+    const uint16_t  *video_place;         /* positions of all DV macro blocks */
+    enum PixelFormat pix_fmt;             /* picture pixel format */
+    
+    int              audio_stride;        /* size of audio_shuffle table */
+    int              audio_min_samples[3];/* min ammount of audio samples */
+                                          /* for 48Khz, 44.1Khz and 32Khz */
+    int              audio_samples_dist[5];/* how many samples are supposed to be */
+                                         /* in each frame in a 5 frames window */
+    const uint16_t (*audio_shuffle)[9];  /* PCM shuffling table */
+} DVprofile;
+
 #define NB_DV_VLC 409
 
-static const UINT16 dv_vlc_bits[409] = {
+static const uint16_t dv_vlc_bits[409] = {
  0x0000, 0x0002, 0x0007, 0x0008, 0x0009, 0x0014, 0x0015, 0x0016,
  0x0017, 0x0030, 0x0031, 0x0032, 0x0033, 0x0068, 0x0069, 0x006a,
  0x006b, 0x006c, 0x006d, 0x006e, 0x006f, 0x00e0, 0x00e1, 0x00e2,
@@ -74,7 +105,7 @@ static const UINT16 dv_vlc_bits[409] = {
  0x0006,
 };
 
-static const UINT8 dv_vlc_len[409] = {
+static const uint8_t dv_vlc_len[409] = {
   2,  3,  4,  4,  4,  5,  5,  5,
   5,  6,  6,  6,  6,  7,  7,  7,
   7,  7,  7,  7,  7,  8,  8,  8,
@@ -129,7 +160,7 @@ static const UINT8 dv_vlc_len[409] = {
   4,
 };
 
-static const UINT8 dv_vlc_run[409] = {
+static const uint8_t dv_vlc_run[409] = {
   0,  0,  1,  0,  0,  2,  1,  0,
   0,  3,  4,  0,  0,  5,  6,  2,
   1,  1,  0,  0,  0,  7,  8,  9,
@@ -184,7 +215,7 @@ static const UINT8 dv_vlc_run[409] = {
   0,
 };
 
-static const UINT8 dv_vlc_level[409] = {
+static const uint8_t dv_vlc_level[409] = {
   1,  2,  1,  3,  4,  1,  2,  5,
   6,  1,  1,  7,  8,  1,  1,  2,
   3,  4,  9, 10, 11,  1,  1,  1,
@@ -239,21 +270,8 @@ static const UINT8 dv_vlc_level[409] = {
   0,
 };
 
-/* Specific zigzag scan for 248 idct. NOTE that unlike the
-   specification, we interleave the fields */
-static const UINT8 dv_248_zigzag[64] = {
-  0,  8,  1,  9, 16, 24,  2, 10,
- 17, 25, 32, 40, 48, 56, 33, 41,
- 18, 26,  3, 11,  4, 12, 19, 27,
- 34, 42, 49, 57, 50, 58, 35, 43,
- 20, 28,  5, 13,  6, 14, 21, 29,
- 36, 44, 51, 59, 52, 60, 37, 45,
- 22, 30,  7, 15, 23, 31, 38, 46,
- 53, 61, 54, 62, 39, 47, 55, 63,
-};
-
 /* unquant tables (not used directly) */
-static const UINT8 dv_88_areas[64] = {
+static const uint8_t dv_88_areas[64] = {
     0,0,0,1,1,1,2,2,
     0,0,1,1,1,2,2,2,
     0,1,1,1,2,2,2,3,
@@ -264,7 +282,7 @@ static const UINT8 dv_88_areas[64] = {
     2,2,3,3,3,3,3,3,
 };
 
-static const UINT8 dv_248_areas[64] = {
+static const uint8_t dv_248_areas[64] = {
     0,0,1,1,1,2,2,3,
     0,0,1,1,2,2,2,3,
     0,1,1,2,2,2,3,3,
@@ -275,7 +293,7 @@ static const UINT8 dv_248_areas[64] = {
     1,2,2,3,3,3,3,3,
 };
 
-static UINT8 dv_quant_shifts[22][4] = {
+static const uint8_t dv_quant_shifts[22][4] = {
   { 3,3,4,4 }, 
   { 3,3,4,4 }, 
   { 2,3,3,4 }, 
@@ -300,12 +318,12 @@ static UINT8 dv_quant_shifts[22][4] = {
   { 0,0,0,0 },
 };
 
-static const UINT8 dv_quant_offset[4] = { 6, 3, 0, 1 };
+static const uint8_t dv_quant_offset[4] = { 6, 3, 0, 1 };
 
 /* NOTE: I prefer hardcoding the positionning of dv blocks, it is
    simpler :-) */
 
-static const UINT16 dv_place_420[1620] = {
+static const uint16_t dv_place_420[1620] = {
  0x0c24, 0x2412, 0x3036, 0x0000, 0x1848,
  0x0e24, 0x2612, 0x3236, 0x0200, 0x1a48,
  0x1024, 0x2812, 0x3436, 0x0400, 0x1c48,
@@ -632,7 +650,334 @@ static const UINT16 dv_place_420[1620] = {
  0x0a34, 0x2222, 0x2e46, 0x4610, 0x1658,
 };
 
-static const UINT16 dv_place_411[1350] = {
+static const uint16_t dv_place_411P[1620] = {
+ 0x0c24, 0x2710, 0x3334, 0x0000, 0x1848, 
+ 0x0d24, 0x2810, 0x3434, 0x0100, 0x1948, 
+ 0x0e24, 0x2910, 0x3534, 0x0200, 0x1a48, 
+ 0x0f24, 0x2914, 0x3538, 0x0300, 0x1b48, 
+ 0x1024, 0x2814, 0x3438, 0x0400, 0x1c48, 
+ 0x1124, 0x2714, 0x3338, 0x0500, 0x1d48, 
+ 0x1128, 0x2614, 0x3238, 0x0504, 0x1d4c, 
+ 0x1028, 0x2514, 0x3138, 0x0404, 0x1c4c, 
+ 0x0f28, 0x2414, 0x3038, 0x0304, 0x1b4c, 
+ 0x0e28, 0x2418, 0x303c, 0x0204, 0x1a4c, 
+ 0x0d28, 0x2518, 0x313c, 0x0104, 0x194c, 
+ 0x0c28, 0x2618, 0x323c, 0x0004, 0x184c, 
+ 0x0c2c, 0x2718, 0x333c, 0x0008, 0x1850, 
+ 0x0d2c, 0x2818, 0x343c, 0x0108, 0x1950, 
+ 0x0e2c, 0x2918, 0x353c, 0x0208, 0x1a50, 
+ 0x0f2c, 0x291c, 0x3540, 0x0308, 0x1b50, 
+ 0x102c, 0x281c, 0x3440, 0x0408, 0x1c50, 
+ 0x112c, 0x271c, 0x3340, 0x0508, 0x1d50, 
+ 0x1130, 0x261c, 0x3240, 0x050c, 0x1d54, 
+ 0x1030, 0x251c, 0x3140, 0x040c, 0x1c54, 
+ 0x0f30, 0x241c, 0x3040, 0x030c, 0x1b54, 
+ 0x0e30, 0x2420, 0x3044, 0x020c, 0x1a54, 
+ 0x0d30, 0x2520, 0x3144, 0x010c, 0x1954, 
+ 0x0c30, 0x2620, 0x3244, 0x000c, 0x1854, 
+ 0x0c34, 0x2720, 0x3344, 0x0010, 0x1858, 
+ 0x0d34, 0x2820, 0x3444, 0x0110, 0x1a58, 
+ 0x0e34, 0x2920, 0x3544, 0x0210, 0x1c58, 
+ 0x1224, 0x2d10, 0x3934, 0x0600, 0x1e48, 
+ 0x1324, 0x2e10, 0x3a34, 0x0700, 0x1f48, 
+ 0x1424, 0x2f10, 0x3b34, 0x0800, 0x2048, 
+ 0x1524, 0x2f14, 0x3b38, 0x0900, 0x2148, 
+ 0x1624, 0x2e14, 0x3a38, 0x0a00, 0x2248, 
+ 0x1724, 0x2d14, 0x3938, 0x0b00, 0x2348, 
+ 0x1728, 0x2c14, 0x3838, 0x0b04, 0x234c, 
+ 0x1628, 0x2b14, 0x3738, 0x0a04, 0x224c, 
+ 0x1528, 0x2a14, 0x3638, 0x0904, 0x214c, 
+ 0x1428, 0x2a18, 0x363c, 0x0804, 0x204c, 
+ 0x1328, 0x2b18, 0x373c, 0x0704, 0x1f4c, 
+ 0x1228, 0x2c18, 0x383c, 0x0604, 0x1e4c, 
+ 0x122c, 0x2d18, 0x393c, 0x0608, 0x1e50, 
+ 0x132c, 0x2e18, 0x3a3c, 0x0708, 0x1f50, 
+ 0x142c, 0x2f18, 0x3b3c, 0x0808, 0x2050, 
+ 0x152c, 0x2f1c, 0x3b40, 0x0908, 0x2150, 
+ 0x162c, 0x2e1c, 0x3a40, 0x0a08, 0x2250, 
+ 0x172c, 0x2d1c, 0x3940, 0x0b08, 0x2350, 
+ 0x1730, 0x2c1c, 0x3840, 0x0b0c, 0x2354, 
+ 0x1630, 0x2b1c, 0x3740, 0x0a0c, 0x2254, 
+ 0x1530, 0x2a1c, 0x3640, 0x090c, 0x2154, 
+ 0x1430, 0x2a20, 0x3644, 0x080c, 0x2054, 
+ 0x1330, 0x2b20, 0x3744, 0x070c, 0x1f54, 
+ 0x1230, 0x2c20, 0x3844, 0x060c, 0x1e54, 
+ 0x1234, 0x2d20, 0x3944, 0x0610, 0x1e58, 
+ 0x1334, 0x2e20, 0x3a44, 0x0710, 0x2058, 
+ 0x1434, 0x2f20, 0x3b44, 0x0810, 0x2258, 
+ 0x1824, 0x3310, 0x3f34, 0x0c00, 0x2448, 
+ 0x1924, 0x3410, 0x4034, 0x0d00, 0x2548, 
+ 0x1a24, 0x3510, 0x4134, 0x0e00, 0x2648, 
+ 0x1b24, 0x3514, 0x4138, 0x0f00, 0x2748, 
+ 0x1c24, 0x3414, 0x4038, 0x1000, 0x2848, 
+ 0x1d24, 0x3314, 0x3f38, 0x1100, 0x2948, 
+ 0x1d28, 0x3214, 0x3e38, 0x1104, 0x294c, 
+ 0x1c28, 0x3114, 0x3d38, 0x1004, 0x284c, 
+ 0x1b28, 0x3014, 0x3c38, 0x0f04, 0x274c, 
+ 0x1a28, 0x3018, 0x3c3c, 0x0e04, 0x264c, 
+ 0x1928, 0x3118, 0x3d3c, 0x0d04, 0x254c, 
+ 0x1828, 0x3218, 0x3e3c, 0x0c04, 0x244c, 
+ 0x182c, 0x3318, 0x3f3c, 0x0c08, 0x2450, 
+ 0x192c, 0x3418, 0x403c, 0x0d08, 0x2550, 
+ 0x1a2c, 0x3518, 0x413c, 0x0e08, 0x2650, 
+ 0x1b2c, 0x351c, 0x4140, 0x0f08, 0x2750, 
+ 0x1c2c, 0x341c, 0x4040, 0x1008, 0x2850, 
+ 0x1d2c, 0x331c, 0x3f40, 0x1108, 0x2950, 
+ 0x1d30, 0x321c, 0x3e40, 0x110c, 0x2954, 
+ 0x1c30, 0x311c, 0x3d40, 0x100c, 0x2854, 
+ 0x1b30, 0x301c, 0x3c40, 0x0f0c, 0x2754, 
+ 0x1a30, 0x3020, 0x3c44, 0x0e0c, 0x2654, 
+ 0x1930, 0x3120, 0x3d44, 0x0d0c, 0x2554, 
+ 0x1830, 0x3220, 0x3e44, 0x0c0c, 0x2454, 
+ 0x1834, 0x3320, 0x3f44, 0x0c10, 0x2458, 
+ 0x1934, 0x3420, 0x4044, 0x0d10, 0x2658, 
+ 0x1a34, 0x3520, 0x4144, 0x0e10, 0x2858, 
+ 0x1e24, 0x3910, 0x4534, 0x1200, 0x2a48, 
+ 0x1f24, 0x3a10, 0x4634, 0x1300, 0x2b48, 
+ 0x2024, 0x3b10, 0x4734, 0x1400, 0x2c48, 
+ 0x2124, 0x3b14, 0x4738, 0x1500, 0x2d48, 
+ 0x2224, 0x3a14, 0x4638, 0x1600, 0x2e48, 
+ 0x2324, 0x3914, 0x4538, 0x1700, 0x2f48, 
+ 0x2328, 0x3814, 0x4438, 0x1704, 0x2f4c, 
+ 0x2228, 0x3714, 0x4338, 0x1604, 0x2e4c, 
+ 0x2128, 0x3614, 0x4238, 0x1504, 0x2d4c, 
+ 0x2028, 0x3618, 0x423c, 0x1404, 0x2c4c, 
+ 0x1f28, 0x3718, 0x433c, 0x1304, 0x2b4c, 
+ 0x1e28, 0x3818, 0x443c, 0x1204, 0x2a4c, 
+ 0x1e2c, 0x3918, 0x453c, 0x1208, 0x2a50, 
+ 0x1f2c, 0x3a18, 0x463c, 0x1308, 0x2b50, 
+ 0x202c, 0x3b18, 0x473c, 0x1408, 0x2c50, 
+ 0x212c, 0x3b1c, 0x4740, 0x1508, 0x2d50, 
+ 0x222c, 0x3a1c, 0x4640, 0x1608, 0x2e50, 
+ 0x232c, 0x391c, 0x4540, 0x1708, 0x2f50, 
+ 0x2330, 0x381c, 0x4440, 0x170c, 0x2f54, 
+ 0x2230, 0x371c, 0x4340, 0x160c, 0x2e54, 
+ 0x2130, 0x361c, 0x4240, 0x150c, 0x2d54, 
+ 0x2030, 0x3620, 0x4244, 0x140c, 0x2c54, 
+ 0x1f30, 0x3720, 0x4344, 0x130c, 0x2b54, 
+ 0x1e30, 0x3820, 0x4444, 0x120c, 0x2a54, 
+ 0x1e34, 0x3920, 0x4544, 0x1210, 0x2a58, 
+ 0x1f34, 0x3a20, 0x4644, 0x1310, 0x2c58, 
+ 0x2034, 0x3b20, 0x4744, 0x1410, 0x2e58, 
+ 0x2424, 0x3f10, 0x0334, 0x1800, 0x3048, 
+ 0x2524, 0x4010, 0x0434, 0x1900, 0x3148, 
+ 0x2624, 0x4110, 0x0534, 0x1a00, 0x3248, 
+ 0x2724, 0x4114, 0x0538, 0x1b00, 0x3348, 
+ 0x2824, 0x4014, 0x0438, 0x1c00, 0x3448, 
+ 0x2924, 0x3f14, 0x0338, 0x1d00, 0x3548, 
+ 0x2928, 0x3e14, 0x0238, 0x1d04, 0x354c, 
+ 0x2828, 0x3d14, 0x0138, 0x1c04, 0x344c, 
+ 0x2728, 0x3c14, 0x0038, 0x1b04, 0x334c, 
+ 0x2628, 0x3c18, 0x003c, 0x1a04, 0x324c, 
+ 0x2528, 0x3d18, 0x013c, 0x1904, 0x314c, 
+ 0x2428, 0x3e18, 0x023c, 0x1804, 0x304c, 
+ 0x242c, 0x3f18, 0x033c, 0x1808, 0x3050, 
+ 0x252c, 0x4018, 0x043c, 0x1908, 0x3150, 
+ 0x262c, 0x4118, 0x053c, 0x1a08, 0x3250, 
+ 0x272c, 0x411c, 0x0540, 0x1b08, 0x3350, 
+ 0x282c, 0x401c, 0x0440, 0x1c08, 0x3450, 
+ 0x292c, 0x3f1c, 0x0340, 0x1d08, 0x3550, 
+ 0x2930, 0x3e1c, 0x0240, 0x1d0c, 0x3554, 
+ 0x2830, 0x3d1c, 0x0140, 0x1c0c, 0x3454, 
+ 0x2730, 0x3c1c, 0x0040, 0x1b0c, 0x3354, 
+ 0x2630, 0x3c20, 0x0044, 0x1a0c, 0x3254, 
+ 0x2530, 0x3d20, 0x0144, 0x190c, 0x3154, 
+ 0x2430, 0x3e20, 0x0244, 0x180c, 0x3054, 
+ 0x2434, 0x3f20, 0x0344, 0x1810, 0x3058, 
+ 0x2534, 0x4020, 0x0444, 0x1910, 0x3258, 
+ 0x2634, 0x4120, 0x0544, 0x1a10, 0x3458, 
+ 0x2a24, 0x4510, 0x0934, 0x1e00, 0x3648, 
+ 0x2b24, 0x4610, 0x0a34, 0x1f00, 0x3748, 
+ 0x2c24, 0x4710, 0x0b34, 0x2000, 0x3848, 
+ 0x2d24, 0x4714, 0x0b38, 0x2100, 0x3948, 
+ 0x2e24, 0x4614, 0x0a38, 0x2200, 0x3a48, 
+ 0x2f24, 0x4514, 0x0938, 0x2300, 0x3b48, 
+ 0x2f28, 0x4414, 0x0838, 0x2304, 0x3b4c, 
+ 0x2e28, 0x4314, 0x0738, 0x2204, 0x3a4c, 
+ 0x2d28, 0x4214, 0x0638, 0x2104, 0x394c, 
+ 0x2c28, 0x4218, 0x063c, 0x2004, 0x384c, 
+ 0x2b28, 0x4318, 0x073c, 0x1f04, 0x374c, 
+ 0x2a28, 0x4418, 0x083c, 0x1e04, 0x364c, 
+ 0x2a2c, 0x4518, 0x093c, 0x1e08, 0x3650, 
+ 0x2b2c, 0x4618, 0x0a3c, 0x1f08, 0x3750, 
+ 0x2c2c, 0x4718, 0x0b3c, 0x2008, 0x3850, 
+ 0x2d2c, 0x471c, 0x0b40, 0x2108, 0x3950, 
+ 0x2e2c, 0x461c, 0x0a40, 0x2208, 0x3a50, 
+ 0x2f2c, 0x451c, 0x0940, 0x2308, 0x3b50, 
+ 0x2f30, 0x441c, 0x0840, 0x230c, 0x3b54, 
+ 0x2e30, 0x431c, 0x0740, 0x220c, 0x3a54, 
+ 0x2d30, 0x421c, 0x0640, 0x210c, 0x3954, 
+ 0x2c30, 0x4220, 0x0644, 0x200c, 0x3854, 
+ 0x2b30, 0x4320, 0x0744, 0x1f0c, 0x3754, 
+ 0x2a30, 0x4420, 0x0844, 0x1e0c, 0x3654, 
+ 0x2a34, 0x4520, 0x0944, 0x1e10, 0x3658, 
+ 0x2b34, 0x4620, 0x0a44, 0x1f10, 0x3858, 
+ 0x2c34, 0x4720, 0x0b44, 0x2010, 0x3a58, 
+ 0x3024, 0x0310, 0x0f34, 0x2400, 0x3c48, 
+ 0x3124, 0x0410, 0x1034, 0x2500, 0x3d48, 
+ 0x3224, 0x0510, 0x1134, 0x2600, 0x3e48, 
+ 0x3324, 0x0514, 0x1138, 0x2700, 0x3f48, 
+ 0x3424, 0x0414, 0x1038, 0x2800, 0x4048, 
+ 0x3524, 0x0314, 0x0f38, 0x2900, 0x4148, 
+ 0x3528, 0x0214, 0x0e38, 0x2904, 0x414c, 
+ 0x3428, 0x0114, 0x0d38, 0x2804, 0x404c, 
+ 0x3328, 0x0014, 0x0c38, 0x2704, 0x3f4c, 
+ 0x3228, 0x0018, 0x0c3c, 0x2604, 0x3e4c, 
+ 0x3128, 0x0118, 0x0d3c, 0x2504, 0x3d4c, 
+ 0x3028, 0x0218, 0x0e3c, 0x2404, 0x3c4c, 
+ 0x302c, 0x0318, 0x0f3c, 0x2408, 0x3c50, 
+ 0x312c, 0x0418, 0x103c, 0x2508, 0x3d50, 
+ 0x322c, 0x0518, 0x113c, 0x2608, 0x3e50, 
+ 0x332c, 0x051c, 0x1140, 0x2708, 0x3f50, 
+ 0x342c, 0x041c, 0x1040, 0x2808, 0x4050, 
+ 0x352c, 0x031c, 0x0f40, 0x2908, 0x4150, 
+ 0x3530, 0x021c, 0x0e40, 0x290c, 0x4154, 
+ 0x3430, 0x011c, 0x0d40, 0x280c, 0x4054, 
+ 0x3330, 0x001c, 0x0c40, 0x270c, 0x3f54, 
+ 0x3230, 0x0020, 0x0c44, 0x260c, 0x3e54, 
+ 0x3130, 0x0120, 0x0d44, 0x250c, 0x3d54, 
+ 0x3030, 0x0220, 0x0e44, 0x240c, 0x3c54, 
+ 0x3034, 0x0320, 0x0f44, 0x2410, 0x3c58, 
+ 0x3134, 0x0420, 0x1044, 0x2510, 0x3e58, 
+ 0x3234, 0x0520, 0x1144, 0x2610, 0x4058, 
+ 0x3624, 0x0910, 0x1534, 0x2a00, 0x4248, 
+ 0x3724, 0x0a10, 0x1634, 0x2b00, 0x4348, 
+ 0x3824, 0x0b10, 0x1734, 0x2c00, 0x4448, 
+ 0x3924, 0x0b14, 0x1738, 0x2d00, 0x4548, 
+ 0x3a24, 0x0a14, 0x1638, 0x2e00, 0x4648, 
+ 0x3b24, 0x0914, 0x1538, 0x2f00, 0x4748, 
+ 0x3b28, 0x0814, 0x1438, 0x2f04, 0x474c, 
+ 0x3a28, 0x0714, 0x1338, 0x2e04, 0x464c, 
+ 0x3928, 0x0614, 0x1238, 0x2d04, 0x454c, 
+ 0x3828, 0x0618, 0x123c, 0x2c04, 0x444c, 
+ 0x3728, 0x0718, 0x133c, 0x2b04, 0x434c, 
+ 0x3628, 0x0818, 0x143c, 0x2a04, 0x424c, 
+ 0x362c, 0x0918, 0x153c, 0x2a08, 0x4250, 
+ 0x372c, 0x0a18, 0x163c, 0x2b08, 0x4350, 
+ 0x382c, 0x0b18, 0x173c, 0x2c08, 0x4450, 
+ 0x392c, 0x0b1c, 0x1740, 0x2d08, 0x4550, 
+ 0x3a2c, 0x0a1c, 0x1640, 0x2e08, 0x4650, 
+ 0x3b2c, 0x091c, 0x1540, 0x2f08, 0x4750, 
+ 0x3b30, 0x081c, 0x1440, 0x2f0c, 0x4754, 
+ 0x3a30, 0x071c, 0x1340, 0x2e0c, 0x4654, 
+ 0x3930, 0x061c, 0x1240, 0x2d0c, 0x4554, 
+ 0x3830, 0x0620, 0x1244, 0x2c0c, 0x4454, 
+ 0x3730, 0x0720, 0x1344, 0x2b0c, 0x4354, 
+ 0x3630, 0x0820, 0x1444, 0x2a0c, 0x4254, 
+ 0x3634, 0x0920, 0x1544, 0x2a10, 0x4258, 
+ 0x3734, 0x0a20, 0x1644, 0x2b10, 0x4458, 
+ 0x3834, 0x0b20, 0x1744, 0x2c10, 0x4658, 
+ 0x3c24, 0x0f10, 0x1b34, 0x3000, 0x0048, 
+ 0x3d24, 0x1010, 0x1c34, 0x3100, 0x0148, 
+ 0x3e24, 0x1110, 0x1d34, 0x3200, 0x0248, 
+ 0x3f24, 0x1114, 0x1d38, 0x3300, 0x0348, 
+ 0x4024, 0x1014, 0x1c38, 0x3400, 0x0448, 
+ 0x4124, 0x0f14, 0x1b38, 0x3500, 0x0548, 
+ 0x4128, 0x0e14, 0x1a38, 0x3504, 0x054c, 
+ 0x4028, 0x0d14, 0x1938, 0x3404, 0x044c, 
+ 0x3f28, 0x0c14, 0x1838, 0x3304, 0x034c, 
+ 0x3e28, 0x0c18, 0x183c, 0x3204, 0x024c, 
+ 0x3d28, 0x0d18, 0x193c, 0x3104, 0x014c, 
+ 0x3c28, 0x0e18, 0x1a3c, 0x3004, 0x004c, 
+ 0x3c2c, 0x0f18, 0x1b3c, 0x3008, 0x0050, 
+ 0x3d2c, 0x1018, 0x1c3c, 0x3108, 0x0150, 
+ 0x3e2c, 0x1118, 0x1d3c, 0x3208, 0x0250, 
+ 0x3f2c, 0x111c, 0x1d40, 0x3308, 0x0350, 
+ 0x402c, 0x101c, 0x1c40, 0x3408, 0x0450, 
+ 0x412c, 0x0f1c, 0x1b40, 0x3508, 0x0550, 
+ 0x4130, 0x0e1c, 0x1a40, 0x350c, 0x0554, 
+ 0x4030, 0x0d1c, 0x1940, 0x340c, 0x0454, 
+ 0x3f30, 0x0c1c, 0x1840, 0x330c, 0x0354, 
+ 0x3e30, 0x0c20, 0x1844, 0x320c, 0x0254, 
+ 0x3d30, 0x0d20, 0x1944, 0x310c, 0x0154, 
+ 0x3c30, 0x0e20, 0x1a44, 0x300c, 0x0054, 
+ 0x3c34, 0x0f20, 0x1b44, 0x3010, 0x0058, 
+ 0x3d34, 0x1020, 0x1c44, 0x3110, 0x0258, 
+ 0x3e34, 0x1120, 0x1d44, 0x3210, 0x0458, 
+ 0x4224, 0x1510, 0x2134, 0x3600, 0x0648, 
+ 0x4324, 0x1610, 0x2234, 0x3700, 0x0748, 
+ 0x4424, 0x1710, 0x2334, 0x3800, 0x0848, 
+ 0x4524, 0x1714, 0x2338, 0x3900, 0x0948, 
+ 0x4624, 0x1614, 0x2238, 0x3a00, 0x0a48, 
+ 0x4724, 0x1514, 0x2138, 0x3b00, 0x0b48, 
+ 0x4728, 0x1414, 0x2038, 0x3b04, 0x0b4c, 
+ 0x4628, 0x1314, 0x1f38, 0x3a04, 0x0a4c, 
+ 0x4528, 0x1214, 0x1e38, 0x3904, 0x094c, 
+ 0x4428, 0x1218, 0x1e3c, 0x3804, 0x084c, 
+ 0x4328, 0x1318, 0x1f3c, 0x3704, 0x074c, 
+ 0x4228, 0x1418, 0x203c, 0x3604, 0x064c, 
+ 0x422c, 0x1518, 0x213c, 0x3608, 0x0650, 
+ 0x432c, 0x1618, 0x223c, 0x3708, 0x0750, 
+ 0x442c, 0x1718, 0x233c, 0x3808, 0x0850, 
+ 0x452c, 0x171c, 0x2340, 0x3908, 0x0950, 
+ 0x462c, 0x161c, 0x2240, 0x3a08, 0x0a50, 
+ 0x472c, 0x151c, 0x2140, 0x3b08, 0x0b50, 
+ 0x4730, 0x141c, 0x2040, 0x3b0c, 0x0b54, 
+ 0x4630, 0x131c, 0x1f40, 0x3a0c, 0x0a54, 
+ 0x4530, 0x121c, 0x1e40, 0x390c, 0x0954, 
+ 0x4430, 0x1220, 0x1e44, 0x380c, 0x0854, 
+ 0x4330, 0x1320, 0x1f44, 0x370c, 0x0754, 
+ 0x4230, 0x1420, 0x2044, 0x360c, 0x0654, 
+ 0x4234, 0x1520, 0x2144, 0x3610, 0x0658, 
+ 0x4334, 0x1620, 0x2244, 0x3710, 0x0858, 
+ 0x4434, 0x1720, 0x2344, 0x3810, 0x0a58, 
+ 0x0024, 0x1b10, 0x2734, 0x3c00, 0x0c48, 
+ 0x0124, 0x1c10, 0x2834, 0x3d00, 0x0d48, 
+ 0x0224, 0x1d10, 0x2934, 0x3e00, 0x0e48, 
+ 0x0324, 0x1d14, 0x2938, 0x3f00, 0x0f48, 
+ 0x0424, 0x1c14, 0x2838, 0x4000, 0x1048, 
+ 0x0524, 0x1b14, 0x2738, 0x4100, 0x1148, 
+ 0x0528, 0x1a14, 0x2638, 0x4104, 0x114c, 
+ 0x0428, 0x1914, 0x2538, 0x4004, 0x104c, 
+ 0x0328, 0x1814, 0x2438, 0x3f04, 0x0f4c, 
+ 0x0228, 0x1818, 0x243c, 0x3e04, 0x0e4c, 
+ 0x0128, 0x1918, 0x253c, 0x3d04, 0x0d4c, 
+ 0x0028, 0x1a18, 0x263c, 0x3c04, 0x0c4c, 
+ 0x002c, 0x1b18, 0x273c, 0x3c08, 0x0c50, 
+ 0x012c, 0x1c18, 0x283c, 0x3d08, 0x0d50, 
+ 0x022c, 0x1d18, 0x293c, 0x3e08, 0x0e50, 
+ 0x032c, 0x1d1c, 0x2940, 0x3f08, 0x0f50, 
+ 0x042c, 0x1c1c, 0x2840, 0x4008, 0x1050, 
+ 0x052c, 0x1b1c, 0x2740, 0x4108, 0x1150, 
+ 0x0530, 0x1a1c, 0x2640, 0x410c, 0x1154, 
+ 0x0430, 0x191c, 0x2540, 0x400c, 0x1054, 
+ 0x0330, 0x181c, 0x2440, 0x3f0c, 0x0f54, 
+ 0x0230, 0x1820, 0x2444, 0x3e0c, 0x0e54, 
+ 0x0130, 0x1920, 0x2544, 0x3d0c, 0x0d54, 
+ 0x0030, 0x1a20, 0x2644, 0x3c0c, 0x0c54, 
+ 0x0034, 0x1b20, 0x2744, 0x3c10, 0x0c58, 
+ 0x0134, 0x1c20, 0x2844, 0x3d10, 0x0e58, 
+ 0x0234, 0x1d20, 0x2944, 0x3e10, 0x1058, 
+ 0x0624, 0x2110, 0x2d34, 0x4200, 0x1248, 
+ 0x0724, 0x2210, 0x2e34, 0x4300, 0x1348, 
+ 0x0824, 0x2310, 0x2f34, 0x4400, 0x1448, 
+ 0x0924, 0x2314, 0x2f38, 0x4500, 0x1548, 
+ 0x0a24, 0x2214, 0x2e38, 0x4600, 0x1648, 
+ 0x0b24, 0x2114, 0x2d38, 0x4700, 0x1748, 
+ 0x0b28, 0x2014, 0x2c38, 0x4704, 0x174c, 
+ 0x0a28, 0x1f14, 0x2b38, 0x4604, 0x164c, 
+ 0x0928, 0x1e14, 0x2a38, 0x4504, 0x154c, 
+ 0x0828, 0x1e18, 0x2a3c, 0x4404, 0x144c, 
+ 0x0728, 0x1f18, 0x2b3c, 0x4304, 0x134c, 
+ 0x0628, 0x2018, 0x2c3c, 0x4204, 0x124c, 
+ 0x062c, 0x2118, 0x2d3c, 0x4208, 0x1250, 
+ 0x072c, 0x2218, 0x2e3c, 0x4308, 0x1350, 
+ 0x082c, 0x2318, 0x2f3c, 0x4408, 0x1450, 
+ 0x092c, 0x231c, 0x2f40, 0x4508, 0x1550, 
+ 0x0a2c, 0x221c, 0x2e40, 0x4608, 0x1650, 
+ 0x0b2c, 0x211c, 0x2d40, 0x4708, 0x1750, 
+ 0x0b30, 0x201c, 0x2c40, 0x470c, 0x1754, 
+ 0x0a30, 0x1f1c, 0x2b40, 0x460c, 0x1654, 
+ 0x0930, 0x1e1c, 0x2a40, 0x450c, 0x1554, 
+ 0x0830, 0x1e20, 0x2a44, 0x440c, 0x1454, 
+ 0x0730, 0x1f20, 0x2b44, 0x430c, 0x1354, 
+ 0x0630, 0x2020, 0x2c44, 0x420c, 0x1254, 
+ 0x0634, 0x2120, 0x2d44, 0x4210, 0x1258, 
+ 0x0734, 0x2220, 0x2e44, 0x4310, 0x1458, 
+ 0x0834, 0x2320, 0x2f44, 0x4410, 0x1658,
+};
+
+static const uint16_t dv_place_411[1350] = {
  0x0c24, 0x2710, 0x3334, 0x0000, 0x1848,
  0x0d24, 0x2810, 0x3434, 0x0100, 0x1948,
  0x0e24, 0x2910, 0x3534, 0x0200, 0x1a48,
@@ -905,3 +1250,108 @@ static const UINT16 dv_place_411[1350] = {
  0x0834, 0x2320, 0x2f44, 0x3810, 0x1658,
 };
 
+static const uint16_t dv_audio_shuffle525[10][9] = {
+  {  0, 30, 60, 20, 50, 80, 10, 40, 70 }, /* 1st channel */
+  {  6, 36, 66, 26, 56, 86, 16, 46, 76 },
+  { 12, 42, 72,  2, 32, 62, 22, 52, 82 },
+  { 18, 48, 78,  8, 38, 68, 28, 58, 88 },
+  { 24, 54, 84, 14, 44, 74,  4, 34, 64 },
+  
+  {  1, 31, 61, 21, 51, 81, 11, 41, 71 }, /* 2nd channel */
+  {  7, 37, 67, 27, 57, 87, 17, 47, 77 },
+  { 13, 43, 73,  3, 33, 63, 23, 53, 83 },
+  { 19, 49, 79,  9, 39, 69, 29, 59, 89 },
+  { 25, 55, 85, 15, 45, 75,  5, 35, 65 },
+};
+
+static const uint16_t dv_audio_shuffle625[12][9] = {
+  {   0,  36,  72,  26,  62,  98,  16,  52,  88}, /* 1st channel */
+  {   6,  42,  78,  32,  68, 104,  22,  58,  94},
+  {  12,  48,  84,   2,  38,  74,  28,  64, 100},
+  {  18,  54,  90,   8,  44,  80,  34,  70, 106},
+  {  24,  60,  96,  14,  50,  86,   4,  40,  76},  
+  {  30,  66, 102,  20,  56,  92,  10,  46,  82},
+	
+  {   1,  37,  73,  27,  63,  99,  17,  53,  89}, /* 2nd channel */
+  {   7,  43,  79,  33,  69, 105,  23,  59,  95},
+  {  13,  49,  85,   3,  39,  75,  29,  65, 101},
+  {  19,  55,  91,   9,  45,  81,  35,  71, 107},
+  {  25,  61,  97,  15,  51,  87,   5,  41,  77},  
+  {  31,  67, 103,  21,  57,  93,  11,  47,  83},
+};
+
+static const __attribute__((unused)) int dv_audio_frequency[3] = {
+    48000, 44100, 32000,
+};
+    
+static const DVprofile dv_profiles[] = {
+    { .dsf = 0,
+      .frame_size = 120000,        /* IEC 61834, SMPTE-314M - 525/60 (NTSC) */
+      .difseg_size = 10,
+      .frame_rate = 30000,
+      .ltc_divisor = 30,
+      .frame_rate_base = 1001,
+      .height = 480,
+      .width = 720,
+      .video_place = dv_place_411,
+      .pix_fmt = PIX_FMT_YUV411P,
+      .audio_stride = 90,
+      .audio_min_samples = { 1580, 1452, 1053 }, /* for 48, 44.1 and 32Khz */
+      .audio_samples_dist = { 1602, 1601, 1602, 1601, 1602 },
+      .audio_shuffle = dv_audio_shuffle525,
+    }, 
+    { .dsf = 1,
+      .frame_size = 144000,        /* IEC 61834 - 625/50 (PAL) */
+      .difseg_size = 12,
+      .frame_rate = 25,
+      .frame_rate_base = 1,
+      .ltc_divisor = 25,
+      .height = 576,
+      .width = 720,
+      .video_place = dv_place_420,
+      .pix_fmt = PIX_FMT_YUV420P,
+      .audio_stride = 108,
+      .audio_min_samples = { 1896, 1742, 1264 }, /* for 48, 44.1 and 32Khz */
+      .audio_samples_dist = { 1920, 1920, 1920, 1920, 1920 },
+      .audio_shuffle = dv_audio_shuffle625,
+    },
+    { .dsf = 1,
+      .frame_size = 144000,        /* SMPTE-314M - 625/50 (PAL) */
+      .difseg_size = 12,
+      .frame_rate = 25,
+      .frame_rate_base = 1,
+      .ltc_divisor = 25,
+      .height = 576,
+      .width = 720,
+      .video_place = dv_place_411P,
+      .pix_fmt = PIX_FMT_YUV411P,
+      .audio_stride = 108,
+      .audio_min_samples = { 1896, 1742, 1264 }, /* for 48, 44.1 and 32Khz */
+      .audio_samples_dist = { 1920, 1920, 1920, 1920, 1920 },
+      .audio_shuffle = dv_audio_shuffle625,
+     }
+};
+
+static inline const DVprofile* dv_frame_profile(uint8_t* frame)
+{
+    if ((frame[3] & 0x80) == 0) {      /* DSF flag */
+        return &dv_profiles[0];
+    }
+    else if ((frame[5] & 0x07) == 0) { /* APT flag */
+        return &dv_profiles[1];
+    }
+    else 
+        return &dv_profiles[2];
+}
+
+static inline const DVprofile* dv_codec_profile(AVCodecContext* codec)
+{
+    if (codec->width != 720) {
+        return NULL;
+    } 
+    else if (codec->height == 480) {
+        return &dv_profiles[0];
+    } 
+    else 
+        return &dv_profiles[1];
+}
