@@ -16,6 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/* Element-Checklist-Version: 5 */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -42,15 +43,12 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_templ,
+static GstStaticPadTemplate sink_templ =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "avidemux_sink",
-    "video/x-msvideo",
-      NULL
-  )
+  GST_STATIC_CAPS ("video/x-msvideo")
 );
 
 static void 	gst_avi_demux_base_init	        (GstAviDemuxClass *klass);
@@ -138,20 +136,19 @@ gst_avi_demux_base_init (GstAviDemuxClass *klass)
   audiosrctempl = gst_pad_template_new ("audio_%02d",
 					GST_PAD_SRC,
 					GST_PAD_SOMETIMES,
-					audcaps, NULL);
+					audcaps);
 
-  vidcaps = gst_caps_append (
-		gst_riff_create_video_template_caps (),
-		gst_riff_create_iavs_template_caps ());
+  vidcaps = gst_riff_create_video_template_caps ();
+  gst_caps_append (vidcaps, gst_riff_create_iavs_template_caps ());
   videosrctempl = gst_pad_template_new ("video_%02d",
 					GST_PAD_SRC,
 					GST_PAD_SOMETIMES,
-					vidcaps, NULL);
+					vidcaps);
 
   gst_element_class_add_pad_template (element_class, audiosrctempl);
   gst_element_class_add_pad_template (element_class, videosrctempl);
   gst_element_class_add_pad_template (element_class,
-	GST_PAD_TEMPLATE_GET (sink_templ));
+      gst_static_pad_template_get (&sink_templ));
   gst_element_class_set_details (element_class, &gst_avi_demux_details);
 }
 
@@ -185,7 +182,7 @@ gst_avi_demux_init (GstAviDemux *avi)
   GST_FLAG_SET (avi, GST_ELEMENT_EVENT_AWARE);
 
   avi->sinkpad = gst_pad_new_from_template (
-	GST_PAD_TEMPLATE_GET (sink_templ), "sink");
+	gst_static_pad_template_get (&sink_templ), "sink");
   gst_element_add_pad (GST_ELEMENT (avi), avi->sinkpad);
   GST_RIFF_READ (avi)->sinkpad = avi->sinkpad;
 
@@ -232,16 +229,10 @@ gst_avi_demux_reset (GstAviDemux *avi)
 static void
 gst_avi_demux_streaminfo (GstAviDemux *avi)
 {
-  GstProps *props;
-
-  props = gst_props_empty_new ();
-
   /* compression formats are added later - a bit hacky */
 
-  gst_caps_replace_sink (&avi->streaminfo,
-		  	 gst_caps_new ("avi_streaminfo",
-                                       "application/x-gst-streaminfo",
-                                       props));
+  gst_caps_replace (&avi->streaminfo,
+      gst_caps_new_simple ("application/x-gst-streaminfo", NULL));
 
   /*g_object_notify(G_OBJECT(avi), "streaminfo");*/
 }
@@ -507,6 +498,14 @@ gst_avi_demux_handle_src_query (GstPad      *pad,
   return res;
 }
 
+static GstCaps *
+gst_avi_demux_src_getcaps (GstPad *pad)
+{
+  avi_stream_context *stream = gst_pad_get_element_private (pad);
+
+  return gst_caps_copy (stream->caps);
+}
+
 static gint32
 gst_avi_demux_sync_streams (GstAviDemux *avi,
 			    guint64      time)
@@ -688,7 +687,7 @@ gst_avi_demux_stream_init (GstAviDemux *avi)
   if (!gst_riff_read_header (riff, &doctype))
     return FALSE;
   if (doctype != GST_RIFF_RIFF_AVI) {
-    gst_element_error (GST_ELEMENT (avi), "Not an AVI file");
+    GST_ELEMENT_ERROR (avi, STREAM, WRONG_TYPE, (NULL), (NULL));
     return FALSE;
   }
 
@@ -707,7 +706,7 @@ gst_avi_demux_stream_avih (GstAviDemux *avi,
   GstRiffRead *riff = GST_RIFF_READ (avi);
   guint32 tag;
   GstBuffer *buf;
-  gst_riff_avih *avih;
+  gst_riff_avih avih, *_avih;
 
   if (!gst_riff_read_data (riff, &tag, &buf))
     return FALSE;
@@ -724,46 +723,43 @@ gst_avi_demux_stream_avih (GstAviDemux *avi,
     return FALSE;
   }
 
-  avih = (gst_riff_avih *) GST_BUFFER_DATA (buf);
-
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  avih->us_frame    = GUINT32_FROM_LE (avih->us_frame);
-  avih->max_bps     = GUINT32_FROM_LE (avih->max_bps);
-  avih->pad_gran    = GUINT32_FROM_LE (avih->pad_gran);
-  avih->flags       = GUINT32_FROM_LE (avih->flags);
-  avih->tot_frames  = GUINT32_FROM_LE (avih->tot_frames);
-  avih->init_frames = GUINT32_FROM_LE (avih->init_frames);
-  avih->streams     = GUINT32_FROM_LE (avih->streams);
-  avih->bufsize     = GUINT32_FROM_LE (avih->bufsize);
-  avih->width       = GUINT32_FROM_LE (avih->width);
-  avih->height      = GUINT32_FROM_LE (avih->height);
-  avih->scale       = GUINT32_FROM_LE (avih->scale);
-  avih->rate        = GUINT32_FROM_LE (avih->rate);
-  avih->start       = GUINT32_FROM_LE (avih->start);
-  avih->length      = GUINT32_FROM_LE (avih->length);
-#endif
+  _avih = (gst_riff_avih *) GST_BUFFER_DATA (buf);
+  avih.us_frame    = GUINT32_FROM_LE (_avih->us_frame);
+  avih.max_bps     = GUINT32_FROM_LE (_avih->max_bps);
+  avih.pad_gran    = GUINT32_FROM_LE (_avih->pad_gran);
+  avih.flags       = GUINT32_FROM_LE (_avih->flags);
+  avih.tot_frames  = GUINT32_FROM_LE (_avih->tot_frames);
+  avih.init_frames = GUINT32_FROM_LE (_avih->init_frames);
+  avih.streams     = GUINT32_FROM_LE (_avih->streams);
+  avih.bufsize     = GUINT32_FROM_LE (_avih->bufsize);
+  avih.width       = GUINT32_FROM_LE (_avih->width);
+  avih.height      = GUINT32_FROM_LE (_avih->height);
+  avih.scale       = GUINT32_FROM_LE (_avih->scale);
+  avih.rate        = GUINT32_FROM_LE (_avih->rate);
+  avih.start       = GUINT32_FROM_LE (_avih->start);
+  avih.length      = GUINT32_FROM_LE (_avih->length);
 
   /* debug stuff */
   GST_INFO ("avih tag found:");
-  GST_INFO (" us_frame    %u",     avih->us_frame);
-  GST_INFO (" max_bps     %u",     avih->max_bps);
-  GST_INFO (" pad_gran    %u",     avih->pad_gran);
-  GST_INFO (" flags       0x%08x", avih->flags);
-  GST_INFO (" tot_frames  %u",     avih->tot_frames);
-  GST_INFO (" init_frames %u",     avih->init_frames);
-  GST_INFO (" streams     %u",     avih->streams);
-  GST_INFO (" bufsize     %u",     avih->bufsize);
-  GST_INFO (" width       %u",     avih->width);
-  GST_INFO (" height      %u",     avih->height);
-  GST_INFO (" scale       %u",     avih->scale);
-  GST_INFO (" rate        %u",     avih->rate);
-  GST_INFO (" start       %u",     avih->start);
-  GST_INFO (" length      %u",     avih->length);
+  GST_INFO (" us_frame    %u",     avih.us_frame);
+  GST_INFO (" max_bps     %u",     avih.max_bps);
+  GST_INFO (" pad_gran    %u",     avih.pad_gran);
+  GST_INFO (" flags       0x%08x", avih.flags);
+  GST_INFO (" tot_frames  %u",     avih.tot_frames);
+  GST_INFO (" init_frames %u",     avih.init_frames);
+  GST_INFO (" streams     %u",     avih.streams);
+  GST_INFO (" bufsize     %u",     avih.bufsize);
+  GST_INFO (" width       %u",     avih.width);
+  GST_INFO (" height      %u",     avih.height);
+  GST_INFO (" scale       %u",     avih.scale);
+  GST_INFO (" rate        %u",     avih.rate);
+  GST_INFO (" start       %u",     avih.start);
+  GST_INFO (" length      %u",     avih.length);
 
-  avi->num_frames = avih->tot_frames;
-  avi->us_per_frame = avih->us_frame;
-  *streams = avih->streams;
-  *flags = avih->flags;
+  avi->num_frames = avih.tot_frames;
+  avi->us_per_frame = avih.us_frame;
+  *streams = avih.streams;
+  *flags = avih.flags;
 
   gst_buffer_unref (buf);
 
@@ -806,8 +802,8 @@ gst_avi_demux_add_stream (GstAviDemux *avi)
   if (!(tag = gst_riff_peek_tag (riff, NULL)))
     return FALSE;
   if (tag != GST_RIFF_TAG_strf) {
-    gst_element_error (GST_ELEMENT (avi),
-		       "Invalid AVI header (no strf as second tag)");
+    GST_ELEMENT_ERROR (avi, STREAM, DEMUX, (NULL),
+		       ("Invalid AVI header (no strf as second tag)"));
     goto skip_stream;
   }
   switch (strh->type) {
@@ -894,8 +890,6 @@ gst_avi_demux_add_stream (GstAviDemux *avi)
   /* set proper settings and add it */
   pad =  gst_pad_new_from_template (templ, padname);
   g_free (padname);
-  if (caps != NULL)
-    gst_pad_try_set_caps (pad, caps);
 
   gst_pad_set_formats_function (pad, gst_avi_demux_get_src_formats);
   gst_pad_set_event_mask_function (pad, gst_avi_demux_get_event_mask);
@@ -903,8 +897,10 @@ gst_avi_demux_add_stream (GstAviDemux *avi)
   gst_pad_set_query_type_function (pad, gst_avi_demux_get_src_query_types);
   gst_pad_set_query_function (pad, gst_avi_demux_handle_src_query);
   gst_pad_set_convert_function (pad, gst_avi_demux_src_convert);
+  gst_pad_set_getcaps_function (pad, gst_avi_demux_src_getcaps);
 
   stream = &avi->stream[avi->num_streams];
+  stream->caps = caps ? caps : gst_caps_new_empty ();
   stream->pad = pad;
   stream->strh = strh;
   stream->num = avi->num_streams;
@@ -918,6 +914,7 @@ gst_avi_demux_add_stream (GstAviDemux *avi)
   gst_pad_set_element_private (pad, stream);
   avi->num_streams++;
 
+  /* auto-negotiates */
   gst_element_add_pad (GST_ELEMENT (avi), pad);
 
   return TRUE;
@@ -961,7 +958,7 @@ gst_avi_demux_stream_odml (GstAviDemux *avi)
 
     switch (tag) {
       case GST_RIFF_TAG_dmlh: {
-        gst_riff_dmlh *dmlh;
+        gst_riff_dmlh dmlh, *_dmlh;
         GstBuffer *buf;
 
         if (!gst_riff_read_data (riff, &tag, &buf))
@@ -972,16 +969,13 @@ gst_avi_demux_stream_odml (GstAviDemux *avi)
           gst_buffer_unref (buf);
           break;
         }
-        dmlh = (gst_riff_dmlh *) GST_BUFFER_DATA (buf);
-
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-        dmlh->totalframes = GUINT32_FROM_LE (dmlh->totalframes);
-#endif
+        _dmlh = (gst_riff_dmlh *) GST_BUFFER_DATA (buf);
+        dmlh.totalframes = GUINT32_FROM_LE (_dmlh->totalframes);
 
         GST_INFO ("dmlh tag found:");
-        GST_INFO (" totalframes: %u", dmlh->totalframes);
+        GST_INFO (" totalframes: %u", dmlh.totalframes);
 
-        avi->num_frames = dmlh->totalframes;
+        avi->num_frames = dmlh.totalframes;
         gst_buffer_unref (buf);
         break;
       }
@@ -1055,20 +1049,20 @@ gst_avi_demux_stream_index (GstAviDemux *avi)
   GST_INFO ("%u index entries", avi->index_size);
 
   for (i = 0; i < avi->index_size; i++) {
-    gst_riff_index_entry *entry;
+    gst_riff_index_entry entry, *_entry;
     avi_stream_context *stream;
     gint stream_nr;
     gst_avi_index_entry *target;
     GstFormat format;
 
-    entry = &((gst_riff_index_entry *) GST_BUFFER_DATA (buf))[i];
-    entry->id     = GUINT32_FROM_LE (entry->id);
-    entry->offset = GUINT32_FROM_LE (entry->offset);
-    entry->flags  = GUINT32_FROM_LE (entry->flags);
-    entry->size   = GUINT32_FROM_LE (entry->size);
+    _entry = &((gst_riff_index_entry *) GST_BUFFER_DATA (buf))[i];
+    entry.id     = GUINT32_FROM_LE (_entry->id);
+    entry.offset = GUINT32_FROM_LE (_entry->offset);
+    entry.flags  = GUINT32_FROM_LE (_entry->flags);
+    entry.size   = GUINT32_FROM_LE (_entry->size);
     target = &avi->index_entries[i];
 
-    stream_nr = CHUNKID_TO_STREAMNR (entry->id);
+    stream_nr = CHUNKID_TO_STREAMNR (entry.id);
     if (stream_nr >= avi->num_streams || stream_nr < 0) {
       g_warning ("Index entry %d has invalid stream nr %d",
 		 i, stream_nr);
@@ -1079,9 +1073,9 @@ gst_avi_demux_stream_index (GstAviDemux *avi)
     stream = &avi->stream[stream_nr];
 
     target->index_nr = i;
-    target->flags    = entry->flags;
-    target->size     = entry->size;
-    target->offset   = entry->offset;
+    target->flags    = entry.flags;
+    target->size     = entry.size;
+    target->offset   = entry.offset;
 
     /* figure out if the index is 0 based or relative to the MOVI start */
     if (i == 0) {
@@ -1163,17 +1157,17 @@ gst_avi_demux_stream_header (GstAviDemux *avi)
   if (!(tag = gst_riff_peek_tag (riff, NULL)))
     return FALSE;
   if (tag != GST_RIFF_TAG_LIST) {
-    gst_element_error (GST_ELEMENT (avi),
-		       "Invalid AVI header (no LIST at start): "
-		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag));
+    GST_ELEMENT_ERROR (avi, STREAM, DEMUX, (NULL),
+		       ("Invalid AVI header (no LIST at start): "
+		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag)));
     return FALSE;
   }
   if (!gst_riff_read_list (riff, &tag))
     return FALSE;
   if (tag != GST_RIFF_LIST_hdrl) {
-    gst_element_error (GST_ELEMENT (avi),
-		       "Invalid AVI header (no hdrl at start): "
-		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag));
+    GST_ELEMENT_ERROR (avi, STREAM, DEMUX, (NULL),
+		       ("Invalid AVI header (no hdrl at start): "
+		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag)));
     return FALSE;
   }
 
@@ -1181,9 +1175,9 @@ gst_avi_demux_stream_header (GstAviDemux *avi)
   if (!(tag = gst_riff_peek_tag (riff, NULL)))
     return FALSE;
   if (tag != GST_RIFF_TAG_avih) {
-    gst_element_error (GST_ELEMENT (avi),
-		       "Invalid AVI header (no avih at start): "
-		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag));
+    GST_ELEMENT_ERROR (avi, STREAM, DEMUX, (NULL),
+		       ("Invalid AVI header (no avih at start): "
+		       GST_FOURCC_FORMAT, GST_FOURCC_ARGS (tag)));
     return FALSE;
   }
   if (!gst_avi_demux_stream_avih (avi, &flags, &streams))
@@ -1216,12 +1210,6 @@ gst_avi_demux_stream_header (GstAviDemux *avi)
               return FALSE;
             break;
 
-          case GST_RIFF_LIST_INFO:
-            if (!gst_riff_read_list (riff, &tag) ||
-                !gst_riff_read_info (riff))
-              return FALSE;
-            break;
-
           default:
             GST_WARNING ("Unknown list " GST_FOURCC_FORMAT " in AVI header",
 			 GST_FOURCC_ARGS (tag));
@@ -1272,8 +1260,13 @@ gst_avi_demux_stream_header (GstAviDemux *avi)
     if (!(tag = gst_riff_peek_list (riff)))
       return FALSE;
     if (tag != GST_RIFF_LIST_movi) {
-      if (!gst_riff_read_skip (riff))
+      if (tag == GST_RIFF_LIST_INFO) {
+        if (!gst_riff_read_list (riff, &tag) ||
+            !gst_riff_read_info (riff))
+         return FALSE;
+      } else  if (!gst_riff_read_skip (riff)) {
         return FALSE;
+      }
       continue;
     }
     break;
