@@ -23,20 +23,18 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <gst/floatcast/floatcast.h>
 
-#include <gstfloat2int.h>
+#include "gstfloat2int.h"
 #include <gst/audio/audio.h>
 
 /* elementfactory information */
 static GstElementDetails float2int_details = {
   "Float to Integer effect",
-  "Filter/Audio/Conversion",
-  "LGPL",
+  "Filter/Converter/Audio",
   "Convert from floating point to integer audio data",
-  VERSION,
-  "Steve Baker <stevebaker_org@yahoo.co.uk>, Andy Wingo <wingo at pobox dot com>",
-  "(C) 2001, 2002, 2003",
+  "Steve Baker <stevebaker_org@yahoo.co.uk>, Andy Wingo <wingo at pobox dot com>"
 };
 
 
@@ -48,43 +46,38 @@ static GstElementDetails float2int_details = {
 #endif
 
 
-GST_PAD_TEMPLATE_FACTORY (float2int_sink_factory,
+static GstStaticPadTemplate float2int_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  GST_CAPS_NEW (
-    "float2int_sink",
-    "audio/x-raw-float",
-    "rate",		GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "width",		GST_PROPS_INT (32),
-    "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
-    "buffer-frames",	GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "channels",		GST_PROPS_INT (1)
-  )
+  GST_STATIC_CAPS (GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS)
 );
 
-GST_PAD_TEMPLATE_FACTORY (float2int_src_factory,
+static GstStaticPadTemplate float2int_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "float2int_src",
-    "audio/x-raw-int",
-    "channels",   GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-    "width",      GST_PROPS_INT (16),
-    "depth",      GST_PROPS_INT (16),
-    "signed",     GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS ( "audio/x-raw-int, "
+    "rate = (int) [ 1, MAX ], "
+    "channels = (int) [ 1, MAX ], "
+    "endianness = (int) BYTE_ORDER, "
+    "width = (int) 16, "
+    "depth = (int) 16, "
+    "signed = (boolean) true, "
+    "buffer-frames = (int) [ 1, MAX]"
   )
 );
 
-static GstPadTemplate *srctempl, *sinktempl;
-
 static void	gst_float2int_class_init		(GstFloat2IntClass *klass);
+static void	gst_float2int_base_init			(GstFloat2IntClass *klass);
 static void	gst_float2int_init			(GstFloat2Int *plugin);
 
-static GstPadLinkReturn gst_float2int_link		(GstPad *pad, GstCaps *caps);
+static GstCaps * gst_float2int_sink_getcaps (GstPad *pad);
+static GstCaps * gst_float2int_src_getcaps (GstPad *pad);
+static GstPadLinkReturn gst_float2int_src_link (GstPad *pad, const GstCaps *caps);
+static GstPadLinkReturn gst_float2int_sink_link (GstPad *pad, const GstCaps *caps);
 
 static GstPad*  gst_float2int_request_new_pad		(GstElement *element, GstPadTemplate *temp,
                                                          const gchar *unused);
@@ -102,7 +95,8 @@ gst_float2int_get_type(void) {
 
   if (!float2int_type) {
     static const GTypeInfo float2int_info = {
-      sizeof(GstFloat2IntClass),      NULL,
+      sizeof(GstFloat2IntClass),
+      (GBaseInitFunc)gst_float2int_base_init,
       NULL,
       (GClassInitFunc)gst_float2int_class_init,
       NULL,
@@ -116,6 +110,18 @@ gst_float2int_get_type(void) {
   return float2int_type;
 }
 
+static void
+gst_float2int_base_init (GstFloat2IntClass *klass)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_set_details (gstelement_class, &float2int_details);
+  gst_element_class_add_pad_template (gstelement_class,
+	gst_static_pad_template_get (&float2int_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+	gst_static_pad_template_get (&float2int_sink_factory));
+}
+
 static void
 gst_float2int_class_init (GstFloat2IntClass *klass)
 {
@@ -137,153 +143,256 @@ gst_float2int_init (GstFloat2Int *plugin)
 {
   gst_element_set_loop_function (GST_ELEMENT (plugin), gst_float2int_loop);
 
-  plugin->srcpad = gst_pad_new_from_template (srctempl, "src");
+  plugin->srcpad = gst_pad_new_from_template (
+			gst_static_pad_template_get (&float2int_src_factory), "src");
   gst_element_add_pad (GST_ELEMENT (plugin), plugin->srcpad);
-  gst_pad_set_link_function (plugin->srcpad, gst_float2int_link);
+  gst_pad_set_link_function (plugin->srcpad, gst_float2int_src_link);
+  gst_pad_set_getcaps_function (plugin->srcpad, gst_float2int_src_getcaps);
 
   plugin->numchannels = 0;
   plugin->channelcount = 0;
   plugin->channels = NULL;
 }
 
-static GstBufferPool*
-gst_float2int_get_bufferpool (GstPad *pad)
-{
-  gint frames;
-  GstFloat2Int *filter = (GstFloat2Int *) gst_pad_get_parent (pad);
 
-  frames = filter->buffer_frames;
-  if (!frames)
-    frames = GST_FLOAT2INT_DEFAULT_BUFFER_FRAMES;
-  
-  return gst_buffer_pool_get_default (frames * sizeof (float), 5);
+static GstCaps *
+gst_float2int_get_allowed_sinkcaps (GstFloat2Int *this, GstPad *ignorepad)
+{
+  GSList *channels;
+  GstCaps *caps;
+  GstCaps *allowed_caps;
+  GstCaps *icaps;
+  GstPad *sinkpad;
+  GstFloat2IntInputChannel *channel;
+
+  GST_DEBUG ("here");
+
+  channels = this->channels;
+  caps = gst_caps_from_string (GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS);
+  while (channels) {
+    channel = GST_FLOAT2INT_CHANNEL (channels);
+    sinkpad = channel->sinkpad;
+    channels = g_slist_next (channels);
+    if (sinkpad == ignorepad) continue;
+
+    allowed_caps = gst_pad_get_allowed_caps(sinkpad);
+    icaps = gst_caps_intersect (caps, allowed_caps);
+    gst_caps_free (allowed_caps);
+    gst_caps_free (caps);
+    caps = icaps;
+
+    channels = g_slist_next (channels);
+  }
+
+  GST_DEBUG ("allowed sinkcaps %s", gst_caps_to_string (caps));
+
+  return caps;
 }
 
-/* this is a very brittle function. it's hard to come up with a "correct
-   behavior". what I (wingo) have chosen is as follows:
-   * if the rate of the float2int plugin has already been set by a previous connection
-     * if the new rate is different from the old rate, bail
-     * otherwise return 'ok' (don't set other caps, no need -- only the rate needs
-       setting, and it's the same)
-   * otherwise set all of the caps (except for on the connecting pad, of course)
-     * bail if that fails, possibly leaving the plugin in an inconsistent state
-
-   FIXME: we really should do something about the inconsistent state case,
-   possibly unlinking the offending pad. the assumption with this element,
-   though, is that the pipelines on the float side will be parallel (more or
-   less the same) and so we should never run into that case. that might be a
-   programmer error more than anything.
- */
-static GstPadLinkReturn
-gst_float2int_link (GstPad *pad, GstCaps *caps)
+static void
+gst_float2int_caps_remove_format (GstCaps *caps, const char *name)
 {
-  GstFloat2Int *filter;
-  GstCaps *intcaps, *floatcaps;
-  GSList *l;
-  gint rate, channels, buffer_frames;
-  
-  filter = GST_FLOAT2INT (GST_PAD_PARENT (pad));
-  
-  /* we have three variables to play with:
-     "rate": both sides
-     "buffer-frames": float side
-     "channels": int side. */
-  if (GST_CAPS_IS_FIXED (caps)) {
-    if (pad == filter->srcpad) {
-      /* it's the int pad */
-      if (!filter->intcaps) {
-        gst_caps_get_int (caps, "channels", &channels);
+  int i;
+  GstStructure *structure;
 
-        if (channels != filter->numchannels) {
-          GST_DEBUG ("Tried to set caps with %d channels but there are %d source pads",
-                     channels, filter->numchannels);
-          return GST_PAD_LINK_REFUSED;
-        }
-        
-        if (!filter->buffer_frames)
-          filter->buffer_frames = GST_FLOAT2INT_DEFAULT_BUFFER_FRAMES;
-        
-        floatcaps = gst_caps_copy (gst_pad_template_get_caps (float2int_sink_factory ()));
-        gst_caps_get_int (caps, "rate", &filter->rate);
-        gst_caps_set (floatcaps, "rate", GST_PROPS_INT (filter->rate), NULL);
-        gst_caps_set (floatcaps, "buffer-frames", GST_PROPS_INT (filter->buffer_frames), NULL);
-        
-        for (l=filter->channels; l; l=l->next)
-          if (gst_pad_try_set_caps (GST_FLOAT2INT_CHANNEL (l)->sinkpad, floatcaps) <= 0)
-            return GST_PAD_LINK_REFUSED;
-        
-        filter->floatcaps = gst_caps_ref (floatcaps);
-        filter->intcaps = gst_caps_ref (caps);
-        return GST_PAD_LINK_OK;
-      } else if (gst_caps_intersect (caps, filter->intcaps)) {
-        /* they have to be the same since both are fixed */
-        return GST_PAD_LINK_OK;
-      } else {
-        GST_DEBUG ("Tried to connect the source of float2int, but with bad caps");
-        gst_caps_debug (filter->intcaps, "float2int internal int source pad caps");
-        return GST_PAD_LINK_REFUSED;
-      }
-    } else {
-      /* it's a float pad */
-      if (!filter->intcaps) {
-        intcaps = gst_caps_copy (gst_pad_template_get_caps (float2int_src_factory ()));
-        gst_caps_get_int (caps, "rate", &rate);
-        gst_caps_set (intcaps, "rate", GST_PROPS_INT (rate));
-        gst_caps_set (intcaps, "channels", GST_PROPS_INT (filter->numchannels));
-        gst_caps_debug (intcaps, "int source pad caps going into try_set_caps()");
-        
-        if (gst_pad_try_set_caps (filter->srcpad, intcaps) <= 0) {
-          return GST_PAD_LINK_REFUSED;
-	}
+  GST_DEBUG ("here");
 
-        if (!gst_caps_get_int (caps, "buffer-frames", &buffer_frames)) {
-          g_warning ("You have an old float plugin that needs to be converted "
-                     "to the new caps format. Please file a bug!");
-          return GST_PAD_LINK_REFUSED;
-        }
-        if (filter->buffer_frames && filter->buffer_frames != buffer_frames) {
-          GST_DEBUG ("Tried to set buffer_frames=%d but it was already set to %d",
-                     buffer_frames, filter->buffer_frames);
-          return GST_PAD_LINK_REFUSED;
-        }
-
-        filter->floatcaps = gst_caps_ref (caps);
-        filter->intcaps = gst_caps_ref (intcaps);
-        filter->rate = rate;
-        filter->buffer_frames = buffer_frames;
-
-        for (l=filter->channels; l; l=l->next) {
-	  if (GST_FLOAT2INT_CHANNEL (l)->sinkpad == pad) {
-	    continue;
-	  }
-	  
-          if (gst_pad_try_set_caps (GST_FLOAT2INT_CHANNEL (l)->sinkpad, caps) == FALSE) {
-	    /* We don't have the caps set, so remove them */
-            if (filter->floatcaps) {
-              gst_caps_unref (filter->floatcaps);
-              filter->floatcaps = NULL;
-            }
-            if (filter->intcaps) {
-              gst_caps_unref (filter->intcaps);
-              filter->intcaps = NULL;
-            }
-            return GST_PAD_LINK_REFUSED;
-	  }
-	}
-        	
-        return GST_PAD_LINK_OK;
-      } else if (gst_caps_intersect (caps, filter->floatcaps)) {
-        /* they have to be the same since both are fixed */
-        return GST_PAD_LINK_OK;
-      } else {
-        return GST_PAD_LINK_REFUSED;
-      }
-    }
-  } else {
-    return GST_PAD_LINK_DELAYED;
+  for(i=0;i<gst_caps_get_size(caps);i++){
+    structure = gst_caps_get_structure(caps, i);
+    gst_structure_set_name (structure, name);
+    gst_structure_remove_field (structure, "width");
+    gst_structure_remove_field (structure, "channels");
+    gst_structure_remove_field (structure, "endianness");
+    gst_structure_remove_field (structure, "depth");
+    gst_structure_remove_field (structure, "signed");
+    //gst_structure_remove_field (structure, "buffer-frames");
   }
 }
 
+static GstCaps *
+gst_float2int_src_getcaps (GstPad *pad)
+{
+  GstFloat2Int *this;
+  GstCaps *caps;
+  GstCaps *icaps;
+  int i;
+  GstStructure *structure;
+
+  GST_DEBUG ("here");
+
+  this = GST_FLOAT2INT (gst_pad_get_parent (pad));
+
+  caps = gst_float2int_get_allowed_sinkcaps (this, NULL);
+  gst_float2int_caps_remove_format (caps, "audio/x-raw-int");
+
+  GST_DEBUG ("no format %s", gst_caps_to_string (caps));
+
+  if (this->numchannels > 0) {
+    for(i=0;i<gst_caps_get_size(caps);i++){
+      structure = gst_caps_get_structure(caps, i);
+      gst_structure_set (structure, "channels", G_TYPE_INT,
+          this->numchannels, NULL);
+    }
+  }
+
+  GST_DEBUG ("channels %s", gst_caps_to_string (caps));
+
+  GST_DEBUG ("pad template %s", gst_caps_to_string (gst_pad_get_pad_template_caps(pad)));
+
+  icaps = gst_caps_intersect (caps, gst_pad_get_pad_template_caps(pad));
+  gst_caps_free (caps);
+
+  return icaps;
+}
+
+static GstCaps *
+gst_float2int_sink_getcaps (GstPad *pad)
+{
+  GstFloat2Int *this;
+  GstCaps *sinkcaps;
+  GstCaps *srccaps;
+  GstCaps *icaps;
+  GstCaps *caps;
+  int i;
+  GstStructure *structure;
+
+  this = GST_FLOAT2INT (gst_pad_get_parent (pad));
+
+  GST_DEBUG ("here");
+
+  sinkcaps = gst_float2int_get_allowed_sinkcaps (this, pad);
+  gst_float2int_caps_remove_format (sinkcaps, "audio/x-raw-float");
+
+  srccaps = gst_pad_get_allowed_caps (this->srcpad);
+  gst_float2int_caps_remove_format (srccaps, "audio/x-raw-float");
+
+  icaps = gst_caps_intersect (srccaps, sinkcaps);
+  gst_caps_free (sinkcaps);
+  gst_caps_free (srccaps);
+
+  for(i=0;i<gst_caps_get_size(icaps);i++){
+    structure = gst_caps_get_structure(icaps, i);
+    gst_structure_set (structure, "channels", G_TYPE_INT, 1, NULL);
+  }
+
+  caps = gst_caps_intersect (icaps, gst_pad_get_pad_template_caps(pad));
+  gst_caps_free (icaps);
+
+  return caps;
+}
+
+static GstPadLinkReturn
+gst_float2int_src_link (GstPad *pad, const GstCaps *caps)
+{
+  GstFloat2Int *this;
+  GstStructure *structure;
+  GstCaps *fcaps;
+  GstPad *sinkpad;
+  int rate;
+  int n_channels;
+  int buffer_frames;
+  GSList *channels;
+  GstPadLinkReturn ret;
+  GstFloat2IntInputChannel *channel;
+
+  GST_DEBUG ("here");
+
+  this = GST_FLOAT2INT (GST_PAD_PARENT (pad));
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int (structure, "rate", &rate);
+  gst_structure_get_int (structure, "channels", &n_channels);
+  gst_structure_get_int (structure, "buffer-frames", &buffer_frames);
+
+  fcaps = gst_caps_from_string (GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS);
+  gst_caps_set_simple (fcaps,
+      "rate", G_TYPE_INT, rate,
+      "buffer-frames", G_TYPE_INT, buffer_frames,
+      NULL);
+
+  channels = this->channels;
+  while (channels) {
+    channel = GST_FLOAT2INT_CHANNEL (channels);
+    sinkpad = channel->sinkpad;
+    channels = g_slist_next (channels);
+    if (sinkpad == pad) continue;
+
+    ret = gst_pad_try_set_caps (sinkpad, fcaps);
+    if(GST_PAD_LINK_FAILED (ret)) {
+      /* FIXME we should revert the other changes */
+      return ret;
+    }
+
+  }
+
+  /* ok, it worked */
+
+  this->rate = rate;
+  //this->n_channels = n_channels;
+  this->buffer_frames = buffer_frames;
+
+  return GST_PAD_LINK_OK;
+}
+
+static GstPadLinkReturn
+gst_float2int_sink_link (GstPad *pad, const GstCaps *caps)
+{
+  GstFloat2Int *this;
+  GstStructure *structure;
+  GstCaps *icaps;
+  GstPad *sinkpad;
+  int rate;
+  int n_channels;
+  int buffer_frames;
+  GSList *channels;
+  GstPadLinkReturn ret;
+  GstFloat2IntInputChannel *channel;
+
+  GST_DEBUG ("here");
+
+  this = GST_FLOAT2INT (GST_PAD_PARENT (pad));
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int (structure, "rate", &rate);
+  gst_structure_get_int (structure, "channels", &n_channels);
+  gst_structure_get_int (structure, "buffer-frames", &buffer_frames);
+
+  icaps = gst_caps_copy (gst_pad_get_pad_template_caps (this->srcpad));
+  gst_caps_set_simple (icaps,
+      "rate", G_TYPE_INT, rate,
+      "buffer-frames", G_TYPE_INT, buffer_frames,
+      "channels", G_TYPE_INT, this->numchannels,
+      NULL);
+  ret = gst_pad_try_set_caps (this->srcpad, icaps);
+  if(GST_PAD_LINK_FAILED (ret)) {
+    return ret;
+  }
+
+  channels = this->channels;
+  while (channels) {
+    channel = GST_FLOAT2INT_CHANNEL (channels);
+    sinkpad = channel->sinkpad;
+    channels = g_slist_next (channels);
+    if (sinkpad == pad) continue;
+
+    ret = gst_pad_try_set_caps (sinkpad, caps);
+    if(GST_PAD_LINK_FAILED (ret)) {
+      /* FIXME we should revert the other changes */
+      return ret;
+    }
+  }
+
+  /* ok, it worked */
+
+  this->rate = rate;
+  this->buffer_frames = buffer_frames;
+
+  return GST_PAD_LINK_OK;
+}
+
+
 static GstPad*
 gst_float2int_request_new_pad (GstElement *element, GstPadTemplate *templ, const gchar *unused) 
 {
@@ -305,20 +414,13 @@ gst_float2int_request_new_pad (GstElement *element, GstPadTemplate *templ, const
   name = g_strdup_printf ("sink%d", plugin->channelcount);
   channel->sinkpad = gst_pad_new_from_template (templ, name);
   gst_element_add_pad (GST_ELEMENT (plugin), channel->sinkpad);
-  gst_pad_set_link_function (channel->sinkpad, gst_float2int_link);
-  gst_pad_set_bufferpool_function (channel->sinkpad, gst_float2int_get_bufferpool);
+  gst_pad_set_link_function (channel->sinkpad, gst_float2int_sink_link);
+  gst_pad_set_getcaps_function (channel->sinkpad, gst_float2int_sink_getcaps);
   
   plugin->channels = g_slist_append (plugin->channels, channel);
   plugin->numchannels++;
   plugin->channelcount++;
   
-  if (plugin->intcaps) {
-    gst_caps_set (plugin->intcaps, "channels", GST_PROPS_INT (plugin->numchannels));
-    if (gst_pad_try_set_caps (plugin->srcpad, plugin->intcaps) <= 0) {
-      g_warning ("float2int: added a pad, but failed to update src caps");
-    }
-  }
-
   GST_DEBUG ("float2int added pad %s\n", name);
 
   g_free (name);
@@ -373,18 +475,13 @@ gst_float2int_loop (GstElement *element)
   gint numchannels = plugin->numchannels;
     
   if (!plugin->channels) {
-    gst_element_error (element, "float2int: at least one sink pad needs to be connected");
+    GST_ELEMENT_ERROR (element, CORE, PAD, (NULL), ("float2int: at least one sink pad needs to be connected"));
     return;
   }
 
-  if (!plugin->pool)
-    if (! (plugin->pool = gst_pad_get_bufferpool (plugin->srcpad)))
-      plugin->pool = gst_buffer_pool_get_default (sizeof(gint16) * 1024 * numchannels, 1);
-
-  g_assert (plugin->pool);
-
   /* get new output buffer */
-  buf_out = gst_buffer_new_from_pool (plugin->pool, 0, 0);
+  /* FIXME the 1024 is arbitrary */
+  buf_out = gst_buffer_new_and_alloc (1024);
   num_frames = GST_BUFFER_SIZE (buf_out) / numchannels / sizeof (gint16);
   g_return_if_fail (GST_BUFFER_SIZE (buf_out) == num_frames * numchannels * sizeof (gint16));
   
@@ -503,14 +600,6 @@ gst_float2int_change_state (GstElement *element)
       break;
       
     case GST_STATE_PAUSED_TO_READY:
-      if (plugin->floatcaps) {
-        gst_caps_unref (plugin->floatcaps);
-        plugin->floatcaps = NULL;
-      }
-      if (plugin->intcaps) {
-        gst_caps_unref (plugin->intcaps);
-        plugin->intcaps = NULL;
-      }
       for (p = plugin->channels; p; p = p->next) {
 	GstFloat2IntInputChannel *c = p->data;
 
@@ -531,24 +620,3 @@ gst_float2int_change_state (GstElement *element)
 
   return GST_STATE_SUCCESS;
 }
-
-gboolean 
-gst_float2int_factory_init (GstPlugin *plugin) 
-{
-  GstElementFactory *factory;
-  
-  factory = gst_element_factory_new ("float2int", GST_TYPE_FLOAT2INT,
-                                     &float2int_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_SECONDARY);
-  
-  sinktempl = float2int_sink_factory();
-  gst_element_factory_add_pad_template (factory, sinktempl);
-
-  srctempl = float2int_src_factory();
-  gst_element_factory_add_pad_template (factory, srctempl);
-  
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-   
-  return TRUE;
-}
