@@ -25,9 +25,9 @@
 #define __GST_FLOAT2INT_H__
 
 
-#include <config.h>
 #include <gst/gst.h>
 
+G_BEGIN_DECLS
 
 #define GST_TYPE_FLOAT2INT \
   (gst_float2int_get_type())
@@ -57,13 +57,10 @@ struct _GstFloat2Int {
 
   GstPad *srcpad;
   GSList *channels;
-  GstBufferPool *pool;
   
   gint numchannels; /* Number of pads on the element */
   gint channelcount; /* counter to get safest pad name */
   
-  GstCaps *intcaps;
-  GstCaps *floatcaps;
   gint rate;
   gint buffer_frames;
   gint64 offset;
@@ -77,5 +74,6 @@ struct _GstFloat2IntClass {
 GType gst_float2int_get_type(void);
 gboolean gst_float2int_factory_init (GstPlugin *plugin);
 
+G_END_DECLS
 
 #endif /* __GST_FLOAT2INT_H__ */
