@@ -63,6 +63,10 @@ tonal.c
  * 8/05/93 Masahiro Iwadare     noise_label modification "option"     *
  **********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "common.h"
 #include "encoder.h"
 #define LONDON                  /* enable "LONDON" modification */
@@ -192,9 +196,9 @@ mask FAR power[HAN_SIZE];     /* domain                      */
     power[i].next = STOP;
     power[i].type = FALSE;
  }
- mpegaudio_mem_free((void **) &x_r);
- mpegaudio_mem_free((void **) &x_i);
- mpegaudio_mem_free((void **) &energy);
+ mpegaudio_mem_free(x_r);
+ mpegaudio_mem_free(x_i);
+ mpegaudio_mem_free(energy);
 }
 
 /**************************************************************** 
@@ -629,8 +633,8 @@ frame_params *fr_ps;
     mpegaudio_II_minimum_mask(ltg, &ltmin[k][0], sblimit);
     mpegaudio_II_smr(&ltmin[k][0], &spike[k][0], &scale[k][0], sblimit);        
  }
- mpegaudio_mem_free((void **) &sample);
- mpegaudio_mem_free((void **) &spike);
+ mpegaudio_mem_free(sample);
+ mpegaudio_mem_free(spike);
 }
 
 /********************************************************************** 
@@ -728,9 +732,9 @@ mask FAR power[HAN_SIZE/2];     /* freq. domain             */
        power[i].next = STOP;
        power[i].type = FALSE;
  }
- mpegaudio_mem_free((void **) &x_r);
- mpegaudio_mem_free((void **) &x_i);
- mpegaudio_mem_free((void **) &energy);
+ mpegaudio_mem_free(x_r);
+ mpegaudio_mem_free(x_i);
+ mpegaudio_mem_free(energy);
 }
 
 /**************************************************************** 
@@ -970,6 +974,6 @@ frame_params *fr_ps;
     mpegaudio_I_minimum_mask(ltg, &ltmin[k][0]);
     mpegaudio_I_smr(&ltmin[k][0], &spike[k][0], &scale[k][0]);        
  }
- mpegaudio_mem_free((void **) &sample);
- mpegaudio_mem_free((void **) &spike);
+ mpegaudio_mem_free(sample);
+ mpegaudio_mem_free(spike);
 }
