@@ -22,7 +22,6 @@
 #define __MPEG_DEMUX_H__
 
 
-#include <config.h>
 #include <gst/gst.h>
 #include "gstmpegparse.h"
 
@@ -76,17 +75,24 @@ struct _GstMPEGDemux {
   gint64	 total_size_bound;
 
 #define NUM_PRIVATE_1_STREAMS 	 8
+#define NUM_PCM_STREAMS 	 8
 #define NUM_SUBTITLE_STREAMS 	16
 #define NUM_VIDEO_STREAMS 	16
 #define NUM_AUDIO_STREAMS 	32
 
   /* stream output */
   GstMPEGStream *private_1_stream[NUM_PRIVATE_1_STREAMS];	/* up to 8 ac3 audio tracks */
+  GstMPEGStream *pcm_stream[NUM_PCM_STREAMS];
   GstMPEGStream *subtitle_stream[NUM_SUBTITLE_STREAMS];
   GstMPEGStream *private_2_stream;
   GstMPEGStream *video_stream[NUM_VIDEO_STREAMS];
   GstMPEGStream *audio_stream[NUM_AUDIO_STREAMS];
 
+  /* The type of linear PCM samples associated to each channel. The
+     values are bit fields with the same format of the sample_info
+     field in the linear PCM header. */
+  guint8	 lpcm_sample_info[NUM_PCM_STREAMS];		
+
   GstIndex	*index;
 };
 
@@ -96,7 +102,7 @@ struct _GstMPEGDemuxClass {
 
 GType gst_mpeg_demux_get_type(void);
 
-gboolean 	gst_mpeg_demux_plugin_init 	(GModule *module, GstPlugin *plugin);
+gboolean gst_mpeg_demux_plugin_init 	(GstPlugin *plugin);
 
 #ifdef __cplusplus
 }
