@@ -26,7 +26,7 @@
 #include <string.h>
 #include <zlib.h>
 
-#define g_print(x...)
+#define g_print(...)
 
 #define QTDEMUX_GUINT32_GET(a) GUINT32_FROM_BE(*(guint32 *)(a))
 #define QTDEMUX_GUINT16_GET(a) GUINT16_FROM_BE(*(guint16 *)(a))
@@ -77,8 +77,11 @@ struct _QtDemuxStream {
   int width;
   int height;
   float fps;
+  
   double rate;
   int n_channels;
+  guint bytes_per_frame;
+  guint samples_per_packet;
 };
 
 enum QtDemuxState {
@@ -111,23 +114,30 @@ enum {
   ARG_0
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_templ,
+static GstStaticPadTemplate gst_qtdemux_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "qtdemux_sink",
-    "video/quicktime",
-      NULL
-  ),
-  GST_CAPS_NEW (
-    "qtdemux_sink",
-    "audio/x-m4a",
-    NULL
-  )
-)
+  GST_PAD_SOMETIMES,
+  GST_STATIC_CAPS ("video/quicktime")
+);
+
+static GstStaticPadTemplate gst_qtdemux_videosrc_template =
+GST_STATIC_PAD_TEMPLATE (
+  "audio_%02d",
+  GST_PAD_SRC,
+  GST_PAD_SOMETIMES,
+  GST_STATIC_CAPS_ANY
+);
+
+static GstStaticPadTemplate gst_qtdemux_audiosrc_template =
+GST_STATIC_PAD_TEMPLATE (
+  "video_%02d",
+  GST_PAD_SRC,
+  GST_PAD_SOMETIMES,
+  GST_STATIC_CAPS_ANY
+);
 
-static GstPadTemplate *videosrctempl, *audiosrctempl;
 static GstElementClass *parent_class = NULL;
 
 static void gst_qtdemux_class_init (GstQTDemuxClass *klass);
@@ -142,7 +152,7 @@ static void qtdemux_parse(GstQTDemux *qtdemux, GNode *node, void *buffer, int le
 static QtNodeType *qtdemux_type_get(guint32 fourcc);
 static void qtdemux_node_dump(GstQTDemux *qtdemux, GNode *node);
 static void qtdemux_parse_tree(GstQTDemux *qtdemux);
-static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc);
+static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc, const guint8 *stsd_data);
 static GstCaps *qtdemux_audio_caps(GstQTDemux *qtdemux, guint32 fourcc);
 
 static GType gst_qtdemux_get_type (void) 
@@ -167,10 +177,13 @@ static void gst_qtdemux_base_init (GstQTDemuxClass *klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
-		GST_PAD_TEMPLATE_GET (sink_templ));
-  gst_element_class_add_pad_template (element_class, videosrctempl);
-  gst_element_class_add_pad_template (element_class, audiosrctempl);
+      gst_static_pad_template_get (&gst_qtdemux_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_qtdemux_videosrc_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_qtdemux_audiosrc_template));
   gst_element_class_set_details (element_class, &gst_qtdemux_details);
+
 }
 
 static void gst_qtdemux_class_init (GstQTDemuxClass *klass) 
@@ -189,14 +202,17 @@ static void gst_qtdemux_class_init (GstQTDemuxClass *klass)
 static void 
 gst_qtdemux_init (GstQTDemux *qtdemux) 
 {
-  qtdemux->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_templ), "sink");
-  gst_element_set_loop_function (GST_ELEMENT (qtdemux), gst_qtdemux_loop_header);
+  qtdemux->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&gst_qtdemux_sink_template), "sink");
+  gst_element_set_loop_function (GST_ELEMENT (qtdemux),
+      gst_qtdemux_loop_header);
   gst_element_add_pad (GST_ELEMENT (qtdemux), qtdemux->sinkpad);
 }
 
 static gboolean
 plugin_init (GstPlugin *plugin)
 {
+#if 0
   GstCaps *audiocaps = NULL, *videocaps = NULL, *temp;
   const guint32 audio_fcc[] = {
     /* FILLME */
@@ -227,6 +243,13 @@ plugin_init (GstPlugin *plugin)
 					GST_PAD_SRC,
 					GST_PAD_SOMETIMES,
 					videocaps, NULL);
+#endif
+
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
+
+  if (!gst_library_load ("gstgetbits"))
+    return FALSE;
 
   return gst_element_register (plugin, "qtdemux",
 			       GST_RANK_PRIMARY, GST_TYPE_QTDEMUX);
@@ -460,6 +483,7 @@ static void gst_qtdemux_loop_header (GstElement *element)
     cur_offset = gst_bytestream_tell(qtdemux->bs);
     if(offset != cur_offset){
       GST_DEBUG ("seeking to offset %d",offset);
+      g_print ("seeking to offset %d\n",offset);
       ret = gst_bytestream_seek(qtdemux->bs, offset, GST_SEEK_METHOD_SET);
       GST_DEBUG ("seek returned %d",ret);
       return;
@@ -484,11 +508,10 @@ static void gst_qtdemux_loop_header (GstElement *element)
       if(stream->subtype == GST_MAKE_FOURCC('v','i','d','e')){
         float fps = 1. * GST_SECOND / stream->samples[stream->sample_index].duration;
         if (fps != stream->fps) {
-          gst_props_remove_entry_by_name(stream->caps->properties, "framerate");
-          gst_props_add_entry(stream->caps->properties,
-                              gst_props_entry_new("framerate", GST_PROPS_FLOAT(fps)));
+	  gst_caps_set_simple (stream->caps, "framerate", G_TYPE_DOUBLE, fps,
+	      NULL);
           stream->fps = fps;
-          gst_pad_try_set_caps(stream->pad, stream->caps);
+          gst_pad_set_explicit_caps(stream->pad, stream->caps);
         }
       }
 
@@ -506,108 +529,43 @@ static void gst_qtdemux_loop_header (GstElement *element)
 
 }
 
-static GstCaps *gst_qtdemux_src_getcaps(GstPad *pad, GstCaps *caps)
-{
-  GstQTDemux *qtdemux;
-  QtDemuxStream *stream;
-  int i;
-
-  GST_DEBUG ("gst_qtdemux_src_getcaps");
-
-  qtdemux = GST_QTDEMUX(gst_pad_get_parent(pad));
-
-  g_return_val_if_fail(GST_IS_QTDEMUX(qtdemux), NULL);
-
-  GST_DEBUG ("looking for pad %p in qtdemux %p", pad, qtdemux);
-  GST_DEBUG ("n_streams is %d", qtdemux->n_streams);
-  for(i=0;i<qtdemux->n_streams;i++){
-    stream = qtdemux->streams[i];
-    if(stream->pad == pad){
-      return stream->caps;
-    }
-  }
-
-  GST_DEBUG ("Couldn't find stream cooresponding to pad\n");
-
-  return NULL;
-}
-
-static GstPadLinkReturn
-gst_qtdemux_src_link(GstPad *pad, GstCaps *caps)
-{
-  GstQTDemux *qtdemux;
-  QtDemuxStream *stream;
-  int i;
-
-  GST_DEBUG ("gst_qtdemux_src_link");
-
-  qtdemux = GST_QTDEMUX(gst_pad_get_parent(pad));
-
-  GST_DEBUG ("looking for pad %p in qtdemux %p", pad, qtdemux);
-  g_return_val_if_fail(GST_IS_QTDEMUX(qtdemux), GST_PAD_LINK_REFUSED);
-
-  GST_DEBUG ("n_streams is %d", qtdemux->n_streams);
-  for(i=0;i<qtdemux->n_streams;i++){
-    stream = qtdemux->streams[i];
-    GST_DEBUG ("pad[%d] is %p", i, stream->pad);
-    if(stream->pad == pad){
-      return GST_PAD_LINK_OK;
-    }
-  }
-
-  GST_DEBUG ("Couldn't find stream cooresponding to pad\n");
-
-  return GST_PAD_LINK_REFUSED;
-}
-
 void gst_qtdemux_add_stream(GstQTDemux *qtdemux, QtDemuxStream *stream)
 {
   if(stream->subtype == GST_MAKE_FOURCC('v','i','d','e')){
-    stream->pad = gst_pad_new_from_template (videosrctempl,
+    stream->pad = gst_pad_new_from_template (
+	gst_static_pad_template_get(&gst_qtdemux_videosrc_template),
         g_strdup_printf ("video_%02d", qtdemux->n_video_streams));
     stream->fps = 1. * GST_SECOND / stream->samples[0].duration;
     if(stream->caps){
-      GstProps *properties = gst_props_intersect(
-	  stream->caps->properties,
-	  gst_props_new("width",GST_PROPS_INT(stream->width),
-              "height",GST_PROPS_INT(stream->height),
-              "framerate", GST_PROPS_FLOAT(stream->fps), NULL));
-      if (stream->caps->properties != NULL)
-        gst_props_unref (stream->caps->properties);
-      stream->caps->properties = properties;
+      gst_caps_set_simple(stream->caps,
+	  "width", G_TYPE_INT, stream->width,
+	  "height", G_TYPE_INT, stream->height,
+	  "framerate", G_TYPE_DOUBLE, stream->fps, NULL);
     }
     qtdemux->n_video_streams++;
   }else{
-    stream->pad = gst_pad_new_from_template (audiosrctempl,
+    stream->pad = gst_pad_new_from_template (
+	gst_static_pad_template_get(&gst_qtdemux_audiosrc_template),
         g_strdup_printf ("audio_%02d", qtdemux->n_audio_streams));
     if(stream->caps){
-      GstProps *properties = gst_props_intersect(
-	  stream->caps->properties,
-	  gst_props_new("rate",GST_PROPS_INT((int)stream->rate),
-              "channels",GST_PROPS_INT(stream->n_channels), NULL));
-      if (stream->caps->properties != NULL)
-        gst_props_unref (stream->caps->properties);
-      stream->caps->properties = properties;
+      gst_caps_set_simple(stream->caps,
+	  "rate", G_TYPE_INT, (int)stream->rate,
+	  "channels", G_TYPE_INT, stream->n_channels, NULL);
     }
     qtdemux->n_audio_streams++;
   }
 
-  gst_pad_set_getcaps_function(stream->pad, gst_qtdemux_src_getcaps);
-  gst_pad_set_link_function(stream->pad, gst_qtdemux_src_link);
+  gst_pad_use_explicit_caps (stream->pad);
 
+  GST_PAD_ELEMENT_PRIVATE(stream->pad) = stream;
   qtdemux->streams[qtdemux->n_streams] = stream;
   qtdemux->n_streams++;
   GST_DEBUG ("n_streams is now %d", qtdemux->n_streams);
 
+  gst_pad_set_explicit_caps(stream->pad, stream->caps);
+
   GST_DEBUG ("adding pad %p to qtdemux %p", stream->pad, qtdemux);
   gst_element_add_pad(GST_ELEMENT (qtdemux), stream->pad);
-
-  /* Note: we need to have everything set up before calling try_set_caps */
-  if(stream->caps){
-    g_print("setting caps to %s\n",gst_caps_to_string(stream->caps));
-
-    gst_pad_try_set_caps(stream->pad, stream->caps);
-  }
 }
 
 
@@ -1323,7 +1281,7 @@ static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
     g_print("frame count:   %u\n", QTDEMUX_GUINT16_GET(stsd->data+offset+48));
     
     stream->caps = qtdemux_video_caps(qtdemux,
-        QTDEMUX_FOURCC_GET(stsd->data+offset+4));
+        QTDEMUX_FOURCC_GET(stsd->data+offset+4), stsd->data);
     g_print("caps %s\n",gst_caps_to_string(stream->caps));
   }else if(stream->subtype == FOURCC_soun){
     int version;
@@ -1345,9 +1303,14 @@ static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
 
     if(version == 0x00010000){
       g_print("samples/packet:   %d\n", QTDEMUX_GUINT32_GET(stsd->data+offset + 20));
+      stream->samples_per_packet = QTDEMUX_GUINT32_GET(stsd->data+offset + 20);
       g_print("bytes/packet:     %d\n", QTDEMUX_GUINT32_GET(stsd->data+offset + 24));
       g_print("bytes/frame:      %d\n", QTDEMUX_GUINT32_GET(stsd->data+offset + 28));
+      stream->bytes_per_frame = QTDEMUX_GUINT32_GET(stsd->data+offset + 28);
       g_print("bytes/sample:     %d\n", QTDEMUX_GUINT32_GET(stsd->data+offset + 32));
+    } else {
+      stream->bytes_per_frame = stream->n_channels * QTDEMUX_GUINT16_GET(stsd->data+offset + 10);
+      stream->samples_per_packet = 1;
     }
 
     stream->caps = qtdemux_audio_caps(qtdemux,
@@ -1358,11 +1321,6 @@ static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
     return;
   }
 
-  if(stream->caps){
-    gst_caps_ref(stream->caps);
-    gst_caps_sink(stream->caps);
-  }
-
   /* sample to chunk */
   stsc = qtdemux_tree_get_child_by_type(stbl, FOURCC_stsc);
   g_assert(stsc);
@@ -1479,14 +1437,16 @@ done:
         }
 	samples[j].chunk = j;
 	samples[j].offset = chunk_offset;
-	samples[j].size = samples_per_chunk * stream->n_channels * sample_width;
+	samples[j].size = samples_per_chunk * stream->bytes_per_frame / stream->samples_per_packet;
+	samples[j].duration = samples_per_chunk * GST_SECOND / stream->rate;
+	samples[j].timestamp = j == 0 ? 0 : samples[j - 1].timestamp + samples[j - 1].duration;
 	samples[j].sample_index = sample_index;
 	sample_index += samples_per_chunk;
 	if(j>=n_samples)goto done2;
       }
     }
+/*
 done2:
-
     n_sample_times = QTDEMUX_GUINT32_GET(stts->data + 12);
     g_print("n_sample_times = %d\n",n_sample_times);
     timestamp = 0;
@@ -1503,13 +1463,14 @@ done2:
 
         samples[index].timestamp = timestamp;
 	size = samples[index+1].sample_index - samples[index].sample_index;
-	time = (GST_SECOND * duration * samples[index].size)/stream->timescale ;
+	time = GST_SECOND / stream->rate; //(GST_SECOND * duration * samples[index].size)/stream->timescale ;
         timestamp += time;
         samples[index].duration = time;
       }
     }
+*/
   }
-
+done2:
 #if 0
   for(i=0;i<n_samples;i++){
     g_print("%d: %d %d %d %d %" G_GUINT64_FORMAT "\n",i,
@@ -1523,92 +1484,105 @@ done2:
 }
 
 
-static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc)
+static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc, const guint8 *stsd_data)
 {
   switch(fourcc){
     case GST_MAKE_FOURCC('j','p','e','g'):
       /* JPEG */
-      return GST_CAPS_NEW("jpeg_caps","image/jpeg",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("image/jpeg");
     case GST_MAKE_FOURCC('m','j','p','a'):
       /* Motion-JPEG (format A) */
-      return GST_CAPS_NEW("mjpa_caps","image/jpeg",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("image/jpeg");
     case GST_MAKE_FOURCC('m','j','p','b'):
       /* Motion-JPEG (format B) */
-      return GST_CAPS_NEW("mjpb_caps","image/jpeg",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("image/jpeg");
     case GST_MAKE_FOURCC('S','V','Q','3'):
-      return GST_CAPS_NEW("SVQ3_caps","video/x-svq",
-	  "svqversion", GST_PROPS_INT(3),
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      if (stsd_data != NULL) {
+        gst_getbits_t gb;
+	gint halfpel_flag;
+	gint thirdpel_flag;
+	gint unknown_svq3_flag;
+	gint low_delay;
+	gint size;
+
+	size = QTDEMUX_GUINT32_GET(stsd_data + 16);
+
+        gst_getbits_init (&gb, NULL, NULL);
+        gst_getbits_newbuf (&gb, (unsigned char *)stsd_data + 98 + 16 + 4 , (size - 102 + 16));
+	
+        /* Infos ripped from ffmpeg see libavcodec/svq3.c */
+	
+	/* 'frame size code' and optional 'width, height' */
+        if (gst_getbitsn (&gb, 3) == 7) {
+          gst_getbitsn (&gb, 12);
+          gst_getbitsn (&gb, 12);
+	}
+
+	halfpel_flag = gst_get1bit (&gb);
+        thirdpel_flag = gst_get1bit (&gb);
+
+        /* unknown fields */
+        gst_get1bit (&gb);
+        gst_get1bit (&gb);
+        gst_get1bit (&gb);
+        gst_get1bit (&gb);
+
+        low_delay = gst_get1bit (&gb);
+
+        /* unknown field */
+        gst_get1bit (&gb);
+
+	while (gst_get1bit (&gb)) {
+          gst_getbitsn (&gb, 8);
+        }
+	
+        unknown_svq3_flag = gst_get1bit (&gb);
+
+        return gst_caps_new_simple ("video/x-svq",
+		"svqversion",        G_TYPE_INT, 3,
+                "halfpel_flag",      G_TYPE_INT, halfpel_flag,
+                "thirdpel_flag",     G_TYPE_INT, thirdpel_flag,
+                "low_delay",         G_TYPE_INT, low_delay,
+                "unknown_svq3_flag", G_TYPE_INT, unknown_svq3_flag,
+		NULL);
+      }
+      return gst_caps_from_string ("video/x-svq, "
+	  "svqversion = (int) 3");
     case GST_MAKE_FOURCC('s','v','q','i'):
     case GST_MAKE_FOURCC('S','V','Q','1'):
-      return GST_CAPS_NEW("SVQ1_caps","video/x-svq",
-	  "svqversion", GST_PROPS_INT(1),
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("video/x-svq, "
+	  "svqversion = (int) 1");
     case GST_MAKE_FOURCC('r','a','w',' '):
       /* uncompressed RGB */
-      return GST_CAPS_NEW("raw__caps","video/x-raw-rgb",
-	  "endianness",GST_PROPS_INT(G_BIG_ENDIAN),
+      return gst_caps_from_string ("video/x-raw-rgb, "
+	  "endianness = (int) BIG_ENDIAN");
           /*"bpp", GST_PROPS_INT(x),
           "depth", GST_PROPS_INT(x),
           "red_mask", GST_PROPS_INT(x),
           "green_mask", GST_PROPS_INT(x),
           "blue_mask", GST_PROPS_INT(x), FIXME! */
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
     case GST_MAKE_FOURCC('Y','u','v','2'):
       /* uncompressed YUV2 */
-      return GST_CAPS_NEW("Yuv2_caps","video/x-raw-yuv",
-	  "format",GST_PROPS_FOURCC(GST_MAKE_FOURCC('Y','U','V','2')),
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("video/x-raw-yuv, "
+	  "format = (fourcc) YUY2");
     case GST_MAKE_FOURCC('m','p','e','g'):
       /* MPEG */
-      return GST_CAPS_NEW("mpeg_caps","video/mpeg",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096),
-          "systemstream", GST_PROPS_BOOLEAN(FALSE),
-          "mpegversion", GST_PROPS_INT(1));
+      return gst_caps_from_string ("video/mpeg, "
+          "systemstream = (boolean) false, "
+          "mpegversion = (int) 1");
     case GST_MAKE_FOURCC('g','i','f',' '):
-      return GST_CAPS_NEW("gif__caps","image/gif",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("image/gif");
     case GST_MAKE_FOURCC('h','2','6','3'):
       /* H.263 */
       /* ffmpeg uses the height/width props, don't know why */
-      return GST_CAPS_NEW("h263_caps","video/x-h263",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("video/x-h263");
     case GST_MAKE_FOURCC('m','p','4','v'):
       /* MPEG-4 */
-      return GST_CAPS_NEW("mp4v_caps", "video/mpeg",
-	  "mpegversion",GST_PROPS_INT(4),
-          "systemstream", GST_PROPS_BOOLEAN(FALSE),
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("video/mpeg, "
+	  "mpegversion = (int) 4, "
+          "systemstream = (boolean) false");
     case GST_MAKE_FOURCC('3','I','V','1'):
-      return GST_CAPS_NEW("3IV1_caps", "video/x-3ivx",
-          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-          "width", GST_PROPS_INT_RANGE (16, 4096),
-          "height", GST_PROPS_INT_RANGE (16, 4096));
+      return gst_caps_from_string ("video/x-3ivx");
     case GST_MAKE_FOURCC('r','p','z','a'):
     case GST_MAKE_FOURCC('c','v','i','d'):
       /* Cinepak */
@@ -1617,7 +1591,7 @@ static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc)
     case GST_MAKE_FOURCC('s','m','c',' '):
     case GST_MAKE_FOURCC('k','p','c','d'):
     default:
-      g_print("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
+      g_critical ("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
 	  "' to caps\n", GST_FOURCC_ARGS(fourcc));
       return NULL;
   }
@@ -1627,134 +1601,94 @@ static GstCaps *qtdemux_audio_caps(GstQTDemux *qtdemux, guint32 fourcc)
 {
   switch(fourcc){
     case GST_MAKE_FOURCC('N','O','N','E'):
-      return NULL; /*GST_CAPS_NEW("NONE_caps","audio/raw",NULL);*/
+      return NULL; /*gst_caps_from_string ("audio/raw");*/
     case GST_MAKE_FOURCC('r','a','w',' '):
       /* FIXME */
-      return GST_CAPS_NEW("raw__caps","audio/x-raw-int",
-	  "width",GST_PROPS_INT(8),
-	  "depth",GST_PROPS_INT(8),
-	  "signed",GST_PROPS_BOOLEAN(FALSE),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-int, "
+	  "width = (int) 8, "
+	  "depth = (int) 8, "
+	  "signed = (boolean) true");
     case GST_MAKE_FOURCC('t','w','o','s'):
       /* FIXME */
-      return GST_CAPS_NEW("twos_caps","audio/x-raw-int",
-	  "width",GST_PROPS_INT(16),
-	  "depth",GST_PROPS_INT(16),
-	  "endianness",GST_PROPS_INT(G_BIG_ENDIAN),
-	  "signed",GST_PROPS_BOOLEAN(TRUE),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-int, "
+	  "width = (int) 16, "
+	  "depth = (int) 16, "
+	  "endianness = (int) G_BIG_ENDIAN, "
+	  "signed = (boolean) true");
     case GST_MAKE_FOURCC('s','o','w','t'):
       /* FIXME */
-      return GST_CAPS_NEW("sowt_caps","audio/x-raw-int",
-	  "width",GST_PROPS_INT(16),
-	  "depth",GST_PROPS_INT(16),
-	  "endianness",GST_PROPS_INT(G_LITTLE_ENDIAN),
-	  "signed",GST_PROPS_BOOLEAN(TRUE),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-int, "
+	  "width = (int) 16, "
+	  "depth = (int) 16, "
+	  "endianness = (int) G_LITTLE_ENDIAN, "
+	  "signed = (boolean) true");
     case GST_MAKE_FOURCC('f','l','6','4'):
-      return GST_CAPS_NEW("fl64_caps","audio/x-raw-float",
-          "width",GST_PROPS_INT (64),
-          "endianness",GST_PROPS_INT (G_BIG_ENDIAN),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-float, "
+	  "width = (int) 64, "
+	  "endianness = (int) G_BIG_ENDIAN");
     case GST_MAKE_FOURCC('f','l','3','2'):
-      return GST_CAPS_NEW("fl32_caps","audio/x-raw-float",
-          "width",GST_PROPS_INT (32),
-          "endianness",GST_PROPS_INT (G_BIG_ENDIAN),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-float, "
+	  "width = (int) 32, "
+	  "endianness = (int) G_BIG_ENDIAN");
     case GST_MAKE_FOURCC('i','n','2','4'):
       /* FIXME */
-      return GST_CAPS_NEW("in24_caps","audio/x-raw-int",
-	  "width",GST_PROPS_INT(24),
-	  "depth",GST_PROPS_INT(32),
-	  "endianness",GST_PROPS_INT(G_BIG_ENDIAN),
-	  "signed",GST_PROPS_BOOLEAN(TRUE),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-int, "
+	  "width = (int) 24, "
+	  "depth = (int) 32, "
+	  "endianness = (int) G_BIG_ENDIAN, "
+	  "signed = (boolean) true");
     case GST_MAKE_FOURCC('i','n','3','2'):
       /* FIXME */
-      return GST_CAPS_NEW("in32_caps","audio/x-raw-int",
-	  "width",GST_PROPS_INT(24),
-	  "depth",GST_PROPS_INT(32),
-	  "endianness",GST_PROPS_INT(G_BIG_ENDIAN),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-raw-int, "
+	  "width = (int) 32, "
+	  "depth = (int) 32, "
+	  "endianness = (int) G_BIG_ENDIAN, "
+	  "signed = (boolean) true");
     case GST_MAKE_FOURCC('u','l','a','w'):
       /* FIXME */
-      return GST_CAPS_NEW("ulaw_caps","audio/x-mulaw",
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-mulaw");
     case GST_MAKE_FOURCC('a','l','a','w'):
       /* FIXME */
-      return GST_CAPS_NEW("alaw_caps","audio/x-alaw",
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT));
+      return gst_caps_from_string ("audio/x-alaw");
     case 0x6d730002:
       /* Microsoft ADPCM-ACM code 2 */
-      return GST_CAPS_NEW("msxx_caps","audio/x-adpcm",
-	  "layout", GST_PROPS_STRING("microsoft"),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/x-adpcm, "
+	  "layout = (string) microsoft");
     case 0x6d730011:
       /* FIXME DVI/Intel IMA ADPCM/ACM code 17 */
-      return GST_CAPS_NEW("msxx_caps","audio/x-adpcm",
-	  "layout", GST_PROPS_STRING("quicktime"),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/x-adpcm, "
+	  "layout = (string) quicktime");
     case 0x6d730055:
       /* MPEG layer 3, CBR only (pre QT4.1) */
     case 0x5500736d:
     case GST_MAKE_FOURCC('.','m','p','3'):
       /* MPEG layer 3, CBR & VBR (QT4.1 and later) */
-      return GST_CAPS_NEW("_mp3_caps","audio/mpeg",
-	  "mpegversion", GST_PROPS_INT(1),
-	  "layer", GST_PROPS_INT(3),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/mpeg, "
+	  "layer = (int) 3, "
+          "mpegversion = (int) 1");
     case GST_MAKE_FOURCC('M','A','C','3'):
       /* MACE 3:1 */
-      return GST_CAPS_NEW("MAC3_caps","audio/x-mace",
-	  "maceversion",GST_PROPS_INT(3),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/x-mace, "
+	  "maceversion = (int) 3");
     case GST_MAKE_FOURCC('M','A','C','6'):
       /* MACE 6:1 */
-      return GST_CAPS_NEW("MAC3_caps","audio/x-mace",
-	  "maceversion",GST_PROPS_INT(6),
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/x-mace, "
+	  "maceversion = (int) 6");
     case GST_MAKE_FOURCC('O','g','g','V'):
       /* Ogg Vorbis */
-      return GST_CAPS_NEW("OggV_caps","application/ogg",
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("application/ogg");
     case GST_MAKE_FOURCC('d','v','c','a'):
       /* DV audio */
-      return GST_CAPS_NEW("dvca_caps","audio/x-dv",
-	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
-	  NULL);
+      return gst_caps_from_string ("audio/x-dv");
     case GST_MAKE_FOURCC('m','p','4','a'):
       /* MPEG-4 AAC */
-      return GST_CAPS_NEW("mp4a_caps", "audio/mpeg",
-	   "mpegversion", GST_PROPS_INT(4),
-	   "rate", GST_PROPS_INT_RANGE(1, G_MAXINT),
-	   "channels", GST_PROPS_INT_RANGE(1, G_MAXINT),
-	   "systemstream", GST_PROPS_BOOLEAN(FALSE), NULL);
-    case GST_MAKE_FOURCC('q','t','v','r'):
-      /* ? */
+      return gst_caps_from_string ("audio/mpeg, "
+	  "mpegversion = (int) 4");
     case GST_MAKE_FOURCC('Q','D','M','2'):
       /* QDesign music version 2 (no constant) */
+      return gst_caps_from_string ("audio/x-qdm2");
+    case GST_MAKE_FOURCC('q','t','v','r'):
+      /* ? */
     case GST_MAKE_FOURCC('Q','D','M','C'):
       /* QDesign music */
     case GST_MAKE_FOURCC('i','m','a','4'):
@@ -1764,7 +1698,7 @@ static GstCaps *qtdemux_audio_caps(GstQTDemux *qtdemux, guint32 fourcc)
     case GST_MAKE_FOURCC('a','g','s','m'):
       /* ? */
     default:
-      g_print("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
+      g_critical ("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
 	  "' to caps\n", GST_FOURCC_ARGS(fourcc));
       return NULL;
   }
