@@ -48,33 +48,31 @@ enum {
 
 enum {
   ARG_0,
-  /* ARG_WIDTH, */ /* width is not even implemented so no use in having this */
-  ARG_SAMPLERATE,
   ARG_TABLESIZE,
   ARG_SAMPLES_PER_BUFFER,
   ARG_FREQ,
   ARG_VOLUME,
 };
 
-GST_PAD_TEMPLATE_FACTORY (sinesrc_src_factory,
+static GstStaticPadTemplate gst_sinesrc_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "sinesrc_src",
-    "audio/x-raw-int",
-      "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
-      "signed",   	GST_PROPS_BOOLEAN (TRUE),
-      "width",   	GST_PROPS_INT (16),
-      "depth",    	GST_PROPS_INT (16),
-      "rate",     	GST_PROPS_INT_RANGE (8000, 48000),
-      "channels", 	GST_PROPS_INT (1)
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) 1"
   )
 );
 
 static void 	    gst_sinesrc_class_init          (GstSineSrcClass *klass);
 static void 	    gst_sinesrc_base_init           (GstSineSrcClass *klass);
 static void 	    gst_sinesrc_init	            (GstSineSrc *src);
+static void	    gst_sinesrc_dispose		    (GObject *object);
 static void 	    gst_sinesrc_set_property        (GObject *object, 
 		                                     guint prop_id, 
 					             const GValue *value, 
@@ -83,18 +81,26 @@ static void         gst_sinesrc_get_property        (GObject *object,
 		                                     guint prop_id, 
 					             GValue *value, 
 					             GParamSpec *pspec);
-/*static gboolean gst_sinesrc_change_state(GstElement *element, */
-/*                                          GstElementState state); */
-/*static void gst_sinesrc_close_audio(GstSineSrc *src); */
-/*static gboolean gst_sinesrc_open_audio(GstSineSrc *src); */
+static GstPadLinkReturn
+                    gst_sinesrc_link                (GstPad  *pad,
+						     const GstCaps *caps);
+static GstElementStateReturn
+                    gst_sinesrc_change_state        (GstElement *element);
 
 static void 	    gst_sinesrc_update_freq         (const GValue *value, 
 		                                     gpointer data);
 static void 	    gst_sinesrc_populate_sinetable  (GstSineSrc *src);
 static inline void  gst_sinesrc_update_table_inc    (GstSineSrc *src);
-static gboolean     gst_sinesrc_force_caps	    (GstSineSrc *src);
+
+static const GstQueryType *
+		    gst_sinesrc_get_query_types     (GstPad      *pad);
+static gboolean     gst_sinesrc_src_query           (GstPad      *pad,
+						     GstQueryType type,
+						     GstFormat   *format,
+						     gint64      *value);
 
 static GstData*   gst_sinesrc_get		    (GstPad *pad);
+static GstCaps * gst_sinesrc_src_fixate (GstPad *pad, const GstCaps *caps);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_sinesrc_signals[LAST_SIGNAL] = { 0 }; */
@@ -123,8 +129,8 @@ gst_sinesrc_base_init (GstSineSrcClass *klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class, 
-		GST_PAD_TEMPLATE_GET (sinesrc_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_sinesrc_src_template));
   gst_element_class_set_details (element_class, &gst_sinesrc_details);
 }
 
@@ -139,14 +145,6 @@ gst_sinesrc_class_init (GstSineSrcClass *klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
-  /*
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_WIDTH,
-    g_param_spec_int("width", "Width", "Width of audio data in bits",
-                     1, 32, 0, G_PARAM_READWRITE));
-		     */
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SAMPLERATE,
-    g_param_spec_int ("samplerate","Sample Rate","Sample Rate (in Hz)",
-                      8000, 48000, 44100, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TABLESIZE,
     g_param_spec_int ("tablesize", "tablesize", "tablesize",
                       1, G_MAXINT, 1024, G_PARAM_READWRITE));
@@ -156,40 +154,40 @@ gst_sinesrc_class_init (GstSineSrcClass *klass)
 	              "Number of samples in each outgoing buffer",
                       1, G_MAXINT, 1024, G_PARAM_READWRITE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FREQ,
-    g_param_spec_float ("freq", "Frequency", "Frequency of sine source",
+    g_param_spec_double ("freq", "Frequency", "Frequency of sine source",
                         0.0, 20000.0, 440.0, G_PARAM_READWRITE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VOLUME,
-    g_param_spec_float ("volume", "Volume", "Volume",
+    g_param_spec_double ("volume", "Volume", "Volume",
                         0.0, 1.0, 0.8, G_PARAM_READWRITE)); 
                                      
   gobject_class->set_property = gst_sinesrc_set_property;
   gobject_class->get_property = gst_sinesrc_get_property;
+  gobject_class->dispose = gst_sinesrc_dispose;
 
-/*  gstelement_class->change_state = gst_sinesrc_change_state; */
+  gstelement_class->change_state = gst_sinesrc_change_state;
 }
 
 static void 
 gst_sinesrc_init (GstSineSrc *src) 
 {
- 
   src->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sinesrc_src_factory), "src");
+      gst_static_pad_template_get (&gst_sinesrc_src_template), "src");
+  gst_pad_set_link_function (src->srcpad, gst_sinesrc_link);
+  gst_pad_set_fixate_function (src->srcpad, gst_sinesrc_src_fixate);
+  gst_pad_set_get_function (src->srcpad, gst_sinesrc_get);
+  gst_pad_set_query_function (src->srcpad, gst_sinesrc_src_query);
+  gst_pad_set_query_type_function (src->srcpad, gst_sinesrc_get_query_types);
   gst_element_add_pad (GST_ELEMENT(src), src->srcpad);
   
-  gst_pad_set_get_function (src->srcpad, gst_sinesrc_get);
-
-  src->width = 16;
   src->samplerate = 44100;
   src->volume = 1.0;
   src->freq = 440.0;
   
-  src->newcaps = TRUE;
-  
   src->table_pos = 0.0;
   src->table_size = 1024;
   src->samples_per_buffer=1024;
-  src->timestamp=0LL;
-  src->bufpool=NULL;
+  src->timestamp=0LLU;
+  src->offset=0LLU;
   
   src->seq = 0;
 
@@ -197,7 +195,7 @@ gst_sinesrc_init (GstSineSrc *src)
 
   gst_dpman_add_required_dparam_callback (
     src->dpman, 
-    g_param_spec_float("freq","Frequency (Hz)","Frequency of the tone",
+    g_param_spec_double("freq","Frequency (Hz)","Frequency of the tone",
                        10.0, 10000.0, 350.0, G_PARAM_READWRITE),
     "hertz",
     gst_sinesrc_update_freq, 
@@ -206,7 +204,7 @@ gst_sinesrc_init (GstSineSrc *src)
   
   gst_dpman_add_required_dparam_direct (
     src->dpman, 
-    g_param_spec_float("volume","Volume","Volume of the tone",
+    g_param_spec_double("volume","Volume","Volume of the tone",
                        0.0, 1.0, 0.8, G_PARAM_READWRITE),
     "scalar",
     &(src->volume)
@@ -219,31 +217,143 @@ gst_sinesrc_init (GstSineSrc *src)
 
 }
 
+static void
+gst_sinesrc_dispose (GObject *object)
+{
+  GstSineSrc *sinesrc = GST_SINESRC (object);
+
+  g_free (sinesrc->table_data);
+  sinesrc->table_data = NULL;
+  
+  GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+}
+
+static GstCaps *
+gst_sinesrc_src_fixate (GstPad *pad, const GstCaps *caps)
+{
+  GstStructure *structure;
+  GstCaps *newcaps;
+
+  if (gst_caps_get_size (caps) > 1) return NULL;
+
+  newcaps = gst_caps_copy (caps);
+  structure = gst_caps_get_structure (newcaps, 0);
+
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "rate", 44100)) {
+    return newcaps;
+  }
+
+  gst_caps_free (newcaps);
+  return NULL;
+}
+
+static GstPadLinkReturn
+gst_sinesrc_link (GstPad *pad, const GstCaps *caps)
+{
+  GstSineSrc *sinesrc;
+  const GstStructure *structure;
+  gboolean ret;
+
+  GST_DEBUG ("gst_sinesrc_src_link");
+  sinesrc = GST_SINESRC (gst_pad_get_parent (pad));
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  ret = gst_structure_get_int (structure, "rate", &sinesrc->samplerate);
+
+  if (!ret) return GST_PAD_LINK_REFUSED;
+
+  return GST_PAD_LINK_OK;
+}
+
+static const GstQueryType *
+gst_sinesrc_get_query_types (GstPad *pad)
+{
+  static const GstQueryType query_types[] = {
+    GST_QUERY_POSITION,
+    0,
+  };
+
+  return query_types;
+} 
+
+static gboolean
+gst_sinesrc_src_query (GstPad      *pad,
+		       GstQueryType type,
+		       GstFormat   *format,
+		       gint64      *value)
+{
+  gboolean res = FALSE;
+  GstSineSrc *src;
+	      
+  src = GST_SINESRC (gst_pad_get_parent (pad));
+	        
+  switch (type) {
+    case GST_QUERY_POSITION:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = src->timestamp;
+          res = TRUE;
+          break;
+        case GST_FORMAT_DEFAULT: /* samples */
+          *value = src->offset / 2; /* 16bpp audio */
+          res = TRUE;
+          break;
+        case GST_FORMAT_BYTES:
+          *value = src->offset;
+          res = TRUE;
+          break;
+        default:
+          break;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return res;
+}
+
 static GstData *
 gst_sinesrc_get (GstPad *pad)
 {
   GstSineSrc *src;
   GstBuffer *buf;
+  guint tdiff;
   
   gint16 *samples;
   gint i=0;
   
   g_return_val_if_fail (pad != NULL, NULL);
-  src = GST_SINESRC(gst_pad_get_parent (pad));
+  src = GST_SINESRC (gst_pad_get_parent (pad));
 
-  if (src->bufpool == NULL) {
-    src->bufpool = gst_buffer_pool_get_default (2 * src->samples_per_buffer, 8);
+  if (!src->tags_pushed) {
+    GstTagList *taglist;
+
+    taglist = gst_tag_list_new ();
+
+    gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND,
+        GST_TAG_DESCRIPTION, "sine wave", NULL);
+
+    gst_element_found_tags_for_pad (GST_ELEMENT (src), pad, 0, taglist);
+    src->tags_pushed = TRUE;
   }
-  
-  buf = (GstBuffer *) gst_buffer_new_from_pool (src->bufpool, 0, 0);
-  GST_BUFFER_TIMESTAMP(buf) = src->timestamp;
 
-  samples = (gint16*)GST_BUFFER_DATA(buf);
-  GST_BUFFER_DATA(buf) = (gpointer) samples;
+  tdiff = src->samples_per_buffer * GST_SECOND / src->samplerate;
+
+  /* note: the 2 is because of the format we use */
+  buf = gst_buffer_new_and_alloc (src->samples_per_buffer * 2);
+
+  GST_BUFFER_TIMESTAMP(buf) = src->timestamp;
+  GST_BUFFER_OFFSET (buf) = src->offset;
+  GST_BUFFER_DURATION (buf) = tdiff;
+
+  samples = (gint16 *) GST_BUFFER_DATA(buf);
 
   GST_DPMAN_PREPROCESS(src->dpman, src->samples_per_buffer, src->timestamp);
   
-  src->timestamp += (gint64)src->samples_per_buffer * GST_SECOND / (gint64)src->samplerate;
+  src->timestamp += tdiff;
+  src->offset += GST_BUFFER_SIZE (buf);
    
   while(GST_DPMAN_PROCESS(src->dpman, i)) {
 #if 0
@@ -283,13 +393,14 @@ gst_sinesrc_get (GstPad *pad)
     i++;
   }
 
-  
-  if (src->newcaps) {
-    if (!gst_sinesrc_force_caps(src)) {
-      gst_element_error (GST_ELEMENT (src), "Could not set caps");
+  if (!GST_PAD_CAPS (src->srcpad)) {
+    if (gst_sinesrc_link (src->srcpad,
+			  gst_pad_get_allowed_caps (src->srcpad)) <= 0) {
+      GST_ELEMENT_ERROR (src, CORE, NEGOTIATION, (NULL), (NULL));
       return NULL;
     }
   }
+
   return GST_DATA (buf);
 }
 
@@ -303,18 +414,6 @@ gst_sinesrc_set_property (GObject *object, guint prop_id,
   src = GST_SINESRC (object);
 
   switch (prop_id) {
-/*
-    case ARG_WIDTH:
-      src->width = g_value_get_int (value);
-      src->newcaps = TRUE;
-      break;
-*/
-    case ARG_SAMPLERATE:
-      src->samplerate = g_value_get_int (value);
-      gst_dpman_set_rate (src->dpman, src->samplerate);
-      src->newcaps = TRUE;
-      gst_sinesrc_update_table_inc (src);
-      break;
     case ARG_TABLESIZE:
       src->table_size = g_value_get_int (value);
       gst_sinesrc_populate_sinetable (src);
@@ -329,7 +428,7 @@ gst_sinesrc_set_property (GObject *object, guint prop_id,
       break;
     case ARG_VOLUME:
       gst_dpman_bypass_dparam (src->dpman, "volume");
-      src->volume = g_value_get_float (value);
+      src->volume = g_value_get_double (value);
       break;
     default:
       break;
@@ -347,14 +446,6 @@ gst_sinesrc_get_property (GObject *object, guint prop_id,
   src = GST_SINESRC(object);
 
   switch (prop_id) {
-/*
-    case ARG_WIDTH:
-      g_value_set_int (value, src->width);
-      break;
-*/
-    case ARG_SAMPLERATE:
-      g_value_set_int (value, src->samplerate);
-      break;
     case ARG_TABLESIZE:
       g_value_set_int (value, src->table_size);
       break;
@@ -362,10 +453,10 @@ gst_sinesrc_get_property (GObject *object, guint prop_id,
       g_value_set_int (value, src->samples_per_buffer);
       break;
     case ARG_FREQ:
-      g_value_set_float (value, src->freq);
+      g_value_set_double (value, src->freq);
       break;
     case ARG_VOLUME:
-      g_value_set_float (value, src->volume);
+      g_value_set_double (value, src->volume);
       break;      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -373,38 +464,35 @@ gst_sinesrc_get_property (GObject *object, guint prop_id,
   }
 }
 
-/*
-static gboolean gst_sinesrc_change_state(GstElement *element,
-                                          GstElementState state) {
-  g_return_if_fail(GST_IS_SINESRC(element));
+static GstElementStateReturn
+gst_sinesrc_change_state (GstElement *element)
+{
+  GstSineSrc *src = GST_SINESRC (element);
 
-  switch (state) {
-    case GST_STATE_RUNNING:
-      if (!gst_sinesrc_open_audio(GST_SINESRC(element)))
-        return FALSE;
-      break;
-    case ~GST_STATE_RUNNING:
-      gst_sinesrc_close_audio(GST_SINESRC(element));
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      src->timestamp = 0LLU;
+      src->offset = 0LLU;
       break;
     default:
       break;
   }
 
   if (GST_ELEMENT_CLASS(parent_class)->change_state)
-    return GST_ELEMENT_CLASS(parent_class)->change_state(element,state);
-  return TRUE;
+    return GST_ELEMENT_CLASS(parent_class)->change_state (element);
+
+  return GST_STATE_SUCCESS;
 }
-*/
 
 static void 
 gst_sinesrc_populate_sinetable (GstSineSrc *src)
 {
   gint i;
   gdouble pi2scaled = M_PI * 2 / src->table_size;
-  gfloat *table = g_new (gfloat, src->table_size);
+  gdouble *table = g_new (gdouble, src->table_size);
 
   for(i=0 ; i < src->table_size ; i++){
-    table[i] = (gfloat) sin(i * pi2scaled);
+    table[i] = (gdouble) sin(i * pi2scaled);
   }
   
   g_free (src->table_data);
@@ -417,7 +505,7 @@ gst_sinesrc_update_freq (const GValue *value, gpointer data)
   GstSineSrc *src = (GstSineSrc *) data;
   g_return_if_fail (GST_IS_SINESRC (src));
 
-  src->freq = g_value_get_float (value);
+  src->freq = g_value_get_double (value);
   src->table_inc = src->table_size * src->freq / src->samplerate;
   
   /*GST_DEBUG ("freq %f", src->freq); */
@@ -429,27 +517,35 @@ gst_sinesrc_update_table_inc (GstSineSrc *src)
   src->table_inc = src->table_size * src->freq / src->samplerate;
 }
 
+#if 0
 static gboolean 
-gst_sinesrc_force_caps (GstSineSrc *src) {
+gst_sinesrc_force_caps (GstSineSrc *src)
+{
+  static GstStaticCaps static_caps = GST_STATIC_CAPS ("audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) 1"
+  );
   GstCaps *caps;
+  GstStructure *structure;
 
   if (!src->newcaps)
     return TRUE;
   
-  caps = GST_CAPS_NEW (
-	   "sinesrc_src_caps",
-	   "audio/x-raw-int",
-    	     "endianness",     	GST_PROPS_INT (G_BYTE_ORDER),
-    	     "signed",   	GST_PROPS_BOOLEAN (TRUE),
-    	     "width",   	GST_PROPS_INT (16),
-	     "depth", 		GST_PROPS_INT (16),
-	     "rate", 		GST_PROPS_INT (src->samplerate),
-	     "channels", 	GST_PROPS_INT (1)
-	  );
+  caps = gst_caps_copy (gst_static_caps_get (&static_caps));
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_set (structure, "rate", G_TYPE_INT, src->samplerate, NULL);
   
   src->newcaps = gst_pad_try_set_caps (src->srcpad, caps) < GST_PAD_LINK_OK;
+
   return !src->newcaps;
 }
+#endif
 
 static gboolean
 plugin_init (GstPlugin *plugin)
@@ -469,7 +565,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
