@@ -74,32 +74,21 @@ enum {
   LAST_SIGNAL
 };
 
-GST_PAD_TEMPLATE_FACTORY (vorbis_tag_src_template_factory,
+static GstStaticPadTemplate gst_vorbis_tag_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "vorbis_tag_data_src",
-    "audio/x-vorbis",
-    NULL
-  ),
-  GST_CAPS_NEW (
-    "vorbis_tag_tag_src",
-    "application/x-gst-tags",
-    NULL
-  )
-)
+  GST_STATIC_CAPS ("audio/x-vorbis; application/x-gst-tags")
+);
 
-GST_PAD_TEMPLATE_FACTORY (vorbis_tag_sink_template_factory,
+static GstStaticPadTemplate gst_vorbis_tag_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "vorbis_tag_data_sink",
-    "audio/x-vorbis",
-    NULL
-  )
-)
+  GST_STATIC_CAPS ( "audio/x-vorbis" )
+);
 
 
 static void		gst_vorbis_tag_base_init		(gpointer		g_class);
@@ -155,9 +144,9 @@ gst_vorbis_tag_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_vorbis_tag_details);
 
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (vorbis_tag_sink_template_factory));
+      gst_static_pad_template_get (&gst_vorbis_tag_sink_template));
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (vorbis_tag_src_template_factory));
+      gst_static_pad_template_get (&gst_vorbis_tag_src_template));
 }
 static void
 gst_vorbis_tag_class_init (gpointer g_class, gpointer class_data)
@@ -175,31 +164,35 @@ gst_vorbis_tag_init (GTypeInstance *instance, gpointer g_class)
     
   /* create the sink and src pads */
   tag->sinkpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (vorbis_tag_sink_template_factory), "sink");
+      gst_static_pad_template_get (&gst_vorbis_tag_sink_template), "sink");
   gst_element_add_pad (GST_ELEMENT (tag), tag->sinkpad);
   gst_pad_set_chain_function (tag->sinkpad, GST_DEBUG_FUNCPTR (gst_vorbis_tag_chain));
 
   tag->srcpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (vorbis_tag_src_template_factory), "src");
+      gst_static_pad_template_get (&gst_vorbis_tag_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (tag), tag->srcpad);
 }
 static GstTagEntryMatch tag_matches[] = {
-  { GST_TAG_TITLE,	"TITLE"		},
-  { GST_TAG_VERSION,	"VERSION"	},
-  { GST_TAG_ALBUM,	"ALBUM"		},
-  { GST_TAG_TRACK_NUMBER,"TRACKNUMBER"	},
-  { GST_TAG_ARTIST,	"ARTIST"	},
-  { GST_TAG_PERFORMER,	"PERFORMER"	},
-  { GST_TAG_COPYRIGHT,	"COPYRIGHT"   	},
-  { GST_TAG_LICENSE, 	"LICENSE"    	},
-  { GST_TAG_ORGANIZATION,"ORGANIZATION" },
-  { GST_TAG_DESCRIPTION,"DESCRIPTION"   },
-  { GST_TAG_GENRE, 	"GENRE"    	},
-  { GST_TAG_DATE, 	"DATE"    	},
-  { GST_TAG_CONTACT, 	"CONTACT"    	},
-  { GST_TAG_ISRC, 	"ISRC"    	},
-  { GST_TAG_COMMENT,	"COMMENT"	},
-  { NULL,		NULL		}
+  { GST_TAG_TITLE,		"TITLE"			},
+  { GST_TAG_VERSION,		"VERSION"		},
+  { GST_TAG_ALBUM,		"ALBUM"			},
+  { GST_TAG_TRACK_NUMBER,	"TRACKNUMBER"		},
+  { GST_TAG_ARTIST,		"ARTIST"		},
+  { GST_TAG_PERFORMER,		"PERFORMER"		},
+  { GST_TAG_COPYRIGHT,		"COPYRIGHT"   		},
+  { GST_TAG_LICENSE, 		"LICENSE"    		},
+  { GST_TAG_ORGANIZATION,	"ORGANIZATION"		},
+  { GST_TAG_DESCRIPTION,	"DESCRIPTION"   	},
+  { GST_TAG_GENRE, 		"GENRE"    		},
+  { GST_TAG_DATE, 		"DATE"    		},
+  { GST_TAG_CONTACT, 		"CONTACT"    		},
+  { GST_TAG_ISRC, 		"ISRC"    		},
+  { GST_TAG_COMMENT,		"COMMENT"		},
+  { GST_TAG_TRACK_GAIN,		"REPLAYGAIN_TRACK_GAIN"	},
+  { GST_TAG_TRACK_PEAK,		"REPLAYGAIN_TRACK_PEAK"	},
+  { GST_TAG_ALBUM_GAIN,		"REPLAYGAIN_ALBUM_GAIN"	},
+  { GST_TAG_ALBUM_PEAK,		"REPLAYGAIN_ALBUM_PEAK"	},
+  { NULL,			NULL			}
 };
 /**
  * gst_tag_from_vorbis_tag:
@@ -299,6 +292,9 @@ gst_vorbis_tag_add (GstTagList *list, const gchar *tag, const gchar *value)
     case G_TYPE_STRING:
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, value, NULL);
       break;
+    case G_TYPE_DOUBLE:
+      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, g_strtod (value, NULL), NULL);
+      break;
     default:
       break;
   }
@@ -377,6 +373,7 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer *buffer, const guint8 *i
 error:
   gst_tag_list_free (list);
   return NULL;
+#undef ADVANCE
 }
 typedef struct {
   guint count;
@@ -419,6 +416,11 @@ gst_tag_to_vorbis_comments (const GstTagList *list, const gchar *tag)
 	result = g_strdup_printf ("%s=%s", vorbis_tag, str);
 	break;
       }
+      case G_TYPE_DOUBLE: {
+	gdouble value;
+	g_assert (gst_tag_list_get_double_index (list, tag, i, &value));
+	result = g_strdup_printf ("%s=%f", vorbis_tag, value);
+      }
       default:
 	GST_DEBUG ("Couldn't write tag %s", tag);
 	continue;
@@ -520,12 +522,15 @@ gst_vorbis_tag_chain (GstPad *pad, GstData *data)
   if (tag->output == OUTPUT_UNKNOWN) {
     /* caps nego */
     do {
-      if (gst_pad_try_set_caps (tag->srcpad, GST_CAPS_NEW ("vorbis_tag_data_src", "audio/x-vorbis", NULL)) >= 0) {
+      if (gst_pad_try_set_caps (tag->srcpad, gst_caps_new_simple (
+	      "audio/x-vorbis", NULL)) >= 0) {
 	tag->output = OUTPUT_DATA;
-      } else if (gst_pad_try_set_caps (tag->srcpad, GST_CAPS_NEW ("vorbis_tag_tag_src", "application/x-gst-tags", NULL)) >= 0) {
+      } else if (gst_pad_try_set_caps (tag->srcpad, gst_caps_new_simple (
+	      "application/x-gst-tags", NULL)) >= 0) {
 	tag->output = OUTPUT_TAGS;
       } else {
-	GstCaps *caps = gst_pad_template_get_caps (GST_PAD_TEMPLATE_GET (vorbis_tag_src_template_factory));
+	const GstCaps *caps = gst_static_caps_get (
+	    &gst_vorbis_tag_src_template.static_caps);
 	if (gst_pad_recover_caps_error (tag->srcpad, caps))
 	  continue;
 	return;
@@ -534,7 +539,7 @@ gst_vorbis_tag_chain (GstPad *pad, GstData *data)
   }
   
   if (GST_BUFFER_SIZE (buffer) == 0)
-    gst_element_error (GST_ELEMENT (tag), "empty buffers are not allowed in vorbis data");
+    GST_ELEMENT_ERROR (tag, CORE, TAG, (NULL), ("empty buffers are not allowed in vorbis data"));
   
   if (GST_BUFFER_DATA (buffer)[0] == 3) {
     gchar *vendor;
@@ -542,7 +547,7 @@ gst_vorbis_tag_chain (GstPad *pad, GstData *data)
 
     gst_data_unref (data);
     if (list == NULL) {
-      gst_element_error (GST_ELEMENT (tag), "invalid data in vorbis comments");
+      GST_ELEMENT_ERROR (tag, CORE, TAG, (NULL), ("invalid data in vorbis comments"));
       return;
     }
     gst_element_found_tags_for_pad (GST_ELEMENT (tag), tag->srcpad, 0,
