@@ -38,7 +38,8 @@ GST_DEBUG_CATEGORY_STATIC (type_find_debug);
 
 /*** text/plain ****************************************************************/
 
-#define UTF8_CAPS GST_CAPS_NEW ("cdxa_type_find", "text/plain", NULL)
+static GstStaticCaps utf8_caps = GST_STATIC_CAPS ("text/plain");
+#define UTF8_CAPS gst_static_caps_get(&utf8_caps)
 static void
 utf8_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -67,7 +68,8 @@ utf8_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** text/uri-list ************************************************************/
 
-#define URI_CAPS GST_CAPS_NEW ("uri_type_find", "text/uri-list", NULL)
+static GstStaticCaps uri_caps = GST_STATIC_CAPS ("text/uri-list");
+#define URI_CAPS (gst_static_caps_get(&uri_caps))
 #define BUFFER_SIZE 16 /* If the string is < 16 bytes we're screwed */
 #define INC_BUFFER { 							\
   pos++;								\
@@ -128,7 +130,8 @@ uri_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** video/x-fli **************************************************************/
 
-#define FLX_CAPS GST_CAPS_NEW ("flx_type_find", "video/x-fli", NULL)
+static GstStaticCaps flx_caps = GST_STATIC_CAPS ("video/x-fli");
+#define FLX_CAPS gst_static_caps_get(&flx_caps)
 static void
 flx_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -158,7 +161,8 @@ flx_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** application/x-id3 **************************************************************/
 
-#define ID3_CAPS GST_CAPS_NEW ("id3_type_find", "application/x-id3", NULL)
+static GstStaticCaps id3_caps = GST_STATIC_CAPS ("application/x-id3");
+#define ID3_CAPS gst_static_caps_get(&id3_caps)
 static void
 id3_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -223,8 +227,7 @@ mp3_type_frame_length_from_header (guint32 header, guint *put_layer,
 				   guint *put_channels, guint *put_bitrate,
 				   guint *put_samplerate)
 {
-  guint length;
-  gulong mode, samplerate, bitrate, layer, version, channels;
+  guint bitrate, layer, length, mode, samplerate, version, channels;
 
   if ((header & 0xffe00000) != 0xffe00000)
     return 0;
@@ -275,12 +278,12 @@ mp3_type_frame_length_from_header (guint32 header, guint *put_layer,
   if (layer == 1) {
     length = ((12000 * bitrate / samplerate) + length) * 4;
   } else {
-    length += ((layer == 3 && version == 0) ? 72000 : 144000) * bitrate / samplerate;
+    length += ((layer == 3 && version != 3) ? 72000 : 144000) * bitrate / samplerate;
   }
   
   GST_LOG ("mp3typefind: alculated mp3 frame length of %u bytes", length);
-  GST_LOG ("mp3typefind: samplerate = %lu - bitrate = %lu - layer = %lu - version = %lu"
-	     " - channels = %lu",
+  GST_LOG ("mp3typefind: samplerate = %u - bitrate = %u - layer = %u - version = %u"
+	     " - channels = %u",
 	     samplerate, bitrate, layer, version, channels);
   
   if (put_layer)
@@ -296,13 +299,9 @@ mp3_type_frame_length_from_header (guint32 header, guint *put_layer,
 }
 
 
-#define MP3_CAPS(layer) (layer == 0 ?						\
-    GST_CAPS_NEW ("mp3_type_find", "audio/mpeg",				\
-		    "mpegversion", GST_PROPS_INT (1),				\
-		    "layer", GST_PROPS_INT_RANGE (1, 3)) :			\
-    GST_CAPS_NEW ("mp3_type_find", "audio/mpeg",				\
-		    "mpegversion", GST_PROPS_INT (1),				\
-		    "layer", GST_PROPS_INT (layer)))
+static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, "
+    "mpegversion = (int) 1, layer = (int) [ 1, 3 ]");
+#define MP3_CAPS (gst_static_caps_get(&mp3_caps))
 /*
  * random values for typefinding
  * if no more data is available, we will return a probability of
@@ -349,6 +348,7 @@ mp3_type_find (GstTypeFind *tf, gpointer unused)
 	head = GUINT32_FROM_BE(*((guint32 *) head_data));
         if (!(length = mp3_type_frame_length_from_header (head, &layer,
 			&channels, &bitrate, &samplerate))) {
+	  GST_LOG ("%d. header at offset %"G_GUINT64_FORMAT" was not an mp3 header", found + 1, offset);
 	  break;
 	}
 	if ((prev_layer && prev_layer != layer) ||
@@ -387,8 +387,13 @@ mp3_type_find (GstTypeFind *tf, gpointer unused)
 	}
 	g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
 	if (probability > 0) {
+	  GstCaps *caps;
 	  g_assert (layer > 0);
-	  gst_type_find_suggest (tf, probability, MP3_CAPS (layer)); 
+	  caps = gst_caps_copy (MP3_CAPS);
+	  gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
+	      G_TYPE_INT, layer, 0);
+	  gst_type_find_suggest (tf, probability, caps); 
+	  gst_caps_free (caps);
 	}
 	return;
       }
@@ -401,13 +406,9 @@ mp3_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** video/mpeg systemstream **************************************************/
 
-#define MPEG_SYS_CAPS(version) (version == 0 ? 				\
-    GST_CAPS_NEW ("mpeg_type_find", "video/mpeg",			\
-	          "systemstream", GST_PROPS_BOOLEAN (TRUE),		\
-		  "mpegversion", GST_PROPS_INT_RANGE (1, 2)) :		\
-    GST_CAPS_NEW ("mpeg_type_find", "video/mpeg",			\
-	    	  "systemstream", GST_PROPS_BOOLEAN (TRUE),		\
-		  "mpegversion", GST_PROPS_INT (version)))
+static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
+    "systemstream = (boolean) true, mpegversion = (int) [ 1, 2 ]");
+#define MPEG_SYS_CAPS gst_static_caps_get(&mpeg_sys_caps)
 #define IS_MPEG_HEADER(data)		((((guint8 *)data)[0] == 0x00) &&	\
 					 (((guint8 *)data)[1] == 0x00) &&	\
 					 (((guint8 *)data)[2] == 0x01) &&	\
@@ -428,7 +429,16 @@ mpeg2_sys_type_find (GstTypeFind *tf, gpointer unused)
   if (data && IS_MPEG_HEADER (data)) {
     if ((data[4] & 0xC0) == 0x40) {
       /* type 2 */
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MPEG_SYS_CAPS (2));
+      GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
+      gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+	  G_TYPE_INT, 2, 0);
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
+    } else if ((data[4] & 0xF0) == 0x20) {
+      GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
+      gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+			 G_TYPE_INT, 1, 0);
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
+      gst_caps_free (caps);
     }
   }
 };
@@ -440,74 +450,94 @@ mpeg2_sys_type_find (GstTypeFind *tf, gpointer unused)
 static guint
 mpeg1_parse_header (GstTypeFind *tf, guint64 offset)
 {
-  guint8 *data = gst_type_find_peek (tf, offset, 18);
+  guint8 *data = gst_type_find_peek (tf, offset, 4);
   guint size;
   
   if (!data) {
-    GST_LOG ("couldn't get 18 bytes to parse MPEG header");
+    GST_LOG ("couldn't get MPEG header bytes");
     return 1;
   }
-  
-  /* check header */
-  if (!IS_MPEG_HEADER (data)) {
-    GST_LOG ("This isn't an MPEG header");
-    return 0;
-  }
-  data += 4;
 
-  /* check marker bits */
-  if ((*data & 0xF1) != 0x21) {
-    GST_LOG ("marker bits in byte 4 don't match");
+  if (data[0] != 0 || data[1] != 0 || data[2] != 1) {
     return 0;
   }
-  data += 2;
-  if ((*data & 0x01) != 0x01) {
-    GST_LOG ("marker bits in byte 6 don't match");
-    return 0;
+  offset += 4;
+
+  switch (data[3]) {
+    case 0xBA: /* pack header */
+      data = gst_type_find_peek (tf, offset, 8);
+      if (!data) {
+        GST_LOG ("couldn't get MPEG pack header bytes");
+        return 1;
+      }
+      size = 12;
+      /* check marker bits */
+      if ((data[0] & 0xF1) != 0x21 ||
+          (data[2] & 0x01) != 0x01 ||
+          (data[4] & 0x01) != 0x01 ||
+          (data[5] & 0x80) != 0x80 ||
+          (data[7] & 0x01) != 0x01)
+        return 0;
+      break;
+
+    case 0xB9: /* ISO end code */
+      size = 4;
+      break;
+
+    case 0xBB: /* system header */
+      data = gst_type_find_peek (tf, offset, 2);
+      if (!data) {
+        GST_LOG ("couldn't get MPEG pack header bytes");
+        return 1;
+      }
+      size = GUINT16_FROM_BE (* (guint16 *) data) + 6;
+      offset += 2;
+      data = gst_type_find_peek (tf, offset, size - 6);
+      if (!data) {
+        GST_LOG ("couldn't get MPEG pack header bytes");
+        return 1;
+      }
+      /* check marker bits */
+      if ((data[0] & 0x80) != 0x80 ||
+          (data[2] & 0x01) != 0x01 ||
+          (data[4] & 0x20) != 0x20)
+        return 0;
+      /* check stream marker bits */
+      for (offset = 6; offset < (size - 6); offset += 3) {
+        if (data[offset] <= 0xBB ||
+            (data[offset + 1] & 0xC0) != 0xC0)
+          return 0;
+      }
+      break;
+
+    default:
+      if (data[3] < 0xB9)
+        return 0;
+      data = gst_type_find_peek (tf, offset, 2);
+      if (!data) {
+        GST_LOG ("couldn't get MPEG pack header bytes");
+        return 1;
+      }
+      size = GUINT16_FROM_BE (* (guint16 *) data) + 6;
+      /* FIXME: we could check PTS/DTS marker bits here... (bit overkill) */
+      break;
   }
-  data += 2;
-  if ((*data & 0x01) != 0x01) {
-    GST_LOG ("marker bits in byte 8 don't match");
-    return 0;
-  }
-  data ++;
-  if ((*data & 0x80) != 0x80) {
-    GST_LOG ("marker bits in byte 9 don't match");
-    return 0;
-  }
-  data += 2;
-  if ((*data & 0x01) != 0x01) {
-    GST_LOG ("marker bits in byte 11 don't match");
-    return 0;
-  }
-  data++;
-  
-  if (!IS_MPEG_PACKET_HEADER (data) &&
-      !IS_MPEG_SYSTEM_HEADER (data)) {
-    GST_LOG ("MPEG packet header doesn't match: %8.8X", GUINT32_FROM_BE (*((guint32 *) data)));
-    return 0;
-  }
-    
-  data += 4;
-  
-  size = GUINT16_FROM_BE (*((guint16 *) data)) + 18;
 
-  GST_DEBUG ("found mpeg1 packet at offset %"G_GUINT64_FORMAT" with size %u", offset, size);
   return size;
 }
 /* calculation of possibility to identify random data as mpeg systemstream:
- * bits that must match in header detection:		65
- * chance that random data is identifed:		1/2^65
+ * bits that must match in header detection:		32 (or more)
+ * chance that random data is identifed:		1/2^32
  * chance that GST_MPEG_TYPEFIND_TRY_HEADERS headers are identified:	
- *					1/2^(65*GST_MPEG_TYPEFIND_TRY_HEADERS)
+ *					1/2^(32*GST_MPEG_TYPEFIND_TRY_HEADERS)
  * chance that this happens in GST_MPEG_TYPEFIND_TRY_SYNC bytes:
- *					1-(1-1/2^(65*GST_MPEG_TYPEFIND_TRY_HEADERS))^GST_MPEG_TYPEFIND_TRY_SYNC
+ *					1-(1+1/2^(32*GST_MPEG_TYPEFIND_TRY_HEADERS)^GST_MPEG_TYPEFIND_TRY_SYNC)
  * for current values:
- *					1-(1-1/2^(65*2)^50000
- *				      = 3.6734..*10^-35
+ *					1-(1+1/2^(32*4)^101024)
+ *				      = <some_number>
  */
-#define GST_MPEG_TYPEFIND_TRY_HEADERS 2
-#define GST_MPEG_TYPEFIND_TRY_SYNC (GST_TYPE_FIND_MAXIMUM * 500) /* 50kB */
+#define GST_MPEG_TYPEFIND_TRY_HEADERS 4
+#define GST_MPEG_TYPEFIND_TRY_SYNC (100 * 1024) /* 100kB */
 #define GST_MPEG_TYPEFIND_SYNC_SIZE 2048
 static void
 mpeg1_sys_type_find (GstTypeFind *tf, gpointer unused)
@@ -515,6 +545,7 @@ mpeg1_sys_type_find (GstTypeFind *tf, gpointer unused)
   guint8 *data = NULL;
   guint size = 0;
   guint64 skipped = 0;
+  GstCaps *caps;
 
   while (skipped < GST_MPEG_TYPEFIND_TRY_SYNC) {
     if (size < 4) {
@@ -528,7 +559,7 @@ mpeg1_sys_type_find (GstTypeFind *tf, gpointer unused)
       guint found = 0;
       guint packet_size = 0;
       guint64 offset = skipped;
-      
+
       while (found < GST_MPEG_TYPEFIND_TRY_HEADERS) {
 	packet_size = mpeg1_parse_header (tf, offset);
 	if (packet_size <= 1)
@@ -539,14 +570,11 @@ mpeg1_sys_type_find (GstTypeFind *tf, gpointer unused)
       g_assert (found <= GST_MPEG_TYPEFIND_TRY_HEADERS);
       if (found == GST_MPEG_TYPEFIND_TRY_HEADERS ||
 	  packet_size == 1) {
-	guint probability = found * GST_TYPE_FIND_MAXIMUM * 
-			    (GST_MPEG_TYPEFIND_TRY_SYNC - skipped) /
-			    GST_MPEG_TYPEFIND_TRY_HEADERS / GST_MPEG_TYPEFIND_TRY_SYNC;
-	
-	if (probability < GST_TYPE_FIND_MINIMUM)
-	  probability = GST_TYPE_FIND_MINIMUM;
-	g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
-	gst_type_find_suggest (tf, probability, MPEG_SYS_CAPS (1)); 
+	caps = gst_caps_copy (MPEG_SYS_CAPS);
+	gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+	    G_TYPE_INT, 1, 0);
+	gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps); 
+	gst_caps_free (caps);
 	return;
       }
     }
@@ -558,9 +586,9 @@ mpeg1_sys_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** video/mpeg video stream **************************************************/
 
-#define MPEG_VIDEO_CAPS                                                 \
-    GST_CAPS_NEW ("mpeg_type_find", "video/mpeg",			\
-	          "systemstream", GST_PROPS_BOOLEAN (FALSE))
+static GstStaticCaps mpeg_video_caps = GST_STATIC_CAPS ("video/mpeg, "
+    "systemstream = (boolean) false");
+#define MPEG_VIDEO_CAPS gst_static_caps_get(&mpeg_video_caps)
 static void
 mpeg_video_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -570,13 +598,105 @@ mpeg_video_type_find (GstTypeFind *tf, gpointer unused)
   data = gst_type_find_peek (tf, 0, 8);
 
   if (data && memcmp(data, sequence_header, 4)==0){
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MPEG_VIDEO_CAPS);
+    GstCaps *caps = gst_caps_copy (MPEG_VIDEO_CAPS);
+    gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+		       G_TYPE_INT, 1, 0);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps);
+    gst_caps_free (caps);
+  }
+}
+
+/*
+ * Idea is the same as MPEG system stream typefinding: We check each
+ * byte of the stream to see if - from that point on - the stream
+ * matches a predefined set of marker bits as defined in the MPEG
+ * video specs.
+ *
+ * I'm sure someone will do a chance calculation here too.
+ */
+
+#define GST_MPEGVID_TYPEFIND_TRY_PICTURES 6
+#define GST_MPEGVID_TYPEFIND_TRY_SYNC (100 * 1024) /* 100 kB */
+#define GST_MPEGVID_TYPEFIND_SYNC_SIZE 2048
+
+static void
+mpeg_video_stream_type_find (GstTypeFind *tf, gpointer unused)
+{
+  gint size = 0, found = 0;
+  guint64 skipped = 0;
+  guint8 *data = NULL;
+
+  while (1) {
+    if (found >= GST_MPEGVID_TYPEFIND_TRY_PICTURES) {
+      GstCaps *caps = gst_caps_copy (MPEG_VIDEO_CAPS);
+      gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+			 G_TYPE_INT, 1, 0);
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 2, caps);
+      gst_caps_free (caps);
+      return;
+    }
+
+    if (skipped > GST_MPEGVID_TYPEFIND_TRY_SYNC)
+      break;
+
+    if (size < 4) {
+      data = gst_type_find_peek (tf, skipped, GST_MPEGVID_TYPEFIND_SYNC_SIZE);
+      if (!data)
+        break;
+      size = GST_MPEGVID_TYPEFIND_SYNC_SIZE;
+    }
+
+    /* are we a sequence (0xB3) or GOP (0xB8) header? */
+    if (data[0] == 0x0 && data[1] == 0x0 && data[2] == 0x1 &&
+        (data[3] == 0xB3 || data[3] == 0xB8)) {
+      size -= 8;
+      data += 8;
+      skipped += 8;
+      if (data[3] == 0xB3)
+        continue;
+      else if (size < 4) {
+        data = gst_type_find_peek (tf, skipped, GST_MPEGVID_TYPEFIND_SYNC_SIZE);
+        size = GST_MPEGVID_TYPEFIND_SYNC_SIZE;
+        if (!data)
+          break;
+      }
+      /* else, we should now see an image */
+    }
+
+    /* image header (and, when found, slice header) */
+    if (data[0] == 0x0 && data[1] == 0x0 &&
+        data[2] == 0x1 && data[4] == 0x0) {
+      size -= 8;
+      data += 8;
+      skipped += 8;
+      if (size < 5) {
+        data = gst_type_find_peek (tf, skipped, GST_MPEGVID_TYPEFIND_SYNC_SIZE);
+        size = GST_MPEGVID_TYPEFIND_SYNC_SIZE;
+        if (!data)
+          break;
+      }
+      if ((data[0] == 0x0 && data[1] == 0x0 &&
+           data[2] == 0x1 && data[3] == 0x1) ||
+          (data[1] == 0x0 && data[2] == 0x0 &&
+           data[3] == 0x1 && data[4] == 0x1)) {
+        size -= 4;
+        data += 4;
+        skipped += 4;
+        found += 1;
+        continue;
+      }
+    }
+
+    size--;
+    data++;
+    skipped++;
   }
 }
 
 /*** video/quicktime***********************************************************/
 
-#define QT_CAPS gst_caps_new ("qt_typefind", "video/quicktime", NULL)
+static GstStaticCaps qt_caps = GST_STATIC_CAPS ("video/quicktime");
+#define QT_CAPS gst_static_caps_get(&qt_caps)
 static void
 qt_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -609,7 +729,8 @@ qt_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** audio/x-aiff *********************************************/
 
-#define AIFF_CAPS GST_CAPS_NEW ("aiff_type_find", "audio/x-aiff", NULL)
+static GstStaticCaps aiff_caps = GST_STATIC_CAPS ("audio/x-aiff");
+#define AIFF_CAPS gst_static_caps_get(&aiff_caps)
 static void
 aiff_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -624,7 +745,8 @@ aiff_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** audio/x-shorten ****************************************/
 
-#define SHN_CAPS GST_CAPS_NEW ("shn_type_find", "audio/x-shorten", NULL)
+static GstStaticCaps shn_caps = GST_STATIC_CAPS ("audio/x-shorten");
+#define SHN_CAPS gst_static_caps_get(&shn_caps)
 static void
 shn_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -638,9 +760,23 @@ shn_type_find (GstTypeFind *tf, gpointer unused)
   }
 }
 
+/*** audio/x-m4a *********************************************/
+
+static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/x-m4a");
+#define AAC_CAPS (gst_static_caps_get(&aac_caps))
+static void
+m4a_type_find (GstTypeFind *tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 4, 8);
+  if (data && memcmp (data, "ftypM4A ", 8) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, AAC_CAPS);
+  }
+}
+
 /*** audio/x-mod *********************************************/
 
-#define MOD_CAPS gst_caps_new ("mod_type_find", "audio/x-mod", NULL)
+static GstStaticCaps mod_caps = GST_STATIC_CAPS ("audio/x-mod");
+#define MOD_CAPS gst_static_caps_get(&mod_caps)
 /* FIXME: M15 CheckType to do */
 static void
 mod_type_find (GstTypeFind *tf, gpointer unused)
@@ -757,7 +893,8 @@ mod_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** application/x-shockwave-flash ********************************************/
 
-#define SWF_CAPS gst_caps_new ("swf_type_find", "application/x-shockwave-flash", NULL)
+static GstStaticCaps swf_caps = GST_STATIC_CAPS ("audio/x-shockwave-flash");
+#define SWF_CAPS (gst_static_caps_get(&swf_caps))
 static void
 swf_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -771,7 +908,8 @@ swf_type_find (GstTypeFind *tf, gpointer unused)
 
 /*** image/jpeg ***************************************************************/
 
-#define JPEG_CAPS gst_caps_new ("jpeg_type_find", "image/jpeg", NULL)
+static GstStaticCaps jpeg_caps = GST_STATIC_CAPS ("image/jpeg");
+#define JPEG_CAPS (gst_static_caps_get(&jpeg_caps))
 static void
 jpeg_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -790,7 +928,9 @@ jpeg_type_find (GstTypeFind *tf, gpointer unused)
 }
 
 /*** image/bmp *********************/
-#define BMP_CAPS gst_caps_new ("bmp_type_find", "image/bmp", NULL)
+
+static GstStaticCaps bmp_caps = GST_STATIC_CAPS ("image/bmp");
+#define BMP_CAPS (gst_static_caps_get(&bmp_caps))
 static void
 bmp_type_find (GstTypeFind *tf, gpointer unused)
 {
@@ -809,12 +949,15 @@ bmp_type_find (GstTypeFind *tf, gpointer unused)
 }
 
 /*** image/tiff ********************/
-#define TIFF_CAPS(endian) (endian == 0 ? \
-    GST_CAPS_NEW ("tiff_type_find", "image/tiff", \
-    "endianness", GST_PROPS_LIST (GST_PROPS_INT ( \
-	G_LITTLE_ENDIAN), GST_PROPS_INT (G_BIG_ENDIAN))) : \
-    GST_CAPS_NEW ("tiff_type_find", "image/tiff",  \
-    "endianness", GST_PROPS_INT (endian)))
+static GstStaticCaps tiff_caps = GST_STATIC_CAPS ("image/tiff, "
+    "endianness = (int) { BIG_ENDIAN, LITTLE_ENDIAN }" );
+#define TIFF_CAPS (gst_static_caps_get(&tiff_caps))
+static GstStaticCaps tiff_be_caps = GST_STATIC_CAPS ("image/tiff, "
+    "endianness = (int) BIG_ENDIAN");
+#define TIFF_BE_CAPS (gst_static_caps_get(&tiff_be_caps))
+static GstStaticCaps tiff_le_caps = GST_STATIC_CAPS ("image/tiff, "
+    "endianness = (int) LITTLE_ENDIAN");
+#define TIFF_LE_CAPS (gst_static_caps_get(&tiff_le_caps))
 static void
 tiff_type_find (GstTypeFind *tf, gpointer ununsed)
 {
@@ -824,20 +967,66 @@ tiff_type_find (GstTypeFind *tf, gpointer ununsed)
   
   if (data) {
     if (memcmp (data, le_header, 4) == 0) {
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, TIFF_CAPS (G_LITTLE_ENDIAN));
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, TIFF_LE_CAPS);
     } else if (memcmp (data, be_header, 4) == 0) {
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, TIFF_CAPS (G_BIG_ENDIAN));
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, TIFF_BE_CAPS);
     }
   }
 }
 
+/*** video/x-matroska ********************/
+static GstStaticCaps matroska_caps = GST_STATIC_CAPS ("video/x-matroska" );
+#define MATROSKA_CAPS (gst_static_caps_get(&matroska_caps))
+static void
+matroska_type_find (GstTypeFind *tf, gpointer ununsed)
+{
+  /* 4 bytes for EBML ID, 1 byte for header length identifier */
+  guint8 *data = gst_type_find_peek (tf, 0, 4 + 1);
+  gint len_mask = 0x80, size = 1, n = 1, total;
+  guint8 probe_data[] = { 'm', 'a', 't', 'r', 'o', 's', 'k', 'a' };
+
+  if (!data)
+    return;
+
+  /* ebml header? */
+  if (data[0] != 0x1A || data[1] != 0x45 ||
+      data[2] != 0xDF || data[3] != 0xA3)
+    return;
+
+  /* length of header */
+  total = data[4];
+  while (size <= 8 && !(total & len_mask)) {
+    size++;
+    len_mask >>= 1;
+  }
+  if (size > 8)
+    return;
+  total &= (len_mask - 1);
+  while (n < size)
+    total = (total << 8) | data[4 + n++];
+
+  /* get new data for full header, 4 bytes for EBML ID,
+   * EBML length tag and the actual header */
+  data = gst_type_find_peek (tf, 0, 4 + size + total);
+  if (!data)
+    return;
+
+  /* the header must contain the document type 'matroska'. For now,
+   * we don't parse the whole header but simply check for the
+   * availability of that array of characters inside the header.
+   * Not fully fool-proof, but good enough. */
+  for (n = 4 + size; n < 4 + size + total - sizeof (probe_data); n++)
+    if (!memcmp (&data[n], probe_data, sizeof (probe_data))) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MATROSKA_CAPS);
+      break;
+    }
+}
+
 /*** video/x-dv ***************************************************************/
 
-#define DV_CAPS GST_CAPS_NEW ("dv_type_find", "video/x-dv", \
-          "systemstream", GST_PROPS_BOOLEAN (TRUE))
-#define DV_CAPS_FORMAT(format) GST_CAPS_NEW ("dv_type_find", "video/x-dv", \
-          "systemstream", GST_PROPS_BOOLEAN (TRUE), \
-          "format", GST_PROPS_STRING(format))
+static GstStaticCaps dv_caps = GST_STATIC_CAPS ("video/x-dv, "
+    "systemstream = (boolean) true");
+#define DV_CAPS (gst_static_caps_get(&dv_caps))
 static void
 dv_type_find (GstTypeFind *tf, gpointer private)
 {
@@ -849,17 +1038,74 @@ dv_type_find (GstTypeFind *tf, gpointer private)
   if (data && (data[0] == 0x1f) && (data[1] == 0x07) && (data[2] == 0x00) &&
       ((data[4]&0x01) == 0)){
     gchar *format;
+    GstCaps *caps = gst_caps_copy (DV_CAPS);
 
     if (data[3] & 0x80) {
       format = "PAL";
     } else {
       format = "NTSC";
     }
+    gst_structure_set (gst_caps_get_structure (caps, 0), "format",
+	G_TYPE_STRING, format, NULL);
 
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, DV_CAPS_FORMAT(format));
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
+    gst_caps_free (caps);
   }
 }
 
+/*** audio/x-vorbis ***********************************************************/
+
+static GstStaticCaps vorbis_caps = GST_STATIC_CAPS ("audio/x-vorbis");
+#define VORBIS_CAPS (gst_static_caps_get(&vorbis_caps))
+static void
+vorbis_type_find (GstTypeFind *tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 30);
+
+  if (data) {
+    guint blocksize_0;
+    guint blocksize_1;
+    /* 1 byte packet type (identification=0x01)
+       6 byte string "vorbis"
+       4 byte vorbis version */
+    if (memcmp (data, "\001vorbis\000\000\000\000", 11) != 0) return;
+    data += 11;
+    /* 1 byte channels must be != 0 */
+    if (data[0] == 0) return;
+    data++;
+    /* 4 byte samplerate must be != 0 */
+    if (*((guint32 *) data) == 0) return;
+    data += 16;
+    /* blocksize checks */
+    blocksize_0 = data[0] & 0x0F;
+    blocksize_1 = (data[0] & 0xF0) >> 4;
+    if (blocksize_0 > blocksize_1) return;
+    if (blocksize_0 < 6 || blocksize_0 > 13) return;
+    if (blocksize_1 < 6 || blocksize_1 > 13) return;
+    data++;
+    /* framing bit */
+    if ((data[0] & 0x01) != 1) return;
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, VORBIS_CAPS);
+  } 
+}
+
+/*** video/x-theora ***********************************************************/
+
+static GstStaticCaps theora_caps = GST_STATIC_CAPS ("video/x-theora");
+#define THEORA_CAPS (gst_static_caps_get(&theora_caps))
+static void
+theora_type_find (GstTypeFind *tf, gpointer private)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 7); //42);
+
+  if (data) {
+    if (data[0] != 0x80) return;
+    if (memcmp (&data[1], "theora", 6) != 0) return;
+    /* FIXME: make this more reliable when specs are out */
+  }
+  gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, THEORA_CAPS);
+}
+
 /*** generic typefind for streams that have some data at a specific position***/
 
 typedef struct {
@@ -875,7 +1121,8 @@ start_with_type_find (GstTypeFind *tf, gpointer private)
   guint8 *data;
 
   GST_LOG ("trying to find mime type %s with the first %u bytes of data", 
-	   gst_caps_get_mime (start_with->caps), start_with->size);
+	   gst_structure_get_name (gst_caps_get_structure(start_with->caps, 0)),
+	   start_with->size);
   data = gst_type_find_peek (tf, 0, start_with->size);
   if (data && memcmp (data, start_with->data, start_with->size)==0) {
     gst_type_find_suggest (tf, start_with->probability, start_with->caps);
@@ -887,7 +1134,7 @@ G_BEGIN_DECLS{									\
   sw_data->data = _data;						      	\
   sw_data->size = _size;						      	\
   sw_data->probability = _probability;						\
-  sw_data->caps = gst_caps_new ("start_with_caps", name, NULL);			\
+  sw_data->caps = gst_caps_new_simple (name, NULL);			\
   TYPE_FIND_REGISTER (plugin, name, rank, start_with_type_find,			\
 		      ext, sw_data->caps, sw_data);				\
 }G_END_DECLS
@@ -912,7 +1159,7 @@ G_BEGIN_DECLS{									\
   sw_data->data = _data;						      	\
   sw_data->size = 4;							      	\
   sw_data->probability = GST_TYPE_FIND_MAXIMUM;					\
-  sw_data->caps = gst_caps_new ("riff_caps", name, NULL);			\
+  sw_data->caps = gst_caps_new_simple (name, NULL);			\
   TYPE_FIND_REGISTER (plugin, name, rank, riff_type_find,			\
 		      ext, sw_data->caps, sw_data);				\
 }G_END_DECLS
@@ -968,6 +1215,7 @@ plugin_init (GstPlugin *plugin)
   static gchar * gz_exts[] = {"gz", NULL};
   static gchar * zip_exts[] = {"zip", NULL};
   static gchar * compress_exts[] = {"Z", NULL};
+  static gchar * m4a_exts[] = {"m4a", NULL};
   
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions", GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
 
@@ -977,7 +1225,7 @@ plugin_init (GstPlugin *plugin)
 	  16, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-au", GST_RANK_MARGINAL, au_exts,
 	  ".snd", 4, GST_TYPE_FIND_MAXIMUM);
-  TYPE_FIND_REGISTER_RIFF (plugin, "video/avi", GST_RANK_PRIMARY, avi_exts, "AVI ");
+  TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY, avi_exts, "AVI ");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_SECONDARY, cdxa_exts, "CDXA");
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-flac", GST_RANK_PRIMARY,
 	  flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
@@ -988,15 +1236,17 @@ plugin_init (GstPlugin *plugin)
   TYPE_FIND_REGISTER (plugin, "audio/x-mod", GST_RANK_SECONDARY,
 	  mod_type_find, mod_exts, MOD_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/mpeg", GST_RANK_PRIMARY,
-	  mp3_type_find, mp3_exts, MP3_CAPS (0), NULL);
+	  mp3_type_find, mp3_exts, MP3_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg1", GST_RANK_PRIMARY,
-	  mpeg1_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS (1), NULL);
+	  mpeg1_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/mpeg2", GST_RANK_SECONDARY,
-	  mpeg2_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS (2), NULL);
+	  mpeg2_sys_type_find, mpeg_sys_exts, MPEG_SYS_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/ogg", GST_RANK_PRIMARY,
 	  ogg_exts, "OggS", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_SECONDARY,
 	  mpeg_video_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_MARGINAL,
+          mpeg_video_stream_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/quicktime", GST_RANK_SECONDARY,
 	  qt_type_find, qt_exts, QT_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/vnd.rn-realmedia", GST_RANK_SECONDARY, 
@@ -1021,9 +1271,9 @@ plugin_init (GstPlugin *plugin)
   TYPE_FIND_REGISTER (plugin, "image/bmp", GST_RANK_PRIMARY,
 	  bmp_type_find, bmp_exts, BMP_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "image/tiff", GST_RANK_PRIMARY,
-	  tiff_type_find, tiff_exts, TIFF_CAPS(0), NULL);
-  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-matroska", GST_RANK_SECONDARY,
-	  matroska_exts, "\032\105\337\243\223\102\202\210matroska", 16, GST_TYPE_FIND_MAXIMUM);
+	  tiff_type_find, tiff_exts, TIFF_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/x-matroska", GST_RANK_PRIMARY,
+	  matroska_type_find, matroska_exts, MATROSKA_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/x-dv", GST_RANK_SECONDARY,
 	  dv_type_find, dv_exts, DV_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-sid", GST_RANK_MARGINAL,
@@ -1046,6 +1296,12 @@ plugin_init (GstPlugin *plugin)
 	  zip_exts, "PK\003\004", 4, GST_TYPE_FIND_LIKELY);
   TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-compress", GST_RANK_SECONDARY,
 	  compress_exts, "\037\235", 2, GST_TYPE_FIND_LIKELY);
+  TYPE_FIND_REGISTER (plugin, "audio/x-vorbis", GST_RANK_PRIMARY,
+	  vorbis_type_find, NULL, VORBIS_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "video/x-theora", GST_RANK_PRIMARY,
+	  theora_type_find, NULL, THEORA_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY,
+	  m4a_type_find, m4a_exts, AAC_CAPS, NULL);
   
   return TRUE;
 }
@@ -1057,7 +1313,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
