@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include <string.h>
 /*#define DEBUG_ENABLED */
 #include <gstvideofilter.h>
 
@@ -107,195 +108,154 @@ static void gst_videofilter_class_init (gpointer g_class, gpointer class_data)
   gobject_class->get_property = gst_videofilter_get_property;
 }
 
-static GstCaps *gst_videofilter_format_get_caps(GstVideofilterFormat *format)
+static GstStructure *gst_videofilter_format_get_structure(GstVideofilterFormat *format)
 {
   unsigned int fourcc;
-  GstCaps *caps;
+  GstStructure *structure;
 
   if(format->filter_func==NULL)
     return NULL;
 
   fourcc = GST_MAKE_FOURCC(format->fourcc[0],format->fourcc[1],format->fourcc[2],format->fourcc[3]);
 
-  if(format->bpp){
-    caps = GST_CAPS_NEW ("videofilter", "video/x-raw-rgb",
-		"format", GST_PROPS_FOURCC (fourcc),
-		"depth", GST_PROPS_INT(format->bpp),
-		"bpp", GST_PROPS_INT(format->depth),
-		"endianness", GST_PROPS_INT(format->endianness),
-		"red_mask", GST_PROPS_INT(format->red_mask),
-		"green_mask", GST_PROPS_INT(format->green_mask),
-		"blue_mask", GST_PROPS_INT(format->blue_mask));
+  if(format->depth){
+    structure = gst_structure_new ("video/x-raw-rgb",
+	"depth", G_TYPE_INT, format->depth,
+	"bpp", G_TYPE_INT, format->bpp,
+	"endianness", G_TYPE_INT, format->endianness,
+	"red_mask", G_TYPE_INT, format->red_mask,
+	"green_mask", G_TYPE_INT, format->green_mask,
+	"blue_mask", G_TYPE_INT, format->blue_mask, NULL);
   }else{
-    caps = GST_CAPS_NEW ("videoflip", "video/x-raw-yuv",
-		"format", GST_PROPS_FOURCC (fourcc),
-		"height", GST_PROPS_INT_RANGE (1,G_MAXINT),
-		"width", GST_PROPS_INT_RANGE (1,G_MAXINT),
-		"framerate", GST_PROPS_FLOAT_RANGE (0,G_MAXFLOAT)
-		);
+    structure = gst_structure_new ("video/x-raw-yuv",
+	"format", GST_TYPE_FOURCC, fourcc, NULL);
+  }
+
+  gst_structure_set(structure,
+      "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+      "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
+
+  return structure;
+}
+
+GstCaps * gst_videofilter_class_get_capslist(GstVideofilterClass *klass)
+{
+  GstCaps *caps;
+  GstStructure *structure;
+  int i;
+
+  caps = gst_caps_new_empty();
+  for(i=0;i<klass->formats->len;i++){
+    structure = gst_videofilter_format_get_structure(g_ptr_array_index(klass->formats,i));
+    gst_caps_append_structure (caps, structure);
   }
 
   return caps;
 }
 
-GstCaps * gst_videofilter_class_get_capslist(GstVideofilterClass *klass)
-{
-  static GstCaps *capslist = NULL;
-  GstCaps *caps;
-  int i;
-
-  if (capslist){
-    gst_caps_ref(capslist);
-    return capslist;
-  }
-
-  for(i=0;i<klass->formats->len;i++){
-    caps = gst_videofilter_format_get_caps(g_ptr_array_index(klass->formats,i));
-    capslist = gst_caps_append(capslist, caps);
-  }
-
-  gst_caps_ref(capslist);
-  return capslist;
-}
-
 static GstCaps *
-gst_videofilter_sink_getcaps (GstPad *pad, GstCaps *caps)
+gst_videofilter_getcaps (GstPad *pad)
 {
   GstVideofilter *videofilter;
   GstVideofilterClass *klass;
-  GstCaps *capslist = NULL;
-  GstCaps *peercaps;
-  GstCaps *sizecaps;
-  int i;
+  //GstCaps *caps;
+  GstCaps *othercaps;
+  GstPad *otherpad;
+  //int i;
 
-  GST_DEBUG("gst_videofilter_sink_getcaps");
+  GST_DEBUG("gst_videofilter_getcaps");
   videofilter = GST_VIDEOFILTER (gst_pad_get_parent (pad));
   
   klass = GST_VIDEOFILTER_CLASS(G_OBJECT_GET_CLASS(videofilter));
 
-  /* get list of peer's caps */
-  peercaps = gst_pad_get_allowed_caps (videofilter->srcpad);
+  otherpad = (pad == videofilter->srcpad) ? videofilter->sinkpad :
+    videofilter->srcpad;
 
+  othercaps = gst_pad_get_allowed_caps (otherpad);
+
+  return othercaps;
+#if 0
   /* FIXME videofilter doesn't allow passthru of video formats it
    * doesn't understand. */
   /* Look through our list of caps and find those that match with
    * the peer's formats.  Create a list of them. */
   /* FIXME optimize if peercaps == NULL */
+  caps = gst_caps_new_empty ();
   for(i=0;i<klass->formats->len;i++){
     GstCaps *icaps;
-    GstCaps *fromcaps = gst_videofilter_format_get_caps(g_ptr_array_index(
-	  klass->formats,i));
+    GstCaps *fromcaps;
+    
+    fromcaps = gst_caps_new_full (gst_videofilter_format_get_structure (
+	  g_ptr_array_index (klass->formats,i)), NULL);
 
-    icaps = gst_caps_intersect(fromcaps, peercaps);
-    //if(gst_caps_is_always_compatible(fromcaps, peercaps)){
+    icaps = gst_caps_intersect (fromcaps, peercaps);
     if(icaps != NULL){
-      capslist = gst_caps_append(capslist, fromcaps);
+      gst_caps_append (caps, fromcaps);
+    } else {
+      gst_caps_free (fromcaps);
     }
-    //gst_caps_unref (fromcaps);
-    if(icaps) gst_caps_unref (icaps);
+    if(icaps) gst_caps_free (icaps);
   }
-  gst_caps_unref (peercaps);
-
-  sizecaps = GST_CAPS_NEW("videofilter_size","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-
-  caps = gst_caps_intersect(capslist, sizecaps);
-  gst_caps_unref (sizecaps);
+  gst_caps_free (peercaps);
 
   return caps;
+#endif
 }
 
 static GstPadLinkReturn
-gst_videofilter_src_link (GstPad *pad, GstCaps *caps)
+gst_videofilter_link (GstPad *pad, const GstCaps *caps)
 {
   GstVideofilter *videofilter;
-  GstPadLinkReturn ret;
-  GstCaps *peercaps;
+  GstStructure *structure;
+  gboolean ret;
+  int width, height;
+  double framerate;
+  GstPadLinkReturn lret;
+  GstPad *otherpad;
 
   GST_DEBUG("gst_videofilter_src_link");
   videofilter = GST_VIDEOFILTER (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_LINK_DELAYED;
-  }
+  otherpad = (pad == videofilter->srcpad) ? videofilter->sinkpad :
+    videofilter->srcpad;
 
-  gst_caps_debug(caps,"ack");
+  structure = gst_caps_get_structure (caps, 0);
 
-  videofilter->format = gst_videofilter_find_format_by_caps (videofilter,caps);
+  videofilter->format = gst_videofilter_find_format_by_structure (
+      videofilter, structure);
   g_return_val_if_fail(videofilter->format, GST_PAD_LINK_REFUSED);
 
-  gst_caps_get_int (caps, "width", &videofilter->to_width);
-  gst_caps_get_int (caps, "height", &videofilter->to_height);
+  ret = gst_structure_get_int (structure, "width", &width);
+  ret &= gst_structure_get_int (structure, "height", &height);
+  ret &= gst_structure_get_double (structure, "framerate", &framerate);
 
-  GST_DEBUG("width %d height %d",videofilter->to_width,videofilter->to_height);
+  if (!ret) return GST_PAD_LINK_REFUSED;
 
-  peercaps = gst_caps_copy(caps);
+  lret = gst_pad_try_set_caps (otherpad, caps);
+  if (GST_PAD_LINK_FAILED (lret)) return lret;
 
-  gst_caps_set(peercaps, "width", GST_PROPS_INT_RANGE (0, G_MAXINT));
-  gst_caps_set(peercaps, "height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+  GST_DEBUG("width %d height %d",width,height);
 
-  ret = gst_pad_try_set_caps (videofilter->srcpad, peercaps);
-
-  gst_caps_unref(peercaps);
-
-  if(ret==GST_PAD_LINK_OK){
-    caps = gst_pad_get_caps (videofilter->srcpad);
-
-    gst_caps_get_int (caps, "width", &videofilter->from_width);
-    gst_caps_get_int (caps, "height", &videofilter->from_height);
-    //gst_videofilter_setup(videofilter);
+#if 0
+  if (pad == videofilter->srcpad) {
+    videofilter->to_width = width;
+    videofilter->to_height = height;
+  } else {
+    videofilter->from_width = width;
+    videofilter->from_height = height;
   }
-
-  return ret;
-}
-
-static GstPadLinkReturn
-gst_videofilter_sink_link (GstPad *pad, GstCaps *caps)
-{
-  GstVideofilter *videofilter;
-  GstPadLinkReturn ret;
-  GstCaps *peercaps;
-
-  GST_DEBUG("gst_videofilter_sink_link");
-  videofilter = GST_VIDEOFILTER (gst_pad_get_parent (pad));
-
-  if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_LINK_DELAYED;
-  }
-
-  videofilter->format = gst_videofilter_find_format_by_caps (videofilter,caps);
-  GST_DEBUG("sink_link: %s\n",gst_caps_to_string(caps));
-  g_return_val_if_fail(videofilter->format, GST_PAD_LINK_REFUSED);
-
-  gst_caps_get_int (caps, "width", &videofilter->from_width);
-  gst_caps_get_int (caps, "height", &videofilter->from_height);
-  gst_caps_get_float (caps, "framerate", &videofilter->framerate);
+#endif
+  videofilter->to_width = width;
+  videofilter->to_height = height;
+  videofilter->from_width = width;
+  videofilter->from_height = height;
+  videofilter->framerate = framerate;
 
   gst_videofilter_setup(videofilter);
 
-  peercaps = gst_caps_copy(caps);
-
-  gst_caps_set(peercaps, "width", GST_PROPS_INT (videofilter->to_width));
-  gst_caps_set(peercaps, "height", GST_PROPS_INT (videofilter->to_height));
-  gst_caps_set(peercaps, "framerate", GST_PROPS_FLOAT (videofilter->framerate));
-
-  GST_DEBUG("setting %s\n",gst_caps_to_string(peercaps));
-
-  ret = gst_pad_try_set_caps (videofilter->srcpad, peercaps);
-
-  //gst_caps_unref(peercaps);
-
-  if(ret==GST_PAD_LINK_OK || ret==GST_PAD_LINK_DONE){
-    caps = gst_pad_get_caps (videofilter->srcpad);
-
-    //gst_caps_get_int (caps, "width", &videofilter->to_width);
-    //gst_caps_get_int (caps, "height", &videofilter->to_height);
-    //gst_videofilter_setup(videofilter);
-  }
-
-  return ret;
+  return GST_PAD_LINK_OK;
 }
 
 static void
@@ -312,16 +272,16 @@ gst_videofilter_init (GTypeInstance *instance, gpointer g_class)
   videofilter->sinkpad = gst_pad_new_from_template(pad_template, "sink");
   gst_element_add_pad(GST_ELEMENT(videofilter),videofilter->sinkpad);
   gst_pad_set_chain_function(videofilter->sinkpad,gst_videofilter_chain);
-  gst_pad_set_link_function(videofilter->sinkpad,gst_videofilter_sink_link);
-  gst_pad_set_getcaps_function(videofilter->sinkpad,gst_videofilter_sink_getcaps);
+  gst_pad_set_link_function(videofilter->sinkpad,gst_videofilter_link);
+  gst_pad_set_getcaps_function(videofilter->sinkpad,gst_videofilter_getcaps);
 
   pad_template = gst_element_class_get_pad_template(GST_ELEMENT_CLASS(g_class),
       "src");
   g_return_if_fail(pad_template != NULL);
   videofilter->srcpad = gst_pad_new_from_template(pad_template, "src");
   gst_element_add_pad(GST_ELEMENT(videofilter),videofilter->srcpad);
-  gst_pad_set_link_function(videofilter->srcpad,gst_videofilter_src_link);
-  //gst_pad_set_getcaps_function(videofilter->srcpad,gst_videofilter_src_getcaps);
+  gst_pad_set_link_function(videofilter->srcpad,gst_videofilter_link);
+  gst_pad_set_getcaps_function(videofilter->srcpad,gst_videofilter_getcaps);
 
   videofilter->inited = FALSE;
 }
@@ -362,13 +322,17 @@ gst_videofilter_chain (GstPad *pad, GstData *_data)
   	size, videofilter->from_buf_size,
   	videofilter->to_buf_size);
 
-  g_return_if_fail (size == videofilter->from_buf_size);
+  g_return_if_fail (size >= videofilter->from_buf_size);
 
-  outbuf = gst_buffer_new();
-  /* FIXME: handle bufferpools */
-  GST_BUFFER_SIZE(outbuf) = videofilter->to_buf_size;
-  GST_BUFFER_DATA(outbuf) = g_malloc (videofilter->to_buf_size);
+  if (size > videofilter->from_buf_size) {
+    GST_INFO("buffer size %ld larger than expected (%d)",
+  	size, videofilter->from_buf_size);
+  }
+
+  outbuf = gst_pad_alloc_buffer(videofilter->srcpad, GST_BUFFER_OFFSET_NONE,
+      videofilter->to_buf_size);
   GST_BUFFER_TIMESTAMP(outbuf) = GST_BUFFER_TIMESTAMP(buf);
+  GST_BUFFER_DURATION(outbuf) = GST_BUFFER_DURATION(buf);
 
   g_return_if_fail(videofilter->format);
   GST_DEBUG ("format %s",videofilter->format->fourcc);
@@ -437,6 +401,7 @@ void gst_videofilter_set_output_size(GstVideofilter *videofilter,
 {
   int ret;
   GstCaps *srccaps;
+  GstStructure *structure;
 
   g_return_if_fail(GST_IS_VIDEOFILTER(videofilter));
 
@@ -444,20 +409,19 @@ void gst_videofilter_set_output_size(GstVideofilter *videofilter,
   videofilter->to_height = height;
 
   videofilter->to_buf_size = (videofilter->to_width * videofilter->to_height
-      * videofilter->format->depth)/8;
+      * videofilter->format->bpp)/8;
 
-  srccaps = gst_caps_copy(gst_pad_get_caps(videofilter->srcpad));
+  srccaps = gst_caps_copy (gst_pad_get_negotiated_caps(videofilter->srcpad));
+  structure = gst_caps_get_structure (srccaps, 0);
 
-  if(!GST_CAPS_IS_FIXED(srccaps)){
-    return;
-  }
-
-  gst_caps_set(srccaps, "width", GST_PROPS_INT (videofilter->to_width));
-  gst_caps_set(srccaps, "height", GST_PROPS_INT (videofilter->to_height));
+  gst_structure_set (structure, "width", G_TYPE_INT, width,
+      "height", G_TYPE_INT, height, NULL);
 
   ret = gst_pad_try_set_caps (videofilter->srcpad, srccaps);
 
-  g_return_if_fail(ret<0);
+  if (ret < 0) {
+    g_critical ("could not set output size");
+  }
 }
 
 static void gst_videofilter_setup(GstVideofilter *videofilter)
@@ -484,36 +448,62 @@ static void gst_videofilter_setup(GstVideofilter *videofilter)
   g_return_if_fail(videofilter->to_height > 0);
 
   videofilter->from_buf_size = (videofilter->from_width * videofilter->from_height *
-      videofilter->format->depth) / 8;
+      videofilter->format->bpp) / 8;
   videofilter->to_buf_size = (videofilter->to_width * videofilter->to_height *
-      videofilter->format->depth) / 8;
+      videofilter->format->bpp) / 8;
 
   videofilter->inited = TRUE;
 }
 
-GstVideofilterFormat *gst_videofilter_find_format_by_caps(GstVideofilter *videofilter,
-    GstCaps *caps)
+GstVideofilterFormat *gst_videofilter_find_format_by_structure (
+    GstVideofilter *videofilter, const GstStructure *structure)
 {
   int i;
-  GstCaps *c;
   GstVideofilterClass *klass;
   GstVideofilterFormat *format;
+  gboolean ret;
 
   klass = GST_VIDEOFILTER_CLASS(G_OBJECT_GET_CLASS(videofilter));
 
-  g_return_val_if_fail(caps != NULL, NULL);
+  g_return_val_if_fail(structure != NULL, NULL);
 
-  for(i=0;i<klass->formats->len;i++){
-    format = g_ptr_array_index(klass->formats,i);
-    c = gst_videofilter_format_get_caps(format);
+  if (strcmp (gst_structure_get_name (structure), "video/x-raw-yuv") == 0) {
+    guint32 fourcc;
 
-    if(c){
-      if(gst_caps_is_always_compatible(caps, c)){
-	gst_caps_unref(c);
-	return format;
+    ret = gst_structure_get_fourcc (structure, "format", &fourcc);
+    if (!ret) return NULL;
+    for(i=0;i<klass->formats->len;i++){
+      guint32 format_fourcc;
+      format = g_ptr_array_index(klass->formats,i);
+      format_fourcc = GST_STR_FOURCC (format->fourcc);
+      if (format->depth == 0 && format_fourcc == fourcc) {
+        return format;
+      }
+    }
+  } else if (strcmp (gst_structure_get_name (structure), "video/x-raw-rgb")
+      == 0) {
+    int bpp;
+    int depth;
+    int endianness;
+    int red_mask;
+    int green_mask;
+    int blue_mask;
+
+    ret = gst_structure_get_int (structure, "bpp", &bpp);
+    ret &= gst_structure_get_int (structure, "depth", &depth);
+    ret &= gst_structure_get_int (structure, "endianness", &endianness);
+    ret &= gst_structure_get_int (structure, "red_mask", &red_mask);
+    ret &= gst_structure_get_int (structure, "green_mask", &green_mask);
+    ret &= gst_structure_get_int (structure, "blue_mask", &blue_mask);
+    if (!ret) return NULL;
+    for(i=0;i<klass->formats->len;i++){
+      format = g_ptr_array_index(klass->formats,i);
+      if (format->bpp == bpp && format->depth == depth &&
+          format->endianness == endianness && format->red_mask == red_mask &&
+          format->green_mask == green_mask && format->blue_mask == blue_mask) {
+        return format;
       }
     }
-    gst_caps_unref(c);
   }
 
   return NULL;
@@ -527,23 +517,15 @@ void gst_videofilter_class_add_format(GstVideofilterClass *videofilterclass,
 
 void gst_videofilter_class_add_pad_templates (GstVideofilterClass *videofilter_class)
 {
-  GstCaps *caps;
   GstElementClass *element_class = GST_ELEMENT_CLASS (videofilter_class);
 
-  caps = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+  gst_element_class_add_pad_template (element_class,
+      gst_pad_template_new("src", GST_PAD_SRC, GST_PAD_ALWAYS,
+	gst_videofilter_class_get_capslist (videofilter_class)));
 
   gst_element_class_add_pad_template (element_class,
-      GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, 
-        gst_caps_intersect(caps,
-          gst_videofilter_class_get_capslist (videofilter_class))));
-
-  gst_element_class_add_pad_template (element_class,
-      GST_PAD_TEMPLATE_NEW("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
-        gst_caps_intersect(caps,
-          gst_videofilter_class_get_capslist (videofilter_class))));
+      gst_pad_template_new("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
+	gst_videofilter_class_get_capslist (videofilter_class)));
 }
 
 static gboolean
@@ -560,7 +542,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
