@@ -68,85 +68,36 @@ gst_videoscale_method_get_type (void)
 static GstCaps *
 gst_videoscale_get_capslist(void)
 {
-  static GstCaps *capslist = NULL;
   GstCaps *caps;
   int i;
 
-  if (capslist){
-    gst_caps_ref(capslist);
-    return capslist;
-  }
-
+  caps = gst_caps_new_empty();
   for(i=0;i<videoscale_n_formats;i++){
-    caps = videoscale_get_caps(videoscale_formats + i);
-    capslist = gst_caps_append(capslist, caps);
+    gst_caps_append_structure (caps,
+	videoscale_get_structure (videoscale_formats + i));
   }
 
-  gst_caps_ref(capslist);
-  return capslist;
+  return caps;
 }
 
 static GstPadTemplate *
 gst_videoscale_src_template_factory(void)
 {
-  static GstPadTemplate *templ = NULL;
-
-  if(!templ){
-    GstCaps *caps;
-    GstCaps *caps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-    GstCaps *caps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-
-    caps = gst_caps_intersect(caps1, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps1);
-    caps1 = caps;
-    caps = gst_caps_intersect(caps2, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps2);
-    caps2 = caps;
-    caps = gst_caps_append(caps1, caps2);
-
-    templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps);
-  }
-  return templ;
+  return gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
+      gst_videoscale_get_capslist());
 }
 
 static GstPadTemplate *
 gst_videoscale_sink_template_factory(void)
 {
-  static GstPadTemplate *templ = NULL;
-
-  if(!templ){
-    GstCaps *caps;
-    GstCaps *caps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-    GstCaps *caps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-
-    caps = gst_caps_intersect(caps1, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps1);
-    caps1 = caps;
-    caps = gst_caps_intersect(caps2, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps2);
-    caps2 = caps;
-    caps = gst_caps_append(caps1, caps2);
-
-    templ = GST_PAD_TEMPLATE_NEW("sink", GST_PAD_SINK, GST_PAD_ALWAYS, caps);
-  }
-  return templ;
+  return gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
+      gst_videoscale_get_capslist());
 }
 
 static void	gst_videoscale_base_init	(gpointer g_class);
 static void	gst_videoscale_class_init	(GstVideoscaleClass *klass);
 static void	gst_videoscale_init		(GstVideoscale *videoscale);
+static gboolean gst_videoscale_handle_src_event (GstPad *pad, GstEvent *event);
 
 static void	gst_videoscale_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void	gst_videoscale_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
@@ -186,8 +137,8 @@ gst_videoscale_base_init (gpointer g_class)
 
   gst_element_class_set_details (element_class, &videoscale_details);
 
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (gst_videoscale_sink_template_factory));
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (gst_videoscale_src_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_videoscale_sink_template_factory());
+  gst_element_class_add_pad_template (element_class, gst_videoscale_src_template_factory());
 }
 static void
 gst_videoscale_class_init (GstVideoscaleClass *klass)
@@ -210,169 +161,113 @@ gst_videoscale_class_init (GstVideoscaleClass *klass)
 }
 
 static GstCaps *
-gst_videoscale_getcaps (GstPad *pad, GstCaps *caps)
+gst_videoscale_getcaps (GstPad *pad)
 {
   GstVideoscale *videoscale;
-  //GstCaps *capslist = NULL;
-  GstCaps *peercaps;
-  //GstCaps *sizecaps1, *sizecaps2;
-  //GstCaps *sizecaps;
-  GstCaps *handled_caps;
-  GstCaps *icaps;
+  GstCaps *othercaps;
+  GstCaps *caps;
   GstPad *otherpad;
+  int i;
 
   GST_DEBUG ("gst_videoscale_getcaps");
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
   
-  /* get list of peer's caps */
-  if(pad == videoscale->srcpad){
-    GST_DEBUG("getting caps of srcpad");
-    otherpad = videoscale->sinkpad;
-  }else{
-    GST_DEBUG("getting caps of sinkpad");
-    otherpad = videoscale->srcpad;
-  }
-  if (!GST_PAD_IS_LINKED (otherpad)){
-    GST_DEBUG ("otherpad not linked");
-    return GST_PAD_LINK_DELAYED;
-  }
-  peercaps = gst_pad_get_allowed_caps (GST_PAD_PEER(otherpad));
+  otherpad = (pad == videoscale->srcpad) ? videoscale->sinkpad :
+    videoscale->srcpad;
+  othercaps = gst_pad_get_allowed_caps (otherpad);
 
-  GST_DEBUG("othercaps are %s", gst_caps_to_string(peercaps));
+  GST_DEBUG ("othercaps are: " GST_PTR_FORMAT, othercaps);
 
-  {
-    GstCaps *caps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-    GstCaps *caps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+  caps = gst_caps_copy (othercaps);
+  for(i=0;i<gst_caps_get_size(caps);i++) {
+    GstStructure *structure = gst_caps_get_structure (caps, i);
 
-    caps = gst_caps_intersect(caps1, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps1);
-    caps1 = caps;
-    caps = gst_caps_intersect(caps2, gst_videoscale_get_capslist ());
-    gst_caps_unref (caps2);
-    caps2 = caps;
-    handled_caps = gst_caps_append(caps1, caps2);
+    gst_structure_set (structure,
+	"width", GST_TYPE_INT_RANGE, 16, G_MAXINT,
+	"height", GST_TYPE_INT_RANGE, 16, G_MAXINT,
+	NULL);
   }
 
-  icaps = gst_caps_intersect (handled_caps, gst_caps_copy(peercaps));
+  GST_DEBUG ("returning caps: " GST_PTR_FORMAT, caps);
 
-  GST_DEBUG("returning caps %s", gst_caps_to_string (icaps));
-
-  return icaps;
+  return caps;
 }
 
 
 static GstPadLinkReturn
-gst_videoscale_link (GstPad *pad, GstCaps *caps)
+gst_videoscale_link (GstPad *pad, const GstCaps *caps)
 {
   GstVideoscale *videoscale;
   GstPadLinkReturn ret;
-  GstCaps *othercaps;
   GstPad *otherpad;
-  GstCaps *icaps;
+  GstStructure *structure;
+  struct videoscale_format_struct *format;
+  int height, width;
 
-  GST_DEBUG ("gst_videoscale_link");
+  GST_DEBUG ("gst_videoscale_link %s\n", gst_caps_to_string (caps));
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_LINK_DELAYED;
+  otherpad = (pad == videoscale->srcpad) ? videoscale->sinkpad :
+    videoscale->srcpad;
+
+  structure = gst_caps_get_structure (caps, 0);
+  ret = gst_structure_get_int (structure, "width", &width);
+  ret &= gst_structure_get_int (structure, "height", &height);
+
+  format = videoscale_find_by_structure (structure);
+
+  if(!ret || format==NULL) return GST_PAD_LINK_REFUSED;
+
+  ret = gst_pad_try_set_caps (otherpad, caps);
+  if (ret == GST_PAD_LINK_OK) {
+    /* cool, we can use passthru */
+
+    videoscale->format = format;
+    videoscale->to_width = width;
+    videoscale->to_height = height;
+    videoscale->from_width = width;
+    videoscale->from_height = height;
+
+    gst_videoscale_setup (videoscale);
+
+    return GST_PAD_LINK_OK;
   }
 
+  if (gst_pad_is_negotiated (otherpad)) {
+    GstCaps *newcaps = gst_caps_copy (caps);
+
+    if (pad == videoscale->srcpad) {
+      gst_caps_set_simple (newcaps,
+          "width", G_TYPE_INT, videoscale->from_width,
+          "height", G_TYPE_INT, videoscale->from_height,
+          NULL);
+    } else {
+      gst_caps_set_simple (newcaps,
+          "width", G_TYPE_INT, videoscale->to_width,
+          "height", G_TYPE_INT, videoscale->to_height,
+          NULL);
+    }
+    ret = gst_pad_try_set_caps (otherpad, newcaps);
+    if (GST_PAD_LINK_FAILED (ret)) {
+      return GST_PAD_LINK_REFUSED;
+    }
+  }
+
+  videoscale->passthru = FALSE;
+
   if (pad == videoscale->srcpad) {
-    otherpad = videoscale->sinkpad;
+    videoscale->to_width = width;
+    videoscale->to_height = height;
   } else {
-    otherpad = videoscale->srcpad;
+    videoscale->from_width = width;
+    videoscale->from_height = height;
   }
-  if (otherpad == NULL) {
-    return GST_PAD_LINK_DELAYED;
+  videoscale->format = format;
+
+  if (gst_pad_is_negotiated (otherpad)) {
+    gst_videoscale_setup (videoscale);
   }
 
-  othercaps = GST_PAD_CAPS (otherpad);
-
-  if (othercaps) {
-    GstCaps *othercaps_wh;
-
-    GST_DEBUG ("otherpad caps are already set");
-    GST_DEBUG ("otherpad caps are %s", gst_caps_to_string(othercaps));
-
-    othercaps_wh = gst_caps_copy (othercaps);
-    gst_caps_set(othercaps_wh, "width", GST_PROPS_INT_RANGE (1, G_MAXINT),
-        "height", GST_PROPS_INT_RANGE (1, G_MAXINT), NULL);
-
-#if 1
-    icaps = othercaps;
-#else
-    /* FIXME this is disabled because videotestsrc ! videoscale ! ximagesink
-     * doesn't negotiate caps correctly the first time, so we pretend it's
-     * still ok here. */
-    icaps = gst_caps_intersect (othercaps_wh, caps);
-
-    GST_DEBUG ("intersected caps are %s", gst_caps_to_string(icaps));
-
-    if (icaps == NULL) {
-      /* the new caps are not compatible with the existing, set caps */
-      /* currently, we don't force renegotiation */
-      return GST_PAD_LINK_REFUSED;
-    }
-#endif
-
-    if (!GST_CAPS_IS_FIXED (icaps)) {
-      return GST_PAD_LINK_REFUSED;
-    }
-
-    /* ok, we have a candidate caps */
-  } else {
-    GstCaps *othercaps_wh;
-
-    GST_DEBUG ("otherpad caps are unset");
-
-    othercaps = gst_pad_get_allowed_caps (pad);
-
-    othercaps_wh = gst_caps_copy (othercaps);
-    gst_caps_set(othercaps_wh, "width", GST_PROPS_INT_RANGE (1, G_MAXINT),
-        "height", GST_PROPS_INT_RANGE (1, G_MAXINT), NULL);
-
-    icaps = gst_caps_intersect (othercaps_wh, caps);
-
-    GST_DEBUG ("intersected caps are %s", gst_caps_to_string(icaps));
-
-    if (icaps == NULL) {
-      /* the new caps are not compatible with the existing, set caps */
-      /* currently, we don't force renegotiation */
-      return GST_PAD_LINK_REFUSED;
-    }
-
-    if (!GST_CAPS_IS_FIXED (icaps)) {
-      return GST_PAD_LINK_REFUSED;
-    }
-
-    /* ok, we have a candidate caps */
-    ret = gst_pad_try_set_caps (otherpad, gst_caps_copy(icaps));
-    if (ret == GST_PAD_LINK_DELAYED || ret == GST_PAD_LINK_REFUSED) {
-      return ret;
-    }
-  }
-
-  if (pad == videoscale->srcpad) {
-    gst_caps_get_int (icaps, "width", &videoscale->from_width);
-    gst_caps_get_int (icaps, "height", &videoscale->from_height);
-    gst_caps_get_int (caps, "width", &videoscale->to_width);
-    gst_caps_get_int (caps, "height", &videoscale->to_height);
-  } else {
-    gst_caps_get_int (icaps, "width", &videoscale->to_width);
-    gst_caps_get_int (icaps, "height", &videoscale->to_height);
-    gst_caps_get_int (caps, "width", &videoscale->from_width);
-    gst_caps_get_int (caps, "height", &videoscale->from_height);
-  }
-
-  videoscale->format = videoscale_find_by_caps (caps);
-  gst_videoscale_setup(videoscale);
-
   return GST_PAD_LINK_OK;
 }
 
@@ -381,7 +276,7 @@ gst_videoscale_init (GstVideoscale *videoscale)
 {
   GST_DEBUG ("gst_videoscale_init");
   videoscale->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (gst_videoscale_sink_template_factory),
+		  gst_videoscale_sink_template_factory(),
 		  "sink");
   gst_element_add_pad(GST_ELEMENT(videoscale),videoscale->sinkpad);
   gst_pad_set_chain_function(videoscale->sinkpad,gst_videoscale_chain);
@@ -389,9 +284,10 @@ gst_videoscale_init (GstVideoscale *videoscale)
   gst_pad_set_getcaps_function(videoscale->sinkpad,gst_videoscale_getcaps);
 
   videoscale->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (gst_videoscale_src_template_factory),
+		  gst_videoscale_src_template_factory(),
 		  "src");
   gst_element_add_pad(GST_ELEMENT(videoscale),videoscale->srcpad);
+  gst_pad_set_event_function (videoscale->srcpad, gst_videoscale_handle_src_event);
   gst_pad_set_link_function(videoscale->srcpad,gst_videoscale_link);
   gst_pad_set_getcaps_function(videoscale->srcpad,gst_videoscale_getcaps);
 
@@ -402,6 +298,39 @@ gst_videoscale_init (GstVideoscale *videoscale)
   /*videoscale->method = GST_VIDEOSCALE_POINT_SAMPLE; */
 }
 
+static gboolean
+gst_videoscale_handle_src_event (GstPad *pad, GstEvent *event)
+{
+  GstVideoscale *videoscale;
+  double a;
+  GstStructure *structure;
+  GstEvent *new_event;
+
+  videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_NAVIGATION:
+      structure = gst_structure_copy (event->event_data.structure.structure);
+      if (gst_structure_get_double (event->event_data.structure.structure,
+          "pointer_x", &a)) {
+        gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE,
+            a*videoscale->from_width/videoscale->to_width, NULL);
+      }
+      if (gst_structure_get_double (event->event_data.structure.structure,
+          "pointer_y", &a)) {
+        gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE,
+            a*videoscale->from_height/videoscale->to_height, NULL);
+      }
+      gst_event_unref (event);
+      new_event = gst_event_new (GST_EVENT_NAVIGATION);
+      new_event->event_data.structure.structure = structure;
+      return gst_pad_event_default (pad, new_event);
+      break;
+    default:
+      return gst_pad_event_default (pad, event);
+      break;
+  }
+}
 
 static void
 gst_videoscale_chain (GstPad *pad, GstData *_data)
@@ -441,14 +370,13 @@ gst_videoscale_chain (GstPad *pad, GstData *_data)
 
   g_return_if_fail (size == videoscale->from_buf_size);
 
-  outbuf = gst_buffer_new();
-  /* FIXME: handle bufferpools */
-  GST_BUFFER_SIZE(outbuf) = videoscale->to_buf_size;
-  GST_BUFFER_DATA(outbuf) = g_malloc (videoscale->to_buf_size);
+  outbuf = gst_pad_alloc_buffer (videoscale->srcpad,
+                                 GST_BUFFER_OFFSET_NONE, videoscale->to_buf_size);
+  
   GST_BUFFER_TIMESTAMP(outbuf) = GST_BUFFER_TIMESTAMP(buf);
 
   g_return_if_fail(videoscale->format);
-  GST_DEBUG ("format %s",videoscale->format->fourcc);
+  GST_DEBUG ("format " GST_FOURCC_FORMAT,GST_FOURCC_ARGS(videoscale->format->fourcc));
   g_return_if_fail(videoscale->format->scale);
 
   videoscale->format->scale(videoscale, GST_BUFFER_DATA(outbuf), data);
