@@ -18,22 +18,35 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
 #include <gst/control/control.h>
+#include <gst/mixer/mixer.h>
 #include "gstvolume.h"
 
+/* some defines for audio processing */
+/* the volume factor is a range from 0.0 to (arbitrary) 4.0
+ * we map 1.0 to VOLUME_UNITY_INT
+ */
+#define VOLUME_UNITY_INT	8192		/* internal int for unity */
+#define VOLUME_UNITY_BIT_SHIFT	13		/* number of bits to shift
+                                                   for unity */
+#define VOLUME_MAX_FLOAT	4.0
+#define VOLUME_MAX_INT16	32767
+#define VOLUME_MIN_INT16	-32768
 
+/* number of steps we use for the mixer interface to go from 0.0 to 1.0 */
+# define VOLUME_STEPS		100
 
 static GstElementDetails volume_details = {
   "Volume",
-  "Filter/Audio/Effect",
-  "LGPL",
+  "Filter/Effect/Audio",
   "Set volume on audio/raw streams",
-  VERSION,
   "Andy Wingo <apwingo@eos.ncsu.edu>",
-  "(C) 2001"
 };
 
 
@@ -50,63 +63,52 @@ enum {
   ARG_VOLUME
 };
 
-GST_PAD_TEMPLATE_FACTORY (volume_sink_factory,
+static GstStaticPadTemplate volume_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "volume_float_sink",
-    "audio/raw",
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "format",     GST_PROPS_STRING ("float"),
-    "layout",     GST_PROPS_STRING ("gfloat"),
-    "intercept",  GST_PROPS_FLOAT(0.0),
-    "slope",      GST_PROPS_FLOAT(1.0),
-    "channels",   GST_PROPS_INT (1)
-  ),
-  GST_CAPS_NEW (
-    "volume_int_sink",
-    "audio/raw",
-    "format",     GST_PROPS_STRING ("int"),
-    "channels",   GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "law",        GST_PROPS_INT (0),
-    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-    "width",      GST_PROPS_INT (16),
-    "depth",      GST_PROPS_INT (16),
-    "signed",     GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS (
+    "audio/x-raw-float, "
+      "rate = (int) [ 1, MAX ], "
+      "channels = (int) [ 1, MAX ], "
+      "endianness = (int) BYTE_ORDER, "
+      "width = (int) 32, "
+      "buffer-frames = (int) [ 1, MAX]; "
+    "audio/x-raw-int, "
+      "channels = (int) [ 1, MAX ], "
+      "rate = (int) [ 1,  MAX ], "
+      "endianness = (int) BYTE_ORDER, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "signed = (bool) TRUE"
   )
 );
 
-GST_PAD_TEMPLATE_FACTORY (volume_src_factory,
+static GstStaticPadTemplate volume_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "volume_float_src",
-    "audio/raw",
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "format",     GST_PROPS_STRING ("float"),
-    "layout",     GST_PROPS_STRING ("gfloat"),
-    "intercept",  GST_PROPS_FLOAT(0.0),
-    "slope",      GST_PROPS_FLOAT(1.0),
-    "channels",   GST_PROPS_INT (1)
-  ), 
-  GST_CAPS_NEW (
-    "volume_int_src",
-    "audio/raw",
-    "format",     GST_PROPS_STRING ("int"),
-    "channels",   GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "law",        GST_PROPS_INT (0),
-    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-    "width",      GST_PROPS_INT (16),
-    "depth",      GST_PROPS_INT (16),
-    "signed",     GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS (
+    "audio/x-raw-float, "
+      "rate = (int) [ 1, MAX ], "
+      "channels = (int) [ 1, MAX ], "
+      "endianness = (int) BYTE_ORDER, "
+      "width = (int) 32, "
+      "buffer-frames = (int) [ 1, MAX]; "
+    "audio/x-raw-int, "
+      "channels = (int) [ 1, MAX ], "
+      "rate = (int) [ 1,  MAX ], "
+      "endianness = (int) BYTE_ORDER, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "signed = (bool) TRUE"
   )
 );
 
-static void		volume_class_init		(GstVolumeClass *klass);
+static void		volume_base_init	(gpointer g_class);
+static void		volume_class_init	(GstVolumeClass *klass);
 static void		volume_init		(GstVolume *filter);
 
 static void		volume_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -114,67 +116,175 @@ static void		volume_get_property     (GObject *object, guint prop_id, GValue *va
 static void		volume_update_volume    (const GValue *value, gpointer data);
 static void		volume_update_mute      (const GValue *value, gpointer data);
 
-static gboolean		volume_parse_caps          (GstVolume *filter, GstCaps *caps);
+static gboolean		volume_parse_caps          (GstVolume *filter, GstStructure *structure);
 
-static void		volume_chain_float         (GstPad *pad, GstBuffer *buf);
-static void		volume_chain_int16         (GstPad *pad, GstBuffer *buf);
+static void		volume_chain_float         (GstPad *pad, GstData *_data);
+static void		volume_chain_int16         (GstPad *pad, GstData *_data);
+
+static void gst_volume_interface_init (GstImplementsInterfaceClass *klass);
+static void gst_volume_mixer_init (GstMixerClass *iface);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_filter_signals[LAST_SIGNAL] = { 0 }; */
 
-static GstBufferPool*
-volume_get_bufferpool (GstPad *pad)
+static gboolean
+gst_volume_interface_supported (GstImplementsInterface *iface, GType type)
 {
-  GstVolume *filter;
+  g_assert (type == GST_TYPE_MIXER);
+  return TRUE;
+}
 
-  filter = GST_VOLUME (gst_pad_get_parent (pad));
+static void
+gst_volume_interface_init (GstImplementsInterfaceClass *klass)
+{
+  klass->supported = gst_volume_interface_supported;
+}
 
-  return gst_pad_get_bufferpool (filter->srcpad);
+static const GList *
+gst_volume_list_tracks (GstMixer *mixer)
+{
+  GstVolume *filter = GST_VOLUME (mixer);
+  
+  g_return_val_if_fail (filter != NULL, NULL);
+  g_return_val_if_fail (GST_IS_VOLUME (filter), NULL);
+  
+  return filter->tracklist;
+}
+
+static void
+gst_volume_set_volume (GstMixer *mixer, GstMixerTrack *track,
+                       gint *volumes)
+{
+  GstVolume *filter = GST_VOLUME (mixer);
+  
+  g_return_if_fail (filter != NULL);
+  g_return_if_fail (GST_IS_VOLUME (filter));
+  
+  gst_dpman_bypass_dparam (filter->dpman, "volume");
+  
+  filter->volume_f       = (gfloat) volumes[0] / VOLUME_STEPS;
+  filter->volume_i       = filter->volume_f * VOLUME_UNITY_INT;
+  
+  if (filter->mute) {
+    filter->real_vol_f = 0.0;
+    filter->real_vol_i = 0;
+  }
+  else {
+    filter->real_vol_f = filter->volume_f;
+    filter->real_vol_i = filter->volume_i;
+  }
+}
+
+static void
+gst_volume_get_volume (GstMixer *mixer, GstMixerTrack *track,
+                       gint *volumes)
+{
+  GstVolume *filter = GST_VOLUME (mixer);
+  
+  g_return_if_fail (filter != NULL);
+  g_return_if_fail (GST_IS_VOLUME (filter));
+  
+  volumes[0] = (gint) filter->volume_f * VOLUME_STEPS;
+}
+
+static void
+gst_volume_set_mute (GstMixer *mixer, GstMixerTrack *track,
+                     gboolean mute)
+{
+  GstVolume *filter = GST_VOLUME (mixer);
+  
+  g_return_if_fail (filter != NULL);
+  g_return_if_fail (GST_IS_VOLUME (filter));
+  
+  gst_dpman_bypass_dparam (filter->dpman, "volume");
+  
+  filter->mute = mute;
+  
+  if (filter->mute) {
+    filter->real_vol_f = 0.0;
+    filter->real_vol_i = 0;
+  }
+  else {
+    filter->real_vol_f = filter->volume_f;
+    filter->real_vol_i = filter->volume_i;
+  }
+}
+
+static void
+gst_volume_mixer_init (GstMixerClass *klass)
+{
+  GST_MIXER_TYPE (klass) = GST_MIXER_SOFTWARE;
+  
+  /* default virtual functions */
+  klass->list_tracks = gst_volume_list_tracks;
+  klass->set_volume = gst_volume_set_volume;
+  klass->get_volume = gst_volume_get_volume;
+  klass->set_mute = gst_volume_set_mute;
+}
+
+static void
+gst_volume_dispose (GObject *object)
+{
+  GstVolume *volume;
+
+  volume = GST_VOLUME (object);
+  
+  if (volume->tracklist)
+    {
+      if (volume->tracklist->data)
+        g_object_unref (volume->tracklist->data);
+      g_list_free (volume->tracklist);
+      volume->tracklist = NULL;
+    }
+    
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static GstPadLinkReturn
-volume_connect (GstPad *pad, GstCaps *caps)
+volume_connect (GstPad *pad, const GstCaps *caps)
 {
   GstVolume *filter;
   GstPad *otherpad;
   gint rate;
+  GstPadLinkReturn link_ret;
+  GstStructure *structure;
   
   filter = GST_VOLUME (gst_pad_get_parent (pad));
-  g_return_val_if_fail (filter != NULL, GST_PAD_LINK_REFUSED);
   g_return_val_if_fail (GST_IS_VOLUME (filter), GST_PAD_LINK_REFUSED);
   otherpad = (pad == filter->srcpad ? filter->sinkpad : filter->srcpad);
-  
-  if (GST_CAPS_IS_FIXED (caps)) {
-    GstPadLinkReturn set_retval;
-    if (!volume_parse_caps (filter, caps))
-      return GST_PAD_LINK_REFUSED;
 
-    if ((set_retval = gst_pad_try_set_caps(otherpad, caps)) > 0)
-      if (gst_caps_get_int (caps, "rate", &rate)){
-        gst_dpman_set_rate(filter->dpman, rate);
-      }
-    return set_retval;
-  }
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "rate", &rate);
   
-  return GST_PAD_LINK_DELAYED;
+  link_ret = gst_pad_try_set_caps (otherpad, caps);
+  if (GST_PAD_LINK_FAILED (link_ret)){
+    return link_ret;
+  }
+
+  if (!volume_parse_caps (filter, structure))
+    return GST_PAD_LINK_REFUSED;
+
+  gst_dpman_set_rate(filter->dpman, rate);
+  
+  return GST_PAD_LINK_OK;
 }
 
 static gboolean
-volume_parse_caps (GstVolume *filter, GstCaps *caps)
+volume_parse_caps (GstVolume *filter, GstStructure *structure)
 {
-  const gchar *format;
+  const gchar *mimetype;
   
   g_return_val_if_fail(filter!=NULL,FALSE);
-  g_return_val_if_fail(caps!=NULL,FALSE);
+  g_return_val_if_fail(structure!=NULL,FALSE);
   
-  gst_caps_get_string (caps, "format", &format);
+  mimetype = gst_structure_get_name (structure);
   
-  if (strcmp(format, "int")==0) {
+  if (strcmp(mimetype, "audio/x-raw-int")==0) {
     gst_pad_set_chain_function(filter->sinkpad,volume_chain_int16);
     return TRUE;
   }
   
-  if (strcmp(format, "float")==0) {
+  if (strcmp(mimetype, "audio/x-raw-float")==0) {
     gst_pad_set_chain_function(filter->sinkpad,volume_chain_float);
     return TRUE;
   }
@@ -189,20 +299,48 @@ gst_volume_get_type(void) {
 
   if (!volume_type) {
     static const GTypeInfo volume_info = {
-      sizeof(GstVolumeClass),      NULL,
+      sizeof(GstVolumeClass),
+      volume_base_init,
       NULL,
       (GClassInitFunc)volume_class_init,
       NULL,
       NULL,
       sizeof(GstVolume),
       0,
-      (GInstanceInitFunc)volume_init,
+      (GInstanceInitFunc)volume_init
     };
+    static const GInterfaceInfo voliface_info = {
+      (GInterfaceInitFunc) gst_volume_interface_init,
+      NULL,
+      NULL
+    };
+    static const GInterfaceInfo volmixer_info = {
+      (GInterfaceInitFunc) gst_volume_mixer_init,
+      NULL,
+      NULL
+    };
+    
     volume_type = g_type_register_static(GST_TYPE_ELEMENT, "GstVolume", &volume_info, 0);
+    g_type_add_interface_static (volume_type,
+                                 GST_TYPE_IMPLEMENTS_INTERFACE,
+                                 &voliface_info);
+    g_type_add_interface_static (volume_type,
+                                 GST_TYPE_MIXER,
+                                 &volmixer_info);
   }
   return volume_type;
 }
-
+static void
+volume_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_add_pad_template (element_class, 
+      gst_static_pad_template_get (&volume_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&volume_sink_factory));
+  gst_element_class_set_details (element_class, &volume_details);
+}
 static void
 volume_class_init (GstVolumeClass *klass)
 {
@@ -220,29 +358,39 @@ volume_class_init (GstVolumeClass *klass)
   
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_VOLUME,
     g_param_spec_float("volume","volume","volume",
-                       0.0,4.0,1.0,G_PARAM_READWRITE));
+                       0.0,VOLUME_MAX_FLOAT,1.0,G_PARAM_READWRITE));
   
   gobject_class->set_property = volume_set_property;
   gobject_class->get_property = volume_get_property;
+  gobject_class->dispose = gst_volume_dispose;
 }
 
 static void
 volume_init (GstVolume *filter)
 {
-  filter->sinkpad = gst_pad_new_from_template(volume_sink_factory (),"sink");
-  gst_pad_set_link_function(filter->sinkpad,volume_connect);
-  gst_pad_set_bufferpool_function(filter->sinkpad,volume_get_bufferpool);
-  filter->srcpad = gst_pad_new_from_template(volume_src_factory (),"src");
+  GstMixerTrack *track = NULL;
+  
+  filter->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&volume_sink_factory), "sink");
+  gst_pad_set_getcaps_function (filter->sinkpad, gst_pad_proxy_getcaps);
+  gst_pad_set_link_function (filter->sinkpad, volume_connect);
+  
+  filter->srcpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&volume_src_factory), "src");
+  gst_pad_set_getcaps_function (filter->srcpad, gst_pad_proxy_getcaps);
   gst_pad_set_link_function(filter->srcpad,volume_connect);
   
-  gst_element_add_pad(GST_ELEMENT(filter),filter->sinkpad);
-  gst_element_add_pad(GST_ELEMENT(filter),filter->srcpad);
-  gst_pad_set_chain_function(filter->sinkpad,volume_chain_int16);
+  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
+  gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
+  
+  gst_pad_set_chain_function (filter->sinkpad, volume_chain_int16);
+  
   filter->mute = FALSE;
-  filter->volume_i = 8192;
+  filter->volume_i = VOLUME_UNITY_INT;
   filter->volume_f = 1.0;
-  filter->real_vol_i = 8192;
+  filter->real_vol_i = VOLUME_UNITY_INT;
   filter->real_vol_f = 1.0;
+  filter->tracklist = NULL;
 
   filter->dpman = gst_dpman_new ("volume_dpman", GST_ELEMENT(filter));
   gst_dpman_add_required_dparam_callback (
@@ -256,17 +404,29 @@ volume_init (GstVolume *filter)
   gst_dpman_add_required_dparam_callback (
     filter->dpman, 
     g_param_spec_float("volume","Volume","Volume of the audio",
-                       0.0, 4.0, 1.0, G_PARAM_READWRITE),
+                       0.0, VOLUME_MAX_FLOAT, 1.0, G_PARAM_READWRITE),
     "scalar",
     volume_update_volume, 
     filter
   );
-
+  
+  track = g_object_new (GST_TYPE_MIXER_TRACK, NULL);
+  
+  if (GST_IS_MIXER_TRACK (track))
+    {
+      track->label = g_strdup ("volume");
+      track->num_channels = 1;
+      track->min_volume = 0;
+      track->max_volume = VOLUME_STEPS;
+      track->flags = GST_MIXER_TRACK_SOFTWARE;
+      filter->tracklist = g_list_append (filter->tracklist, track);
+    }
 }
 
 static void
-volume_chain_float (GstPad *pad, GstBuffer *buf)
+volume_chain_float (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstVolume *filter;
   GstBuffer *out_buf;
   gfloat *data;
@@ -289,13 +449,14 @@ volume_chain_float (GstPad *pad, GstBuffer *buf)
     data[i++] *= filter->real_vol_f;
   }
   
-  gst_pad_push(filter->srcpad,out_buf);
+  gst_pad_push(filter->srcpad,GST_DATA (out_buf));
   
 }
 
 static void
-volume_chain_int16 (GstPad *pad, GstBuffer *buf)
+volume_chain_int16 (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstVolume *filter;
   GstBuffer *out_buf;
   gint16 *data;
@@ -317,21 +478,23 @@ volume_chain_int16 (GstPad *pad, GstBuffer *buf)
  
   while(GST_DPMAN_PROCESS(filter->dpman, i)) {
     /* only clamp if the gain is greater than 1.0 */
-    if (filter->real_vol_i > 8192){
+    if (filter->real_vol_i > VOLUME_UNITY_INT){
       while (i < GST_DPMAN_NEXT_UPDATE_FRAME(filter->dpman)){
-        data[i] = (gint16)CLAMP(filter->real_vol_i * (gint)data[i] / 8192, -32768, 32767);
+        /* we use bitshifting instead of dividing by UNITY_INT for speed */
+        data[i] = (gint16) CLAMP((filter->real_vol_i * (gint) data[i]) >> VOLUME_UNITY_BIT_SHIFT, VOLUME_MIN_INT16, VOLUME_MAX_INT16);
 	i++;
       }
     }
     else {
       while (i < GST_DPMAN_NEXT_UPDATE_FRAME(filter->dpman)){
-        data[i] = (gint16)(filter->real_vol_i * (gint)data[i] / 8192);
+        /* we use bitshifting instead of dividing by UNITY_INT for speed */
+        data[i] = (gint16) ((filter->real_vol_i * (gint) data[i]) >> VOLUME_UNITY_BIT_SHIFT);
         i++;
       }
     }
   }
 
-  gst_pad_push(filter->srcpad,out_buf);   
+  gst_pad_push(filter->srcpad,GST_DATA (out_buf));   
   
 }
 
@@ -365,7 +528,7 @@ volume_update_volume(const GValue *value, gpointer data)
   g_return_if_fail(GST_IS_VOLUME(filter));
 
   filter->volume_f       = g_value_get_float (value);
-  filter->volume_i       = filter->volume_f*8192;
+  filter->volume_i       = filter->volume_f*VOLUME_UNITY_INT;
   if (filter->mute){
     filter->real_vol_f = 0.0;
     filter->real_vol_i = 0;
@@ -424,27 +587,21 @@ volume_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new("volume",GST_TYPE_VOLUME,
-                                   &volume_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, volume_src_factory ());
-  gst_element_factory_add_pad_template (factory, volume_sink_factory ());
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  /* initialize dparam support library */
   gst_control_init(NULL,NULL);
-  return TRUE;
+  
+  return gst_element_register (plugin, "volume", GST_RANK_PRIMARY, GST_TYPE_VOLUME);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "volume",
-  plugin_init
-};
+  "element for controlling audio volume",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
