@@ -60,61 +60,40 @@ static GstElementDetails gst_wavparse_details = GST_ELEMENT_DETAILS (
   "Erik Walthinsen <omega@cse.ogi.edu>"
 );
 
-GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
+static GstStaticPadTemplate sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "wavparse_sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "wavparse_wav",
-    "audio/x-wav",
-    NULL
-  )
-)
+  GST_STATIC_CAPS ("audio/x-wav")
+);
 
-GST_PAD_TEMPLATE_FACTORY (src_template_factory,
+static GstStaticPadTemplate src_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "wavparse_src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "wavparse_raw",
-    "audio/x-raw-int",
-       "endianness",       GST_PROPS_INT (G_LITTLE_ENDIAN),
-       "signed",           GST_PROPS_LIST (
-				GST_PROPS_BOOLEAN (FALSE),
-				GST_PROPS_BOOLEAN (TRUE)
-			   ),
-       "width",            GST_PROPS_LIST (
-	                     GST_PROPS_INT (8),
-	                     GST_PROPS_INT (16)
-			   ),
-       "depth",            GST_PROPS_LIST (
-	                     GST_PROPS_INT (8),
-	                     GST_PROPS_INT (16)
-			   ),
-       "rate",             GST_PROPS_INT_RANGE (8000, 48000),
-       "channels",         GST_PROPS_INT_RANGE (1, 2)
-  ),
-  GST_CAPS_NEW (
-    "wavparse_mpeg",
-    "audio/mpeg",
-      "mpegversion",       GST_PROPS_INT (1),
-      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
-      "channels",          GST_PROPS_INT_RANGE (1, 2),
-      "layer",             GST_PROPS_INT_RANGE (1, 3)
-  ),
-  GST_CAPS_NEW (
-    "parsewav_law",
-    "audio/x-alaw",
-      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
-      "channels",          GST_PROPS_INT_RANGE (1, 2)
-  ),
-  GST_CAPS_NEW (
-    "parsewav_law",
-    "audio/x-mulaw",
-      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
-      "channels",          GST_PROPS_INT_RANGE (1, 2)
+  GST_STATIC_CAPS (
+    "audio/x-raw-int, "
+      "endianness = (int) little_endian, "
+      "signed = (boolean) { true, false }, "
+      "width = (int) { 8, 16 }, "
+      "depth = (int) { 8, 16 }, "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) [ 1, 2 ]; "
+    "audio/mpeg, "
+      "mpegversion = (int) 1, "
+      "layer = (int) [ 1, 3 ], "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) [ 1, 2 ]; "
+    "audio/x-alaw, "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) [ 1, 2 ]; "
+    "audio/x-mulaw, "
+      "rate = (int) [ 8000, 48000 ], "
+      "channels = (int) [ 1, 2 ]"
   )
-)
+);
 
 /* WavParse signals and args */
 enum {
@@ -124,7 +103,6 @@ enum {
 
 enum {
   PROP_0,
-  PROP_METADATA
 };
 
 static GstElementClass *parent_class = NULL;
@@ -161,8 +139,8 @@ gst_wavparse_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_wavparse_details);
 
   /* register src pads */
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (sink_template_factory));
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (src_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&src_template_factory));
 }
 static void
 gst_wavparse_class_init (GstWavParseClass *klass) 
@@ -177,21 +155,13 @@ gst_wavparse_class_init (GstWavParseClass *klass)
 
   object_class->get_property = gst_wavparse_get_property;
   gstelement_class->change_state = gst_wavparse_change_state;
-
-  g_object_class_install_property (object_class, PROP_METADATA,
-				   g_param_spec_boxed ("metadata",
-						       "Metadata", "Metadata",
-						       GST_TYPE_CAPS,
-						       G_PARAM_READABLE));
 }
 
 static void 
 gst_wavparse_init (GstWavParse *wavparse) 
 {
-  GstProps *props;
-  
   /* sink */
-  wavparse->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_template_factory), "sink");
+  wavparse->sinkpad = gst_pad_new_from_template (gst_static_pad_template_get (&sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (wavparse), wavparse->sinkpad);
 
   gst_pad_set_formats_function (wavparse->sinkpad, gst_wavparse_get_formats);
@@ -201,7 +171,8 @@ gst_wavparse_init (GstWavParse *wavparse)
   gst_pad_set_query_function (wavparse->sinkpad, gst_wavparse_pad_query);
 
   /* source */
-  wavparse->srcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (src_template_factory), "src");
+  wavparse->srcpad = gst_pad_new_from_template (gst_static_pad_template_get (&src_template_factory), "src");
+  gst_pad_use_explicit_caps (wavparse->srcpad);
   gst_element_add_pad (GST_ELEMENT (wavparse), wavparse->srcpad);
   gst_pad_set_formats_function (wavparse->srcpad, gst_wavparse_get_formats);
   gst_pad_set_convert_function (wavparse->srcpad, gst_wavparse_pad_convert);
@@ -217,14 +188,6 @@ gst_wavparse_init (GstWavParse *wavparse)
   wavparse->bps = 0;
   wavparse->seek_pending = FALSE;
   wavparse->seek_offset = 0;
-
-  props = gst_props_empty_new ();
-
-  /* Metadata is added later when we find it */
-  gst_caps_replace_sink (&wavparse->metadata,
-			 gst_caps_new ("wav_metadata",
-				       "application/x-gst-metadata",
-				       props));
 }
 
 static void
@@ -238,15 +201,12 @@ gst_wavparse_get_property (GObject *object,
   wavparse = GST_WAVPARSE (object);
 
   switch (prop_id) {
-  case PROP_METADATA:
-    g_value_set_boxed (value, wavparse->metadata);
-    break;
-
   default:
     break;
   }
 }
 
+#if 0
 static void
 gst_wavparse_parse_adtl (GstWavParse *wavparse,
 			 int len)
@@ -310,8 +270,8 @@ gst_wavparse_parse_adtl (GstWavParse *wavparse,
       new_caps = gst_caps_new ("label",
 															 "application/x-gst-metadata",
 															 gst_props_new (
-																 "identifier", GST_PROPS_INT (labl.identifier),
-																 "name", GST_PROPS_STRING (label_name),
+																 "identifier", G_TYPE_INT (labl.identifier),
+																 "name", G_TYPE_STRING (label_name),
 																 NULL));
       
       if (gst_props_get (props, "labels", &caps, NULL)) {
@@ -362,9 +322,9 @@ gst_wavparse_parse_adtl (GstWavParse *wavparse,
       new_caps = gst_caps_new ("ltxt",
 															 "application/x-gst-metadata",
 															 gst_props_new (
-																 "identifier", GST_PROPS_INT (ltxt.identifier),
-																 "name", GST_PROPS_STRING (label_name),
-																 "length", GST_PROPS_INT (ltxt.length),
+																 "identifier", G_TYPE_INT (ltxt.identifier),
+																 "name", G_TYPE_STRING (label_name),
+																 "length", G_TYPE_INT (ltxt.length),
 																 NULL));
 			
       if (gst_props_get (props, "ltxts", &caps, NULL)) {
@@ -405,8 +365,8 @@ gst_wavparse_parse_adtl (GstWavParse *wavparse,
       new_caps = gst_caps_new ("note",
 															 "application/x-gst-metadata",
 															 gst_props_new (
-																 "identifier", GST_PROPS_INT (note.identifier),
-																 "name", GST_PROPS_STRING (label_name),
+																 "identifier", G_TYPE_INT (note.identifier),
+																 "name", G_TYPE_STRING (label_name),
 																 NULL));
       
       if (gst_props_get (props, "notes", &caps, NULL)) {
@@ -428,7 +388,9 @@ gst_wavparse_parse_adtl (GstWavParse *wavparse,
 
   g_object_notify (G_OBJECT (wavparse), "metadata");
 }
+#endif
 
+#if 0
 static void
 gst_wavparse_parse_info (GstWavParse *wavparse,
 												 int len)
@@ -536,14 +498,16 @@ gst_wavparse_parse_info (GstWavParse *wavparse,
     if (type) {
       GstPropsEntry *entry;
 
-      entry = gst_props_entry_new (type, GST_PROPS_STRING (name));
+      entry = gst_props_entry_new (type, G_TYPE_STRING (name));
       gst_props_add_entry (wavparse->metadata->properties, entry);
     }
   }
 
   g_object_notify (G_OBJECT (wavparse), "metadata");
 }
+#endif
 
+#if 0
 static void
 gst_wavparse_parse_cues (GstWavParse *wavparse,
 			 int len)
@@ -595,8 +559,8 @@ gst_wavparse_parse_cues (GstWavParse *wavparse,
       caps = gst_caps_new ("cues",
 													 "application/x-gst-metadata",
 													 gst_props_new (
-														 "identifier", GST_PROPS_INT (points[i].identifier),
-														 "position", GST_PROPS_INT (points[i].offset),
+														 "identifier", G_TYPE_INT (points[i].identifier),
+														 "position", G_TYPE_INT (points[i].offset),
 														 NULL));
       cues = g_list_append (cues, caps);
     }
@@ -607,9 +571,10 @@ gst_wavparse_parse_cues (GstWavParse *wavparse,
   
   g_object_notify (G_OBJECT (wavparse), "metadata");
 }
+#endif
 
 static void
-gst_wavparse_parse_fmt (GstWavParse *wavparse)
+gst_wavparse_parse_fmt (GstWavParse *wavparse, guint size)
 {
   GstWavParseFormat *format;
   GstCaps *caps = NULL;
@@ -621,7 +586,7 @@ gst_wavparse_parse_fmt (GstWavParse *wavparse)
   format = (GstWavParseFormat *) fmtdata;
 
   if (got_bytes == sizeof (GstWavParseFormat)) {
-    gst_bytestream_flush (bs, got_bytes);
+    gst_bytestream_flush (bs, size);
     wavparse->bps = GUINT16_FROM_LE (format->wBlockAlign);
     wavparse->rate = GUINT32_FROM_LE (format->dwSamplesPerSec);
     wavparse->channels = GUINT16_FROM_LE (format->wChannels);
@@ -640,49 +605,46 @@ gst_wavparse_parse_fmt (GstWavParse *wavparse)
 				return;
       }
 			
-      caps = GST_CAPS_NEW ("parsewav_src",
-			   mime,
-			     "rate", GST_PROPS_INT (wavparse->rate),
-			     "channels", GST_PROPS_INT (wavparse->channels)
-				);
+      caps = gst_caps_new_simple (mime,
+        "rate", G_TYPE_INT, wavparse->rate,
+        "channels", G_TYPE_INT, wavparse->channels,
+        NULL);
     }
 			
     case GST_RIFF_WAVE_FORMAT_PCM:
-      caps = GST_CAPS_NEW ("parsewav_src",
-			   "audio/x-raw-int",
-			     "endianness", GST_PROPS_INT (G_LITTLE_ENDIAN),
-			     "signed", GST_PROPS_BOOLEAN ((wavparse->width > 8) ? TRUE : FALSE),
-			     "width", GST_PROPS_INT (wavparse->width),
-			     "depth", GST_PROPS_INT (wavparse->width),
-			     "rate", GST_PROPS_INT (wavparse->rate),
-			     "channels", GST_PROPS_INT (wavparse->channels)
-				);
+      caps = gst_caps_new_simple ("audio/x-raw-int",
+        "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+        "signed", G_TYPE_BOOLEAN, (wavparse->width > 8) ? TRUE : FALSE,
+        "width", G_TYPE_INT, wavparse->width,
+        "depth", G_TYPE_INT, wavparse->width,
+        "rate", G_TYPE_INT, wavparse->rate,
+        "channels", G_TYPE_INT, wavparse->channels,
+        NULL);
       break;
 			
     case GST_RIFF_WAVE_FORMAT_MPEGL12:
     case GST_RIFF_WAVE_FORMAT_MPEGL3: {
       int layer = (wavparse->format == GST_RIFF_WAVE_FORMAT_MPEGL12) ? 2 : 3;
 			
-      caps = GST_CAPS_NEW ("parsewav_src",
-			   "audio/mpeg",
-			     "mpegversion", GST_PROPS_INT (1),
-			     "layer", GST_PROPS_INT (layer),
-			     "rate", GST_PROPS_INT (wavparse->rate),
-			     "channels", GST_PROPS_INT (wavparse->channels)
-				);
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 1,
+          "layer", G_TYPE_INT, layer,
+          "rate", G_TYPE_INT, wavparse->rate,
+          "channels", G_TYPE_INT, wavparse->channels,
+          NULL);
     }
       break;
 			
     default:
-      gst_element_error (GST_ELEMENT (wavparse), "wavparse: format %d not handled", wavparse->format);
+      GST_ELEMENT_ERROR (wavparse, STREAM, NOT_IMPLEMENTED, (NULL), ("format %d not handled", wavparse->format));
       return;
     }
-		
-    if (gst_pad_try_set_caps (wavparse->srcpad, caps) <= 0) {
-      gst_element_error (GST_ELEMENT (wavparse), "Could not set caps");
-      return;
+
+    if (caps) {
+      gst_pad_set_explicit_caps (wavparse->srcpad, caps);
+      gst_caps_free (caps);
     }
-		
+
     GST_DEBUG ("frequency %d, channels %d",
 							 wavparse->rate, wavparse->channels);
   }
@@ -763,7 +725,16 @@ gst_wavparse_loop (GstElement *element)
       desired = MIN (wavparse->dataleft, MAX_BUFFER_SIZE);
       got_bytes = gst_bytestream_peek (bs, &buf, desired);
 
-      if (got_bytes == 0) {
+      if (got_bytes != desired) {
+        /* EOS? */
+        GstEvent *event;
+        guint32 remaining;
+        gst_bytestream_get_status (bs, &remaining, &event);
+        if (event && GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+          gst_pad_event_default (wavparse->sinkpad, event);
+        } else {
+          GST_ELEMENT_ERROR (element, RESOURCE, READ, (NULL), (NULL));
+        }
 	return;
       }
 
@@ -839,7 +810,8 @@ gst_wavparse_loop (GstElement *element)
   case GST_WAVPARSE_START:
     if (chunk.id != GST_RIFF_TAG_RIFF &&
 				chunk.type != GST_RIFF_RIFF_WAVE) {
-      gst_element_error (element, "This doesn't appear to be a WAV file %08x %08x", chunk.id, chunk.type);
+      GST_ELEMENT_ERROR (element, STREAM, WRONG_TYPE, (NULL),
+                           ("chunk.id %08x chunk.type %08x", chunk.id, chunk.type));
       return;
     }
 		
@@ -854,42 +826,45 @@ gst_wavparse_loop (GstElement *element)
     switch (chunk.id) {
     case GST_RIFF_TAG_data:
       wavparse->state = GST_WAVPARSE_DATA;
-			wavparse->dataleft = chunk.size;
-			wavparse->byteoffset = 0;
-
-			flush = 0;
+      wavparse->dataleft = chunk.size;
+      wavparse->byteoffset = 0;
+      flush = 0;
       break;
       
     case GST_RIFF_TAG_fmt:
-      gst_wavparse_parse_fmt (wavparse);
+      gst_wavparse_parse_fmt (wavparse, chunk.size);
+      flush = 0;
       break;
 
     case GST_RIFF_TAG_cue:
-      gst_wavparse_parse_cues (wavparse, chunk.size);
+      //gst_wavparse_parse_cues (wavparse, chunk.size);
       break;
       
     case GST_RIFF_TAG_LIST:
       GST_DEBUG ("list type: %4.4s", (char *) &chunk.type);
       switch (chunk.type) {
       case GST_RIFF_LIST_INFO:
-	gst_wavparse_parse_info (wavparse, chunk.size - 4);
-	flush = 0;
+	//gst_wavparse_parse_info (wavparse, chunk.size - 4);
+	//flush = 0;
 
 	break;
 				
       case GST_RIFF_LIST_adtl:
-	gst_wavparse_parse_adtl (wavparse, chunk.size - 4);
-	flush = 0;
+	//gst_wavparse_parse_adtl (wavparse, chunk.size - 4);
+	//flush = 0;
 	break;
 
       default:
-	flush = 0;
+	//flush = 0;
 	break;
       }
       
-      flush = 0;
+    default:
+      GST_DEBUG ("  *****  unknown chunkid %08x", chunk.id);
+      //flush = 0;
       break;      
     }
+    break;
 		
   case GST_WAVPARSE_DATA:
 		/* Should have been handled up there ^^^^ */
@@ -898,7 +873,8 @@ gst_wavparse_loop (GstElement *element)
 		
   default:
     /* Unknown */
-    GST_DEBUG ("  *****  unknown chunkid %08x", chunk.id);
+    g_warning ("Unknown state %d\n", wavparse->state);
+    //GST_DEBUG ("  *****  unknown chunkid %08x", chunk.id);
     break;
   }
 
@@ -1108,8 +1084,6 @@ gst_wavparse_change_state (GstElement *element)
       wavparse->bps = 0;
       wavparse->seek_pending = FALSE;
       wavparse->seek_offset = 0;
-      gst_caps_replace (&wavparse->metadata, NULL);
-      
       break;
     case GST_STATE_READY_TO_NULL:
       break;
