@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -88,22 +89,22 @@ gst_osselement_get_type (void)
       NULL,
       sizeof(GstOssElement),
       0,
-      (GInstanceInitFunc)gst_osselement_init,
+      (GInstanceInitFunc)gst_osselement_init
     };
     static const GInterfaceInfo ossiface_info = {
       (GInterfaceInitFunc) gst_oss_interface_init,
       NULL,
-      NULL,
+      NULL
     };
     static const GInterfaceInfo ossmixer_info = {
       (GInterfaceInitFunc) gst_ossmixer_interface_init,
       NULL,
-      NULL,
+      NULL
     };
     static const GInterfaceInfo ossprobe_info = {
       (GInterfaceInitFunc) gst_ossprobe_interface_init,
       NULL,
-      NULL,
+      NULL
     };
 
     osselement_type = g_type_register_static (GST_TYPE_ELEMENT,
@@ -206,8 +207,24 @@ static gboolean
 gst_osselement_class_probe_devices (GstOssElementClass *klass,
 				    gboolean            check)
 {
+  GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
   static gboolean init = FALSE;
   static GList *device_combinations;
+  GList *padtempllist;
+  gint openmode = O_RDONLY;
+  gboolean mixer = FALSE;
+
+  /* Ok, so how do we open the device? We assume that we have (max.) one
+   * pad, and if this is a sinkpad, we're osssink (w). else, we're osssrc
+   * (r) */
+  padtempllist = gst_element_class_get_pad_template_list (eklass);
+  if (padtempllist != NULL) {
+    GstPadTemplate *firstpadtempl = padtempllist->data;
+    if (GST_PAD_TEMPLATE_DIRECTION (firstpadtempl) == GST_PAD_SINK) {
+      openmode = O_WRONLY;
+    }
+    mixer = TRUE;
+  }
 
   if (!init && !check) {
     gchar *dsp_base[] = { "/dev/dsp", "/dev/sound/dsp", NULL };
@@ -248,8 +265,10 @@ gst_osselement_class_probe_devices (GstOssElementClass *klass,
         continue;
 
       /* we just check the dsp. we assume the mixer always works.
-       * we don't need a mixer anyway (says OSS)... */
-      if ((fd = open (devices[n].dsp, O_RDONLY)) > 0 || errno == EBUSY) {
+       * we don't need a mixer anyway (says OSS)... If we are a
+       * mixer element, we use the mixer anyway. */
+      if ((fd = open (mixer ? devices[n].mixer :
+                         devices[n].dsp, openmode)) > 0 || errno == EBUSY) {
         GstOssDeviceCombination *combi;
 
         if (fd > 0)
@@ -486,19 +505,22 @@ gst_ossformat_get (gint law, gint endianness, gboolean sign, gint width, gint de
 }
 
 gboolean 
-gst_osselement_parse_caps (GstOssElement *oss, GstCaps *caps) 
+gst_osselement_parse_caps (GstOssElement *oss, const GstCaps *caps) 
 {
   gint bps, format;
+  GstStructure *structure;
 
-  gst_caps_get_int (caps, "width", &oss->width);
-  gst_caps_get_int (caps, "depth", &oss->depth);
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int  (structure, "width", &oss->width);
+  gst_structure_get_int  (structure, "depth", &oss->depth);
 	        
   if (oss->width != oss->depth) 
     return FALSE;
 		  
-  gst_caps_get_int (caps, "law", &oss->law); 
-  gst_caps_get_int (caps, "endianness", &oss->endianness);
-  gst_caps_get_boolean (caps, "signed", &oss->sign);
+  gst_structure_get_int  (structure, "law", &oss->law); 
+  gst_structure_get_int  (structure, "endianness", &oss->endianness);
+  gst_structure_get_boolean  (structure, "signed", &oss->sign);
 			    
   if (!gst_ossformat_get (oss->law, oss->endianness, oss->sign, 
                           oss->width, oss->depth, &format, &bps))
@@ -507,8 +529,8 @@ gst_osselement_parse_caps (GstOssElement *oss, GstCaps *caps)
      return FALSE;
   }
 
-  gst_caps_get_int (caps, "channels", &oss->channels);
-  gst_caps_get_int (caps, "rate", &oss->rate);
+  gst_structure_get_int  (structure, "channels", &oss->channels);
+  gst_structure_get_int  (structure, "rate", &oss->rate);
 			      
   oss->bps = bps * oss->channels * oss->rate;
   oss->format = format;
@@ -519,25 +541,28 @@ gst_osselement_parse_caps (GstOssElement *oss, GstCaps *caps)
 #define GET_FIXED_INT(caps, name, dest)         \
 G_STMT_START {                                  \
   if (gst_caps_has_fixed_property (caps, name)) \
-    gst_caps_get_int (caps, name, dest);        \
+    gst_structure_get_int  (structure, name, dest);        \
 } G_STMT_END
 #define GET_FIXED_BOOLEAN(caps, name, dest)     \
 G_STMT_START {                                  \
   if (gst_caps_has_fixed_property (caps, name)) \
-    gst_caps_get_boolean (caps, name, dest);    \
+    gst_structure_get_boolean  (structure, name, dest);    \
 } G_STMT_END
 
 gboolean 
 gst_osselement_merge_fixed_caps (GstOssElement *oss, GstCaps *caps) 
 {
   gint bps, format;
+  GstStructure *structure;
 
+  structure = gst_caps_get_structure (caps, 0);
+  
   /* peel off fixed stuff from the caps */
-  GET_FIXED_INT         (caps, "law",        &oss->law);
-  GET_FIXED_INT         (caps, "endianness", &oss->endianness);
-  GET_FIXED_BOOLEAN     (caps, "signed",     &oss->sign);
-  GET_FIXED_INT         (caps, "width",      &oss->width);
-  GET_FIXED_INT         (caps, "depth",      &oss->depth);
+  gst_structure_get_int (structure, "law",        &oss->law);
+  gst_structure_get_int (structure, "endianness", &oss->endianness);
+  gst_structure_get_boolean (structure, "signed",     &oss->sign);
+  gst_structure_get_int (structure, "width",      &oss->width);
+  gst_structure_get_int (structure, "depth",      &oss->depth);
 
   if (!gst_ossformat_get (oss->law, oss->endianness, oss->sign, 
                           oss->width, oss->depth, &format, &bps))
@@ -545,8 +570,8 @@ gst_osselement_merge_fixed_caps (GstOssElement *oss, GstCaps *caps)
      return FALSE;
   }
 
-  GET_FIXED_INT         (caps, "rate",       &oss->rate);
-  GET_FIXED_INT         (caps, "channels",   &oss->channels);
+  gst_structure_get_int (structure, "rate",       &oss->rate);
+  gst_structure_get_int (structure, "channels",   &oss->channels);
 			      
   oss->bps = bps * oss->channels * oss->rate;
   oss->format = format;
@@ -619,7 +644,12 @@ gst_osselement_sync_parms (GstOssElement *oss)
       target_channels != oss->channels ||
       target_rate     != oss->rate) 
   {
-    g_warning ("couldn't set requested OSS parameters, enjoy the noise :)");
+    if (target_channels != oss->channels)
+      g_warning ("couldn't set the right number of channels, enjoy the tone difference");
+    if (target_rate != oss->rate)
+      g_warning ("couldn't set the right number of channels, enjoy the speed difference");
+    if (target_format != oss->format)
+      g_warning ("couldn't set requested OSS parameters, enjoy the noise :)");
     /* we could eventually return FALSE here, or just do some additional tests
      * to see that the frequencies don't differ too much etc.. */
   }
@@ -668,37 +698,38 @@ gst_osselement_open_audio (GstOssElement *oss)
   if (oss->fd < 0) {
     switch (errno) {
       case EBUSY:
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Unable to open %s (in use ?)",
-			   oss->device);
-	break;
-      case EISDIR:
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Device %s is a directory",
-			   oss->device);
+	GST_ELEMENT_ERROR (oss, RESOURCE, BUSY,
+                           (_("OSS device \"%s\" is already in use by another program."), oss->device),
+                           (NULL));
 	break;
       case EACCES:
       case ETXTBSY:
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Cannot access %s, check permissions",
-			   oss->device);
+        if (mode == GST_OSSELEMENT_WRITE)
+	  GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_WRITE,
+			     (_("Could not access device \"%s\", check its permissions."), oss->device),
+                             GST_ERROR_SYSTEM);
+        else
+	  GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
+			     (_("Could not access device \"%s\", check its permissions."), oss->device),
+                             GST_ERROR_SYSTEM);
 	break;
       case ENXIO:
       case ENODEV:
       case ENOENT:
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Cannot access %s, does it exist ?",
-			   oss->device);
+	GST_ELEMENT_ERROR (oss, RESOURCE, NOT_FOUND,
+			   (_("Device \"%s\" does not exist."), oss->device),
+                           GST_ERROR_SYSTEM);
 	break;
-      case EROFS:
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Cannot access %s, read-only filesystem ?",
-			   oss->device);
       default:
 	/* FIXME: strerror is not threadsafe */
-	gst_element_error (GST_ELEMENT (oss),
-			   "osselement: Cannot open %s, generic error: %s",
-			   oss->device, strerror (errno));
+        if (mode == GST_OSSELEMENT_WRITE)
+	  GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_WRITE,
+			     (_("Could not open device \"%s\" for writing."), oss->device),
+                             GST_ERROR_SYSTEM);
+        else
+	  GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
+			     (_("Could not open device \"%s\" for reading."), oss->device),
+                             GST_ERROR_SYSTEM);
 	break;
     }
     return FALSE;
@@ -791,7 +822,7 @@ gst_osselement_convert (GstOssElement *oss,
 	  *dest_value = src_value * GST_SECOND / oss->bps;
           break;
         case GST_FORMAT_DEFAULT:
-	  *dest_value = src_value / (oss->channels * oss->width);
+	  *dest_value = src_value / (oss->width * oss->channels / 8);
           break;
         default:
           res = FALSE;
@@ -815,7 +846,7 @@ gst_osselement_convert (GstOssElement *oss,
 	  *dest_value = src_value * GST_SECOND / oss->rate;
           break;
         case GST_FORMAT_BYTES:
-	  *dest_value = src_value * oss->channels * oss->width;
+	  *dest_value = src_value * oss->width * oss->channels / 8;
           break;
         default:
           res = FALSE;
@@ -926,4 +957,3 @@ gst_osselement_change_state (GstElement *element)
 
   return GST_STATE_SUCCESS;
 }
-
