@@ -25,6 +25,9 @@
 
 #include <gst/gst.h>
 
+GST_DEBUG_CATEGORY_EXTERN (oss_debug);
+#define GST_CAT_DEFAULT oss_debug
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_OSSELEMENT \
@@ -43,6 +46,11 @@ G_BEGIN_DECLS
 typedef struct _GstOssElement GstOssElement;
 typedef struct _GstOssElementClass GstOssElementClass;
 
+/* debugging category */
+GST_DEBUG_CATEGORY_EXTERN (oss_debug);
+//#define GST_CAT_DEFAULT oss_debug
+
+
 typedef enum {
   GST_OSSELEMENT_READ,
   GST_OSSELEMENT_WRITE,
@@ -69,7 +77,7 @@ struct _GstOssElement
   gint		 fragment_size;
   GstOssOpenMode mode;
 
-  /* stats */
+  /* stats bytes per *second* */
   guint		 bps;
 
   /* parameters */
@@ -101,9 +109,9 @@ GType		gst_osselement_get_type		(void);
 
 /* some useful functions */
 gboolean 	gst_osselement_parse_caps 	(GstOssElement *oss,
-						 GstCaps       *caps);
+						 const GstCaps      *caps);
 gboolean	gst_osselement_merge_fixed_caps (GstOssElement *oss,
-						 GstCaps       *caps);
+						 GstCaps      *caps);
 	
 gboolean 	gst_osselement_sync_parms 	(GstOssElement *oss);
 void		gst_osselement_reset 		(GstOssElement *oss);
