@@ -58,7 +58,8 @@ static gboolean 		gst_osssink_query 		(GstElement *element, GstQueryType type,
 static gboolean 		gst_osssink_sink_query 		(GstPad *pad, GstQueryType type,
 								 GstFormat *format, gint64 *value);
 
-static GstPadLinkReturn		gst_osssink_sinkconnect		(GstPad *pad, GstCaps *caps);
+static GstCaps *                gst_osssink_sink_fixate         (GstPad *pad, const GstCaps *caps);
+static GstPadLinkReturn		gst_osssink_sinkconnect		(GstPad *pad, const GstCaps *caps);
 
 static void 			gst_osssink_set_property	(GObject *object, guint prop_id, const GValue *value, 
 		  						 GParamSpec *pspec);
@@ -83,28 +84,18 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (osssink_sink_factory,
+static GstStaticPadTemplate osssink_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "osssink_sink",
-    "audio/x-raw-int",
-      "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-      "signed",     GST_PROPS_LIST (
-         	      GST_PROPS_BOOLEAN (FALSE),
-     		      GST_PROPS_BOOLEAN (TRUE)
-      		    ),
-      "width",      GST_PROPS_LIST (
-         	      GST_PROPS_INT (8),
-     		      GST_PROPS_INT (16)
-      		    ),
-      "depth",      GST_PROPS_LIST (
-    		      GST_PROPS_INT (8),
-     		      GST_PROPS_INT (16)
-     		    ),
-      "rate",       GST_PROPS_INT_RANGE (1000, 48000),
-      "channels",   GST_PROPS_INT_RANGE (1, 2)
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) { TRUE, FALSE }, "
+      "width = (int) { 8, 16 }, "
+      "depth = (int) { 8, 16 }, "
+      "rate = (int) [ 1000, 48000 ], "
+      "channels = (int) [ 1, 2 ]"
   )
 );
 
@@ -112,7 +103,7 @@ static GstElementClass *parent_class = NULL;
 static guint gst_osssink_signals[LAST_SIGNAL] = { 0 };
 
 GType
-gst_osssink_get_type (void) 
+gst_osssink_get_type (void)
 {
   static GType osssink_type = 0;
 
@@ -134,20 +125,6 @@ gst_osssink_get_type (void)
   return osssink_type;
 }
 
-static GstBufferPool*
-gst_osssink_get_bufferpool (GstPad *pad)
-{
-  GstOssSink *oss;
-  
-  oss = GST_OSSSINK (gst_pad_get_parent(pad));
-
-  /* 6 buffers per chunk by default */
-  if (!oss->sinkpool)
-    oss->sinkpool = gst_buffer_pool_get_default (oss->bufsize, 6);
-
-  return oss->sinkpool;
-}
-
 static void
 gst_osssink_dispose (GObject *object)
 {
@@ -164,7 +141,7 @@ gst_osssink_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
   
   gst_element_class_set_details (element_class, &gst_osssink_details);
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (osssink_sink_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&osssink_sink_factory));
 }
 static void
 gst_osssink_class_init (GstOssSinkClass *klass) 
@@ -214,10 +191,10 @@ static void
 gst_osssink_init (GstOssSink *osssink) 
 {
   osssink->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (osssink_sink_factory), "sink");
+		  gst_static_pad_template_get (&osssink_sink_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (osssink), osssink->sinkpad);
   gst_pad_set_link_function (osssink->sinkpad, gst_osssink_sinkconnect);
-  gst_pad_set_bufferpool_function (osssink->sinkpad, gst_osssink_get_bufferpool);
+  gst_pad_set_fixate_function (osssink->sinkpad, gst_osssink_sink_fixate);
   gst_pad_set_convert_function (osssink->sinkpad, gst_osssink_convert);
   gst_pad_set_query_function (osssink->sinkpad, gst_osssink_sink_query);
   gst_pad_set_query_type_function (osssink->sinkpad, gst_osssink_get_query_types);
@@ -225,12 +202,11 @@ gst_osssink_init (GstOssSink *osssink)
 
   gst_pad_set_chain_function (osssink->sinkpad, gst_osssink_chain);
 
+  GST_DEBUG ("initializing osssink");
   osssink->bufsize = 4096;
   osssink->chunk_size = 4096;
-  osssink->resync = FALSE;
   osssink->mute = FALSE;
   osssink->sync = TRUE;
-  osssink->sinkpool = NULL;
   osssink->provided_clock = gst_audio_clock_new ("ossclock", gst_osssink_get_time, osssink);
   gst_object_set_parent (GST_OBJECT (osssink->provided_clock), GST_OBJECT (osssink));
   osssink->handled = 0;
@@ -239,15 +215,38 @@ gst_osssink_init (GstOssSink *osssink)
   GST_FLAG_SET (osssink, GST_ELEMENT_EVENT_AWARE);
 }
 
+static GstCaps *
+gst_osssink_sink_fixate (GstPad *pad, const GstCaps *caps)
+{
+  GstCaps *newcaps;
+  GstStructure *structure;
+
+  newcaps = gst_caps_new_full (gst_structure_copy(gst_caps_get_structure (caps, 0)), NULL);
+  structure = gst_caps_get_structure (newcaps, 0);
+
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "rate", 44100)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "depth", 16)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "width", 16)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "channels", 2)) {
+    return newcaps;
+  }
+
+  gst_caps_free (newcaps);
+
+  return NULL;
+}
 
 static GstPadLinkReturn 
-gst_osssink_sinkconnect (GstPad *pad, GstCaps *caps) 
+gst_osssink_sinkconnect (GstPad *pad, const GstCaps *caps) 
 {
   GstOssSink *osssink = GST_OSSSINK (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-
   if (!gst_osselement_parse_caps (GST_OSSELEMENT (osssink), caps))
     return GST_PAD_LINK_REFUSED;
 
@@ -326,7 +325,9 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstOssSink *osssink;
-  GstClockTime buftime;
+  GstClockTime buftime, elementtime, soundtime;
+  guchar *data;
+  guint to_write;
 
   /* this has to be an audio buffer */
   osssink = GST_OSSSINK (gst_pad_get_parent (pad));
@@ -340,77 +341,61 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
 	gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
 	gst_pad_event_default (pad, event);
         return;
-      case GST_EVENT_DISCONTINUOUS:
-      {
-	gint64 value;
-
-        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET);
-	if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
-          if (!gst_clock_handle_discont (osssink->clock, value))
-            gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
-	  osssink->handled = 0;
-	}
-        osssink->resync = TRUE;
-
-        break;
-      }
       default:
 	gst_pad_event_default (pad, event);
         return;
     }
-    gst_event_unref (event);
-    return;
+    g_assert_not_reached ();
   }
 
   if (!GST_OSSELEMENT (osssink)->bps) {
     gst_buffer_unref (buf);
-    gst_element_error (GST_ELEMENT (osssink), "capsnego was never performed, unknown data type");
+    GST_ELEMENT_ERROR (osssink, CORE, NEGOTIATION, (NULL), ("format wasn't negotiated before chain function"));
     return;
   }
 
-  buftime = GST_BUFFER_TIMESTAMP (buf);
+  data = GST_BUFFER_DATA (buf);
+  to_write = GST_BUFFER_SIZE (buf);
+  /* sync audio with buffers timestamp */
+  elementtime = gst_element_get_time (GST_ELEMENT (osssink));
+  soundtime = elementtime + gst_osssink_get_delay (osssink) * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+    buftime = GST_BUFFER_TIMESTAMP (buf);
+  } else {
+    buftime = soundtime;
+  }
+  GST_LOG_OBJECT (osssink, "time: element %"G_GUINT64_FORMAT", real %"G_GUINT64_FORMAT", buffer: %"G_GUINT64_FORMAT,
+      elementtime, soundtime, buftime);
+  if (MAX (buftime, soundtime) - MIN (buftime, soundtime) > GST_SECOND / 10) {
+    /* we need to adjust to the buffers here */
+    GST_INFO_OBJECT (osssink, "need sync: element %"G_GUINT64_FORMAT", real %"G_GUINT64_FORMAT", buffer: %"G_GUINT64_FORMAT, 
+	elementtime, soundtime, buftime);
+    if (soundtime > buftime) {
+      /* full frames */
+      guint throw_away = (soundtime - buftime) * GST_OSSELEMENT (osssink)->bps / GST_SECOND
+	  / GST_OSSELEMENT (osssink)->width / GST_OSSELEMENT (osssink)->channels
+	  * GST_OSSELEMENT (osssink)->width * GST_OSSELEMENT (osssink)->channels;
+      if (throw_away >= GST_BUFFER_SIZE (buf)) {
+	gst_data_unref (_data);
+	return;
+      }
+      to_write -= throw_away;
+      data += throw_away;
+    } else {
+      guint64 to_handle = (buftime - soundtime) * GST_OSSELEMENT (osssink)->bps / GST_SECOND
+	          / GST_OSSELEMENT (osssink)->width / GST_OSSELEMENT (osssink)->channels
+		  * GST_OSSELEMENT (osssink)->width * GST_OSSELEMENT (osssink)->channels;
+      /* FIXME: we really should output silence here */
+      /* round to full frames */
+      g_usleep (to_handle * G_USEC_PER_SEC / GST_OSSELEMENT (osssink)->bps);
+      osssink->handled += (buftime - soundtime) * GST_OSSELEMENT (osssink)->bps / GST_SECOND 
+	  / GST_OSSELEMENT (osssink)->width / GST_OSSELEMENT (osssink)->channels
+	  * GST_OSSELEMENT (osssink)->width * GST_OSSELEMENT (osssink)->channels;
+    }
+  }
 
   if (GST_OSSELEMENT (osssink)->fd >= 0) {
     if (!osssink->mute) {
-      guchar *data = GST_BUFFER_DATA (buf);
-      gint size = GST_BUFFER_SIZE (buf);
-      gint to_write = 0;
-
-      if (osssink->clock) {
-        gint delay = 0;
-	gint64 queued;
-        GstClockTimeDiff jitter;
-    
-	delay = gst_osssink_get_delay (osssink);
-	queued = delay * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
-
-	if  (osssink->resync && osssink->sync) {
-          GstClockID id = gst_clock_new_single_shot_id (osssink->clock, buftime - queued);
-
-          gst_element_clock_wait (GST_ELEMENT (osssink), id, &jitter);
-	  gst_clock_id_free (id);
-
-          if (jitter >= 0) {
-            gst_clock_handle_discont (osssink->clock, buftime - queued + jitter);
-	    to_write = size;
-            gst_audio_clock_set_active ((GstAudioClock*)osssink->provided_clock, TRUE);
-	    osssink->resync = FALSE;
-          }
-	}
-	else {
-	  to_write = size;
-        }
-      }
-      /* no clock, try to be as fast as possible */
-      else {
-        audio_buf_info ospace;
-
-        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETOSPACE, &ospace);
-
-        if (ospace.bytes >= size) {
-	  to_write = size;
-	}
-      }
 
       while (to_write > 0) {
         gint done = write (GST_OSSELEMENT (osssink)->fd, data, 
@@ -426,10 +411,12 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
 	  osssink->handled += done;
 	}
       }
+    } else {
+      g_warning ("muting osssinks unimplemented wrt clocks!");
     }
   }
 
-  gst_audio_clock_update_time ((GstAudioClock*)osssink->provided_clock, buftime);
+  gst_audio_clock_update_time ((GstAudioClock*)osssink->provided_clock, gst_osssink_get_time (osssink->provided_clock, osssink));
 
   gst_buffer_unref (buf);
 }
@@ -489,7 +476,7 @@ gst_osssink_sink_query (GstPad *pad, GstQueryType type, GstFormat *format, gint6
       break;
     case GST_QUERY_POSITION:
       if (!gst_osssink_convert (pad, 
-			        GST_FORMAT_TIME, gst_clock_get_time (osssink->provided_clock),
+			        GST_FORMAT_TIME, gst_element_get_time (GST_ELEMENT (osssink)),
 		                format, value)) 
       {
         res = FALSE;
@@ -529,7 +516,6 @@ gst_osssink_set_property (GObject *object, guint prop_id, const GValue *value, G
       break;
     case ARG_BUFFER_SIZE:
       osssink->bufsize = g_value_get_uint (value);
-      osssink->sinkpool = gst_buffer_pool_get_default (osssink->bufsize, 6);
       g_object_notify (object, "buffer_size");
       break;
     case ARG_SYNC:
@@ -585,18 +571,18 @@ gst_osssink_change_state (GstElement *element)
     case GST_STATE_READY_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
-      osssink->resync = TRUE;
+      gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), TRUE);
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN)) 
         ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET, 0);
       gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
-      osssink->resync = TRUE;
       break;
     case GST_STATE_PAUSED_TO_READY:
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN))
         ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET, 0);
       gst_osselement_reset (GST_OSSELEMENT (osssink));
+      osssink->handled = 0;
       break;
     default:
       break;
