@@ -413,7 +413,7 @@ gst_v4lelement_init (GstV4lElement *v4lelement)
   v4lelement->video_fd = -1;
   v4lelement->buffer = NULL;
   v4lelement->videodev = g_strdup ("/dev/video");
-  v4lelement->display = g_strdup(g_getenv("DISPLAY"));;
+  v4lelement->display = g_strdup(g_getenv("DISPLAY"));
 
   v4lelement->norms = NULL;
   v4lelement->channels = NULL;
