@@ -18,9 +18,6 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include <config.h>
 #endif
 
@@ -31,11 +28,8 @@
 static GstElementDetails gst_v4lmjpegsrc_details = {
   "Video (video4linux/MJPEG) Source",
   "Source/Video",
-  "LGPL",
   "Reads MJPEG-encoded frames from a zoran MJPEG/video4linux device",
-  VERSION,
-  "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 2001",
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
 };
 
 /* V4lMjpegSrc signals and args */
@@ -64,22 +58,32 @@ enum {
   ARG_USE_FIXED_FPS
 };
 
+GST_FORMATS_FUNCTION (GstPad *, gst_v4lmjpegsrc_get_formats,
+		      GST_FORMAT_TIME, GST_FORMAT_DEFAULT);
+GST_QUERY_TYPE_FUNCTION (GstPad *, gst_v4lmjpegsrc_get_query_types,
+			 GST_QUERY_POSITION);
 
 /* init functions */
+static void                  gst_v4lmjpegsrc_base_init	  (gpointer g_class);
 static void                  gst_v4lmjpegsrc_class_init   (GstV4lMjpegSrcClass *klass);
 static void                  gst_v4lmjpegsrc_init         (GstV4lMjpegSrc *v4lmjpegsrc);
 
-/* pad/buffer functions */
-static gboolean              gst_v4lmjpegsrc_srcconvert   (GstPad         *pad,
+/* pad/info functions */
+static gboolean              gst_v4lmjpegsrc_src_convert  (GstPad         *pad,
                                                            GstFormat      src_format,
                                                            gint64         src_value,
                                                            GstFormat      *dest_format,
                                                            gint64         *dest_value);
+static gboolean              gst_v4lmjpegsrc_src_query    (GstPad         *pad,
+                                                           GstQueryType   type, 
+                                                           GstFormat      *format,
+                                                           gint64         *value);
+
+/* buffer functions */
 static GstPadLinkReturn      gst_v4lmjpegsrc_srcconnect   (GstPad         *pad,
-                                                           GstCaps        *caps);
+                                                           const GstCaps        *caps);
 static GstData*            gst_v4lmjpegsrc_get          (GstPad         *pad);
-static GstCaps*              gst_v4lmjpegsrc_getcaps      (GstPad         *pad,
-                                                           GstCaps        *caps);
+static GstCaps*              gst_v4lmjpegsrc_getcaps      (GstPad         *pad);
 
 /* get/set params */
 static void                  gst_v4lmjpegsrc_set_property (GObject        *object,
@@ -98,18 +102,8 @@ static void                  gst_v4lmjpegsrc_set_clock    (GstElement     *eleme
 /* state handling */
 static GstElementStateReturn gst_v4lmjpegsrc_change_state (GstElement     *element);
 
-/* bufferpool functions */
-static GstBuffer*            gst_v4lmjpegsrc_buffer_new   (GstBufferPool  *pool,
-                                                           guint64        offset,
-                                                           guint          size,
-                                                           gpointer       user_data);
-static void                  gst_v4lmjpegsrc_buffer_free  (GstBufferPool  *pool,
-							   GstBuffer      *buf,
-                                                           gpointer       user_data);
-
-
-static GstCaps *capslist = NULL;
-static GstPadTemplate *src_template;
+/* requeue buffer after use */
+static void                  gst_v4lmjpegsrc_buffer_free  (GstBuffer      *buffer);
 
 static GstElementClass *parent_class = NULL;
 static guint gst_v4lmjpegsrc_signals[LAST_SIGNAL] = { 0 };
@@ -123,7 +117,7 @@ gst_v4lmjpegsrc_get_type (void)
   if (!v4lmjpegsrc_type) {
     static const GTypeInfo v4lmjpegsrc_info = {
       sizeof(GstV4lMjpegSrcClass),
-      NULL,
+      gst_v4lmjpegsrc_base_init,
       NULL,
       (GClassInitFunc)gst_v4lmjpegsrc_class_init,
       NULL,
@@ -139,6 +133,26 @@ gst_v4lmjpegsrc_get_type (void)
 }
 
 
+static void
+gst_v4lmjpegsrc_base_init (gpointer g_class)
+{
+  static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE (
+      "src",
+      GST_PAD_SRC,
+      GST_PAD_ALWAYS,
+      GST_STATIC_CAPS ("video/x-jpeg, "
+        "width = (int) [ 0, MAX ], "
+        "height = (int) [ 0, MAX ], "
+        "framerate = (double) [ 0, MAX ]"
+      )
+  );
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_set_details (gstelement_class, &gst_v4lmjpegsrc_details);
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_template));
+}
 static void
 gst_v4lmjpegsrc_class_init (GstV4lMjpegSrcClass *klass)
 {
@@ -166,15 +180,15 @@ gst_v4lmjpegsrc_class_init (GstV4lMjpegSrcClass *klass)
 #endif
 
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_QUALITY,
-    g_param_spec_int("quality","quality","quality",
-                     G_MININT,G_MAXINT,0,G_PARAM_WRITABLE));
+    g_param_spec_int("quality","Quality","JPEG frame quality",
+                     1,100,50,G_PARAM_READWRITE));
 
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_NUMBUFS,
     g_param_spec_int("num_buffers","Num Buffers","Number of Buffers",
-                     G_MININT,G_MAXINT,0,G_PARAM_READWRITE));
+                     1,256,64,G_PARAM_READWRITE));
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_BUFSIZE,
     g_param_spec_int("buffer_size", "Buffer Size", "Size of buffers",
-                     0, G_MAXINT, 0, G_PARAM_READABLE));
+                     0, 512*1024, 128*1024, G_PARAM_READABLE));
 
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_USE_FIXED_FPS,
     g_param_spec_boolean("use_fixed_fps", "Use Fixed FPS",
@@ -216,23 +230,21 @@ gst_v4lmjpegsrc_class_init (GstV4lMjpegSrcClass *klass)
 static void
 gst_v4lmjpegsrc_init (GstV4lMjpegSrc *v4lmjpegsrc)
 {
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (v4lmjpegsrc);
+
   GST_FLAG_SET(GST_ELEMENT(v4lmjpegsrc), GST_ELEMENT_THREAD_SUGGESTED);
 
-  v4lmjpegsrc->srcpad = gst_pad_new_from_template (src_template, "src");
+  v4lmjpegsrc->srcpad = gst_pad_new_from_template (
+	gst_element_class_get_pad_template (klass, "src"), "src");
   gst_element_add_pad(GST_ELEMENT(v4lmjpegsrc), v4lmjpegsrc->srcpad);
 
   gst_pad_set_get_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_get);
   gst_pad_set_getcaps_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_getcaps);
   gst_pad_set_link_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_srcconnect);
-  gst_pad_set_convert_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_srcconvert);
-
-  v4lmjpegsrc->bufferpool = gst_buffer_pool_new(
-		  			NULL,
-					NULL,
-					(GstBufferPoolBufferNewFunction)gst_v4lmjpegsrc_buffer_new,
-					NULL,
-					(GstBufferPoolBufferFreeFunction)gst_v4lmjpegsrc_buffer_free,
-					v4lmjpegsrc);
+  gst_pad_set_convert_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_src_convert);
+  gst_pad_set_formats_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_get_formats);
+  gst_pad_set_query_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_src_query);
+  gst_pad_set_query_type_function (v4lmjpegsrc->srcpad, gst_v4lmjpegsrc_get_query_types);
 
 #if 0
   v4lmjpegsrc->frame_width = 0;
@@ -283,13 +295,12 @@ gst_v4lmjpegsrc_get_fps (GstV4lMjpegSrc *v4lmjpegsrc)
   return fps;
 }
 
-
 static gboolean
-gst_v4lmjpegsrc_srcconvert (GstPad    *pad,
-                            GstFormat  src_format,
-                            gint64     src_value,
-                            GstFormat *dest_format,
-                            gint64    *dest_value)
+gst_v4lmjpegsrc_src_convert (GstPad    *pad,
+                             GstFormat  src_format,
+                             gint64     src_value,
+                             GstFormat *dest_format,
+                             gint64    *dest_value)
 {
   GstV4lMjpegSrc *v4lmjpegsrc;
   gdouble fps;
@@ -327,6 +338,40 @@ gst_v4lmjpegsrc_srcconvert (GstPad    *pad,
   return TRUE;
 }
 
+static gboolean
+gst_v4lmjpegsrc_src_query (GstPad      *pad,
+                           GstQueryType type, 
+                          GstFormat   *format,
+                           gint64      *value)
+{
+  GstV4lMjpegSrc *v4lmjpegsrc = GST_V4LMJPEGSRC (gst_pad_get_parent (pad));
+  gboolean res = TRUE;
+  gdouble fps;
+
+  if ((fps = gst_v4lmjpegsrc_get_fps(v4lmjpegsrc)) == 0)
+    return FALSE;
+
+  switch (type) {
+    case GST_QUERY_POSITION:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = v4lmjpegsrc->handled * GST_SECOND / fps;
+          break;
+        case GST_FORMAT_DEFAULT:
+          *value = v4lmjpegsrc->handled;
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    default:
+      res = FALSE;
+      break;
+  }
+
+  return res;
+}
 
 static inline gulong
 calc_bufsize (int hor_dec,
@@ -349,24 +394,17 @@ calc_bufsize (int hor_dec,
         return result;
 }
 
-#define gst_caps_get_int_range(caps, name, min, max) \
-  gst_props_entry_get_int_range(gst_props_get_entry((caps)->properties, \
-                                                    name), \
-                                min, max)
-
-
 static GstPadLinkReturn
 gst_v4lmjpegsrc_srcconnect (GstPad  *pad,
-                            GstCaps *caps)
+                            const GstCaps *caps)
 {
-  GstPadLinkReturn ret_val;
   GstV4lMjpegSrc *v4lmjpegsrc = GST_V4LMJPEGSRC(gst_pad_get_parent(pad));
   gint hor_dec, ver_dec;
   gint w, h;
   gint max_w = GST_V4LELEMENT(v4lmjpegsrc)->vcap.maxwidth,
        max_h = GST_V4LELEMENT(v4lmjpegsrc)->vcap.maxheight;
   gulong bufsize;
-  gfloat fps = gst_v4lmjpegsrc_get_fps(v4lmjpegsrc);
+  GstStructure *structure;
 
   /* in case the buffers are active (which means that we already
    * did capsnego before and didn't clean up), clean up anyways */
@@ -386,22 +424,9 @@ gst_v4lmjpegsrc_srcconnect (GstPad  *pad,
    * our own mime type back and it'll work. Other properties are to be set
    * by the src, not by the opposite caps */
 
-  if (gst_caps_has_property(caps, "width")) {
-    if (gst_caps_has_fixed_property(caps, "width")) {
-      gst_caps_get_int(caps, "width", &w);
-    } else {
-      int max;
-      gst_caps_get_int_range(caps, "width",  &max, &w);
-    }
-  }
-  if (gst_caps_has_property(caps, "height")) {
-    if (gst_caps_has_fixed_property(caps, "height")) {
-      gst_caps_get_int(caps, "height", &h);
-    } else {
-      int max;
-      gst_caps_get_int_range(caps, "height", &max, &h);
-    }
-  }
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "width", &w);
+  gst_structure_get_int (structure, "height", &h);
 
   /* figure out decimation */
   if (w >= max_w) {
@@ -463,23 +488,7 @@ gst_v4lmjpegsrc_srcconnect (GstPad  *pad,
   }
 #endif
 
-  /* we now have an actual width/height - *set it* */
-  caps = gst_caps_new("v4lmjpegsrc_caps",
-                      "video/x-jpeg",
-                      gst_props_new(
-                        "width",     GST_PROPS_INT(v4lmjpegsrc->end_width),
-                        "height",    GST_PROPS_INT(v4lmjpegsrc->end_height),
-                        "framerate", GST_PROPS_FLOAT(fps),
-                        NULL       ) );
-  if ((ret_val = gst_pad_try_set_caps(v4lmjpegsrc->srcpad, caps)) == GST_PAD_LINK_REFUSED)
-    return GST_PAD_LINK_REFUSED;
-  else if (ret_val == GST_PAD_LINK_DELAYED)
-    return GST_PAD_LINK_DELAYED;
-
-  if (!gst_v4lmjpegsrc_capture_init(v4lmjpegsrc))
-    return GST_PAD_LINK_REFUSED;
-
-  return GST_PAD_LINK_DONE;
+  return GST_PAD_LINK_OK;
 }
 
 
@@ -499,14 +508,6 @@ gst_v4lmjpegsrc_get (GstPad *pad)
       (fps = gst_v4lmjpegsrc_get_fps(v4lmjpegsrc)) == 0)
     return NULL;
 
-  buf = gst_buffer_new_from_pool(v4lmjpegsrc->bufferpool, 0, 0);
-  if (!buf)
-  {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Failed to create a new GstBuffer");
-    return NULL;
-  }
-
   if (v4lmjpegsrc->need_writes > 0) {
     /* use last frame */
     num = v4lmjpegsrc->last_frame;
@@ -576,8 +577,13 @@ gst_v4lmjpegsrc_get (GstPad *pad)
     v4lmjpegsrc->use_num_times[num] = 1;
   }
 
+  buf = gst_buffer_new ();
+  GST_BUFFER_FREE_DATA_FUNC (buf) = gst_v4lmjpegsrc_buffer_free;
+  GST_BUFFER_PRIVATE (buf) = v4lmjpegsrc;
   GST_BUFFER_DATA(buf) = gst_v4lmjpegsrc_get_buffer(v4lmjpegsrc, num);
   GST_BUFFER_SIZE(buf) = v4lmjpegsrc->last_size;
+  GST_BUFFER_MAXSIZE(buf) = v4lmjpegsrc->breq.size;
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY | GST_BUFFER_DONTFREE);
   if (v4lmjpegsrc->use_fixed_fps)
     GST_BUFFER_TIMESTAMP(buf) = v4lmjpegsrc->handled * GST_SECOND / fps;
   else /* calculate time based on our own clock */
@@ -593,27 +599,20 @@ gst_v4lmjpegsrc_get (GstPad *pad)
 
 
 static GstCaps*
-gst_v4lmjpegsrc_getcaps (GstPad  *pad,
-                         GstCaps *_caps)
+gst_v4lmjpegsrc_getcaps (GstPad  *pad)
 {
   GstV4lMjpegSrc *v4lmjpegsrc = GST_V4LMJPEGSRC(gst_pad_get_parent(pad));
   struct video_capability *vcap = &GST_V4LELEMENT(v4lmjpegsrc)->vcap;
-  GstCaps *caps;
 
   if (!GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lmjpegsrc))) {
     return NULL;
   }
 
-  caps = GST_CAPS_NEW("v4lmjpegsrc_jpeg_caps",
-                      "video/x-jpeg",
-                        "width",  GST_PROPS_INT_RANGE(vcap->maxwidth/4,
-                                                      vcap->maxwidth),
-                        "height", GST_PROPS_INT_RANGE(vcap->maxheight/4,
-                                                      vcap->maxheight),
-                        "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-                        NULL);
-
-  return caps;
+  return gst_caps_new_simple ("video/x-jpeg",
+      "width", GST_TYPE_INT_RANGE, vcap->maxwidth/4, vcap->maxwidth,
+      "height", GST_TYPE_INT_RANGE, vcap->maxheight/4, vcap->maxheight,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
 }
 
 
@@ -691,7 +690,10 @@ gst_v4lmjpegsrc_get_property (GObject    *object,
       g_value_set_int(value, v4lmjpegsrc->quality);
       break;
     case ARG_NUMBUFS:
-      g_value_set_int(value, v4lmjpegsrc->breq.count);
+      if (GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc)))
+        g_value_set_int(value, v4lmjpegsrc->breq.count);
+      else
+        g_value_set_int(value, v4lmjpegsrc->numbufs);
       break;
     case ARG_BUFSIZE:
       g_value_set_int(value, v4lmjpegsrc->breq.size);
@@ -764,6 +766,7 @@ gst_v4lmjpegsrc_set_clock (GstElement *element,
 }
 
 
+#if 0
 static GstBuffer*
 gst_v4lmjpegsrc_buffer_new (GstBufferPool *pool,
                             guint64       offset,
@@ -786,12 +789,12 @@ gst_v4lmjpegsrc_buffer_new (GstBufferPool *pool,
 
   return buffer;
 }
-
+#endif
 
 static void
-gst_v4lmjpegsrc_buffer_free (GstBufferPool *pool, GstBuffer *buf, gpointer user_data)
+gst_v4lmjpegsrc_buffer_free (GstBuffer *buf)
 {
-  GstV4lMjpegSrc *v4lmjpegsrc = GST_V4LMJPEGSRC (user_data);
+  GstV4lMjpegSrc *v4lmjpegsrc = GST_V4LMJPEGSRC (GST_BUFFER_PRIVATE (buf));
   int n;
 
   if (gst_element_get_state(GST_ELEMENT(v4lmjpegsrc)) != GST_STATE_PLAYING)
@@ -808,44 +811,6 @@ gst_v4lmjpegsrc_buffer_free (GstBufferPool *pool, GstBuffer *buf, gpointer user_
     }
 
   if (n == v4lmjpegsrc->breq.count)
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Couldn't find the buffer");
-
-  /* free the buffer struct et all */
-  gst_buffer_default_free(buf);
-}
-
-
-gboolean
-gst_v4lmjpegsrc_factory_init (GstPlugin *plugin)
-{
-  GstElementFactory *factory;
-  GstCaps *caps;
-
-  /* create an elementfactory for the v4lmjpegsrcparse element */
-  factory = gst_element_factory_new("v4lmjpegsrc",GST_TYPE_V4LMJPEGSRC,
-                                   &gst_v4lmjpegsrc_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  caps = gst_caps_new ("v4lmjpegsrc_caps",
-                       "video/x-jpeg",
-                       gst_props_new (
-                          "width",     GST_PROPS_INT_RANGE (0, G_MAXINT),
-                          "height",    GST_PROPS_INT_RANGE (0, G_MAXINT),
-                          "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
-                          NULL       )
-                      );
-  capslist = gst_caps_append(capslist, caps);
-
-  src_template = gst_pad_template_new (
-		  "src",
-                  GST_PAD_SRC,
-  		  GST_PAD_ALWAYS,
-		  capslist, NULL);
-
-  gst_element_factory_add_pad_template (factory, src_template);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, (NULL),
+      ("Couldn't find the buffer"));
 }
