@@ -35,14 +35,19 @@
 #define MAP_FAILED ( (caddr_t) -1 )
 #endif
 
+#define MIN_BUFFERS_QUEUED 2
+
 #define DEBUG(format, args...) \
-	GST_DEBUG_ELEMENT(GST_CAT_PLUGIN_INFO, \
+	GST_DEBUG_OBJECT (\
 		GST_ELEMENT(v4lmjpegsrc), \
 		"V4LMJPEGSRC: " format, ##args)
 
-
-char *input_name[] = { "Composite", "S-Video", "TV-Tuner", "Autodetect" };
-
+enum {
+  QUEUE_STATE_ERROR = -1,
+  QUEUE_STATE_READY_FOR_QUEUE,
+  QUEUE_STATE_QUEUED,
+  QUEUE_STATE_SYNCED,
+};
 
 /******************************************************
  * gst_v4lmjpegsrc_queue_frame():
@@ -56,14 +61,21 @@ gst_v4lmjpegsrc_queue_frame (GstV4lMjpegSrc *v4lmjpegsrc,
 {
   DEBUG("queueing frame %d", num);
 
-  if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_QBUF_CAPT, &num) < 0)
-  {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error queueing a buffer (%d): %s",
-      num, g_strerror(errno));
+  if (v4lmjpegsrc->frame_queue_state[num] != QUEUE_STATE_READY_FOR_QUEUE) {
     return FALSE;
   }
 
+  if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_QBUF_CAPT, &num) < 0)
+  {
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, (NULL),
+      ("Error queueing a buffer (%d): %s",
+      num, g_strerror(errno)));
+    return FALSE;
+  }
+
+  v4lmjpegsrc->frame_queue_state[num] = QUEUE_STATE_QUEUED;
+  v4lmjpegsrc->num_queued++;
+
   return TRUE;
 }
 
@@ -80,113 +92,28 @@ gst_v4lmjpegsrc_sync_next_frame (GstV4lMjpegSrc *v4lmjpegsrc,
 {
   DEBUG("syncing on next frame");
 
-  if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_SYNC, &(v4lmjpegsrc->bsync)) < 0)
-  {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error syncing on a buffer (%ld): %s",
-      v4lmjpegsrc->bsync.frame, g_strerror(errno));
+  if (v4lmjpegsrc->num_queued <= 0) {
     return FALSE;
   }
 
+  while (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd,
+               MJPIOC_SYNC, &(v4lmjpegsrc->bsync)) < 0) {
+    if (errno != EINTR) {
+      GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SYNC, NULL, GST_ERROR_SYSTEM);
+      return FALSE;
+    }
+    DEBUG("Sync got interrupted");
+  }
+
   *num = v4lmjpegsrc->bsync.frame;
 
+  v4lmjpegsrc->frame_queue_state[*num] = QUEUE_STATE_SYNCED;
+  v4lmjpegsrc->num_queued--;
+
   return TRUE;
 }
 
 
-/******************************************************
- * gst_v4lmjpegsrc_set_input_norm():
- *   set input/norm (includes autodetection), norm is
- *   VIDEO_MODE_{PAL|NTSC|SECAM|AUTO}
- * return value: TRUE on success, FALSE on error
- ******************************************************/
-
-gboolean
-gst_v4lmjpegsrc_set_input_norm (GstV4lMjpegSrc       *v4lmjpegsrc,
-                                GstV4lMjpegInputType input,
-                                gint                 norm)
-{
-  struct mjpeg_status bstat;
-
-  DEBUG("setting input = %d (%s), norm = %d (%s)",
-    input, input_name[input], norm, norm_name[norm]);
-
-  GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lmjpegsrc));
-  GST_V4L_CHECK_NOT_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
-
-  if (input == V4L_MJPEG_INPUT_AUTO)
-  {
-    int n;
-
-    for (n=V4L_MJPEG_INPUT_COMPOSITE;n<V4L_MJPEG_INPUT_AUTO;n++)
-    {
-      gst_info("Trying %s as input...\n",
-        input_name[n]);
-      bstat.input = n;
-
-      if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_STATUS, &bstat) < 0)
-      {
-        gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-          "Error getting device status: %s",
-          g_strerror(errno));
-        return FALSE;
-      }
-
-      if (bstat.signal)
-      {
-        input = bstat.input;
-        if (norm == VIDEO_MODE_AUTO)
-          norm = bstat.norm;
-        gst_info("Signal found: on input %s, norm %s\n",
-          input_name[bstat.input], norm_name[bstat.norm]);
-        break;
-      }
-    }
-
-    /* check */
-    if (input == V4L_MJPEG_INPUT_AUTO || norm == VIDEO_MODE_AUTO)
-    {
-      gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-        "Unable to auto-detect an input");
-      return FALSE;
-    }
-
-    /* save */
-    GST_V4LELEMENT(v4lmjpegsrc)->channel = input;
-    GST_V4LELEMENT(v4lmjpegsrc)->norm = norm;
-  }
-  else if (norm == VIDEO_MODE_AUTO && input)
-  {
-    bstat.input = input;
-
-    if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_STATUS, &bstat) < 0)
-    {
-      gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-        "Error getting device status: %s",
-        g_strerror(errno));
-      return FALSE;
-    }
-
-    if (bstat.signal)
-    {
-      norm = bstat.norm;
-      gst_info("Norm %s detected on input %s\n",
-        norm_name[bstat.norm], input_name[input]);
-      GST_V4LELEMENT(v4lmjpegsrc)->norm = norm;
-    }
-    else
-    {
-      gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-        "No signal found on input %s",
-        input_name[input]);
-      return FALSE;
-    }
-  }
-
-  return gst_v4l_set_chan_norm(GST_V4LELEMENT(v4lmjpegsrc), input, norm);
-}
-
-
 /******************************************************
  * gst_v4lmjpegsrc_set_buffer():
  *   set buffer parameters (size/count)
@@ -234,9 +161,7 @@ gst_v4lmjpegsrc_set_capture (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Query params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_PARAMS, &bparm) < 0)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error getting video parameters: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -262,9 +187,7 @@ gst_v4lmjpegsrc_set_capture (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Set params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_S_PARAMS, &bparm) < 0)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error setting video parameters: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -308,9 +231,7 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Query params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_PARAMS, &bparm) < 0)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error getting video parameters: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -318,7 +239,8 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
   bparm.quality = quality;
   bparm.norm = norm;
   bparm.input = input;
-  bparm.APP_len = 0; /* no JPEG markers - TODO: this is definately not right for decimation==1 */
+  bparm.APP_len = 0; /* no JPEG markers - TODO: this is definately
+                      * not right for decimation==1 */
 
   if (width <= 0)
   {
@@ -344,23 +266,23 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
 
   if (width + x_offset > maxwidth)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Image width+offset (%d) bigger than maximum (%d)",
-      width + x_offset, maxwidth);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+      ("Image width+offset (%d) bigger than maximum (%d)",
+      width + x_offset, maxwidth));
     return FALSE;
   }
   if ((width%(bparm.HorDcm*16))!=0) 
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Image width (%d) not multiple of %d (required for JPEG)",
-      width, bparm.HorDcm*16);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, NULL,
+      ("Image width (%d) not multiple of %d (required for JPEG)",
+      width, bparm.HorDcm*16));
     return FALSE;
   }
   if (height + y_offset > (norm==VIDEO_MODE_NTSC ? 480 : 576)) 
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Image height+offset (%d) bigger than maximum (%d)",
-      height + y_offset, (norm==VIDEO_MODE_NTSC ? 480 : 576));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+      ("Image height+offset (%d) bigger than maximum (%d)",
+      height + y_offset, (norm==VIDEO_MODE_NTSC ? 480 : 576)));
     return FALSE;
   }
   /* RJ: Image height must only be a multiple of 8, but geom_height
@@ -368,9 +290,9 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
    */
   if ((height%(bparm.VerDcm*16))!=0) 
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Image height (%d) not multiple of %d (required for JPEG)",
-      height, bparm.VerDcm*16);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, NULL,
+      ("Image height (%d) not multiple of %d (required for JPEG)",
+      height, bparm.VerDcm*16));
     return FALSE;
   }
 
@@ -391,9 +313,7 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Set params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_S_PARAMS, &bparm) < 0)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error setting video parameters: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -415,26 +335,44 @@ gst_v4lmjpegsrc_capture_init (GstV4lMjpegSrc *v4lmjpegsrc)
   GST_V4L_CHECK_NOT_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
 
   /* Request buffers */
-  if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_REQBUFS, &(v4lmjpegsrc->breq)) < 0)
+  if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd,
+            MJPIOC_REQBUFS, &(v4lmjpegsrc->breq)) < 0)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error requesting video buffers: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, NULL, GST_ERROR_SYSTEM);
+    return FALSE;
+  }
+
+  if (v4lmjpegsrc->breq.count < MIN_BUFFERS_QUEUED)
+  {
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, NULL,
+      ("Too little buffers. We got %ld, we want at least %d",
+      v4lmjpegsrc->breq.count, MIN_BUFFERS_QUEUED));
     return FALSE;
   }
 
   gst_info("Got %ld buffers of size %ld KB\n",
     v4lmjpegsrc->breq.count, v4lmjpegsrc->breq.size/1024);
 
+  /* keep track of queued buffers */
+  v4lmjpegsrc->frame_queue_state = (gint8 *)
+    g_malloc(sizeof(gint8) * v4lmjpegsrc->breq.count);
+
+  /* track how often to use each frame */
+  v4lmjpegsrc->use_num_times = (gint *)
+    g_malloc(sizeof(gint) * v4lmjpegsrc->breq.count);
+
+  /* lock for the frame_state */
+  v4lmjpegsrc->mutex_queue_state = g_mutex_new();
+  v4lmjpegsrc->cond_queue_state = g_cond_new();
+
   /* Map the buffers */
   GST_V4LELEMENT(v4lmjpegsrc)->buffer = mmap(0,
     v4lmjpegsrc->breq.count * v4lmjpegsrc->breq.size, 
     PROT_READ|PROT_WRITE, MAP_SHARED, GST_V4LELEMENT(v4lmjpegsrc)->video_fd, 0);
   if (GST_V4LELEMENT(v4lmjpegsrc)->buffer == MAP_FAILED)
   {
-    gst_element_error(GST_ELEMENT(v4lmjpegsrc),
-      "Error mapping video buffers: %s",
-      g_strerror(errno));
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+      ("Error mapping video buffers: %s", g_strerror (errno)));
     GST_V4LELEMENT(v4lmjpegsrc)->buffer = NULL;
     return FALSE;
   }
@@ -458,10 +396,23 @@ gst_v4lmjpegsrc_capture_start (GstV4lMjpegSrc *v4lmjpegsrc)
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lmjpegsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
 
-  /* queue'ing the buffers starts streaming capture */
-  for (n=0;n<v4lmjpegsrc->breq.count;n++)
-    if (!gst_v4lmjpegsrc_queue_frame(v4lmjpegsrc, n))
+  g_mutex_lock(v4lmjpegsrc->mutex_queue_state);
+
+  v4lmjpegsrc->quit = FALSE;
+  v4lmjpegsrc->num_queued = 0;
+  v4lmjpegsrc->queue_frame = 0;
+
+  /* set all buffers ready to queue , this starts streaming capture */
+  for (n=0;n<v4lmjpegsrc->breq.count;n++) {
+    v4lmjpegsrc->frame_queue_state[n] = QUEUE_STATE_READY_FOR_QUEUE;
+    if (!gst_v4lmjpegsrc_queue_frame(v4lmjpegsrc, n)) {
+      g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
+      gst_v4lmjpegsrc_capture_stop(v4lmjpegsrc);
       return FALSE;
+    }
+  }
+
+  g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -482,12 +433,41 @@ gst_v4lmjpegsrc_grab_frame (GstV4lMjpegSrc *v4lmjpegsrc,
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lmjpegsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
 
+  g_mutex_lock(v4lmjpegsrc->mutex_queue_state);
+
+  /* do we have enough frames? */
+  while (v4lmjpegsrc->num_queued < MIN_BUFFERS_QUEUED ||
+         v4lmjpegsrc->frame_queue_state[v4lmjpegsrc->queue_frame] ==
+           QUEUE_STATE_READY_FOR_QUEUE) {
+    while (v4lmjpegsrc->frame_queue_state[v4lmjpegsrc->queue_frame] !=
+             QUEUE_STATE_READY_FOR_QUEUE &&
+           !v4lmjpegsrc->quit) {
+      GST_DEBUG (
+                "Waiting for frames to become available (%d < %d)",
+                v4lmjpegsrc->num_queued, MIN_BUFFERS_QUEUED);
+      g_cond_wait(v4lmjpegsrc->cond_queue_state,
+                  v4lmjpegsrc->mutex_queue_state);
+    }
+    if (v4lmjpegsrc->quit) {
+      g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
+      return TRUE; /* it won't get through anyway */
+    }
+    if (!gst_v4lmjpegsrc_queue_frame(v4lmjpegsrc, v4lmjpegsrc->queue_frame)) {
+      g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
+      return FALSE;
+    }
+    v4lmjpegsrc->queue_frame = (v4lmjpegsrc->queue_frame + 1) % v4lmjpegsrc->breq.count;
+  }
+
   /* syncing on the buffer grabs it */
-  if (!gst_v4lmjpegsrc_sync_next_frame(v4lmjpegsrc, num))
+  if (!gst_v4lmjpegsrc_sync_next_frame(v4lmjpegsrc, num)) {
     return FALSE;
+  }
 
   *size = v4lmjpegsrc->bsync.length;
 
+  g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
+
   return TRUE;
 }
 
@@ -502,6 +482,8 @@ guint8 *
 gst_v4lmjpegsrc_get_buffer (GstV4lMjpegSrc *v4lmjpegsrc,
                             gint           num)
 {
+  /*DEBUG("gst_v4lmjpegsrc_get_buffer(), num = %d", num);*/
+
   if (!GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc)) ||
       !GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lmjpegsrc)))
     return NULL;
@@ -527,8 +509,23 @@ gst_v4lmjpegsrc_requeue_frame (GstV4lMjpegSrc *v4lmjpegsrc,
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lmjpegsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
 
-  if (!gst_v4lmjpegsrc_queue_frame(v4lmjpegsrc, num))
+  /* mark frame as 'ready to requeue' */
+  g_mutex_lock(v4lmjpegsrc->mutex_queue_state);
+
+  if (v4lmjpegsrc->frame_queue_state[num] != QUEUE_STATE_SYNCED) {
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+                      ("Invalid state %d (expected %d), can't requeue",
+                      v4lmjpegsrc->frame_queue_state[num],
+                      QUEUE_STATE_SYNCED));
     return FALSE;
+  }
+
+  v4lmjpegsrc->frame_queue_state[num] = QUEUE_STATE_READY_FOR_QUEUE;
+
+  /* let an optional wait know */
+  g_cond_broadcast(v4lmjpegsrc->cond_queue_state);
+
+  g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -543,13 +540,24 @@ gst_v4lmjpegsrc_requeue_frame (GstV4lMjpegSrc *v4lmjpegsrc,
 gboolean
 gst_v4lmjpegsrc_capture_stop (GstV4lMjpegSrc *v4lmjpegsrc)
 {
+  int n;
+
   DEBUG("stopping capture");
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lmjpegsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lmjpegsrc));
 
-  /* unqueue the buffers */
-  if (!gst_v4lmjpegsrc_queue_frame(v4lmjpegsrc, -1))
-    return FALSE;
+  g_mutex_lock(v4lmjpegsrc->mutex_queue_state);
+
+  /* make an optional pending wait stop */
+  v4lmjpegsrc->quit = TRUE;
+  g_cond_broadcast(v4lmjpegsrc->cond_queue_state);
+                                                                                
+  /* sync on remaining frames */
+  while (v4lmjpegsrc->num_queued > 0) {
+    gst_v4lmjpegsrc_sync_next_frame(v4lmjpegsrc, &n);
+  }
+
+  g_mutex_unlock(v4lmjpegsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -572,5 +580,11 @@ gst_v4lmjpegsrc_capture_deinit (GstV4lMjpegSrc *v4lmjpegsrc)
   munmap(GST_V4LELEMENT(v4lmjpegsrc)->buffer, v4lmjpegsrc->breq.size * v4lmjpegsrc->breq.count);
   GST_V4LELEMENT(v4lmjpegsrc)->buffer = NULL;
 
+  /* free buffer tracker */
+  g_mutex_free(v4lmjpegsrc->mutex_queue_state);
+  g_cond_free(v4lmjpegsrc->cond_queue_state);
+  g_free(v4lmjpegsrc->frame_queue_state);
+  g_free(v4lmjpegsrc->use_num_times);
+
   return TRUE;
 }
