@@ -31,7 +31,7 @@
 #include "v4l2_calls.h"
 
 #define DEBUG(format, args...) \
-	GST_DEBUG_ELEMENT(GST_CAT_PLUGIN_INFO, \
+	GST_DEBUG_OBJECT (\
 		GST_ELEMENT(v4l2element), \
 		"V4L2-overlay: " format, ##args)
 
@@ -43,28 +43,33 @@
  ******************************************************/
 
 gboolean
-gst_v4l2_set_display (GstV4l2Element *v4l2element,
-                      const gchar    *display)
+gst_v4l2_set_display (GstV4l2Element *v4l2element)
 {
 	gchar *buff;
 
-	DEBUG("trying to set overlay to '%s'", display);
+	if (v4l2element->display)
+		g_free(v4l2element->display);
+	v4l2element->display = g_strdup(g_getenv("DISPLAY"));
+
+	DEBUG("trying to set overlay to '%s'", v4l2element->display);
 
 	/* start v4l-conf */
 	buff = g_strdup_printf("v4l-conf -q -c %s -d %s",
-		v4l2element->device?v4l2element->device:"/dev/video", display);
+		v4l2element->device, v4l2element->display);
 
 	switch (system(buff)) {
 		case -1:
-			gst_element_error(GST_ELEMENT(v4l2element),
-				"Could not start v4l-conf: %s", g_strerror(errno));
+			GST_ELEMENT_ERROR (v4l2element, RESOURCE, FAILED,
+                                           (_("Could not start v4l-conf.")),
+                                           GST_ERROR_SYSTEM);
 			g_free(buff);
 			return FALSE;
 		case 0:
 			break;
 		default:
-			gst_element_error(GST_ELEMENT(v4l2element),
-				"v4l-conf failed to run correctly: %s", g_strerror(errno));
+			GST_ELEMENT_ERROR (v4l2element, RESOURCE, FAILED,
+                                           (_("Executing v4l-conf failed.")),
+                                           GST_ERROR_SYSTEM);
 			g_free(buff);
 			return FALSE;
 	}
@@ -81,7 +86,7 @@ gst_v4l2_set_display (GstV4l2Element *v4l2element,
  ******************************************************/
 
 gboolean
-gst_v4l2_set_window (GstV4l2Element   *v4l2element,
+gst_v4l2_set_window (GstElement       *element,
                      gint              x,
                      gint              y,
                      gint              w,
@@ -90,6 +95,7 @@ gst_v4l2_set_window (GstV4l2Element   *v4l2element,
                      gint              num_clips)
 {
 	struct v4l2_format fmt;
+	GstV4l2Element *v4l2element = GST_V4L2ELEMENT(element);
 
 	DEBUG("trying to set video window to %dx%d,%d,%d", x,y,w,h);
 	GST_V4L2_CHECK_OVERLAY(v4l2element);
@@ -106,9 +112,8 @@ gst_v4l2_set_window (GstV4l2Element   *v4l2element,
 	fmt.fmt.win.bitmap = NULL;
 
 	if (ioctl(v4l2element->video_fd, VIDIOC_S_FMT, &fmt) < 0) {
-		gst_element_error(GST_ELEMENT(v4l2element),
-			"Failed to set the video window on device %s: %s",
-			v4l2element->device, g_strerror(errno));
+		GST_ELEMENT_ERROR (v4l2element, RESOURCE, TOO_LAZY, (NULL),
+			("Failed to set the video window: %s", g_strerror (errno)));
 		return FALSE;
 	}
 
@@ -133,9 +138,9 @@ gst_v4l2_enable_overlay (GstV4l2Element *v4l2element,
 	GST_V4L2_CHECK_OVERLAY(v4l2element);
 
 	if (ioctl(v4l2element->video_fd, VIDIOC_OVERLAY, &doit) < 0) {
-		gst_element_error(GST_ELEMENT(v4l2element),
-			"Failed to %s overlay display for device %s: %s",
-			enable?"enable":"disable", v4l2element->device, g_strerror(errno));
+		GST_ELEMENT_ERROR (v4l2element, RESOURCE, TOO_LAZY, (NULL),
+			("Failed to %s overlay display: %s",
+			 enable?"enable":"disable", g_strerror (errno)));
 		return FALSE;
 	}
 
